 /*******************************************************************************
 *
 *                draw_skel
 *
 *******************************************************************************
 *
 *  File         : draw_skel_avs.cc
 *
 *  Type         : Implementation, Interface
 * 
 *  Purpose      : 
 *
 *******************************************************************************
 *                 
 *  Author       : Markus Naef
 *  started      :  6-May-94
 *  last change  :  6-May-94
 *
 *******************************************************************************/
#include <stdio.h>
#include <math.h>
#include <avs/avs.h>
#include <avs/field.h>
#include <avs/geom.h>
#include <avs/colormap.h>
#include <avs/udata.h>
#include <avs/image_draw.h>
#include <avs/image_upstrm.h>

#include "Sy_Skeleton.h"
#include "Sy_Graphics.h"
#include "Sy_Avs.h"

#define BUFSIZE 1024

AVS_STATIC(int Sy_AvsDrawSkeleton_compute,(char *in_file,AVSfield_char *in,image_viewer_id *iv_id,mouse_info *mouseInfo,image_draw **im_draw,int is_raw, char* display, int rank, float *th_p));

int
Sy_AvsDrawSkeleton_desc()
/******************************************************************************
 *
 * 
 * 
 *
 *****************************************************************************/
{
   extern char *AVSpath;
   char		spath[128];
   int port,param;

   AVSset_module_name("draw skeleton",MODULE_RENDER);
   AVScreate_input_port("filename of input","string",REQUIRED);
   AVScreate_input_port("input","field 2D byte",OPTIONAL);
   port=AVScreate_input_port("Image Viewer Id","struct image_viewer_id",
				 INVISIBLE|OPTIONAL);
   AVSset_input_class(port,"Image Draw:image_viewer_id");
   port=AVScreate_input_port("Mouse Info","struct mouse_info",
				 INVISIBLE|OPTIONAL);
   AVSset_input_class(port,"Image Draw:mouse_info");
   AVScreate_output_port("Image Draw","struct image_draw");
   
   AVSadd_parameter("raw data","boolean",0,0,1);
   AVSadd_parameter("display mode","choice","Plain","Plain!Foreground!Background","!");
   AVSadd_parameter("rank","integer",1,0,100);
   AVSadd_float_parameter("th",3.0,0.0,100.0);

   AVSset_compute_proc((AVS_FNCP)Sy_AvsDrawSkeleton_compute);
   if (AVSpath != NULL)
     strcpy(spath, AVSpath);
   else
     strcpy(spath, "/usr/avs");
   strcat(spath, "/include/image_draw.h");
   AVSload_user_data_types(spath);
   return(1);
}

static int
Sy_AvsDrawSkeleton_compute(
/*******************************************************************************
 *
 * AVS computation function
 *
 * 
 *
 *
 ******************************************************************************/
char *in_file,
AVSfield_char	*in,
image_viewer_id *iv_id,
mouse_info      *mouseInfo,
image_draw **im_draw,
int is_raw, 
char* display, 
int rank, 
float *th_p)
{
   enum PruneOp {PRUNE_WITH_IPD, PRUNE_WITH_TRIAN, PRUNE_WITH_IPD_PYR, 
		   NO_PRUNEOP};

   static SyIpoDAC    *ipd;
   static SySkelDAC   *vsk;
   static enum PruneOp oper = NO_PRUNEOP;

   int           no_ipo_found = SY_TRUE,no_skel_found = SY_TRUE;
   int display_mode=SKE_PLAIN;
   SyIOhandle    *iohandle;
   float th=*th_p;


   int dims[2];
   static int not_displayed=1;
   char *outbuf, *errbuf;
   char command[BUFSIZE];
   int imageChangedFlag = 0;
   static char imageName[80];
   static int imageNameInited = 0;
   int mask,i;

   if (strcmp(display,"Plain")) display_mode=SKE_PLAIN;
   if (strcmp(display,"Background")) display_mode=SKE_BACKGROUND;
   if (strcmp(display,"Foreground")) display_mode=SKE_FOREGROUND;

   if (AVSinput_changed("filename of input", 0)) {
      system("pwd;");
      Sy_ManageInput (in_file);
      while ((iohandle = Sy_NextIOhandle()) != NULL) {
	 switch (Sy_GetIOproperty (iohandle, TYPE).type){
	  case SY_IPD_STRIP:
	    ipd = (SyIpoDAC*) (Sy_GetIOproperty(iohandle, DATA).data);
	    oper = PRUNE_WITH_IPD;
	    no_ipo_found=0;
	    break;
	  case SY_SKEL_STRIP:
	    vsk = (SySkelDAC*) (Sy_GetIOproperty(iohandle, DATA).data);
	    oper = PRUNE_WITH_IPD;
	    no_skel_found=0;
	    break;
	 };
      };
      
      if (no_skel_found){
	 Sy_SimpleError ("No SySkelDAC structure found in %s\n", in_file);
      }
      if (no_ipo_found){
	 Sy_SimpleError ("No SyIpoDAC structure found in %s\n", in_file);
      }
   };
   
   
   if (*im_draw == NULL) *im_draw = (image_draw*)AVSdata_alloc("struct image_draw",0);
   if (in != NULL) AVSfield_get_dimensions((AVSfield *)in,dims);

   if (iv_id == NULL) {
      /* no image viewer connection */
      imageName[0] = '\0';
      imageNameInited = 0;
      return(1);
   };
   
   if (AVSinput_changed("Image Viewer Id", 0)) {
      if (iv_id && in)
	imageChangedFlag = 1;
   };

   if (AVSinput_changed("input", 0)) imageChangedFlag=1;


   if (imageChangedFlag && in) {
      /* request that mouse info be sent to us when appropriate (i.e. when
       * image viewer pick mode is set to us)
       */
      sprintf(command,
	      "image_wakeup_when_defined -scene_id %d -mesh_id %d -module \"$Module\"",
	      iv_id->scene_id, in->mesh_id);
      AVScommand("kernel", command, &outbuf, &errbuf);
   }
   
   if (in == NULL && imageNameInited) {
      
      /* input connection was broken - request that no events be sent */
      sprintf(command,
	      "image_stop_events -scene_id %d -image_name \"%s\" -module $Module -func_id draw_skel", 
	      iv_id->scene_id, imageName);
      AVScommand("kernel", command, &outbuf, &errbuf);
      imageName[0] = '\0';
      imageNameInited = 0;
   }

   if (mouseInfo && AVSinput_changed("Mouse Info", 0)) {
      if (mouseInfo->nEvents == 0) {
	 /* image viewer connection established and image selected */
	 if (!imageNameInited) {
	    /* first time - save image name for later check for changes */
	    strcpy(imageName, mouseInfo->image_name);
	    imageNameInited = 1;
	 }
	 else if (strcmp(imageName,mouseInfo->image_name) != 0) {
	    /* new image: first stop events for old image */
	    sprintf(command,
		    "image_clear_all_lines -scene_id %d -image_name \"%s\" -module $Module", iv_id->scene_id, imageName);
	    AVScommand("kernel", command, &outbuf, &errbuf);
	    not_displayed=1;
	    sprintf(command,
		    "image_stop_events -scene_id %d -image_name \"%s\" -module $Module -func_id voronoi", iv_id->scene_id, imageName);
	    AVScommand("kernel", command, &outbuf, &errbuf);
	    strcpy(imageName, mouseInfo->image_name);
	 }

	 /* request mouse events on the image (of the following type) */
	 mask = IMAGE_Button1Mask | IMAGE_PressMask;
	 sprintf(command,
		 "image_select_events -scene_id %d -image_name \"%s\" -module $Module -func_id draw_skel -mask %d", iv_id->scene_id, imageName, mask);
	 AVScommand("kernel", command, &outbuf, &errbuf);
	 
	 /* set the pick mode to us so user doesn't have to hit button */
	 sprintf(command, "image_set_pick_mode draw_skel");
	 AVScommand("kernel", command, &outbuf, &errbuf);
      } else {
	 /* received mouse press/release motion events */
	 /* for each event received... */
	 for (i = 0; i < mouseInfo->nEvents; i++) {
	    if ((mouseInfo->buttonMask[i] & IMAGE_Button1Mask) &&
		(mouseInfo->buttonMask[i] & IMAGE_PressMask)&&
		not_displayed&&in) {
	       /* button 1 pressed -> draw object in green */
	       sprintf(command,"image_set_line_color -scene_id %d -image_name \"%s\" -module $Module %f %f %f",iv_id->scene_id,imageName, 1.0, 1.0 ,0.0);
	       AVScommand("kernel", command, &outbuf, &errbuf);
	       switch (oper){
		case PRUNE_WITH_IPD:
		  if (!is_raw) Sy_Ipo2AVS (ipd, iv_id, imageName);      
		  Sy_Skel2AVS(vsk, ipd, th, display_mode, iv_id,imageName);
		  break;
		case PRUNE_WITH_IPD_PYR:
		  if (!is_raw) Sy_Ipo2AVS (ipd, iv_id, imageName);      
		  Sy_SkelPyr2AVS(vsk, ipd, rank, display_mode, vsk->hier, iv_id,imageName);
		  break; 
	       } 
	       
	       sprintf(command,"image_set_line_color -scene_id %d -image_name \"%s\" -module $Module %f %f %f",iv_id->scene_id,imageName, 1.0, 1.0 ,1.0);
	       AVScommand("kernel", command, &outbuf, &errbuf);
	       not_displayed=0;
	       sprintf(command,"image_set_image_transformation -image \"%s\" -tx 0.0 -ty 0.0 -sx %f -sy %f",imageName,1.0,1.0);
	       AVScommand("kernel", command, &outbuf, &errbuf);
	    } else if ((mouseInfo->buttonMask[i] & IMAGE_Button1Mask) &&
		       (mouseInfo->buttonMask[i] & IMAGE_PressMask)&&
		       !not_displayed) {
	       /* button 1 pressed and image was already drawn -> clear object */
	       sprintf(command,
		       "image_clear_all_lines -scene_id %d -image_name \"%s\" -module $Module", iv_id->scene_id, imageName);
	       AVScommand("kernel", command, &outbuf, &errbuf);
	       sprintf(command,"image_set_line_color -scene_id %d -image_name \"%s\" -module $Module %f %f %f",iv_id->scene_id,imageName, 0.0, 1.0 ,0.0);
	       AVScommand("kernel", command, &outbuf, &errbuf);
	       not_displayed=1;
	    }
	 }
      }
   } 
   return 1;
}




