/*
 *  $Id$
 * 
 *  Author: Robert L. Ogniewicz, Swiss Federal Institute of Technology ETH
 *          Zurich, Switzerland (rlo@vision.ethz.ch)
 */

/* Copyright Notice
 * ================
 * Copyright (C) 1993 by Robert L. Ogniewicz
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more Details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
 */

/*
 * AVS Skeleton Display Module
 *
 */

static char sySkelViewId[] = "$Id";

#include <stdio.h>

#include <avs/avs.h>
#include <avs/avs_data.h>
#include <avs/geom.h>
#include <avs/flow.h>

#include "Sy_Avs.h"
#include "Sy_Skeleton.h"

static char par_inskel_string[] = "Skeleton Project File";
static char par_mode_string[]   = "Render Mode";
static char par_active_string[] = "Module State";
static char par_recon_string[]  = "Add Spheres";
static char par_res_string[]    = "Residual Epsilon";
static char par_loc_string[]    = "Locus Epsilon";
static char par_fgbg_string[]   = "foreground/background/all";
static char par_color_string[]  = "Color Mode";
static char par_cont_string[]   = "Continuity";
static char par_narr_string[]   = "Narrowings";
static char par_prox_string[]   = "Proximity";

static SyIpoDAC   *ipd;
static SyVoroDAC  *vd;
static SySkelDAC  *vsk;
static SyTriDAC   *tri;
static SyObject   olist[128];

static SyObjectList list;
static GEOMobj *geomlist[128];

int Sy_SkelView_compute (char* infilename, 
			 GEOMedit_list *output, 
			 char *filename, 
			 char *mode_string, char *recon_string, 
			 int tri_fgbg, char *active_string, 
			 float *heigth, float *value_limit, 
			 float *loc_eps, float *res_eps, float *thres,
			 char *color_string, float *cont_thres, 
			 float *narr_thres, float *prox_thres);

int Sy_AvsSkelView_desc(void)
{
  int par_inskel,
      par_heigth,
      par_mode,
      par_active,
      par_recon,
      par_res_eps,
      par_loc_eps,
      par_thres,
      par_fgbg,
      par_cut,
      par_color,
      par_cont,
      par_narr,
      par_prox;
  
  AVSset_module_name("skeleton to geom", MODULE_DATA);
   
  AVScreate_input_port("input filename","string", OPTIONAL);

  AVScreate_output_port("Geometry Output", "geom"); 

  /* parameter */

  par_inskel = AVSadd_parameter(par_inskel_string, "string", 
				0, 0, ".ipo.vor.ske.point.crack.voro.skel");

  par_mode   = AVSadd_parameter(par_mode_string, "choice", "2D View", 
				"2D View|3D Stripes|Radii|Residuals|Birth-Vector", "|");

  par_recon  = AVSadd_parameter(par_recon_string, "choice", "NoReconstruct", 
				"Reconstruct|NoReconstruct", "|");

  par_fgbg   = AVSadd_parameter(par_fgbg_string, "tristate", 0, 0, 0);

  par_active = AVSadd_parameter(par_active_string, "choice", "On",
				"On|Off", "|");

  par_heigth = AVSadd_float_parameter("Heigth", 3.0, 1.0, 20.0);
  par_cut    = AVSadd_float_parameter("Cut", 100.0, 1.0, 10000.0);

  par_loc_eps= AVSadd_float_parameter(par_loc_string, 0.05, 0.001, 42.0);
  par_res_eps= AVSadd_float_parameter(par_res_string, 0.05, 0.001, 0.2);

  par_thres  = AVSadd_float_parameter("Threshold", 2.0, -2.0, 100.0);

  par_color  = AVSadd_parameter(par_color_string, "choice", "Mono", 
				"Mono|Color", "|");

  par_cont = AVSadd_float_parameter(par_cont_string,  1.0,  0.0, 3.0);
  par_narr = AVSadd_float_parameter(par_narr_string,  1.0,  0.0, 3.0);
  par_prox = AVSadd_float_parameter(par_prox_string, -0.5, -2.0, 3.0);


  /* widget definition */

  AVSconnect_widget(par_inskel,  "browser");
  AVSconnect_widget(par_thres,   "dial");
  AVSconnect_widget(par_heigth,  "slider");
  AVSconnect_widget(par_cut,     "slider");
  AVSconnect_widget(par_mode,    "radio_buttons");
  AVSconnect_widget(par_recon,   "radio_buttons");
  AVSconnect_widget(par_active,  "radio_buttons");
  AVSconnect_widget(par_res_eps, "typein_real");
  AVSconnect_widget(par_loc_eps, "typein_real");
  AVSconnect_widget(par_fgbg,    "tristate");

  AVSconnect_widget(par_color,   "radio_buttons");
  AVSconnect_widget(par_cont,    "slider");
  AVSconnect_widget(par_narr,    "slider");
  AVSconnect_widget(par_prox,    "slider");

  AVSadd_parameter_prop (par_thres, "accumulator" , "boolean", 1);
  AVSadd_parameter_prop (par_thres, "local_range" , "real", 100.0);

  /* Compute procedure */

  AVSset_compute_proc(Sy_SkelView_compute); 

  ipd = (SyIpoDAC*) NULL;
  vd  = (SyVoroDAC*) NULL;
  vsk = (SySkelDAC*) NULL;
  tri = (SyTriDAC*) NULL;
  list.dim = 0;
  list.maxdim = sizeof(olist)/sizeof(SyObject);
  list.object = olist;
}

/* 
** computation function 
*/

int Sy_SkelView_compute (char* infilename, 
			 GEOMedit_list *output, 
			 char *filename, 
			 char *mode_string, char *recon_string, 
			 int tri_fgbg, char *active_string, 
			 float *heigth, float *value_limit, 
			 float *loc_eps, float *res_eps, float *thres,
			 char *color_string, float *cont_thres, 
			 float *narr_thres, float *prox_thres)
{
  GEOMobj *skeleton, *ipo, *sphere, *voro, *triangulation;
  char    *skelname, *iponame, *voroname, *triname, *current_infilename;

  SyBoolean  is_new_skel, is_reconstruct,
             no_ipo_found  = SY_TRUE, 
             no_skel_found = SY_TRUE,
             no_voro_found = SY_TRUE,
             no_tri_found  = SY_TRUE;

  SyDisplayObject display_object;
  SyIOhandle *iohandle;

  enum SyRenderMethod  render_method;
  enum SyTriangleDecor triangle_decor;
  SySGroupThres sogthres;
  
  int geomlistdim = 0, i;


  if (filename == NULL && infilename == NULL) return(0);

  switch (AVSchoice_number (par_active_string, active_string)){
  case 1: break;
  case 2: return(0); break;
  }

  switch (tri_fgbg){
  case 0: Sy_AvsSetSyDataFocus (FOREGROUND); break;
  case 1: Sy_AvsSetSyDataFocus (BACKGROUND); break;
  case 2: Sy_AvsSetSyDataFocus (ALLDATA);    break;
  }

  if (AVSparameter_changed (par_loc_string)) Sy_SetLocEpsilon (*loc_eps);
  if (AVSparameter_changed (par_res_string)) Sy_SetResEpsilon (*res_eps);

  switch (AVSchoice_number (par_recon_string, recon_string)){
  case 1: is_reconstruct = 1; break;
  case 2: is_reconstruct = 0; break;
  }

  if (AVSparameter_changed(par_inskel_string)){
    is_new_skel = SY_TRUE;
    current_infilename = filename;
  }
  else if (AVSinput_changed("input filename", 0)){
    is_new_skel = SY_TRUE;
    current_infilename = infilename;
  }
  else{is_new_skel = SY_FALSE;} 

  if (is_new_skel){

    list.dim = geomlistdim = 0;

    if (vsk != NULL){
      Sy_DestroySkeleton(vsk);
      Sy_Free(vsk); vsk = (SySkelDAC*) NULL;}

    if (vd != NULL){
      Sy_DestroyVoronoi(vd);
      Sy_Free(vd); vd = (SyVoroDAC*) NULL;}

    if (ipd != NULL){
      Sy_DestroyIpo(ipd);
      Sy_Free (ipd); ipd = (SyIpoDAC*) NULL;}

    if (tri != NULL){
      Sy_DestroyTrian(tri);
      Sy_Free (tri); tri = (SyTriDAC*) NULL;}

    Sy_ManageInput (current_infilename);

    while (((iohandle = Sy_NextIOhandle()) != NULL)){

      switch (Sy_GetIOproperty (iohandle, TYPE).type){
      case SY_IPD_STRIP:
        ipd = (SyIpoDAC*) (Sy_GetIOproperty(iohandle, DATA).data);
	list.object[list.dim].ptr = (SY_VOID*) ipd;
	list.object[list.dim].tag = SY_IPO; list.dim++;
        break;
      case SY_VORO_STRIP:
        vd = (SyVoroDAC*) (Sy_GetIOproperty(iohandle, DATA).data);
	list.object[list.dim].ptr = (SY_VOID*) vd;
	list.object[list.dim].tag = SY_VORO; list.dim++;
        break;
      case SY_SKEL_STRIP:
        vsk = (SySkelDAC*) (Sy_GetIOproperty(iohandle, DATA).data);
	list.object[list.dim].ptr = (SY_VOID*) vsk;
	list.object[list.dim].tag = SY_SKE; list.dim++;
        break;
      case SY_TRIAN_STRIP:
        tri = (SyTriDAC*) (Sy_GetIOproperty(iohandle, DATA).data);
	list.object[list.dim].ptr = (SY_VOID*) tri;
	list.object[list.dim].tag = SY_TRI; list.dim++;
        break;
      default:
	AVSmessage(sySkelViewId, AVS_Warning, AVSmodule, "skeleton to geom",
		   "Ok", "No skeleton project structure found in %s", 
		   current_infilename);
      }
    }
  }

  for (i = 0; i < list.dim; i++){
    switch (list.object[i].tag){
	
    case SY_IPO:
      ipo = GEOMcreate_obj(GEOM_POLYTRI, NULL);
      
      switch (AVSchoice_number (par_mode_string, mode_string)){
      case 1:  render_method   = POLYLINE; break;
      case 2:  render_method   = POLYTRIANGLE; break;
      default: render_method   = POLYLINE;
      }
      iponame = Sy_BaseName (current_infilename);
      
      Sy_AvsSetSyRenderMethod  (render_method);
      Sy_AvsSetTriangleDecor   (CONSTANT);
      Sy_AvsSetTriangleHeight  (*heigth);
      
      ipd = (SyIpoDAC*) list.object[i].ptr;
      Sy_AvsIpo2Geom (ipo, ipd);
      
      GEOMauto_transform(ipo); 
      
      *output = GEOMinit_edit_list(*output); 
      
      GEOMedit_color(*output, iponame, &yellow);    
      GEOMedit_geometry(*output, iponame, ipo);     
      GEOMedit_render_mode (*output, iponame, "flat");
      GEOMdestroy_obj(ipo);     
      break;
      
    case SY_VORO:
      voro = GEOMcreate_obj(GEOM_POLYTRI, NULL);
      
      switch (AVSchoice_number (par_mode_string, mode_string)){
      case 1:  render_method   = POLYLINE; break;
      case 2:  render_method   = POLYTRIANGLE; break;
      default: render_method   = POLYLINE;
      }
      voroname = Sy_BaseName (current_infilename);
      
      Sy_AvsSetSyRenderMethod (render_method);
      Sy_AvsSetTriangleDecor (CONSTANT);
      Sy_AvsSetTriangleHeight (*heigth);
	

      vd = (SyVoroDAC*) list.object[i].ptr;
      Sy_AvsVoro2Geom (voro, vd);
      
      GEOMauto_transform(voro);
      *output = GEOMinit_edit_list(*output);
      GEOMedit_color(*output, voroname, &red);    
      GEOMedit_geometry(*output, voroname, voro);
      GEOMedit_render_mode (*output, voroname, "flat");
      GEOMdestroy_obj(voro); 
      break;
      
    case SY_SKE:
      geomlistdim = 0;
      skeleton = GEOMcreate_obj(GEOM_POLYTRI, NULL);
      ipo      = GEOMcreate_obj(GEOM_POLYTRI, NULL);
      geomlist[geomlistdim++] = ipo;
      geomlist[geomlistdim++] = skeleton;

      if (is_reconstruct){
	sphere = Sy_AvsRegul2Geom(vsk, ipd, *thres);
	geomlist[geomlistdim++] = sphere;
      }
      else{
	sphere = (GEOMobj*)NULL;}
      
      switch (AVSchoice_number (par_mode_string, mode_string)){
      case 1: render_method = POLYLINE;     triangle_decor = CONSTANT; break;
      case 2: render_method = POLYTRIANGLE; triangle_decor = CONSTANT; break;
      case 3: render_method = POLYTRIANGLE; triangle_decor = RADIUS;   break;
      case 4: render_method = POLYTRIANGLE; triangle_decor = RESIDUAL; break;
      case 5: render_method = POLYTRIANGLE; triangle_decor = BIRTH_VECTOR; break;
      default:
	render_method = POLYLINE; triangle_decor = CONSTANT; 
      }  
      
      if ((triangle_decor == RESIDUAL) && !vsk->is_eresid){
	AVSmessage(sySkelViewId, AVS_Warning, AVSmodule, "Skeleton To Geom",
		   "Ok", "Missing extended residual information\
run pruning module with 'rich' option enabled\
or specify -rich on the command line"); 
	return(0);
      }
      
      iponame  = Sy_BaseName (current_infilename);
      skelname = Sy_BaseName (current_infilename);
      
      Sy_AvsSetTriangleHeight (*heigth);
      Sy_AvsSetValueLimit (*value_limit);
      Sy_AvsSetSyRenderMethod (render_method);
      Sy_AvsSetTriangleDecor (triangle_decor);
      
      vsk = (SySkelDAC*) list.object[i].ptr;

      Sy_AvsSkel2Geom (skeleton, vsk, ipd, *thres);
      
      Sy_AvsSetTriangleDecor (CONSTANT);
      Sy_AvsIpo2Geom (ipo, ipd);  
            
      GEOMauto_transform_list(geomlist, geomlistdim); 
      
      /* create edit list */
      *output = GEOMinit_edit_list(*output);
      
      /* insert object into edit list */
      GEOMedit_geometry(*output, skelname, skeleton);
      GEOMedit_geometry(*output, "ipos",  ipo);
      GEOMedit_parent (*output, "ipos", skelname);
      GEOMedit_transform_mode(*output, "ipos", "parent", NULL);

      if (is_reconstruct){
	GEOMedit_geometry(*output, "spheres",  sphere);
	GEOMedit_parent (*output, "spheres", skelname);
	GEOMedit_transform_mode(*output, "spheres", "parent", NULL);
      }
      
      GEOMdestroy_obj(skeleton); 
      GEOMdestroy_obj(ipo); 
      if (is_reconstruct) GEOMdestroy_obj(sphere);
      
      GEOMedit_color(*output, skelname, &red);
      GEOMedit_color(*output, "ipos", &yellow);
      if (is_reconstruct) GEOMedit_color(*output, "spheres", &white);

      GEOMedit_render_mode (*output, skelname, "flat");
      GEOMedit_render_mode (*output, "ipos", "flat");
      if (is_reconstruct) GEOMedit_render_mode (*output, "spheres", "gouraud");
      break;
      
    case SY_TRI:
      triangulation = GEOMcreate_obj(GEOM_POLYTRI, NULL);
      
      switch (AVSchoice_number (par_mode_string, mode_string)){
      case 1: render_method = POLYLINE;     triangle_decor = CONSTANT; break;
      case 2: render_method = POLYTRIANGLE; triangle_decor = CONSTANT; break;
      }  
      
      switch (AVSchoice_number (par_color_string, color_string)){
      case 1: Sy_AvsSetSyColorMode (COARSE_COL); break;
      case 2: Sy_AvsSetSyColorMode (FINE_COL); break;
      }  
      
      triname  = Sy_BaseName (current_infilename);
      
      Sy_AvsSetTriangleHeight (*heigth);
      Sy_AvsSetSyRenderMethod (render_method);
      Sy_AvsSetTriangleDecor (triangle_decor);
      Sy_AvsSetSyColorMode (tri->is_linkinfo ? FINE_COL : COARSE_COL); 
      
      sogthres.cont_thres = *cont_thres;
      sogthres.narr_thres = *narr_thres;
      sogthres.prox_thres = *prox_thres;
      
      tri = (SyTriDAC*) list.object[i].ptr;
      Sy_AvsTri2Geom (triangulation, tri, &sogthres);

      GEOMauto_transform(triangulation);
      
      *output = GEOMinit_edit_list(*output);      
      GEOMedit_geometry(*output, triname, triangulation);
      GEOMdestroy_obj(triangulation);       

      GEOMedit_render_mode (*output, triname, "flat");	
      if (!tri->is_linkinfo) GEOMedit_color(*output, triname, &white);
    }
  }

  return(1);
}

/* ***********************************************************************/
/* Initialization for modules contained in this file.                    */
/* ***********************************************************************/

/* 
   static int ((*mod_list[])()) = {
   Sy_SkelView_desc	
   };

   #define NMODS (sizeof(mod_list) / sizeof(char *))
   
   void AVSinit_modules (void)
   {
   AVSinit_from_module_list(mod_list, NMODS);
   }
   
*/


