/*
 *  $Id$
 * 
 *  Author: M. Naef, Swiss Federal Institute of Technology ETH
 *          Zurich, Switzerland (mnaef@vision.ethz.ch)
 */

/* Copyright Notice
 * ================
 * Copyright (C) 1994 by M. Naef
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
 */

#include <sys/types.h>
#include <avs/udata.h>
#include <avs/image_draw.h>
#include <avs/image_upstrm.h>
#include "Sy_Skeleton.h"
#include "Sy_Graphics.h"

/************************************************************************
 *
 *                             CLI BUFFER 
 *
 * The following structures and definitions are used to buffer cli 
 * commands so a round-trip to the avs-kernel is not performed for
 * every cli command.
 *
 ************************************************************************/

#define BUFSIZE 1024
#define	CLI_BUFFER_SIZE BUFSIZE*32

typedef struct {
   char buffer[CLI_BUFFER_SIZE], *bptr;
} CLIbuffer;

/*----------------------------------------------------------------------*/

/************************************************************************
 *
 * initCLIBuffer 
 *
 ************************************************************************/
CLIbuffer * 
initCLIBuffer()
{
   static CLIbuffer commandBuffer;
   
   commandBuffer.bptr = commandBuffer.buffer;

  return (&commandBuffer);
} 
/*----------------------------------------------------------------------*/






/************************************************************************
 *
 * flush_command_buffer
 *
 ************************************************************************/
void flush_command_buffer(CLIbuffer *clibuf)
{
  char *outbuf, *errbuf;

  if (clibuf->bptr != clibuf->buffer)
  {
    *(clibuf->bptr) = '\0';

    AVScommand("kernel", clibuf->buffer, &outbuf, &errbuf);

    clibuf->bptr = clibuf->buffer;
  }
}
/*----------------------------------------------------------------------*/




/************************************************************************
 *
 * add_command
 *
 ************************************************************************/
void add_command(char *command, CLIbuffer *clibuf, int flush)
{
  int len = strlen(command) + 1;
  if ((CLI_BUFFER_SIZE - (clibuf->bptr - clibuf->buffer)) < len)
    flush_command_buffer(clibuf);

  strcpy(clibuf->bptr, command);
  clibuf->bptr += len;
  *(clibuf->bptr - 1) = '\n';

  if (flush)
    flush_command_buffer(clibuf);
} 
/*----------------------------------------------------------------------*/

void Sy_Ipo2AVS(SyIpoDAC *ipd, image_viewer_id *iv_id,char *imageName) {

   int      tail_ipo, head_ipo;
   int      i;
   float    x1, x2, y1, y2;
   char command[BUFSIZE];
   CLIbuffer *clibuf;
   
   clibuf = initCLIBuffer();

   sprintf(command,"image_set_line_width -scene_id %d -image_name \"%s\" -module $Module %d",iv_id->scene_id,imageName, 5);
   add_command(command, clibuf, 1);
   for (i = 1; i <= ipd->ipodim; i++){
      x1=(float)(ipd->ipo[i].x)+1.0;
      y1=(float)(ipd->ipo[i].y)+1.0;
      x2=x1;
      y2=y1;
      sprintf(command,"image_add_line -scene_id %d -image_name \"%s\" -module $Module %f %f %f %f",iv_id->scene_id,imageName, x1,y1,x2,y2);
      add_command(command, clibuf, 0);
   }
   sprintf(command,"image_set_line_width -scene_id %d -image_name \"%s\" -module $Module %d",iv_id->scene_id,imageName, 1);
   add_command(command, clibuf, 0);
   sprintf(command,"image_set_line_color -scene_id %d -image_name \"%s\" -module $Module %f %f %f",iv_id->scene_id,imageName, 0.0, 0.0 ,1.0);
   add_command(command, clibuf, 0);
   for (i = 1; i <= ipd->icndim; i++) {
      if ((tail_ipo = ipd->icn[i].tail_ipo) && 
	  (head_ipo = ipd->icn[i].head_ipo)){
	 x1 = (float)(ipd->ipo[tail_ipo].x)+1.0;
	 y1 = (float)(ipd->ipo[tail_ipo].y)+1.0; 
	 x2 = (float)(ipd->ipo[head_ipo].x)+1.0;
	 y2 = (float)(ipd->ipo[head_ipo].y)+1.0;
	 sprintf(command,"image_add_line -scene_id %d -image_name \"%s\" -module $Module %f %f %f %f",iv_id->scene_id,imageName, x1,y1,x2,y2);
	 add_command(command, clibuf, 0);
      }
   }
   flush_command_buffer(clibuf);
}

void Sy_Voro2AVS(SyVoroDAC *vd, image_viewer_id *iv_id,char *imageName) {

   SyVertex vhead, vtail;
   float    x1, y1, x2, y2;
   int      i;
   char command[BUFSIZE];
   CLIbuffer *clibuf;
   
   clibuf = initCLIBuffer();

   sprintf(command,"image_set_line_width -scene_id %d -image_name \"%s\" -module $Module %d",iv_id->scene_id,imageName, 5);
   add_command(command, clibuf, 1);
   for (i = 1; i <= vd->pdim; i++){
      x1=(float)(vd->point[i].x)+1.0;
      y1=(float)(vd->point[i].y)+1.0;
      x2=x1;
      y2=y1;
      sprintf(command,"image_add_line -scene_id %d -image_name \"%s\" -module $Module %f %f %f %f",iv_id->scene_id,imageName, x1,y1,x2,y2);
      add_command(command, clibuf, 0);
   }
   sprintf(command,"image_set_line_width -scene_id %d -image_name \"%s\" -module $Module %d",iv_id->scene_id,imageName, 1);
   add_command(command, clibuf, 0);

   sprintf(command,"image_set_line_color -scene_id %d -image_name \"%s\" -module $Module %f %f %f",iv_id->scene_id,imageName, 0.0, 1.0 ,0.0);
   add_command(command, clibuf, 0);
   for (i = 1; i <= vd->edim; i++) {
      vtail = vd->vertex[vd->edge[i].tail_vertex];
      vhead = vd->vertex[vd->edge[i].head_vertex];

      x1 = vtail.xden / vtail.num+1.0;
      y1 = vtail.yden / vtail.num+1.0;
      x2 = vhead.xden / vhead.num+1.0;
      y2 = vhead.yden / vhead.num+1.0;

      switch (vd->edge[i].rank) {
       case 0:
       case 1: break;
       default:
	 sprintf(command,"image_add_line -scene_id %d -image_name \"%s\" -module $Module %f %f %f %f",iv_id->scene_id,imageName, x1,y1,x2,y2);
	 add_command(command, clibuf, 0);
	 break;
      }
   }
   flush_command_buffer(clibuf);
}

void Sy_Skel2AVS(SySkelDAC *vsk, SyIpoDAC *ipd, float thres, enum SyDisplayMode display_mode, image_viewer_id *iv_id,char *imageName)
{
   SyVertex   vhead, vtail;
   int	   i;
   float    x1, y1, x2, y2;
   int      conn_flag;
   int      conn[4];
   int      tail_vertex, head_vertex, 
            left_point, right_point,
            tail_ipo, head_ipo;
   int      tail_flag, head_flag;
   int      is_residual_cut, is_skel, is_foreground, is_render;

   char command[BUFSIZE];
   CLIbuffer *clibuf;
   
   clibuf = initCLIBuffer();

   for (i = 1; i <= vsk->edim; i++) {
    
      tail_vertex = vsk->edge[i].tail_vertex;
      head_vertex = vsk->edge[i].head_vertex;
      left_point  = vsk->edge[i].left_point;
      right_point = vsk->edge[i].right_point;

      is_foreground  = Sy_SkeletonType (vsk, ipd, tail_vertex, i);

      if ((is_foreground && (display_mode == SKE_BACKGROUND)) ||
	  (!is_foreground && (display_mode == SKE_FOREGROUND))) continue;

      tail_flag   = vsk->regul[tail_vertex].birth[2] > thres;
      head_flag   = vsk->regul[head_vertex].birth[2] > thres;

      conn[0]  = ((ipd->ipl[left_point].tail_icn == ipd->ipl[right_point].tail_icn) &&
		  (ipd->ipl[left_point].tail_icn != 0));
      
      conn[1]  = ((ipd->ipl[left_point].tail_icn == ipd->ipl[right_point].head_icn) &&
		  (ipd->ipl[left_point].tail_icn != 0));
      
      conn[2]  = ((ipd->ipl[left_point].head_icn == ipd->ipl[right_point].tail_icn) &&
		  (ipd->ipl[left_point].head_icn != 0));
      
      conn[3]  = ((ipd->ipl[left_point].head_icn == ipd->ipl[right_point].head_icn) &&
		  (ipd->ipl[left_point].head_icn != 0));
      
      conn_flag = (conn[0] || conn[1] || conn[2] || conn[3]);

    
      is_residual_cut = vsk->is_eresid && 
	((vsk->eresid[i].tail_residual > thres) || 
	 (vsk->eresid[i].head_residual > thres));

      is_skel         = tail_flag && head_flag;

      is_render = (is_residual_cut || is_skel) && (!conn_flag || vsk->is_border);

      if (is_render){

	 if (!(tail_flag && head_flag)){
	    Sy_SimpleResidualCut (vsk, i, thres, &vtail, &vhead);
	 }
	 else{
	    vtail = vsk->vertex[vsk->edge[i].tail_vertex];
	    vhead = vsk->vertex[vsk->edge[i].head_vertex];
	 }

	 x1 = vtail.xden / vtail.num+1.0;
	 y1 = vtail.yden / vtail.num+1.0;
	 x2 = vhead.xden / vhead.num+1.0;
	 y2 = vhead.yden / vhead.num+1.0;

	 switch (vsk->edge[i].rank) {
	  case 0:
	  case 1:
	    break;
	  default:
	    sprintf(command,"image_add_line -scene_id %d -image_name \"%s\" -module $Module %f %f %f %f",iv_id->scene_id,imageName, x1,y1,x2,y2);
	    add_command(command, clibuf, 0);
	    break;
	 }
      }
   }
   flush_command_buffer(clibuf);
}


void Sy_SkelPyr2AVS(SySkelDAC *vsk, SyIpoDAC *ipd, int rank, enum SyDisplayMode display_mode, int *pyrlab, image_viewer_id *iv_id,char *imageName)
{
   SyVertex   vhead, vtail;
   int	   i;
   float    x1, y1, x2, y2;
   int      conn_flag;
   int      conn[4];
   int      tail_vertex, head_vertex, 
            left_point, right_point,
            tail_ipo, head_ipo;
   int      tail_flag, head_flag;
   int      is_residual_cut, is_skel, is_foreground, is_render;

   char command[BUFSIZE];
   CLIbuffer *clibuf;
   
   clibuf = initCLIBuffer();

   for (i = 1; i <= vsk->edim; i++) {
      
      tail_vertex = vsk->edge[i].tail_vertex;
      head_vertex = vsk->edge[i].head_vertex;
      left_point  = vsk->edge[i].left_point;
      right_point = vsk->edge[i].right_point;
      
      is_foreground  = Sy_SkeletonType (vsk, ipd, tail_vertex, i);

      if ((is_foreground && (display_mode == SKE_BACKGROUND)) ||
	  (!is_foreground && (display_mode == SKE_FOREGROUND))) continue;
      
      conn[0]  = ((ipd->ipl[left_point].tail_icn == ipd->ipl[right_point].tail_icn) &&
		  (ipd->ipl[left_point].tail_icn != 0));
      
      conn[1]  = ((ipd->ipl[left_point].tail_icn == ipd->ipl[right_point].head_icn) &&
		  (ipd->ipl[left_point].tail_icn != 0));
      
      conn[2]  = ((ipd->ipl[left_point].head_icn == ipd->ipl[right_point].tail_icn) &&
		  (ipd->ipl[left_point].head_icn != 0));
      
      conn[3]  = ((ipd->ipl[left_point].head_icn == ipd->ipl[right_point].head_icn) &&
		  (ipd->ipl[left_point].head_icn != 0));
      
      conn_flag = (conn[0] || conn[1] || conn[2] || conn[3]);
      
      is_render = (pyrlab[i] > 0) && (rank >= pyrlab[i]) && 
	(!conn_flag || vsk->is_border);
      
      if (is_render){
	 
	 vtail = vsk->vertex[vsk->edge[i].tail_vertex];
	 vhead = vsk->vertex[vsk->edge[i].head_vertex];
	 
	 x1 = vtail.xden / vtail.num+1.0;
	 y1 = vtail.yden / vtail.num+1.0;
	 x2 = vhead.xden / vhead.num+1.0;
	 y2 = vhead.yden / vhead.num+1.0;

	 switch (vsk->edge[i].rank) {
	  case 0:
	  case 1:
	    break;
	  default:
	    sprintf(command,"image_add_line -scene_id %d -image_name \"%s\" -module $Module %f %f %f %f",iv_id->scene_id,imageName, x1,y1,x2,y2);
	    add_command(command, clibuf, 0);
	    break;
	 }
      }
   }
   flush_command_buffer(clibuf);
}





