/*
 *  $Id$
 * 
 *  Author: Robert L. Ogniewicz, Swiss Federal Institute of Technology ETH
 *          Zurich, Switzerland (rlo@vision.ethz.ch)
 */

/* Copyright Notice
 * ================
 * Copyright (C) 1993 by Robert L. Ogniewicz
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more Details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
 */

/*
 * Skeleton "Regul" Data Structure to AVS Geometry Transform
 *
 */

static char regulGeom_Id[] = "%W%\t%G%";

#include <stdio.h>

#include <avs/avs.h>
#include <avs/avs_data.h>
#include <avs/geom.h>
#include <avs/flow.h>

#include "Sy_Avs.h"

#include "Sy_Skeleton.h"

extern enum SyRenderMethod  _sy_avs_render_method;
extern enum SyTriangleDecor _sy_avs_triangle_decor;
extern float              _sy_avs_triangle_height;

static GEOMobj*      spheres;
static SySkelDAC*     vsk;
static SyIpoDAC*       ipd;

static SyAvsVector*    vertex;
static float*        radius;
static int           vindex, rrindex, 
                     vdim, rdim;

static void Sy_AvsRegul2GeomInit(void);
static void Sy_AvsRegulInsert      (void);
static void Sy_AvsRegulGeomExit(void);

GEOMobj* Sy_AvsRegul2Geom (SySkelDAC *skel_vsk, SyIpoDAC *skel_ipd, float thres)
{
  SyWalktask wt;
  
  vsk  = skel_vsk;
  ipd  = skel_ipd;

  Sy_ResetWalktask (&wt);

  wt.user_function[WALK] = (void(*)())Sy_AvsRegulInsert;

  wt.user_function[INIT] = (void(*)())Sy_AvsRegul2GeomInit;
  wt.user_function[TRAV] = (void(*)())NULL;
  wt.user_function[TERM] = (void(*)())NULL;
  wt.user_function[ROT ] = (void(*)())NULL;
  wt.user_function[FEXIT] = (void(*)())Sy_AvsRegulGeomExit;

  Sy_SkeletonTraverse (vsk, ipd, thres, 0, &wt);
  return (spheres);
}
 
static void Sy_AvsRegul2GeomInit (void)
{
  vertex = (SyAvsVector*) NULL;
  radius = (float*) NULL;
  vindex = rrindex = 0;
  
  vertex = (SyAvsVector*) Sy_Malloc((vsk->vdim+1)*sizeof(SyAvsVector));
  radius = (float*)     Sy_Malloc((vsk->vdim+1)*sizeof(float));
  vdim   = rdim = vsk->vdim+1;
}

static int cnt = 0;

static void Sy_AvsRegulInsert (void)
{

  SyVertex vtmp;
  float x, y;

  if (syWstate.foreground == 0) return;

  vtmp  = syWstate.vsk->vertex[syWstate.current.head_vertex];
  x = vtmp.xden/vtmp.num;
  y = vtmp.yden/vtmp.num;

  vertex[vindex].x = x;
  vertex[vindex].y = y;
  vertex[vindex].z = 0.0;
  vindex++;
  radius[rrindex] = syWstate.vsk->regul[syWstate.current.head_vertex].radius;
  rrindex++;
}

static void Sy_AvsRegulGeomExit (void)
{
  spheres =  GEOMcreate_sphere (GEOM_NULL, vertex, radius, GEOM_NULL,
                            GEOM_NULL, vindex, GEOM_DONT_COPY_DATA);
} 















