/*
 *  $Id$
 * 
 *  Author: Robert L. Ogniewicz, Swiss Federal Institute of Technology ETH
 *          Zurich, Switzerland (rlo@vision.ethz.ch)
 */

/* Copyright Notice
 * ================
 * Copyright (C) 1993 by Robert L. Ogniewicz
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
 */

#include <stdio.h>
#include "Sy_Skeleton.h"
#include "Sy_Graphics.h"

main(int argc, char **argv)
{
  SyVSkeletonSet *vss;
  FILE *fp;
  int i;

  if (argc <= 1) {
    fprintf (stderr, "Usage: %s input_file [output_file]\n", argv[0]); 
    exit(2);
  }
  
  fp = fopen (argv[1], "r");
  if (fp == NULL){
    fprintf (stderr, "Could not open file `%s'.\n", argv[1]);
    exit(2);
  }

  vss = Sy_ReadSkeletonSet(fp);

  if (vss == NULL){
    fprintf (stderr, "Could not read skeleton set.\n");
    exit(2);
  }

  printf ("> Skeleton set with %d skeleton(s).\n", vss->dim);

  for (i = 1; i <= vss->dim; i++){
    if (vss->skel[i].branchdim > 0 || vss->skel[i].nodedim > 0){
      printf ("Skeleton (%s) no %d contains %d nodes and %d branches\n",
	      vss->skel[i].is_foreground ? "foreground" : "background",
	      vss->skel[i].id,
	      vss->skel[i].nodedim,
	      vss->skel[i].branchdim);
    }
  }

  {
    SyBoolean        is_ascii = SY_TRUE;
    SyIOhandle       *iohandle = NULL;
    SyDisplayControl control;

    Sy_SetVerbose(SY_TRUE);

    if (argc >= 3){
      sprintf (syStrBuf, "Writing Voronoi skeleton(s) to %s", argv[2]);
      Sy_Message (SY_FMTNL, syStrBuf);
      
      iohandle = Sy_CreateIOhandle();
      
      Sy_SetIOproperty (iohandle, FORMAT, (SY_VOID*) (is_ascii ? 
						      SY_ASCII:SY_BINARY));
      Sy_SetIOproperty (iohandle, TYPE, (SY_VOID*) SY_SKELETON_SET);
      Sy_SetIOproperty (iohandle, DATA, (SY_VOID*) vss);
      Sy_SetIOproperty (iohandle, IO_MODE, (SY_VOID*) SY_FREE_AFTER_WRITE);
      
      Sy_PutIOhandle (iohandle);
      Sy_ManageOutput (argv[2]);
    }
    else {
      control.raw_mode = SKE_OFF;
      control.display_mode = SKE_PLAIN;
      control.set_window = SKE_OFF;
      control.window_dimX = 512;
      control.window_dimY = 512;
      control.rank = 0;
      
      Sy_XSkeletonSetDisp (vss, "Skeleton Set", "Skeleton Set", 0, 0, 0, 0, 
			   0.0, NULL, control);
    } /* else */
  }
}

