/*
 *  $Id$
 * 
 *  Author: Robert L. Ogniewicz, Division of Applied Sciences,
 *          Harvard University, Cambridge, Massachusetts (rlo@hrl.harvard.edu)
 */

/* Copyright Notice
 * ================
 * Copyright (C) 1995 by Robert L. Ogniewicz
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
 */

#ifndef	_SY_AVL_TREE
#define	_SY_AVL_TREE 1

#ifdef  __cplusplus
extern "C" {
#endif

typedef	struct _SyAvlTree {
  struct _SyAvlTree *tree_l,
                    *tree_r;
  short	             tree_b;
  void	            *tree_p;
  void              *key;
} SyAvlTree;

void Sy_AvlTreeInit (SyAvlTree **ppr_tree);

void *Sy_AvlTreeSrch (SyAvlTree **ppr_tree, int (*pfi_compare) (void*, void*), 
		      void* key);

int Sy_AvlTreeAdd (SyAvlTree **ppr_tree, int (*pfi_compare) (void*, void*), 
		   void *pc_user, void *key, int (*pfi_delete) (void*));
/* return value: 1 = new node, 0: only replacement */

int Sy_AvlTreeDelete (SyAvlTree **ppr_p, int (*pfi_compare) (void*, void*), 
		      void *key, int (*pfi_uar) (void*, void*));

int Sy_AvlTreeTrav (SyAvlTree **ppr_tree, int (*pfi_uar) (void*));

void Sy_AvlTreeMung (SyAvlTree **ppr_tree, int (*pfi_uar) (void*, void*));

char *Sy_Strcpy (char *oldstr);

#define	MAXPROC	256

#ifdef DEBUG
#define	ENTER(proc) { \
			APC_PROCS[I_PROC] = proc; \
			printf("ENTER(%d:%s.%s)\n", \
				I_PROC, DEBUG, APC_PROCS[I_PROC]); \
			I_PROC++; \
		}
#define	EXITNV(value) { \
			I_PROC--; \
			printf("EXIT(%d:%s.%s)\n", \
				I_PROC, DEBUG, \
				APC_PROCS[I_PROC]); \
			return value; \
		}
#define	EXITV { \
			I_PROC--; \
			printf("EXITV(%d:%s.%s)\n", \
				I_PROC, DEBUG, \
				APC_PROCS[I_PROC]); \
			return value; \
		}
#else
#define	ENTER(proc)
#define	EXITNV(value)	{return value;}
#define	EXITV		return
#endif

#ifdef MAIN
int	I_PROC = 0;
char	*APC_PROCS[MAXPROC];
#else
extern	int	I_PROC;
extern	char	*APC_PROCS[MAXPROC];
#endif

#ifndef TRUE
#define	TRUE		1
#endif
#ifndef FALSE
#define	FALSE		0
#endif

#ifdef  __cplusplus
}
#endif

#endif	/* _SY_AVS_TREE */

