/*
 *  $Id$
 * 
 *  Author: Robert L. Ogniewicz, Swiss Federal Institute of Technology ETH
 *          Zurich, Switzerland (rlo@vision.ethz.ch)
 */

/* Copyright Notice
 * ================
 * Copyright (C) 1993 by Robert L. Ogniewicz
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
 */

#include <avs/udata.h>
#include "Sy_Skeleton.h"
#include "Sy_Graphics.h"

enum SyRenderMethod  {POLYLINE, POLYTRIANGLE};
enum SyTriangleDecor {CONSTANT, RADIUS, BIRTH_VECTOR, RESIDUAL};
enum SyDataFocus     {FOREGROUND, BACKGROUND, ALLDATA};
enum SyColorMode     {COARSE_COL, FINE_COL};

typedef struct _SyAvsVector {
  float x, y, z;
} SyAvsVector;

typedef struct _SyAvsNormal {
  float nx, ny, nz;
} SyAvsNormal;

typedef struct _SyAvsColor {
  float r,g,b;
} SyAvsColor;

typedef struct _SyAvsContext {
  enum SyRenderMethod  render_method;
  enum SyTriangleDecor triangle_decor;
  enum SyColorMode     color_mode;
  int                triangle_heigth;
} SyAvsContext;

typedef struct _SySGroupThres {
  float cont_thres,
        narr_thres,
        prox_thres;
} SySGroupThres;

extern void Sy_AvsSetSyRenderMethod (enum SyRenderMethod rm);
extern void Sy_AvsSetTriangleDecor (enum SyTriangleDecor td);
extern void Sy_AvsSetTriangleHeight (float heigth);
extern void Sy_AvsSetValueLimit (float maxval);
extern void Sy_AvsSetSyDataFocus (enum SyDataFocus focus);

extern void Sy_AvsIpo2Geom (GEOMobj *avs_obj, SyIpoDAC *skel_ipd);
extern void Sy_AvsVoro2Geom (GEOMobj *avs_obj, SyVoroDAC *skel_vd);
extern void Sy_AvsSkel2Geom (GEOMobj *avs_obj, SySkelDAC *skel_vsk,
			     SyIpoDAC *skel_ipd, float thres); 
extern void Sy_AvsTri2Geom (GEOMobj *avs_obj, SyTriDAC *in_tri, 
			    SySGroupThres *sogthres);

extern GEOMobj* Sy_AvsRegul2Geom (SySkelDAC *skel_vsk, SyIpoDAC *skel_ipd, 
				  float thres);

void Sy_Ipo2AVS(SyIpoDAC *ipd, image_viewer_id *iv_id,char *imageName);

void Sy_Voro2AVS(SyVoroDAC *ipd, image_viewer_id *iv_id,char *imageName);

void Sy_Skel2AVS(SySkelDAC *vsk, SyIpoDAC *ipd, float thres, 
		 enum SyDisplayMode display_mode, 
		 image_viewer_id *iv_id,char *imageName);

void Sy_SkelPyr2AVS(SySkelDAC *vsk, SyIpoDAC *ipd, int rank, 
		    enum SyDisplayMode display_mode, int *pyrlab, 
		    image_viewer_id *iv_id,char *imageName);

static SyAvsColor white   = {1.0,1.0,1.0};
static SyAvsColor black   = {0.0,0.0,0.0};

static SyAvsColor red     = {1.0,0.0,0.0};
static SyAvsColor green   = {0.0,1.0,0.0};
static SyAvsColor blue    = {0.0,0.0,1.0};

static SyAvsColor yellow  = {1.0,1.0,0.0};
static SyAvsColor magenta = {1.0,0.0,1.0};
static SyAvsColor cyan    = {0.0,1.0,1.0};





