/*
 *  $Id$
 * 
 *  Author: Robert L. Ogniewicz, Division of Applied Sciences,
 *          Harvard University, Cambridge, Massachusetts. 
	    (rlo@hrl.harvard.edu)
 */

/* Copyright Notice
 * ================
 * Copyright (C) 1995 by Robert L. Ogniewicz
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
 */

#ifndef _SY_INIMAGE_H
#define _SY_INIMAGE_H 1

#include "Sy_Stddef.h"

#ifdef  __cplusplus
extern "C" {
#endif

typedef struct _SyImBitMask {
  int *offset;
  unsigned char *mask;
  int linebytes;
} SyImBitMask;

enum _SyImageType {SY_BITMAP, SY_GRAY, SY_GREY, SY_RGB, 
		   SY_TRUECOLOR, SY_ORIG, SY_GREY_OR_BITMAP};

typedef enum _SyImageType SyImageType;

typedef struct _SyRGBMap {
  int  size;            /* size of RGB map */
  unsigned char *red,
                *green,
                *blue;   /* color values */
} SyRGBMap;

typedef struct _SyImage_t {
  char        *title;    /* name of image */
  SyImageType type;      /* type of image */
  SyRGBMap    rgb;       /* RGB map of image if IRGB type */
  int         width;     /* width of image in pixels */
  int         height;    /* height of image in pixels */
  int         depth;     /* depth of image in bits if IRGB type */
  int         pixlen;    /* length of pixel in bytes if IRGB type */
  int         linebytes; /* number of bytes per line */
  float       gamma;     /* gamma of display the image is adjusted for */
  unsigned char *data;   /* data rounded to full byte for each row */
                         /* additional entries for the skeleton project */
  void        (*expand_line)();
  int         bin_thres;

  SyBoolean   is_pixval;
  int         greyval, r_pix, g_pix, b_pix;
  void        (*filter_line)();
} SyImage_t;



SyImBitMask *Sy_MakeImBitMask (int width);
void Sy_FreeImage(SyImage_t *im);
void Sy_FreeImBitMask (SyImBitMask *ibm);

SyImage_t *Sy_LoadImageFromFile (char *fname, SyImageType typ, 
				 char *formathint, int verbose);

void Sy_CheckPixvalInImage (SyImage_t *ima);

#ifdef  __cplusplus
}
#endif
#endif /* _SY_INIMAGE_H */



