/*
 *  $Id$
 * 
 *  Author: Robert L. Ogniewicz, Swiss Federal Institute of Technology ETH
 *          Zurich, Switzerland (rlo@vision.ethz.ch)
 */

/* Copyright Notice
 * ================
 * Copyright (C) 1993 by Robert L. Ogniewicz
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
 */

#ifndef _SY_INTRINSIC_H
#define _SY_INTRINSIC_H 1

#include "Sy_Stddef.h"
#include "Sy_Lang.h"

#ifdef  __cplusplus
extern "C" {
#endif

extern int getpagesize(void);

#if !defined(SVR4) && !defined(SYSV) && !defined(convex) && !defined(__GNUC__) && !defined(ultrix) && !defined(sgi)
extern char* malloc(size_t);
#endif

/*  
 * crack extraction routines
 *
 */

typedef struct { 
  int     x_start, y_start,
          arc_id,
          ipo_id;
} SyCratranArc;

typedef struct {
  int x;
  int arc_ptr;
  
} SyCarcMemo;
 
typedef struct {
  SyCarcMemo* x_;
  int*       arc_;
  int*       index_;
  
  int        x_ptr;
  int        arc_ptr;
  int        ind_ptr;

  int        width;

} SyCratranStack;

typedef struct {
  SyCratranArc* arc_;
  int*  x_;

  int   width;

} SyCratranTable;

typedef struct {
  int*  offset;
  int*  mask;
  char* blank_line;
  int*  ones16;
  int   width; 
  int   invert_image,
        reverse_bits;
} SyCratranUtil;

typedef struct {
  int* line[4];
  int  width;

} SyCratranBuff;

typedef struct {
  int page_size;
  int nr_ipos;
  int ipodim;
  int icndim;
  int max_y;

} SyCratranInfo;

#define SY_XCORR (-1.0)
#define SY_YCORR ( 1.0)

/*
 * definition of internal data types for the Voronoi diagram routine
 *
 */

typedef                     /* convex hull: CH                          */  
struct{
       int* elem;
       int* left;           /* 'left' half (with respect to VD) of CH   */
       int* right;          /* 'Sy_Right' half of CH                    */
       int  left_index,
            right_index;
       int  left_count;
       int  right_count;    /* current hull size                        */
       SY_UINT dim;
       SY_UINT left_dim;       
       SY_UINT right_dim;   /* memory allocated for CH                  */
} SyHull;

typedef struct{
       SyEdge* elem;        /* SyComb: VD of a single scan-line         */
       int   count;         /* current comb size                        */
       int   plow, phigh;   /* references to comb boundary points       */
       int   n;             /* nr of points (of a single scan-line)     */
       SY_UINT  dim;        /* memory allocated for CH                  */
       int   max_edges;     /* max nr of edges                          */
} SyComb;

typedef
struct{
       int* e_;             /* edge stack               */
       int* v_;             /* vertex stack             */
       int* h_;             /* hull stack               */
       int  e_ptr;          /* edge stack pointer       */
       int  v_ptr;          /* vertex stack pointer     */
       int  h_ptr;          /* hull stack pointer       */
       SY_UINT dim;         /* memory alloc. for stacks */
} SyVoroStack;

typedef
struct{
       int suplow, suphigh; /* support points              */
       int raylow, rayhigh; /* corresponding hull elements */
} SySupport;

typedef                     
struct{
       int page_size;       /* physical memory page size   */ 
       int max_lines;       /* max nr of lines to proceed  */
} SyGenInfo;

enum SyVorOp {VORO_PRESCAN, VORO_SCAN, VORO_NOP};

typedef struct _SyPredat {
  int           stripcount;
  int           line_nr;
  SyPoint         pstart;
  SyIntVector       ref_vec;
  enum SyVorOp vorop;
} SyPredat;

void Sy_AllocVoroEnvironment (SyVoroDAC *vd, SyComb *comb, SyHull *hull, 
			      SySupport *support, SyVoroStack *stack, 
			      SyGenInfo *info);
void Sy_FreeVoroEnvironment (SyVoroDAC *vd, SyComb *comb, SyHull *hull, 
			     SySupport *support, SyVoroStack *stack, 
			     SyGenInfo *info);

void Sy_ReallocVoroComb(SyComb *comb, SyGenInfo *info); 
void Sy_ReallocVoroHullStack(SyHull *hull, SyVoroStack *stack, 
			     SyGenInfo *info); 

void Sy_AllocCratranEnvironment(SyImage_t *ima, SyCratranBuff *buff, 
				SyCratranTable *table, SyCratranStack *stack, 
				SyCratranUtil *util, SyCratranInfo *info,
				SyCratranControl *cntl);
 
void Sy_FreeCratranEnvironment (SyImage_t *ima, SyCratranBuff *buff, 
				SyCratranTable *table, SyCratranStack *stack,
				SyCratranUtil *util, SyCratranInfo *info,
				SyCratranControl *cntl);

int  Sy_AllocCratranHittab(int **hit, int nr_icns);
void Sy_FreeCratranHittab (int **hit);

void Sy_SetKhorosMode (SyBoolean flag);
void Sy_SetAvsMode (SyBoolean flag);

/* IO Language Parser Routines */

void Sy_GetBranchInfo (SyIoToken *tok, int *nelem, int *branch_id, 
		       int *skel_id, SyBoolean *is_binary_io, char* funcname);
int Sy_Load_skeletonbranch (SyIoToken *tok);
int Sy_Load_skeletonnode (SyIoToken *tok);
int Sy_Load_skeleton (SyIoToken *tok);
int Sy_ReadBndryOffsetVectors(SyIoToken *tok);
int Sy_ReadIncList(SyIoToken *tok);
int Sy_ReadRadiusList(SyIoToken *tok);
int Sy_ReadReferences(SyIoToken *tok);
int Sy_ReadSymmAxis(SyIoToken *tok);

SyVSkeletonSet *Sy_GetCurSkeletonSetPtr (void);
void Sy_ClearCurSkeletonSetPtr (void);

/* Declarations of global variables */

extern SyBoolean syIsKhoros, 
                 syIsAvs;

#ifdef  __cplusplus
}
#endif
#endif /* _SY_INTRINSIC_H */





