/*
 *  $Id$
 * 
 *  Author: Robert L. Ogniewicz, Swiss Federal Institute of Technology ETH
 *          Zurich, Switzerland (rlo@vision.ethz.ch)
 */

/* Copyright Notice
 * ================
 * Copyright (C) 1993 by Robert L. Ogniewicz
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
 */

#ifndef _SY_SKELETON_H
#define _SY_SKELETON_H 1

#include <stdio.h>

#ifndef NO_STDLIB
#include <stdlib.h>
#endif

#if ! ( defined(convex) || defined (bsdi) || defined (__bsdi__) )
#include <values.h>
#else
#define BITSPERBYTE  8
#define BITS(type)  (BITSPERBYTE * (int)sizeof(type))
#define HIBITI  (1 << BITS(int) - 1)
#define MAXINT  (~HIBITI)
#endif

#include <math.h>
#include <assert.h>

#if defined(sun) && (defined (SVR4) || defined (SYSV))
#include "solex.h"
#endif

#if defined(SYSV) || defined(SVR4) || defined(sparc)
#include <string.h>
#else
#include <strings.h>
#endif

#include "Sy_Stddef.h"
#include "Sy_InImage.h"
#include "Sy_NR.h"

#include "Sy_Export.h"

#ifdef  __cplusplus
extern "C" {
#endif
 
typedef struct _SyDot {
  int x, y;
} SyDot;

/*
 * definitions for the initiating point generation routines
 *
 */

#define SY_I_LABEL int
#define SY_C_LABEL int

typedef struct _SyIpoLabel {
  SY_I_LABEL* p_;
  SY_C_LABEL* c_;
} SyIpoLabel;

typedef struct {
  COORD x, y;
} SyIpo;

typedef struct _SyIpoFloat {
  float x, y;
} SyIpoFloat;

typedef struct _SyIpoDouble {
  double x, y;
} SyIpoDouble;

typedef struct _SyIpl { 
  SyIndex  tail_icn,
           head_icn;
  SyFloatVector grad;
} SyIpl;

typedef struct _SyIcn {
  SyIndex tail_ipo,
          head_ipo;
} SyIcn;

typedef struct _SyRegpotElement{
  float val;
  short label;
} SyRegpotElement;

typedef struct _SyIpoInfo {
  float orig_len, smooth_len;
  SyIndex contour_id;
  double first_ellipse_b_ref,
         first_ellipse_len,
         first_ellipse_len_smooth,
         first_ellipse_a,
         first_ellipse_b,
         first_ellipse_a_smooth,
         first_ellipse_b_smooth;
  float  clthreshold;
} SyIpoInfo;

typedef struct _SySkContourIndex {
  int contour_id, point_id;
} SySkContourIndex;

typedef struct _SyIpoDAC {
   int       ipodim;
   int       icndim;
   
   SyIpo*      ipo;
   SyIpl*      ipl;
   SyIcn*      icn;

   SyIpoLabel label;

   int       ipocardim;
   int       icncardim;

   SY_VOID*     ipocar;
   SY_VOID*     icncar;

   SyIpoFloat *fipo;

   float max_x, min_x, max_y, min_y;  /* bounding box */

   SyIpoInfo  *info;
   int        contourdim;
   int        maxcontourdim;

   SyBoolean  is_contour_index;
   SySkContourIndex *contour_index;

   SyRegpotElement *rpe;

} SyIpoDAC;

typedef struct _SyHipoData {
  int   hier, max_hier, min_hier, anchor_hier;
  int   fmax_hier, fmin_hier;
  char  is_anchor;
  float weight, fweight;
  char  same_boundary;
  float anchor_dist;
  float sigma;  
  char  is_foreground;
} SyHipoData;

typedef struct _SyHipoContainer {
  int maxdim, size, ptr, total_iposize;
  int *iposize, *ipomaxdim;
  SyIpo **ipo;
  SyIpoFloat  **fipo, **gauss1, **gauss2;
  float **curv;
  SyHipoData **hipodata;
  int **origin;
  SyIpoInfo *info;
} SyHipoContainer;

void Sy_FreeContainer (SyHipoContainer *container);

typedef struct _SyIpoMappingControl {
  char ignore_disjoint,
       use_endpoints,
       human_readable,
       use_endoskeleton,
       use_exoskeleton,
       do_file_output,
       process_contour,
       keep_anchor_points,
       is_float_contour,
       keep_shrinkage,
       set_local_thres,
       do_global_smoothing,
       no_smoothing, 
       no_cleanup;
  float sigma_lo, sigma_hi, adapt_sensitivity;
  float path_thres, thres;
  float value;
  int   lo_hier, hi_hier;
} SyIpoMappingControl;

typedef struct _SySkContour {
  int len,
      is_closed,
      is_polygon,
      is_interior_fg,
      fg_left_side,
      level;
  SyIpoFloat *point;
} SySkContour;

typedef struct _SySkContourSet {
  int dim, maxdim;
  int is_fg_info, is_all_closed, is_polygon;
  SySkContour *contour;
} SySkContourSet;

typedef struct _SySkContourTree {
  int contour_id, point_id, level;
  int is_interior_fg;
  struct _SySkContourTree *next, *down;
} SySkContourTree;

typedef struct _SyTmpFPoint {
  float x, y;
  int contour_id, point_id;
  unsigned char set;
} SyTmpFPoint;

typedef struct _SyTmpIpo {
  int x, y;
  int index;
  int tail_ipo, head_ipo,
      tail_icn, head_icn;
  SyVector grad;
  SySkContourIndex ci;
} SyTmpIpo;

/* 
 * purpose: random deletion of boundary points
 *
 */

int Sy_RandomDots (SyDot *array, unsigned int dimx, unsigned int dimy, 
		   unsigned int max_dots, unsigned int num_dots);
int Sy_IpoRandomDelete (SyIpoDAC *ipdIn, SyIpoDAC *ipdOut, float percent, 
			int is_verbose);

/*
 * Voronoi diagram
 *
 */

/**********************************************************************
 * Definition of the SyLabel structure
 *********************************************************************/

typedef int SyLabel;

#define SY_P_LABEL SyLabel
#define SY_E_LABEL SyLabel
#define SY_V_LABEL SyLabel

typedef
struct{
  SY_P_LABEL* p_;
  SY_E_LABEL* e_;
  SY_V_LABEL* v_;
} SyCompositeLabel;

/**********************************************************************
 * components of the Voronoi-diagram:
 * SyPoint
 *********************************************************************/
typedef 
struct{
	COORD  x, y;        /* x- & y-coordinate */
} SyPoint;

/**********************************************************************
 * SyVertex
 *********************************************************************/
typedef 
struct{                    /* fractal representation */
	double xden, yden; /* x- resp. y-coordinate denominator */
	double num;        /* common numerator */
} SyVertex;

/**********************************************************************
 * SyEdge
 *********************************************************************/
typedef 
struct{
	SyIndex  head_edge,   tail_edge,        /* doubly connected */
	       head_vertex, tail_vertex,      /* edge list (DCEL) */
	       left_point,  right_point;
	UCHAR  rank;                          /* rank = number of endpoints */
	COORD  dirx, diry;                    /* edge direction */
	COORD  xoff, yoff;                    /* edge center coord. x 2 */
} SyEdge; 


/**********************************************************************
 * definition of the VOROnoi-diagram-based DAta Carrier 
 * SyVoroDAC
 *********************************************************************/
typedef 
struct{
	int      pdim, 
                 edim, 
                 vdim;        /* nr of points, edges, vertices    */

	SyPoint  *point; 
	SyEdge   *edge;       /* pointers to the main components  */
	SyVertex *vertex;     /* of the VD */ 
     
	SyCompositeLabel  label; /* labels for points, edges, vertices */

	int      pcardim, 
                 ecardim, 
                 vcardim;     /* nr of supplementary elements     */

	SY_VOID  *pcar,       /* pointers to arbitrary structures */ 
	         *ecar,       /* containing supplementary data of */
	         *vcar;       /* points, edges and verteces       */	
} SyVoroDAC;


enum SyPruneMethod {SKE_CIR, SKE_BIC, SKE_KLE, SKE_CHO, SKE_POT,
		     SKE_SMPOT, SKE_SMCHO, SKE_SMCIR};

typedef 
enum _SyDisplayObject {SY_IPO, SY_VORO, SY_SKE, SY_TRI} SyDisplayObject;

typedef struct _SyObject {
  SY_VOID *ptr;
  SyDisplayObject tag;
} SyObject;

typedef struct _SyObjectList {
  int dim, maxdim;
  SyObject *object; 
} SyObjectList;
  
/**********************************************************************
 * Definition of the SyRegul structure (SyRegularization data)
 *********************************************************************/
typedef struct _SyRegul{
  float birth[3];   
  float radius;
  SyBoolean is_void_nfurcation;
} SyRegul;

typedef struct _SyEresid{
  float tail_residual, 
        head_residual;
  float dist;
  char  tail_corr,
        head_corr;
  char  same_boundary;
} SyEresid;

/**********************************************************************
 * definition of the V-SKELeton DAta Carrier 
 * SySkelDAC
 *********************************************************************/
typedef struct _SySkelDAC {
  int     pdim, 
          edim, 
          vdim;        /* nr of points, edges, vertices        */
  
  int     erdim;

  enum SyPruneMethod prune_method;

  SyPoint   *point; 
  SyEdge    *edge;       /* pointers to the main components      */
  SyVertex  *vertex;     /* of the VD                            */ 
  
  SyCompositeLabel   label;  /* labels for points, edges, vertices   */
  
  SyRegul   *regul;      /* regularization data                  */
  SyRegul   *regul_aux;  /* optional auxiliary vertex info       */
  SyEresid  *eresid;
  
  int       *hier;       /* skeleton pyramid labels             */  
  
  SyBoolean is_hierarchy,
            is_border, 
            is_eresid,
            is_nfurcation_info,
            is_smcir;
  
  int     pcardim, 
          ecardim, 
          vcardim;     /* nr of supplementary elements         */

  SY_VOID *pcar,       /* pointers to arbitrary structures     */ 
          *ecar,       /* containing supplementary data of     */
          *vcar;       /* points, edges and vertices           */
	
  double  max_resid;

} SySkelDAC;


/*
 * 
 * Delaunay Triangulation
 *
 */

typedef int SyEdgeRef;
typedef int SyReference;

enum SyTrianMode {TRIAN_FULL, TRIAN_SHORT};

typedef struct _SyTriThres {
  float cont_thres,
        narr_thres,
        prox_thres;
} SyTriThres;

/**********************************************************************
 * SyTriEdge
 *********************************************************************/
typedef struct _SyTriEdge{

        SyIndex  head_edge,   tail_edge,        /* doubly connected */
               head_point,  tail_point;       /* edge list (DCEL) */

} SyTriEdge;

/**********************************************************************
 * Definition of the SyTriReg structure 
 *********************************************************************/
typedef struct _SyTriReg{
  float cont,
        narr,
        prox;
} SyTriReg;

/**********************************************************************
 * definition of Triangulation Data Carrier
 * SyTriDAC
 *********************************************************************/
typedef struct _SyTriDAC{

	int     pdim, 
                edim,
                regdim,
	        erefdim;     /* nr of points, edges, edge references */
	
	int     is_linkinfo;
        int     is_pointref;

	SyPoint   *point;      /* pointers to the main components      */
	SyTriEdge *edge;       /* of the T.                            */ 
                             
	SyEdgeRef   *eref;
	SyReference *pref;
 
	SyCompositeLabel   label;  /* labels for points, edges             */

	SyTriReg  *regul;      /* "regularization" data                */

	Boolean *is_hardlink;
     
	int     pcardim, 
                ecardim;     /* nr of supplementary elements         */

	SY_VOID *pcar,       /* pointers to arbitrary structures     */ 
	        *ecar;       /* containing supplementary data of     */
	                     /* points and edges                     */	
} SyTriDAC;



/* 
 * definition of structures for boundary potential routines 
 *
 */

#define SY_TRIAN_OBJ_TAB float

typedef struct _SyTriPotElement{

  SyIntVector      a_vec, b_vec;
  SY_TRIAN_OBJ_TAB a_head_val, a_tail_val, 
                   b_head_val, b_tail_val;
  short   a_label,
          b_label;

} SyTriPotElement;

typedef struct _SyTriPot
{
  SyTriPotElement*  edge;
  SY_TRIAN_OBJ_TAB* len;
  int               obj_count;
  int               obj_dim;
  int               count;
  int               edim;

} SyTriPot;

typedef struct _SyTriPotBuff {
  int     tail_point, head_point,
          tail_edge, head_edge;
  int     this_edge, next_edge;
  SY_TRIAN_OBJ_TAB head_val, tail_val;
  SyIntVector vec;
} SyTriPotBuff;

typedef struct _SyTriPotState {

  SyTriDAC* tri;
  SyTriPot* pot;

  int id,
      sub_id;

  Boolean is_verbose;
  Boolean is_rotating;

  SyTriPotBuff start,
             current,
             last;

} SyTriPotState;

SyTriPot* Sy_Tripot (SyTriDAC *tri);
SyTriPot* Sy_AllocTriPot (int nr_elem);  


/*
 * Voronoi to Del. Triangulation
 *         
 */

void Sy_Voronoi2Trian(SyVoroDAC *vd_in, SyTriDAC *ret_tri);
void Sy_SetEdgeref(int val);
void Sy_SetTrianMode(enum SyTrianMode mode);
void Sy_AddHardlink (SyIpoDAC *ipd, SyTriDAC *tri);

/*  
 * crack extraction routines
 *
 */

typedef struct _SyCratranControl {
  int is_verbose,
      invert_image,
      reverse_bits;
} SyCratranControl;

void Sy_Cratran(SyImage_t *ima, SyIpoDAC *crack, SyCratranControl *cntl);
void Sy_CratranBuildIcn(SyIpoDAC *crack);
int  Sy_CratranIpoCheck(SyIpoDAC *crack);

/*
 *
 * regularization routines 
 *
 */

typedef struct _SyRegulControl {
  SyIndex   *edge_select;
  int       edge_select_dim;
  int       is_edge_selection;
  float     sigma_hint;
  float     sigma_lo, sigma_hi, sigma_step;
  float     base_threshold,
            work_threshold,
            search_threshold,
            term_threshold,
            pyr_trigger_threshold,
            pyr_cluster_delta,
            smcir_search_thres,
            smcir_term_thres;
            
  SyBoolean is_base_threshold, is_work_threshold, is_search_threshold, 
            is_term_threshold, is_pyr_trigger_threshold, 
            is_pyr_cluster_delta, is_thorough, no_shrink_estim;
  int       n_iter;
  int       gaussian_smoothing, oliensis_smoothing;
  float     pyr_trigger_hint;
  SyBoolean is_pyr_trigger_hint;
  float     pyr_cluster_hint;
  SyBoolean is_pyr_cluster_hint;
  float     cluster_max;


  int       pdim;              /* total number of boundary points */
  int       vdim;              /* number of vertices of Voronoi diagram */
  SyBoolean *is_concave_point; /* array : is_concave[i] true if 
			  boundary concave at boundary point with index i */
  SyBoolean *is_concave_vertex, *is_concave_vertex_0;

  double    max_resid;

  SyBoolean is_single_shot;

} SyRegulControl;

typedef struct _SyObjTab{
  float len, eff_len, bndry_scale;
} SyObjTab;

void Sy_ClearObjTabPtr();

typedef struct _SyRegpot{
  SyRegpotElement* elem_;
  SyObjTab*        obj_tab_;
  int*             obj_closed_;
  int              obj_count;
  int              obj_dim;
  int              count;
  int              dim;
  SyRegulControl   *control;
} SyRegpot;

typedef struct _SyControl {
  SyBoolean is_multiply;
  float mult_f;
  double mult_d;
  SyBoolean is_gaussian_noise, is_uniform_noise;
  float sigma, mean;
  float maxnoise, onnoise, offnoise, onleft, offleft, 
        onright, offright;
  int noise_model;
  float (*noisefunc)(float,float);

  float threshold;
  SyBoolean stop_at_nodes;
  SyBranchElementType branch_el_type;

  SyBoolean is_automatic,
            is_separate_saliency;
  
  SyRegul *regul;
  float   resolution;
} SyControl;

enum SyResidualCompMode   {RES_NORMAL, RES_FULL};
enum SyResidualBorderMode {RES_NOBORDER, RES_BORDER};

void Sy_Regularize(SySkelDAC *vsk, SyIpoDAC *ipd, 
		   void (*regul_method) (/* ??? */), 
		   int is_verbose);

void Sy_CirResidual(SySkelDAC *vsk, SyIpoDAC *ipd, SyRegpot *pot, 
		    int is_verbose);
void Sy_BicirResidual(SySkelDAC *vsk, SyIpoDAC *ipd, SyRegpot *pot, 
		      int is_verbose);
void Sy_KleinResidual(SySkelDAC *vsk, SyIpoDAC *ipd, SyRegpot *pot, 
		      int is_verbose);
void Sy_ChordResidual(SySkelDAC *vsk, SyIpoDAC *ipd, SyRegpot *pot, 
		      int is_verbose);
void Sy_PotResidual(SySkelDAC *vsk, SyIpoDAC *ipd, SyRegpot *pot, 
		    int is_verbose);

void Sy_SetResidualCompMode(enum SyResidualCompMode mode);
void Sy_SetResidualBorderMode(enum SyResidualBorderMode mode);

int  Sy_SimpleResidualCut(SySkelDAC *vsk, int edge_ref, float threshold, 
			  SyVertex *tvertex, SyVertex *hvertex);
int  Sy_ResidualCut(SySkelDAC *vsk, int edge_ref, float threshold, SyVertex *tvertex, 
		    SyVertex *hvertex, float *ret_tail_radius, float *ret_tail_residual,
		    float *ret_head_radius, float *ret_head_residual);

void Sy_SetResEpsilon(float eps);
void Sy_SetLocEpsilon(float eps);

void Sy_ComplexRegularize(SySkelDAC *vsk, SyTriDAC *tri, 
void (*regul_method)() );

void Sy_ComplexCirResidual(SySkelDAC *vsk, SyTriDAC *tri, SyTriPot *pot);
void Sy_ComplexKleinResidual(SySkelDAC *vsk, SyTriDAC *tri, SyTriPot *pot);
void Sy_ComplexChordResidual(SySkelDAC *vsk, SyTriDAC *tri, SyTriPot *pot);
void Sy_ComplexPotResidual(SySkelDAC *vsk, SyTriDAC *tri, SyTriPot *pot);

void Sy_RelaxPruning(SySkelDAC *vsk_in, SyIpoDAC *ipd_in, SyRegulControl *rc);

double Sy_GetMaxResidual(void);
/*
 * 
 * all-purpose skeleton/ipo traverse
 *
 */

typedef struct{
  int head_vertex,
      this_edge,
      next_edge;
} SyWalkBuff;  

typedef struct _SyIpoWalkBuff {
  int tail_ipo, head_ipo,
      tail_icn, head_icn;
  int this_icn,
      next_icn;
  SyIpo pos;
} SyIpoWalkBuff;

typedef struct _SyTriWalkBuff {
  int tail_point, head_point,
      tail_edge, head_edge;
  int this_edge, next_edge;
} SyTriWalkBuff;
      
#define SY_WALK_USERFUNCTIONS 33

enum SyFunctionTags {INIT, TRAV, TERM, FEXIT, WALK, ROT, SCAN, EOSK}; 

typedef struct _SyWalktask {

  void (*user_function[SY_WALK_USERFUNCTIONS])();

  FILE **user_file;

  int  fildim;

  int  retval;
  int  *error;

  int  verbose;

  float thres,
        search_thres,
        group_thres,
        narr_thres,
        pro_thres;

  SyIndex start_edge, start_vertex, next_edge;
  SyIndex skelid;

  SyBoolean is_search_threshold,
            is_loc_preset;

} SyWalktask;
  

typedef struct _SyWalkState{

  SyVoroDAC         *vd;
  SySkelDAC         *vsk;
  SyIpoDAC          *ipd;
  SyTriDAC          *tri;
  SyVSkeletonSet    *vss;
  SyVSkeleton       *vs;
  SyVSkeletonBranch *branch;
  SyVSkeletonNode   *node;
  
  int   foreground;

  int   id, 
        sub_id;

  float thres,
        search_thres,
        group_thres,
        narr_thres,
        pro_thres;

  SyBoolean is_search_threshold;

  int   verbose;

  SyWalkBuff start,
           current,
           last;

  int   is_connected;
  int   is_border;
  int   is_eresid;

  SyBoolean is_endpoint, is_node, is_loop;
  SyBoolean is_invalid, is_terminating;

  int branch_id, node_id;

} SyWalkState;  

typedef struct{

  SyIpoDAC*   ipd;

  int   id, 
        sub_id;

  int   verbose;
  int   closed_contour;

  SyIpoWalkBuff start,
              current,
              last;

  SyIndex contour_id;

} SyIpoWalkState;  

typedef struct _SyTriWalkState {

  SyTriDAC* tri;

  int   id, 
        sub_id;

  int   verbose;
  int   is_regul;


  float cont_thres,
        narr_thres,
        prox_thres;

  SyTriWalkBuff start,
              current,
              last;

} SyTriWalkState;  

#ifndef SY_WALK_H_INTERNAL
extern SyWalkState    syWstate; 
extern SyIpoWalkState wi;
extern SyTriWalkState wr;
#endif

enum SyInitTravOper {TRAV_NOP, TRAV_IN_EDGE, TRAV_OUT_EDGE};
enum SyTravDir      {TRAV_TAIL, TRAV_HEAD, TRAV_VOID};

void Sy_InitSkelTraverse (SySkelDAC *vsk, SyIpoDAC *ipd, int verbose, 
			  SyWalktask *wt);
void Sy_SkeletonTraverse (SySkelDAC *vsk, SyIpoDAC *ipd, float thres, 
			  int verbose, SyWalktask *wt); 
void Sy_WalkSkeleton (SySkelDAC *vsk, SyIpoDAC *ipd, int *next_edge, 
		      int *current_head_vertex, int skelid, float thres);
 
void Sy_InitIpoTraverse (SyIpoDAC *ipd, SyWalktask *wt);
void Sy_IpoTraverse (SyIpoDAC *ipd, SyWalktask *wt);

void Sy_InitVoronoiTraverse(SyVoroDAC *vd, SyWalktask *wt);
void Sy_VoronoiTraverse(SyVoroDAC *vd, SyWalktask *wt);
void Sy_WalkVoronoi(SyVoroDAC *vd, int *next_edge, int *current_head_vertex, 
		    int voroid);
void Sy_VoronoiRotate(SyVoroDAC *vd, int *next_edge, int *current_head_vertex);
void Sy_VoronoiPolyScan(SyVoroDAC *vd, int *next_edge, 
			int *current_head_vertex);

void Sy_InitTrianTraverse(SyTriDAC *tri, SyWalktask *wt);
void Sy_TrianTraverse(SyTriDAC *tri, SyWalktask *wt);
void Sy_WalkTriangulation(SyTriDAC *tri, int *next_edge, 
			  int *current_head_point, int trian_id);

void Sy_SkelRotate (SySkelDAC *vsk, SyIpoDAC *ipd, int *next_edge, 
		    int *current_head_vertex, float thres);

void Sy_SimplePolyScan (SySkelDAC *vsk, SyIpoDAC *ipd, int *next_edge, 
			int *current_head_vertex);
void Sy_PolyScan (SySkelDAC *vsk, SyIpoDAC *ipd, int *next_edge, 
		  int *current_head_vertex, SyControl *ctrl);

SyBoolean Sy_IsContinuation (SyIpoDAC *ipd, SySkelDAC *vsk, 
			     SyIndex edge_index, enum SyTravDir dir,
			     SyWalktask *wt);
/*
 * 
 * shape-oriented grouping
 *
 */

void Sy_SGroupInit(void);
void Sy_SGroupExit(void);
void Sy_SGroupWalk(void);
void Sy_SkGroupWalk(void);

/*
 *
 * Skeleton Pyramid
 *
 */

void Sy_Pyramid(SySkelDAC *vsk_in, SyIpoDAC *ipd_in, SyRegulControl *rc);

typedef struct _SyResTable {
  float residual;
  int   edge_ref;
} SyResTable;

typedef struct _SySkelTableElem {
  float max_residual;
  int   max_edge;
  int   skelid;
  int   is_foreground;
} SySkelTableElem;

typedef struct _SySkelTable {
  SySkelTableElem *elem;
  int             count;
  int             size;
} SySkelTable;


/*
 * I/O routines
 *
 */

#define SY_PRESERVE_DATA    0
#define SY_FREE_AFTER_WRITE 1

#define SY_IS_FREE_AFTER_WRITE(iomode) ((iomode) & SY_FREE_AFTER_WRITE)

#define SY_HEADER_SIZE 512
#define SY_MAGIC_STRING  "SKELETON"
#define SY_MAGIC_STRING_N "SKELETON"

#define SY_MAGIC_LEN   16
#define SY_OBJECT_LEN  32
#define SY_PARENT_LEN  32
#define SY_TYPE_LEN    16
#define SY_FORMAT_LEN   8
#define SY_XEXTENT_LEN  8
#define SY_YEXTENT_LEN  8
#define SY_NBYTES_LEN  16
#define SY_ARCH_LEN    16
#define SY_BOOL_LEN     1

enum SyStripType {SY_IPD_STRIP, SY_VORO_STRIP, SY_SKEL_STRIP, SY_TRIAN_STRIP, 
	          SY_IPO_STRIP,    SY_ICN_STRIP,     SY_IPL_STRIP, 
                  SY_POINT_STRIP,  SY_EDGE_STRIP,    SY_VERTEX_STRIP, 
		  SY_P_LABEL_STRIP, SY_E_LABEL_STRIP,  SY_V_LABEL_STRIP,
                  SY_LABEL_STRIP,
                  SY_NREF_STRIP,
		  SY_TRIPO_STRIP, SY_TRIEDGE_STRIP, SY_TRIVERTEX_STRIP,
                  SY_BIWI_STRIP,   
                  SY_SUN_STRIP,  
		  SY_CONTOUR_UNORDERED_STRIP,
		  SY_CONTOUR_ORDERED_STRIP,
		  SY_SKELETON_SET,
                  SY_NULL_STRIP};

enum  SySkeDataFormat  {SY_ASCII, SY_BINARY, SY_NONE};

enum  SyIOpropertyId  {OBJECT, PARENT, FORMAT, TYPE, DATA, 
		       X_EXTENT, Y_EXTENT, NUMBYTES, IO_MODE, IO_TYPE, 
		       IO_INDEX};

typedef union _SyIOpropertyValue{
  char               *data;
  char               *object;
  char               *parent;
  enum SyStripType  type;
  enum SySkeDataFormat format;
  float              x_extent;
  float              y_extent;
  int                numbytes;
  char               *io_type;
  unsigned int       io_mode;
  int                index;
} SyIOpropertyValue;

typedef struct _SySkeHeader {
  char               object[SY_OBJECT_LEN];
  char               parent[SY_PARENT_LEN];
  char               type[SY_TYPE_LEN];
  enum SySkeDataFormat format;
  float              x_extent;
  float              y_extent;
  int                numbytes;
} SySkeHeader;

typedef struct _SyFileHeader {
  char magic [SY_MAGIC_LEN];
  char object[SY_OBJECT_LEN];
  char parent[SY_PARENT_LEN];
  char type[SY_TYPE_LEN];
  char format[SY_FORMAT_LEN];
  char x_extent[SY_XEXTENT_LEN];
  char y_extent[SY_YEXTENT_LEN];
  char numbytes[SY_NBYTES_LEN];
  char arch[SY_ARCH_LEN];
} SyFileHeader;

typedef struct _SyIOhandle {
  enum SyStripType tag;
  unsigned int     io_mode;
  char*            data;
  SySkeHeader      val;
  int              index;
} SyIOhandle;

void Sy_ManageInput (char *filename);
void Sy_ManageOutput (char *filename);
void Sy_ManageOutputStream (FILE *stream);

SyIOhandle* Sy_NextIOhandle(void);
SyIOhandle* Sy_GetIOhandle(int index);
void Sy_PutIOhandle (SyIOhandle *handle);

void Sy_ResetIOhandler (void);

void Sy_SetOutputMode (char *str);

void Sy_SetObject (SyIOhandle *handle, char *object);
void Sy_SetParent (SyIOhandle *handle, char *parent);
void Sy_SetFormat (SyIOhandle *handle, enum SySkeDataFormat format);
void Sy_SetXExtent (SyIOhandle *handle, float x_extent);
void Sy_SetYExtent (SyIOhandle *handle, float y_extent);
void Sy_SetNumbytes (SyIOhandle *handle, int numbytes);
void Sy_SetTag (SyIOhandle *handle, enum SyStripType tag);
void Sy_SetData (SyIOhandle *handle, char *data);

char*  Sy_GetObject (SyIOhandle *handle);
char*  Sy_GetParent (SyIOhandle *handle);
char*  Sy_GetType (SyIOhandle *handle);
char*  Sy_GetData (SyIOhandle *handle);
enum SyStripType Sy_GetTag ();
enum SySkeDataFormat Sy_GetFormat ();
float  Sy_GetXExtent (SyIOhandle *handle);
float  Sy_GetYExtent (SyIOhandle *handle);
int    Sy_GetNumBytes (SyIOhandle *handle); 

SyIOhandle* Sy_CreateIOhandle (void);
void         Sy_ResetIOhandle (SyIOhandle *handle);
void         Sy_DestroyIOhandle (SyIOhandle *handle);
void         Sy_SetIOproperty (SyIOhandle *handle, 
			       enum SyIOpropertyId property, 
			       SY_VOID *val);

SyIOpropertyValue  Sy_GetIOproperty (SyIOhandle *handle, 
				     enum SyIOpropertyId property);

int Sy_Ipo_fprintf (FILE *fp, char *indata, unsigned int iomode);
int Sy_Ipo_fscanf (FILE *fp, SyIpoDAC **ret_ipd, unsigned int iomode);
int Sy_Ipo_fwrite (FILE *fp, char *indata, unsigned int iomode);
int Sy_Ipo_fread (FILE *fp, SyIpoDAC **ret_ipd, unsigned int iomode);
int Sy_IpoRaw_fscanf(FILE *fp, SyIpoDAC *ipd);
int Sy_IpoRaw_fread(FILE *fp, SyIpoDAC *ipd);

int Sy_Contour_fscanf (FILE *fp, SySkContourSet **ret_cs, unsigned int iomode);
int Sy_Ipo2ContourSet_fprintf (FILE *fp, char *indata, unsigned int iomode);

int Sy_Voro_fprintf (FILE *fp, char *indata, unsigned int iomode);
int Sy_Voro_fscanf (FILE *fp, SyVoroDAC **ret_vd, unsigned int iomode);
int Sy_Voro_fwrite (FILE *fp, char *indata, unsigned int iomode);
int Sy_Voro_fread (FILE *fp, SyVoroDAC **ret_vd, unsigned int iomode);

int Sy_Skel_fprintf (FILE *fp, char *indata, unsigned int iomode);
int Sy_Skel_fscanf (FILE *fp, SySkelDAC **ret_vsk, unsigned int iomode);
int Sy_Skel_fwrite (FILE *fp, char *indata, unsigned int iomode);
int Sy_Skel_fread (FILE *fp, SySkelDAC **ret_vsk, unsigned int iomode);

int Sy_SkeletonSet_fprintf (FILE *fp, char *indata, unsigned int iomode); 

int Sy_Trian_fprintf (FILE *fp, char *indata, unsigned int iomode);
int Sy_Trian_fscanf (FILE *fp, SyTriDAC **ret_tri, unsigned int iomode);
int Sy_Trian_fwrite (FILE *fp, char *indata, unsigned int iomode);
int Sy_Trian_fread (FILE *fp, SyTriDAC **ret_tri, unsigned int iomode);

/*
 * Memory allocation routines
 *
 */

SY_VOID* Sy_Malloc(int nbytes);
SY_VOID* Sy_Realloc(SY_VOID* old, int nbytes);
void Sy_Free(SY_VOID *ptr);

SyLabel* Sy_AllocLabel (int nr_elem);
SyReference* Sy_AllocRef (int nr_elem);

int* Sy_AllocInt (int nr_elem);

SyPoint*  Sy_AllocPoint(int nrpoints);
SyEdge*   Sy_AllocEdge(int nredges);
SyVertex* Sy_AllocVertex(int nrvertices);
SyPoint*  Sy_ReallocPoint(SyPoint *point, int nrpoints);
SyEdge*   Sy_ReallocEdge(SyEdge *edge, int nredges);
SyVertex* Sy_ReallocVertex(SyVertex *vertex, int nrvertices);

SyIpo* Sy_AllocIpo (int nr_ipos);
SyIpl* Sy_AllocIpl (int nr_ipos);
SyIcn* Sy_AllocIcn (int nr_icn);
SySkContourIndex* Sy_AllocContourIndex (int nr_ipos);

SyIpo* Sy_ReallocIpo (SyIpo* old, int nr_ipos);
SyIpl* Sy_ReallocIpl (SyIpl* old, int nr_ipos);
SyIcn* Sy_ReallocIcn (SyIcn* old, int nr_ipos);

void Sy_AllocRegpot(SyRegpot *pot, int nr_elem); 
void Sy_FreeRegpot (SyRegpot *pot);

SyRegul* Sy_AllocRegul(int nr_elem);
 
SyEresid* Sy_AllocEresid(int nr_elem);

SyTriEdge*  Sy_AllocTriEdge(int nredges);
SyTriReg*   Sy_AllocTriRegul(int nredges);
SyEdgeRef*  Sy_AllocEdgeRef(int nredges);

SY_P_LABEL*  Sy_AllocPointLabel(int nrpoints);
SY_E_LABEL*  Sy_AllocEdgeLabel(int nredges);
SY_V_LABEL*  Sy_AllocVertexLabel(int nrvertices);

Boolean*  Sy_AllocBoolean (int nrelem);

/* 
 * Lisp facts generation
 *
 */

void Sy_SimplePrintWalk(void);

void Sy_LispFactsInit(void);
void Sy_LispFactsTraverse(void);
void Sy_LispFactsWalk(void);
void Sy_LispFactsExit(void);

/* 
 * Utilities
 *
 */

void Sy_SkelBorder (SySkelDAC *vsk, SyIpoDAC *ipd, int is_verbose);

int Sy_IpoConnected  (SyIpoDAC *ipd, int first_point, int second_point);
int Sy_IsResidualCut(SySkelDAC *vsk, SyIpoDAC *ipd, int edge_ref, float thres);
void Sy_ResetWalktask (SyWalktask *wt);

void Sy_Infix (SySkelDAC *vsk_in, SyIpoDAC *ipd_in);
void Sy_SetInfix (Boolean b);

void Sy_Maxfix (SySkelDAC *vsk_in, SyIpoDAC *ipd_in);
void Sy_SetMaxfix (Boolean b);

void Sy_TrianPack(SyTriDAC *tri, SyTriThres trith);
void Sy_SkeletonPack(SySkelDAC *vsk, SyIpoDAC *ipd, float thres);

void Sy_SetRetainBorder(Boolean b);

void  Sy_Usage (char *progName, char **usageStrings);
char* Sy_BaseName (char *str);
char* Sy_RawName  (char *str);
char* Sy_CreateSimpleFilename (char *str, char *ext);
int   Sy_SkeletonType (SySkelDAC *vsk, SyIpoDAC *ipd, 
		       int current_head_vertex, int next_edge);
void  Sy_SimpleError (char *format, char *arg);
void  Sy_Error (char *module, char *function, char *format, char *arg);
void  Sy_Warning (char *format, char *arg);
void  Sy_Message (char *format, char *arg);
void  Sy_SetVerbose (SyBoolean flag);
void  Sy_DestroyIpo (SyIpoDAC *ipd);
void  Sy_DestroyVoronoi (SyVoroDAC *vd);
void  Sy_DestroySkeleton (SySkelDAC *vsk);
void  Sy_DestroyTrian (SyTriDAC *tri);

void Sy_InitIpo (SyIpoDAC *ipd);
void Sy_InitVoronoi (SyVoroDAC *vd);
void Sy_InitSkeleton (SySkelDAC *vsk);
void Sy_InitTrian (SyTriDAC *tri);

#define SY_NOSKEL(regul,thres) \
(thres>(regul).birth[2])

#define SY_EOSKEL(regul,thres) \
(((regul).birth[2]>thres)&&(thres>(regul).birth[1]))

#define SY_COMMONV(regul,thres) \
(((regul).birth[1]>thres)&&(thres>(regul).birth[0]))

#define SY_BRANV(regul,thres) \
((regul).birth[0]>thres)

/*
 * resource usage measurements
 *
 */

#include <sys/types.h>
#include <sys/time.h>
#include <sys/times.h>
#include <sys/resource.h>
#include <sys/param.h>

#define SY_CHRONO_DISPLAY 1
#define SY_CHRONO_NO_DISPLAY 0

typedef struct{
  float wtime;
  float utime;
  float stime;
} SyRusdelta;

void Sy_InitTimer(void);

void Sy_ShowTimer(void);

SyRusdelta Sy_GetTimer(int display_flag);

/* auxiliary regularization routines */


void Sy_DiamRegularize (SySkelDAC *vsk, SyIpoDAC *ipd, SyRegpot *pot, 
			int is_verbose);

void Sy_SmoothRegularize (SySkelDAC *vsk, SyIpoDAC *ipd, SyRegpot *pot, 
			  int is_verbose, SyIpoFloat *fipo);

void Sy_RegFlatten (SySkelDAC *vsk, SyIpoDAC *ipd, 
		    void (*regul_method) (/* ??? */), 
		    int is_verbose, float flatten_th);

void Sy_RegSmooth (SySkelDAC *vsk, SyIpoDAC *ipd, 
		   void (*regul_method) (/* ??? */), int is_verbose, 
		   float sigma, int transform, SyRegulControl *control);

void Sy_OutputBirthSetHistogram (SySkelDAC *vsk, char *filename, 
				  float threshold, float val3d, int is3d);

float Sy_OlienFreq (float ku, float kl, float c, float x);

float* Sy_ComputeOlienMask (int lowfreq, float bndry_len, int *size);

/* The Skeleton-Space */

typedef int SyHierLabel;

typedef struct _SyMScaleVertexInfo {
  float       birth[3];
  float       slope[3];
  float       radius;
  SyIndex    *incident; /* always the three most salient edges */
  SyHierLabel hier;     /* highest hierarchy of incident edges */
  SyBoolean   is_void_nfurcation;
} SyMScaleVertexInfo;

typedef struct _SyMScaleEdgeInfo {
  float resid;
  SyHierLabel hier;
} SyMScaleEdgeInfo;

typedef struct _SyMScaleEdge {
  SyIndex  head_edge,   tail_edge,   /* doubly connected */
           head_vertex, tail_vertex, /* edge list (DCEL) */
           left_point,  right_point;
  UCHAR    rank;                     /* rank = number of endpoints */
  COORD    dirx, diry;               /* edge direction */
  COORD    xoff, yoff;               /* edge center coord. x 2 */
  SyMScaleEdgeInfo *sc;
  int nscale;
} SyMScaleEdge;

typedef struct _SyMScaleVertex {
  double xden, yden; /* x- resp. y-coordinate denominator */
  double num;        /* common numerator */
  float x, y;
  SyMScaleVertexInfo *sc;
  int nscale;
  SyIndex *incident;
  int nincident;
  SyIndex self;
  SyMScaleVertexInfo smcir;
  SyBoolean is_concave;
} SyMScaleVertex;

typedef struct _SyMScaleInfo {
  float sigma;
} SyMScaleInfo;

typedef struct _SyMScaleSkelDAC {
  int     pdim, edim, vdim;        /* nr of points, edges, vertices */

  int     erdim;
  int     is_border;
  int     is_eresid;

  int     nscale;
  SyMScaleInfo *sc;
  
  enum SyPruneMethod prune_method;

  SyPoint         *point; 
  SyMScaleEdge    *edge;    /* pointers to the main components      */
  SyMScaleVertex  *vertex;  /* of the VD                            */ 
  
  SyLabel   label;          /* labels for points, edges, vertices   */
  
  SyIndex *edge_select;
  int     edge_select_dim;
  SyIndex *vertex_select;
  int     vertex_select_dim;

  int     pcardim, 
          ecardim, 
          vcardim;          /* nr of supplementary elements         */
  
  SY_VOID    *pcar,            /* pointers to arbitrary structures     */ 
          *ecar,            /* containing supplementary data of     */
          *vcar;            /* points, edges and vertices           */
	
} SyMScaleSkelDAC;

typedef struct _SyScaleSel {
  float sal, slope, smcir_sal;
  SyIndex self;
  SyIndex vptr;
  char signif;
} SyScaleSel;

void Sy_AddMScaleInfo (SyMScaleSkelDAC *mvsk, SySkelDAC *vsk, float sigma,
		       int n_iter, int cur_iter);

void Sy_DestroyMScaleVertex(SyMScaleVertex *mv);
void Sy_DestroyMScaleEdge(SyMScaleEdge *me);
void Sy_DestroyMScaleSkelDAC (SyMScaleSkelDAC *mvsk);

void Sy_OutputSkelSpaceLog (SyMScaleSkelDAC *mvsk, char *filename, 
			    float threshold);

void Sy_SkelSpaceRegularize (SyMScaleSkelDAC *mvsk, 
			     SySkelDAC *vsk, SyIpoDAC *ipd, 
			     SyRegulControl *rc);

void Sy_CleanupIpoSelect (SyIpoDAC *ipd, SyRegpot *pot); 
void Sy_CleanupIpo       (SyIpoDAC *ipd, SyRegpot *pot);

/* clipping */

void Sy_SetSimpleViewport (int width, int height);
void Sy_SetVectorProc (void(*funcptr)(int x0, int y0, int x1, int y1));
void Sy_DrawClippedVector(int x0, int y0, int x1, int y1);
void Sy_DrawClippedRay (int x0, int y0, int dirx, int diry);
void Sy_DrawClippedLine (int x0, int y0, int dirx, int diry);

SyVSkeletonSet *Sy_ExtractSkeletons (SySkelDAC *vsk, SyIpoDAC *ipd, 
				     SyControl *ctrl);

SyVSkeletonBranch* Sy_ManageBranchData (SyVSkeletonBranch *in_br, 
					int elemptr);

SyVSkeletonNode* Sy_ManageNodeData (SyVSkeletonNode *in_node, int rank);
SyVSkeleton* Sy_ManageSkeletonBranches (SyVSkeleton *in_vs, int branchid);
SyVSkeleton* Sy_ManageSkeletonNodes (SyVSkeleton *in_vs, int nodeid);
SyVSkeletonSet* Sy_ManageSkeletonSet (SyVSkeletonSet *in_vss, int skelid);
SyBoolean Sy_MapString2Bool (char *string);
SyBranchElementType Sy_MapString2BranchElementType (char *string);
SyBranchElementType Sy_MapString2BranchElementType (char *string);


SyIpoDAC* Sy_ContourSet2Ipo(SySkContourSet *cs, SyControl *ctrl);
SySkContourSet *Sy_Polygon2Points (SySkContourSet *cs, SyControl *in_ctrl);
void Sy_AddInsideOutsideInfo2ContourSet (SySkContourSet *cs);
int Sy_IsWithinSkContour (SySkContourSet *cs, 
			  SySkContourTree *t, SyTmpFPoint *p);
void Sy_DeleteSkContourSet (SySkContourSet *cs);
SySkContourTree* Sy_InsertSkContourTree (SySkContourSet *cs, 
					 SySkContourTree *t, 
					 SyTmpFPoint *p, int level);
void Sy_DeleteSkContourTree(SySkContourTree *t);
void Sy_AddFgInfo2ContourSet (SySkContourSet *cs);

SySkContourSet *Sy_AddNoise2ContourSet(SySkContourSet *cs, SyControl *ctrl);

int Sy_CurvatureSign (SyIpoFloat *p, int p2, int p1, int i0, 
		      int n1, int n2);

void Sy_GaussSmoothFFT (SyIpo *ipo, SyIpoFloat *smipo, int pdim, float sigma);


void Sy_DrawDigitalLine (int x1, int y1, int x2, int y2, 
			 void (*dotproc)(int, int));

void Sy_SetBinaryIO (SyBoolean flag);

/* external variables */

#ifndef SY_GLOBAL_C
extern SyBoolean syIsVerbose;
extern SyBoolean syIsBinaryIO;
extern char syStrBuf[SY_GLOBALSTRBUFLEN];
#endif

#ifdef  __cplusplus
}
#endif
#endif /* _SY_SKELETON_H */












