/*
 *  $Id: Sy_Uparse.h,v 1.1 1993/12/12 16:37:04 rlo Exp rlo $
 * 
 *  Author: Robert L. Ogniewicz, Swiss Federal Institute of Technology ETH
 *          Zurich, Switzerland (rlo@vision.ethz.ch)
 */

/* Copyright Notice
 * ================
 * Copyright (C) 1993 by Robert L. Ogniewicz
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
 */

/* 
  ---------------------------------------------------------------------------

   Command-Line Parsing Routines 

   int option (char ch) 
        
      Returns 1 if option 'ch' is part of the command line, 0 otherwise
      Example: if (option ('a')) { ...; ...; }
   
   char* argopt (char ch) 

      Returns a pointer to an (eventual) argument of option 'ch'
      Example: string_var = argopt('a');

   int uparse (argv, argc, optstring, noptarray, noptnr)
     char* argv[];
     char* noptarray[];  
     int   argc;
     char  optstring[];
     int*  noptnr;

     Performs parsing of a command line, returns 1 to indicate successful
     completion, 0 otherwise
     argv      : command line argument list
     argc      : nr of command line arguments
     optstring : string containing the options to parse for, option letters 
                 followed by a colon indicate that this option requires an
                 argument
     noptarray : array of pointers to strings holding the arguments after the
                 last option processed (possibly file names)
                 
     noptnr    : nr of elements in noptarray

  --------------------------------------------------------------------------- 
*/

#ifndef _SY_UPARSE_H
#define _SY_UPARSE_H 1

#ifdef  __cplusplus
extern "C" {
#endif

#ifndef SY_ARG_NONE
#define SY_ARG_NONE 0
#define SY_ARG_REQ  1
#define SY_ARG_OPT  2
#endif

#define SY_NULL_OPTION {(char*)NULL, SY_ARG_NONE}

typedef struct _SyOption {
#if	__STDC__
  const char *name;
#else
  char *name;
#endif
  int has_arg;
} SyOption;

int option(char name);

char* argopt(char name);

int uparse (char **argv, int argc, char *optstring, char **noptarray,
	    int maxnopt, int *noptnr);

int long_option(char* name);

char* long_argopt(char* name);

int uparse_long (char **argv, int argc, char *optstring, 
		 SyOption *longopt, char **noptarray, int maxnopt, 
		 int *noptnr);

#ifdef  __cplusplus
}
#endif
#endif /* _SY_UPARSE_H */







