 /*
  * Khoros: $Id$
  */

#if !defined(lint) && !defined(SABER)
static char rcsid[] = "Khoros: $Id$";
#endif

 /*
  * $Log$
  */

/*
 *----------------------------------------------------------------------
 *
 * Copyright 1992, University of New Mexico.  All rights reserved.
 * Permission to copy and modify this software and its documen-
 * tation only for internal use in your organization is hereby
 * granted, provided that this notice is retained thereon and
 * on all copies.  UNM makes no representations as to the sui-
 * tability and operability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 * 
 * UNM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FIT-
 * NESS.  IN NO EVENT SHALL UNM BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY OTHER DAMAGES WHAT-
 * SOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
 * IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
 * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PER-
 * FORMANCE OF THIS SOFTWARE.
 * 
 * No other rights, including, for example, the right to redis-
 * tribute this software and its documentation or the right to
 * prepare derivative works, are granted unless specifically
 * provided in a separate license agreement.
 *---------------------------------------------------------------------
 */

#include "unmcopyright.h"        /* Copyright 1992 by UNM */

/*>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>  <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 >>>>
 >>>>         File Name: ctran.c
 >>>>
 >>>>      Program Name: ctran
 >>>>
 >>>>      Program Desc: 
 >>>>	
 >>>>	
 >>>>	
 >>>>	
 >>>>	
 >>>>	
 >>>>	This program generates a crack coded description of a binary
 >>>>	image.
 >>>>	
 >>>>	
 >>>>	
 >>>>	
 >>>>	
 >>>>	
 >>>>	
 >>>>	
 >>>>	
 >>>>	
 >>>>	
 >>>>	
 >>>>	
 >>>>	
 >>>>	
 >>>>	
 >>>>	
 >>>>	
 >>>>	
 >>>>	
 >>>>	
 >>>>	
 >>>>	
 >>>>	
 >>>>	
 >>>>	
 >>>>	
 >>>>	
 >>>>	
 >>>>	
 >>>>	
 >>>>	
 >>>>	
 >>>>	
 >>>>	
 >>>>	
 >>>>	
 >>>>	
 >>>>	
 >>>>	
 >>>>	
 >>>>	
 >>>>	
 >>>>	
 >>>>	
 >>>>	
 >>>>	
 >>>>	
 >>>>	
 >>>>	
 >>>>	
 >>>>	
 >>>>	
 >>>>	
 >>>>	
 >>>>	
 >>>>	
 >>>>	
 >>>>	

 >>>>            Author: Robert Ogniewicz and M. Y. Jaisimha
 
 >>>>
 >>>> Date Last Updated: Fri May 13 17:45:02 1994
 >>>>
 >>>>          Routines: main- the main program for ctran
 >>>> 		 gw_usage - gives usage of the program
 >>>> 		 gw_args  - gets arguments of program from command line
 >>>>
 >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>   <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<*/


#include "ctran.h"


/****************************************************************
 *
 * Routine Name:  main program for ctran
 *
 *       Input:  
 *       -i  input image or file 
 *
 *
 *       [-o1] resulting output image or file
 *       [-o2] resulting output crack file
 *       [-timer] selects one or zero
 *       [-verbose] selects one or zero
 *       [-ascii] selects one or zero
 *       [-cleanup] cleanup preprocessing
 *       [-clthreshold] Cleanup Threshold  (0.000 < value < 1.000)
 *
 *
 *
 ****************************************************************/


main(argc, argv)
int argc;
char *argv[];
{

/* -main_variable_list */
  
  SyIpoDAC       ipd;
  SyCratranImage cima;
  SyIOhandle*    iohandle;
  SyRegpot       pot;
  SyIpoInfo      ipoinfo;
  
  char charbuf   [SY_CHARBUF_SIZE];

  struct xvimage *img1, *img2, *readimage(), *createimage();
  int writeimage();
  int flag, is_ascii=SY_FALSE, is_timer=SY_FALSE, is_verbose=SY_FALSE,
      is_cleanup = SY_FALSE;
  int nrows, ncols, type;
  float clthreshold = 0.1; 
  
/* -main_variable_list_end */

	program = VStrcpy(argv[0]);
	ac = argc;
	av = argv;

	khoros_init();

	gw_get_args();

/* -main_before_lib_call */
  if (check_args())exit(1);

  if (ctran->verbose_flag){
    is_verbose = ctran->verbose_logic;}

  if (ctran->timer_flag){
    is_timer = ctran->timer_logic;}

  if (ctran->ascii_flag){
    is_ascii = ctran->ascii_logic;}

  if (ctran->cleanup_flag){
    is_cleanup = ctran->cleanup_logic;
  }

  if (ctran->clthreshold_flag){
    clthreshold = ctran->clthreshold_float;
  }

  img1 = readimage(ctran->i_file);
  if (img1 == NULL) exit(1);       
  nrows = (int )img1->col_size;
  ncols = (int )img1->row_size;
  type  = (int )img1->data_storage_type;
   
  flag = propertype (program, img1, VFF_TYP_1_BYTE, FALSE) ||
         propertype (program, img1, VFF_TYP_BIT, FALSE); 
  if (!flag) exit(1);

  img2 = createimage ((unsigned long) nrows,
                      (unsigned long) ncols,
                      (unsigned long) VFF_TYP_1_BYTE,
                      (unsigned long) 1,
                      (unsigned long) 1,
                      "created by ctran",
                      (unsigned long) 0,
                      (unsigned long) 0,
                      (unsigned long)
                      VFF_MS_NONE,
                      (unsigned long)
                      VFF_MAPTYP_NONE,
                      (unsigned long)
                      VFF_LOC_IMPLICIT,
                      (unsigned long) 0);

  memset((SY_VOID*)img2->imagedata, 0, nrows*ncols*sizeof(char));
 
/* -main_before_lib_call_end */

/* -main_library_call */
  Sy_SetVerbose(is_verbose);

  cima.width      = ncols;
  cima.height     = nrows;
  if (type == VFF_TYP_1_BYTE){
    cima.depth= 8;
    cima.linebytes  = ncols;}
  else{
    cima.depth = 1;
    cima.linebytes  = (ncols%8 == 0) ? ncols/8 : ncols/8 + 1;}

  cima.buff       = (unsigned char*)img1->imagedata;
  
  if (is_timer) Sy_InitTimer();
  
  Sy_SetKhorosMode(SY_TRUE);

  /* run crack extraction */
  
  Sy_Cratran (&cima, &ipd, is_verbose);
  
  /* build initiating contours */
  
  Sy_Message ("\n%s ", "@ building SyIcns");
  
  Sy_CratranBuildIcn (&ipd);
  
  Sy_Message ("%s\n", "done");

  if (is_cleanup){
    ipoinfo.clthreshold = clthreshold;
    ipd.info = &ipoinfo;
    Sy_CleanupIpo (&ipd, &pot);
  }

  Sy_SetKhorosMode(SY_FALSE);

  if (is_timer) Sy_ShowTimer();
 
  sprintf (charbuf, "@ initiating points: %d\n", ipd.ipodim);
  Sy_Message ("%s", charbuf);

/* -main_library_call_end */

/* -main_after_lib_call */
  if (ctran->o1_flag){
    Sy_Ipo2Khoros(&ipd, img2);
    (void) writeimage(ctran->o1_file, img2);
  }
  freeimage(img2);

  if (ctran->o2_flag){
    iohandle = Sy_CreateIOhandle ();
    Sy_SetIOproperty (iohandle, FORMAT, (is_ascii) ? SY_ASCII:SY_BINARY);
    Sy_SetIOproperty (iohandle, TYPE, SY_IPD_STRIP);
    Sy_SetIOproperty (iohandle, DATA, (char*) &ipd);
    Sy_SetIOproperty (iohandle, IO_MODE, SY_FREE_AFTER_WRITE);
    
    Sy_PutIOhandle (iohandle);
    Sy_ManageOutput (ctran->o2_file);
    
    sprintf (charbuf, "@ writing SyIpos to %s \n", ctran->o2_file);
    Sy_Message ("%s", charbuf);
  }  
    
/* -main_after_lib_call_end */


	khoros_close();
}


/****************************************************************
*
* Routine Name:  gw_usage 
*
*
* Purpose:  prints out the usage for ctran 
*
* Input:  none 
*
* Output:  none 
*
* Written By:  automatically generated by ghostwriter
*
****************************************************************/


gw_usage()
{
	fprintf(stderr, "ctran :\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\tThis program generates a crack coded description of a binary\n");
	fprintf(stderr, "\timage.\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\t\n");

/* -usage_additions */
/* -usage_additions_end */

	fprintf(stderr,"\t-i  input image or file  (infile)\n");
	fprintf(stderr, "\n");
	fprintf(stderr, "\n");
	fprintf(stderr,"\t[-o1]  resulting output image or file (outfile) [crack.vif]\n");
	fprintf(stderr,"\t[-o2]  resulting output crack file (outfile) [test.ipo]\n");
	fprintf(stderr,"\t[-timer]  selects one or zero (boolean) [true]\n");
	fprintf(stderr,"\t[-verbose]  selects one or zero (boolean) [true]\n");
	fprintf(stderr,"\t[-ascii]  selects one or zero (boolean) [true]\n");
	fprintf(stderr,"\t[-cleanup]  cleanup preprocessing (boolean) [true]\n");
	fprintf(stderr,"\t[-clthreshold]  Cleanup Threshold (float,  0.000 to 1.000) [0.1]\n");
	fprintf(stderr, " \n");
	fprintf(stderr, "\n");
	fprintf(stderr, "\t[-V] Gives the version for ctran\n");
	fprintf(stderr, "\t[-U] Gives the usage for ctran\n");
	fprintf(stderr, "\t[-P] Prompts for command line options\n");
	fprintf(stderr, "\t[-A [file1]] Creates the answer file called ctran.ans or file1 \n");
	fprintf(stderr, "\t[-a [file1]] Uses ctran.ans or file1 as the answer file \n\n\n");

}


/****************************************************************
*
* Routine Name:  gw_getargs 
*
*
* Purpose:  gets arguments off command line for ctran
*
* Input:  none 
*
* Output:  none 
*
* Written By:  automatically generated by ghostwriter
*
****************************************************************/


gw_get_args()
{

char	*i_pstr = 
	"\nEnter: (i) input image or file \n        {infile}: "; 
char	*o1_pstr = 
	"\nEnter: (o1) resulting output image or file \n        {outfile [crack.vif] }: "; 
char	*o2_pstr = 
	"\nEnter: (o2) resulting output crack file \n        {outfile [test.ipo] }: "; 
char	*timer_pstr = 
	"\nEnter: (timer) selects one or zero \n        {boolean, (y/n)  [yes] }: "; 
char	*verbose_pstr = 
	"\nEnter: (verbose) selects one or zero \n        {boolean, (y/n)  [yes] }: "; 
char	*ascii_pstr = 
	"\nEnter: (ascii) selects one or zero \n        {boolean, (y/n)  [yes] }: "; 
char	*cleanup_pstr = 
	"\nEnter: (cleanup) cleanup preprocessing \n        {boolean, (y/n)  [yes] }: "; 
char	*clthreshold_pstr = 
	"\nEnter: (clthreshold) Cleanup Threshold \n        {float,  0.000 to 1.000 [0.100000] }: "; 
char  answer_file[512], temp[512];



int	fid = -1,
	error, query, ok, count;
	int prompt_flag = FALSE;

	/*
	 * print version if necessary
	 */
	vgparml(fid, "-V", "noprompt", &query, 0, 0, &error);
	if (error == 0)
	{
	    printf ("\n%s: %s: Version %d.%d\n\n",
	    "ctran", RELEASE_NAME, RELEASE_MAJOR, RELEASE_MINOR);
	    exit(1);
	}

	/*
	 * print usage if necessary
	 */
	vgparml(fid, "-U", "noprompt", &query, 0, 0, &error);
	if (error == 0)
	{
	    gw_usage();
	    exit(1);
	}

	/*
	 * see if interactive prompting is desired
	 */
	vgparml(fid, "-P", "noprompt", &query, 0, 0, &error);
	if (error == 0)
	    prompt_flag = TRUE;

	/*
	 * see if answer file is to be used
	 */
	vgparms(fid, "-a", "noprompt", answer_file, 0, 0, &error);
	if (error == 0)
	{
	     if (VStrlen(answer_file) == 0)
		sprintf(answer_file, "ctran.ans");
	     while (fid < 0)
	     {
		if ((fid = open(answer_file, O_RDONLY, 0666))== -1)
		{
		    fprintf(stderr, "ctran: can't open answer file \"%s\"",
			    answer_file); 
		    fprintf(stderr, "please re-enter: ");
		    (void) gets(answer_file);
		    if (VStrlen(answer_file) == 0)
			sprintf(answer_file, "ctran.ans");
		}

	     }

	}

	/*
	 * allocate the ctran structure
	 */
	ctran = (ctran_struct *) 
	       calloc((unsigned)1, (unsigned) sizeof (ctran_struct));

	/*
	 * get required arguments for ctran
	 */
	if (prompt_flag == TRUE)
	    fprintf(stderr,"\nRequired Arguments:\n\n");

	if (!(ctran->i_flag = vget_infile(fid, prompt_flag,
			"-i", i_pstr, &(ctran->i_file))
))
	{
	    if (prompt_flag == TRUE)
	    {
		while(!(ctran->i_flag))
		{
		    fprintf(stderr, "\t\ti is required, please re-enter: \n");
		    ctran->i_flag = vget_infile(fid, prompt_flag,
		    "-i", i_pstr, &(ctran->i_file))
;

		}
	    }
	    else
	    {
		fprintf(stderr, "Error: '-i' is a required argument\n");
		gw_usage();
		exit(1);
	    }
	}

	/*
	 * get optional arguments for ctran
	 */
	if (prompt_flag == TRUE)
	   fprintf(stderr,"\nOptional Arguments:\n\n");

	ctran->o1_flag = vget_outfile(fid, prompt_flag,
	"-o1", o1_pstr, &(ctran->o1_file))
;

	ctran->o2_flag = vget_outfile(fid, prompt_flag,
	"-o2", o2_pstr, &(ctran->o2_file))
;

	ctran->timer_flag = vget_logic(fid, prompt_flag,
	"-timer", timer_pstr, &(ctran->timer_logic), 1);

	ctran->verbose_flag = vget_logic(fid, prompt_flag,
	"-verbose", verbose_pstr, &(ctran->verbose_logic), 1);

	ctran->ascii_flag = vget_logic(fid, prompt_flag,
	"-ascii", ascii_pstr, &(ctran->ascii_logic), 1);

	ctran->cleanup_flag = vget_logic(fid, prompt_flag,
	"-cleanup", cleanup_pstr, &(ctran->cleanup_logic), 1);

	ctran->clthreshold_flag = vget_float(fid, prompt_flag,
	"-clthreshold", clthreshold_pstr, &(ctran->clthreshold_float),
	0.100000, 0.000000, 1.000000);


}



