 /*
  * Khoros: $Id$
  */

#if !defined(lint) && !defined(SABER)
static char rcsid[] = "Khoros: $Id$";
#endif

 /*
  * $Log$
  */

/*
 *----------------------------------------------------------------------
 *
 * Copyright 1992, University of New Mexico.  All rights reserved.
 * Permission to copy and modify this software and its documen-
 * tation only for internal use in your organization is hereby
 * granted, provided that this notice is retained thereon and
 * on all copies.  UNM makes no representations as to the sui-
 * tability and operability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 * 
 * UNM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FIT-
 * NESS.  IN NO EVENT SHALL UNM BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY OTHER DAMAGES WHAT-
 * SOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
 * IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
 * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PER-
 * FORMANCE OF THIS SOFTWARE.
 * 
 * No other rights, including, for example, the right to redis-
 * tribute this software and its documentation or the right to
 * prepare derivative works, are granted unless specifically
 * provided in a separate license agreement.
 *---------------------------------------------------------------------
 */

#include "unmcopyright.h"        /* Copyright 1992 by UNM */

/*>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>  <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 >>>>
 >>>>         File Name: prune.c
 >>>>
 >>>>      Program Name: prune
 >>>>
 >>>>      Program Desc: 
 >>>>

 >>>> 
 >>>> Date Last Updated: Tue May 17 15:30:13 1994
 >>>>
 >>>>          Routines: main- the main program for prune
 >>>> 		 gw_usage - gives usage of the program
 >>>> 		 gw_args  - gets arguments of program from command line
 >>>>
 >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>   <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<*/


#include "prune.h"


/****************************************************************
 *
 * Routine Name:  main program for prune
 *
 *       Input:  
 *       -i2  input file with Voronoi diagram 
 *
 *
 *       [-i1] input image or file
 *       [-o1] resulting output image or file
 *       [-o2] resulting output skeleton file
 *       [-timer] selects one or zero
 *       [-verbose] selects one or zero
 *       [-ascii] selects one or zero
 *       [-circ] Circularity Residual
 *       [-bicirc] selects one or zero
 *       [-chord] selects one or zero
 *       [-pot] selects one or zero
 *       [-smpot] selects one or zero
 *       [-smchord] selects one or zero
 *       [-smooth] selects one or zero
 *       [-bdtrans] selects one or zero
 *       [-split] selects one or zero
 *       [-automatic] selects one or zero
 *       [-bg] selects one or zero
 *       [-fg] selects one or zero
 *       [-pyramid] selects one or zero
 *       [-trigger] the value of integer  (value >= 0)
 *       [-cluster] the value of integer  (value > 0)
 *       [-rank] the value of integer  (value > 0)
 *       [-sigma] the value of float  (value > 0.0)
 *       [-threshold] the value of float 
 *       [-iter] the value of integer  (value > 0)
 *       [-lo] the value of float 
 *       [-hi] the value of float 
 *       [-raw] selects one or zero
 *       [-thorough] selects one or zero
 *
 *
 *
 ****************************************************************/


main(argc, argv)
int argc;
char *argv[];
{

/* -main_variable_list */

enum PruneOp {PRUNE_WITH_IPD, PRUNE_WITH_TRIAN, PRUNE_WITH_IPD_PYR, 
              NO_PRUNEOP};

  struct xvimage *img1, *img2, *readimage(), *createimage();
  int writeimage();

  SyVoroDAC   *vd; 
  SyIpoDAC    *ipd;
  SyTriDAC    *tri;
  SySkelDAC   vsk;
  SyMScaleSkelDAC mvsk;
  SyTriPot    *pot; 

  SyIOhandle  *iohandle, *iohandle_ipo, *iohandle_tri;

  int          max_iter = 0, k, rank=1;
  int          is_verbose=SY_FALSE, is_ascii=SY_FALSE, is_flatten=SY_FALSE,
               is_timer=SY_FALSE, is_eresid=SY_FALSE, is_border=SY_FALSE, 
               is_output=SY_FALSE, is_automatic=SY_FALSE,
               is_infix=SY_FALSE, is_maxfix=SY_FALSE, is_smoothpoti=SY_FALSE, 
               is_smoothcho=SY_FALSE, is_raw=SY_FALSE,
               is_pyramid=SY_FALSE, is_reconstruct=SY_FALSE,
               is_thorough=SY_FALSE;

  int          is_bdtrans = SY_FALSE;

  char         charbuf     [SY_CHARBUF_SIZE],
               th_str      [SY_SMALL_STRING_LEN];

  float        pyr_th, pyr_delta, base_reg_thres;

  float        th, flatten_th, sigma, sigma_lo, sigma_hi, sigma_step;
  int          io_result;
  int          no_ipo_found = 1, no_voro_found = 1, no_tri_found = 1;
  int          nrows, ncols, type, flag;

  enum PruneOp oper = NO_PRUNEOP;

  SyDisplayControl control;
  SyRegulControl   rcntrl;

/* -main_variable_list_end */

	program = VStrcpy(argv[0]);
	ac = argc;
	av = argv;

	khoros_init();

	gw_get_args();

/* -main_before_lib_call */

  if (check_args())exit(1);

  if (prune->verbose_flag){is_verbose = prune->verbose_logic;}
  if (prune->timer_flag){is_timer = prune->timer_logic;}
  if (prune->ascii_flag){is_ascii = prune->ascii_logic;}

  if (prune->bdtrans_flag) is_bdtrans = prune->bdtrans_logic;
  if (prune->raw_flag) is_raw = prune->raw_logic;
  if (prune->thorough_flag) is_thorough = prune->thorough_logic;

  if (prune->automatic_flag){is_automatic = prune->automatic_logic;}
  is_eresid      = SY_FALSE;
  is_infix       = SY_FALSE;
  is_maxfix      = SY_FALSE;
  if (prune->split_flag){is_border = prune->split_logic;}
  is_flatten     = SY_FALSE;
  if (prune->smpot_flag){is_smoothpoti = prune->smpot_logic;}
  if (prune->smchord_flag){is_smoothcho = prune->smchord_logic;}
  is_output      = SY_FALSE;
  if (prune->pyramid_flag){is_pyramid = prune->pyramid_logic;}
  is_reconstruct = SY_FALSE;

  if (prune->threshold_flag) th = prune->threshold_float; else th = 3.0;
  flatten_th = 3.0;
  if (prune->trigger_flag) pyr_th = prune->trigger_int; else pyr_th = 2.0;
  if (prune->cluster_flag) pyr_delta=prune->cluster_int; else pyr_delta = 10.0;
  if (prune->rank_flag) rank = prune->rank_int; else rank = 1;
  if (prune->sigma_flag) sigma = prune->sigma_float; else sigma = 2.0;
  if (prune->hi_flag) sigma_hi = prune->hi_float; else sigma_hi = 50.0;
  if (prune->lo_flag) sigma_lo = prune->lo_float; else sigma_lo = 1.0;
  if (prune->iter_flag) max_iter = prune->iter_int; else max_iter = 0;

  control.display_mode = SKE_PLAIN;
  if (prune->fg_flag && prune->fg_logic)
    control.display_mode  = SKE_FOREGROUND;
  if (prune->bg_flag && prune->bg_logic)
    control.display_mode  = SKE_BACKGROUND;

  sigma_step = 2.0;

  if (max_iter > 0){
    if (max_iter == 1) max_iter = 2;
    sigma_step = (sigma_hi - sigma_lo)/(max_iter - 1);
  }
 
  rcntrl.sigma_lo       = sigma_lo;
  rcntrl.sigma_hi       = sigma_hi;
  rcntrl.sigma_step     = sigma_step;
  rcntrl.base_threshold = 1.0;
  rcntrl.is_edge_selection     = SY_FALSE;
  rcntrl.is_pyr_trigger_hint   = SY_FALSE;
  rcntrl.is_pyr_cluster_hint   = SY_FALSE;
  rcntrl.pyr_cluster_hint      = pyr_delta;

  rcntrl.gaussian_smoothing = SY_TRUE;
  rcntrl.oliensis_smoothing = SY_FALSE;

  if (prune->i1_flag){
    img1 = readimage(prune->i1_file);
    if (img1 == NULL) exit(1);       
    nrows = (int )img1->col_size;
    ncols = (int )img1->row_size;
    type  = (int )img1->data_storage_type;
    
    flag = propertype (program, img1, VFF_TYP_BIT, FALSE) ||
      propertype (program, img1, VFF_TYP_1_BYTE, FALSE);
    if (!flag) exit(1);
    
    img2 = createimage ((unsigned long) nrows,
                        (unsigned long) ncols,
                        (unsigned long) VFF_TYP_1_BYTE,
                        (unsigned long) 1,
                        (unsigned long) 1,
                        "created by prune",
                        (unsigned long) 0,
                        (unsigned long) 0,
                        (unsigned long)
                        VFF_MS_NONE,
                        (unsigned long)
                        VFF_MAPTYP_NONE,
                        (unsigned long)
                        VFF_LOC_IMPLICIT,
                        (unsigned long) 0);

    memset((SY_VOID*)img2->imagedata, 0, nrows*ncols*sizeof(char));
  }
/* -main_before_lib_call_end */

/* -main_library_call */
  Sy_SetVerbose (is_verbose);

  if (is_verbose)
    fprintf (stderr, "%s: loading IPD/SyVoroDAC from %s...\n", 
             argv[0], prune->i2_file);

  if (prune->i2_flag){
    Sy_ManageInput (prune->i2_file);
    while ((iohandle = Sy_NextIOhandle()) != NULL) {
      switch (Sy_GetIOproperty (iohandle, TYPE).type){
      case SY_IPD_STRIP:
        ipd = (SyIpoDAC*) (Sy_GetIOproperty(iohandle, DATA).data);
        iohandle_ipo = iohandle;
        oper = PRUNE_WITH_IPD;
        break;
      case SY_VORO_STRIP:
        vd = (SyVoroDAC*) (Sy_GetIOproperty(iohandle, DATA).data);
        no_voro_found = 0;
        break;
      case SY_TRIAN_STRIP:
        tri = (SyTriDAC*) (Sy_GetIOproperty(iohandle, DATA).data);
        oper = PRUNE_WITH_TRIAN;
        iohandle_tri = iohandle;
        break;
      }
    }
  }

  if (no_voro_found){
    Sy_SimpleError ("No SyVoroDAC structure found in %s\n", prune->i2_file);
  }

  if (is_verbose) fprintf (stderr, "%s: points:%d edges:%d vertices:%d\n",
                  argv[0], vd->pdim, vd->edim, vd->vdim);

  ipd->info  = NULL;
  ipd->contourdim = 0; ipd->maxcontourdim = 0;
  vsk.pdim   = vd->pdim;
  vsk.edim   = vd->edim;
  vsk.vdim   = vd->vdim;
  vsk.point  = vd->point;
  vsk.edge   = vd->edge;
  vsk.vertex = vd->vertex;
  vsk.regul  = (SyRegul*) NULL;

  if (is_eresid) Sy_SetResidualCompMode (RES_FULL);
  if (is_border) Sy_SetResidualBorderMode (RES_FULL);
  if (is_infix)  Sy_SetInfix (is_infix);
  if (is_maxfix) Sy_SetMaxfix (is_maxfix);

  if (is_timer) Sy_InitTimer();

  switch (oper){
  case NO_PRUNEOP:
    Sy_SimpleError ("No IPD/TRIAN structure found in %s\n", prune->i2_file);
    break;
  case PRUNE_WITH_IPD:
    if (is_automatic){
      vsk.prune_method = SKE_SMCHO;
      rcntrl.smcir_term_thres      = is_thorough ? 3.0 : 1.0;
      rcntrl.smcir_search_thres    = SY_SMCIR_SEARCH_THRES;
 #ifdef DEBUG
      fprintf(stderr, "Automatic scale selection:\n");
#endif
      th = 1.0; /* default regularization threshold */
      Sy_SkelSpaceRegularize (&mvsk, &vsk, ipd, &rcntrl);
    }
    else  
    if (is_smoothpoti){
      vsk.prune_method = SKE_SMPOT;
      Sy_RegSmooth (&vsk, ipd, Sy_SmoothRegularize, is_verbose, sigma, 
                    is_bdtrans, &rcntrl);
      if (is_output) 
        Sy_OutputBirthSetHistogramNew (&vsk, "dummy.out", th, 
                                       sigma, 1);
    }
    else
    if (is_smoothcho){
      vsk.prune_method = SKE_SMCHO;
      if (max_iter == 0){
        Sy_RegSmooth (&vsk, ipd, Sy_SmoothRegularize, is_verbose, sigma,
                      is_bdtrans, &rcntrl);
        if (is_output) 
          Sy_OutputBirthSetHistogramm (&vsk, "dummy.out", th, 
                                                    sigma, 1);
      }
      else{
        for (k = 0; k < max_iter; k++){
          if (vsk.regul != (SyRegul*)NULL) free(vsk.regul);
          sigma = sigma_lo + k*sigma_step;
          Sy_RegSmooth (&vsk, ipd, Sy_SmoothRegularize, is_verbose, sigma,
                        is_bdtrans, &rcntrl);
          Sy_AddMScaleInfo (&mvsk, &vsk, sigma, max_iter, k);
        }
        if (is_output)
          Sy_OutputSkelSpaceLog (&mvsk, "dummy.out", th);
      }/* else : max_iter > 0 */
    }
    else
    if (prune->circ_flag && prune->circ_logic) 
      Sy_Regularize (&vsk, ipd, Sy_CirResidual, is_verbose);
    else 
    if (prune->pot_flag && prune->pot_logic) 
      Sy_Regularize (&vsk, ipd, Sy_PotResidual, is_verbose);
    else
    if (SY_FALSE) 
      Sy_Regularize (&vsk, ipd, Sy_KleinResidual, is_verbose);
    else
    if (prune->bicirc_flag && prune->bicirc_logic) 
      Sy_Regularize (&vsk, ipd, Sy_BicirResidual, is_verbose);
    else
      Sy_Regularize (&vsk, ipd, Sy_ChordResidual, is_verbose);
    break;
  case PRUNE_WITH_TRIAN:
    if (prune->circ_flag && prune->circ_logic) 
      Sy_ComplexRegularize (&vsk, tri, Sy_ComplexCirResidual);
    else
    if (prune->pot_flag && prune->pot_logic) 
      Sy_ComplexRegularize (&vsk, tri, Sy_ComplexPotResidual);
    else
    if (SY_FALSE) 
      Sy_ComplexRegularize (&vsk, tri, Sy_ComplexKleinResidual);
    else
      Sy_ComplexRegularize (&vsk, tri, Sy_ComplexChordResidual);
    break;
  }


  if (is_flatten){
#ifdef DEBUG
    fprintf (stderr, "Flattening...\n");
#endif
    Sy_RegFlatten (&vsk, ipd, Sy_DiamRegularize, is_verbose, flatten_th);
  }

  if (is_timer) Sy_ShowTimer();

  vsk.is_hierarchy = SY_FALSE;
  vsk.hier         = NULL;

  if (is_pyramid){
    if (prune->threshold_flag){
      base_reg_thres               = th;
      rcntrl.term_threshold        = th;}
    else{
      base_reg_thres               = 1.0;
      rcntrl.term_threshold        = 0.5;
    }
    rcntrl.base_threshold        = 1.0;
    rcntrl.search_threshold      = 1.0;
    rcntrl.smcir_term_thres      = is_thorough ? 3.0 : SY_SMCIR_TERM_THRES;
    rcntrl.smcir_search_thres    = SY_SMCIR_SEARCH_THRES;
    rcntrl.pyr_trigger_threshold = pyr_th;
    rcntrl.pyr_cluster_delta     = pyr_delta;
    rcntrl.is_search_threshold   = SY_TRUE;
    rcntrl.is_term_threshold     = SY_TRUE;
    if (prune->trigger_flag) 
      rcntrl.is_pyr_trigger_hint = SY_FALSE;
    if (prune->cluster_flag) 
      rcntrl.is_pyr_cluster_hint = SY_FALSE;

#ifdef DEBUG
    fprintf(stderr, "The Skeleton Pyramid\n");
#endif    
    if (is_timer) Sy_InitTimer();
    
    Sy_Pyramid (&vsk, ipd, &rcntrl);
    
    if (is_timer) Sy_ShowTimer();    

    oper = PRUNE_WITH_IPD_PYR;
  }

  if (prune->i1_flag && prune->o1_flag){
    switch (oper){
    case PRUNE_WITH_IPD:
      if (!is_raw) Sy_Ipo2Khoros (ipd, img2);      
      Sy_Skel2Khoros (&vsk, ipd, th, control.display_mode, img2);      
      break;
    case PRUNE_WITH_IPD_PYR:
      if (!is_raw) Sy_Ipo2Khoros (ipd, img2);      
      Sy_SkelPyr2Khoros(&vsk, ipd, rank, control.display_mode, vsk.hier, img2);
      break; 
    } 
    (void) writeimage(prune->o1_file, img2);
    freeimage(img2);
  } 


  if (prune->o2_flag){
     iohandle = Sy_CreateIOhandle ();
     Sy_SetIOproperty (iohandle, FORMAT, (is_ascii) ? SY_ASCII:SY_BINARY);
     Sy_SetIOproperty (iohandle, TYPE, SY_SKEL_STRIP);
     Sy_SetIOproperty (iohandle, DATA, (char*) &vsk);
     Sy_SetIOproperty (iohandle, IO_MODE, SY_FREE_AFTER_WRITE);

     switch (oper){
     case PRUNE_WITH_IPD:
     case PRUNE_WITH_IPD_PYR:
       Sy_SetIOproperty (iohandle_ipo, FORMAT, (is_ascii)?SY_ASCII:SY_BINARY);
       Sy_PutIOhandle (iohandle_ipo);
       break;
     case PRUNE_WITH_TRIAN:
       Sy_SetIOproperty (iohandle_tri, FORMAT, (is_ascii)?SY_ASCII:SY_BINARY);
       Sy_PutIOhandle (iohandle_tri);
       break;
     }

     Sy_PutIOhandle (iohandle);
     Sy_ManageOutput (prune->o2_file);
     
     sprintf (charbuf, "@ writing Voronoi skeleton to %s \n", prune->o2_file);
     Sy_Message ("%s", charbuf);
   }

  
/* -main_library_call_end */

/* -main_after_lib_call */
/* -main_after_lib_call_end */


	khoros_close();
}


/****************************************************************
*
* Routine Name:  gw_usage 
*
*
* Purpose:  prints out the usage for prune 
*
* Input:  none 
*
* Output:  none 
*
* Written By:  automatically generated by ghostwriter
*
****************************************************************/


gw_usage()
{
	fprintf(stderr, "prune :\n");

/* -usage_additions */
/* -usage_additions_end */

	fprintf(stderr,"\t-i2  input file with Voronoi diagram  (infile)\n");
	fprintf(stderr, "\n");
	fprintf(stderr, "\n");
	fprintf(stderr,"\t[-i1]  input image or file (infile) [voro.vif]\n");
	fprintf(stderr,"\t[-o1]  resulting output image or file (outfile) [skel.vif]\n");
	fprintf(stderr,"\t[-o2]  resulting output skeleton file (outfile) [test.ske]\n");
	fprintf(stderr,"\t[-timer]  selects one or zero (boolean) [true]\n");
	fprintf(stderr,"\t[-verbose]  selects one or zero (boolean) [true]\n");
	fprintf(stderr,"\t[-ascii]  selects one or zero (boolean) [true]\n");
	fprintf(stderr,"\t[-circ]  Circularity Residual (boolean) [true]\n");
	fprintf(stderr,"\t[-bicirc]  selects one or zero (boolean) [true]\n");
	fprintf(stderr,"\t[-chord]  selects one or zero (boolean) [true]\n");
	fprintf(stderr,"\t[-pot]  selects one or zero (boolean) [true]\n");
	fprintf(stderr,"\t[-smpot]  selects one or zero (boolean) [true]\n");
	fprintf(stderr,"\t[-smchord]  selects one or zero (boolean) [true]\n");
	fprintf(stderr,"\t[-smooth]  selects one or zero (boolean) [true]\n");
	fprintf(stderr,"\t[-bdtrans]  selects one or zero (boolean) [true]\n");
	fprintf(stderr,"\t[-split]  selects one or zero (boolean) [true]\n");
	fprintf(stderr,"\t[-automatic]  selects one or zero (boolean) [true]\n");
	fprintf(stderr,"\t[-bg]  selects one or zero (boolean) [true]\n");
	fprintf(stderr,"\t[-fg]  selects one or zero (boolean) [true]\n");
	fprintf(stderr,"\t[-pyramid]  selects one or zero (boolean) [true]\n");
	fprintf(stderr,"\t[-trigger]  the value of integer (integer, >= 0) [2]\n");
	fprintf(stderr,"\t[-cluster]  the value of integer (integer, > 0) [10]\n");
	fprintf(stderr,"\t[-rank]  the value of integer (integer, > 0) [1]\n");
	fprintf(stderr,"\t[-sigma]  the value of float (float, > 0.0) [2]\n");
	fprintf(stderr,"\t[-threshold]  the value of float (float) [3]\n");
	fprintf(stderr,"\t[-iter]  the value of integer (integer, > 0) [1]\n");
	fprintf(stderr,"\t[-lo]  the value of float (float) [0]\n");
	fprintf(stderr,"\t[-hi]  the value of float (float) [1]\n");
	fprintf(stderr,"\t[-raw]  selects one or zero (boolean) [true]\n");
	fprintf(stderr,"\t[-thorough]  selects one or zero (boolean) [true]\n");
	fprintf(stderr, " \n");
	fprintf(stderr, "\n");
	fprintf(stderr, "\t[-V] Gives the version for prune\n");
	fprintf(stderr, "\t[-U] Gives the usage for prune\n");
	fprintf(stderr, "\t[-P] Prompts for command line options\n");
	fprintf(stderr, "\t[-A [file1]] Creates the answer file called prune.ans or file1 \n");
	fprintf(stderr, "\t[-a [file1]] Uses prune.ans or file1 as the answer file \n\n\n");

}


/****************************************************************
*
* Routine Name:  gw_getargs 
*
*
* Purpose:  gets arguments off command line for prune
*
* Input:  none 
*
* Output:  none 
*
* Written By:  automatically generated by ghostwriter
*
****************************************************************/


gw_get_args()
{

char	*i2_pstr = 
	"\nEnter: (i2) input file with Voronoi diagram \n        {infile}: "; 
char	*i1_pstr = 
	"\nEnter: (i1) input image or file \n        {infile [voro.vif] }: "; 
char	*o1_pstr = 
	"\nEnter: (o1) resulting output image or file \n        {outfile [skel.vif] }: "; 
char	*o2_pstr = 
	"\nEnter: (o2) resulting output skeleton file \n        {outfile [test.ske] }: "; 
char	*timer_pstr = 
	"\nEnter: (timer) selects one or zero \n        {boolean, (y/n)  [yes] }: "; 
char	*verbose_pstr = 
	"\nEnter: (verbose) selects one or zero \n        {boolean, (y/n)  [yes] }: "; 
char	*ascii_pstr = 
	"\nEnter: (ascii) selects one or zero \n        {boolean, (y/n)  [yes] }: "; 
char	*circ_pstr = 
	"\nEnter: (circ) Circularity Residual \n        {boolean, (y/n)  [yes] }: "; 
char	*bicirc_pstr = 
	"\nEnter: (bicirc) selects one or zero \n        {boolean, (y/n)  [yes] }: "; 
char	*chord_pstr = 
	"\nEnter: (chord) selects one or zero \n        {boolean, (y/n)  [yes] }: "; 
char	*pot_pstr = 
	"\nEnter: (pot) selects one or zero \n        {boolean, (y/n)  [yes] }: "; 
char	*smpot_pstr = 
	"\nEnter: (smpot) selects one or zero \n        {boolean, (y/n)  [yes] }: "; 
char	*smchord_pstr = 
	"\nEnter: (smchord) selects one or zero \n        {boolean, (y/n)  [yes] }: "; 
char	*smooth_pstr = 
	"\nEnter: (smooth) selects one or zero \n        {boolean, (y/n)  [yes] }: "; 
char	*bdtrans_pstr = 
	"\nEnter: (bdtrans) selects one or zero \n        {boolean, (y/n)  [yes] }: "; 
char	*split_pstr = 
	"\nEnter: (split) selects one or zero \n        {boolean, (y/n)  [yes] }: "; 
char	*automatic_pstr = 
	"\nEnter: (automatic) selects one or zero \n        {boolean, (y/n)  [yes] }: "; 
char	*bg_pstr = 
	"\nEnter: (bg) selects one or zero \n        {boolean, (y/n)  [yes] }: "; 
char	*fg_pstr = 
	"\nEnter: (fg) selects one or zero \n        {boolean, (y/n)  [yes] }: "; 
char	*pyramid_pstr = 
	"\nEnter: (pyramid) selects one or zero \n        {boolean, (y/n)  [yes] }: "; 
char	*trigger_pstr = 
	"\nEnter: (trigger) the value of integer \n        {integer, >= 0 [2] }: "; 
char	*cluster_pstr = 
	"\nEnter: (cluster) the value of integer \n        {integer, > 0 [10] }: "; 
char	*rank_pstr = 
	"\nEnter: (rank) the value of integer \n        {integer, > 0 [1] }: "; 
char	*sigma_pstr = 
	"\nEnter: (sigma) the value of float \n        {float, > 0.0 [2.000000] }: "; 
char	*threshold_pstr = 
	"\nEnter: (threshold) the value of float \n        {float [3.000000] }: "; 
char	*iter_pstr = 
	"\nEnter: (iter) the value of integer \n        {integer, > 0 [1] }: "; 
char	*lo_pstr = 
	"\nEnter: (lo) the value of float \n        {float [0.000000] }: "; 
char	*hi_pstr = 
	"\nEnter: (hi) the value of float \n        {float [1.000000] }: "; 
char	*raw_pstr = 
	"\nEnter: (raw) selects one or zero \n        {boolean, (y/n)  [yes] }: "; 
char	*thorough_pstr = 
	"\nEnter: (thorough) selects one or zero \n        {boolean, (y/n)  [yes] }: "; 
char  answer_file[512], temp[512];



int	fid = -1,
	error, query, ok, count;
	int prompt_flag = FALSE;

	/*
	 * print version if necessary
	 */
	vgparml(fid, "-V", "noprompt", &query, 0, 0, &error);
	if (error == 0)
	{
	    printf ("\n%s: %s: Version %d.%d\n\n",
	    "prune", RELEASE_NAME, RELEASE_MAJOR, RELEASE_MINOR);
	    exit(1);
	}

	/*
	 * print usage if necessary
	 */
	vgparml(fid, "-U", "noprompt", &query, 0, 0, &error);
	if (error == 0)
	{
	    gw_usage();
	    exit(1);
	}

	/*
	 * see if interactive prompting is desired
	 */
	vgparml(fid, "-P", "noprompt", &query, 0, 0, &error);
	if (error == 0)
	    prompt_flag = TRUE;

	/*
	 * see if answer file is to be used
	 */
	vgparms(fid, "-a", "noprompt", answer_file, 0, 0, &error);
	if (error == 0)
	{
	     if (VStrlen(answer_file) == 0)
		sprintf(answer_file, "prune.ans");
	     while (fid < 0)
	     {
		if ((fid = open(answer_file, O_RDONLY, 0666))== -1)
		{
		    fprintf(stderr, "prune: can't open answer file \"%s\"",
			    answer_file); 
		    fprintf(stderr, "please re-enter: ");
		    (void) gets(answer_file);
		    if (VStrlen(answer_file) == 0)
			sprintf(answer_file, "prune.ans");
		}

	     }

	}

	/*
	 * allocate the prune structure
	 */
	prune = (prune_struct *) 
	       calloc((unsigned)1, (unsigned) sizeof (prune_struct));

	/*
	 * get required arguments for prune
	 */
	if (prompt_flag == TRUE)
	    fprintf(stderr,"\nRequired Arguments:\n\n");

	if (!(prune->i2_flag = vget_infile(fid, prompt_flag,
			"-i2", i2_pstr, &(prune->i2_file))
))
	{
	    if (prompt_flag == TRUE)
	    {
		while(!(prune->i2_flag))
		{
		    fprintf(stderr, "\t\ti2 is required, please re-enter: \n");
		    prune->i2_flag = vget_infile(fid, prompt_flag,
		    "-i2", i2_pstr, &(prune->i2_file))
;

		}
	    }
	    else
	    {
		fprintf(stderr, "Error: '-i2' is a required argument\n");
		gw_usage();
		exit(1);
	    }
	}

	/*
	 * get optional arguments for prune
	 */
	if (prompt_flag == TRUE)
	   fprintf(stderr,"\nOptional Arguments:\n\n");

	prune->i1_flag = vget_infile(fid, prompt_flag,
	"-i1", i1_pstr, &(prune->i1_file))
;

	prune->o1_flag = vget_outfile(fid, prompt_flag,
	"-o1", o1_pstr, &(prune->o1_file))
;

	prune->o2_flag = vget_outfile(fid, prompt_flag,
	"-o2", o2_pstr, &(prune->o2_file))
;

	prune->timer_flag = vget_logic(fid, prompt_flag,
	"-timer", timer_pstr, &(prune->timer_logic), 1);

	prune->verbose_flag = vget_logic(fid, prompt_flag,
	"-verbose", verbose_pstr, &(prune->verbose_logic), 1);

	prune->ascii_flag = vget_logic(fid, prompt_flag,
	"-ascii", ascii_pstr, &(prune->ascii_logic), 1);

	prune->circ_flag = vget_logic(fid, prompt_flag,
	"-circ", circ_pstr, &(prune->circ_logic), 1);

	prune->bicirc_flag = vget_logic(fid, prompt_flag,
	"-bicirc", bicirc_pstr, &(prune->bicirc_logic), 1);

	prune->chord_flag = vget_logic(fid, prompt_flag,
	"-chord", chord_pstr, &(prune->chord_logic), 1);

	prune->pot_flag = vget_logic(fid, prompt_flag,
	"-pot", pot_pstr, &(prune->pot_logic), 1);

	prune->smpot_flag = vget_logic(fid, prompt_flag,
	"-smpot", smpot_pstr, &(prune->smpot_logic), 1);

	prune->smchord_flag = vget_logic(fid, prompt_flag,
	"-smchord", smchord_pstr, &(prune->smchord_logic), 1);

	prune->smooth_flag = vget_logic(fid, prompt_flag,
	"-smooth", smooth_pstr, &(prune->smooth_logic), 1);

	prune->bdtrans_flag = vget_logic(fid, prompt_flag,
	"-bdtrans", bdtrans_pstr, &(prune->bdtrans_logic), 1);

	prune->split_flag = vget_logic(fid, prompt_flag,
	"-split", split_pstr, &(prune->split_logic), 1);

	prune->automatic_flag = vget_logic(fid, prompt_flag,
	"-automatic", automatic_pstr, &(prune->automatic_logic), 1);

	prune->bg_flag = vget_logic(fid, prompt_flag,
	"-bg", bg_pstr, &(prune->bg_logic), 1);

	prune->fg_flag = vget_logic(fid, prompt_flag,
	"-fg", fg_pstr, &(prune->fg_logic), 1);

	prune->pyramid_flag = vget_logic(fid, prompt_flag,
	"-pyramid", pyramid_pstr, &(prune->pyramid_logic), 1);

	prune->trigger_flag = vget_int(fid, prompt_flag,
	"-trigger", trigger_pstr, &(prune->trigger_int),
	2, 1, 1);

	prune->cluster_flag = vget_int(fid, prompt_flag,
	"-cluster", cluster_pstr, &(prune->cluster_int),
	10, 2, 2);

	prune->rank_flag = vget_int(fid, prompt_flag,
	"-rank", rank_pstr, &(prune->rank_int),
	1, 2, 2);

	prune->sigma_flag = vget_float(fid, prompt_flag,
	"-sigma", sigma_pstr, &(prune->sigma_float),
	2.000000, 2.000000, 2.000000);

	prune->threshold_flag = vget_float(fid, prompt_flag,
	"-threshold", threshold_pstr, &(prune->threshold_float),
	3.000000, 0.000000, 0.000000);

	prune->iter_flag = vget_int(fid, prompt_flag,
	"-iter", iter_pstr, &(prune->iter_int),
	1, 2, 2);

	prune->lo_flag = vget_float(fid, prompt_flag,
	"-lo", lo_pstr, &(prune->lo_float),
	0.000000, 0.000000, 0.000000);

	prune->hi_flag = vget_float(fid, prompt_flag,
	"-hi", hi_pstr, &(prune->hi_float),
	1.000000, 0.000000, 0.000000);

	prune->raw_flag = vget_logic(fid, prompt_flag,
	"-raw", raw_pstr, &(prune->raw_logic), 1);

	prune->thorough_flag = vget_logic(fid, prompt_flag,
	"-thorough", thorough_pstr, &(prune->thorough_logic), 1);


}



