 /*
  * Khoros: $Id$
  */

#if !defined(lint) && !defined(SABER)
static char rcsid[] = "Khoros: $Id$";
#endif

 /*
  * $Log$
  */

/*
 *----------------------------------------------------------------------
 *
 * Copyright 1992, University of New Mexico.  All rights reserved.
 * Permission to copy and modify this software and its documen-
 * tation only for internal use in your organization is hereby
 * granted, provided that this notice is retained thereon and
 * on all copies.  UNM makes no representations as to the sui-
 * tability and operability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 * 
 * UNM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FIT-
 * NESS.  IN NO EVENT SHALL UNM BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY OTHER DAMAGES WHAT-
 * SOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
 * IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
 * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PER-
 * FORMANCE OF THIS SOFTWARE.
 * 
 * No other rights, including, for example, the right to redis-
 * tribute this software and its documentation or the right to
 * prepare derivative works, are granted unless specifically
 * provided in a separate license agreement.
 *---------------------------------------------------------------------
 */

#include "unmcopyright.h"        /* Copyright 1992 by UNM */

/*>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>  <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 >>>>
 >>>>         File Name: voro.c
 >>>>
 >>>>      Program Name: voro
 >>>>
 >>>>      Program Desc: 
 >>>>

 >>>> 
 >>>> Date Last Updated: Wed May 11 09:57:24 1994
 >>>>
 >>>>          Routines: main- the main program for voro
 >>>> 		 gw_usage - gives usage of the program
 >>>> 		 gw_args  - gets arguments of program from command line
 >>>>
 >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>   <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<*/


#include "voro.h"


/****************************************************************
 *
 * Routine Name:  main program for voro
 *
 *       Input:  
 *      -i2  input file of raster crack data 
 *
 *
 *      [-i1] input image or file
 *      [-o1] resulting output image or file
 *      [-o2] resulting output Voronoi file
 *      [-timer] selects one or zero
 *      [-verbose] selects one or zero
 *      [-ascii] selects one or zero
 *
 *
 *
 ****************************************************************/


main(argc, argv)
int argc;
char *argv[];
{

/* -main_variable_list */

  struct xvimage *img1, *img2, *readimage(), *createimage();
  int writeimage();

  SyVoroDAC     vd;
  SyIpoDAC*     ipd;
  SyTriDAC*     tri;
  
  int           e_mem, v_mem;
 
  char          charbuf[SY_CHARBUF_SIZE];

  int           no_ipo_found = SY_TRUE, no_trian_found = SY_FALSE;
  SyIOhandle    *iohandle, *iohandle_point;
  SyRusdelta    delta;

  enum SyStripType  data_type;

  int flag, is_ascii=SY_FALSE, is_timer=SY_FALSE, 
      is_verbose=SY_FALSE;
  int nrows, ncols, type;
/* -main_variable_list_end */

	program = VStrcpy(argv[0]);
	ac = argc;
	av = argv;

	khoros_init();

	gw_get_args();

/* -main_before_lib_call */
  if (check_args())exit(1);

  if (voro->verbose_flag){
    is_verbose = voro->verbose_logic;}

  if (voro->timer_flag){
    is_timer = voro->timer_logic;}

  if (voro->ascii_flag){
    is_ascii = voro->ascii_logic;}

  if (voro->i1_flag){
    img1 = readimage(voro->i1_file);
    if (img1 == NULL) exit(1);       
    nrows = (int )img1->col_size;
    ncols = (int )img1->row_size;
    type  = (int )img1->data_storage_type;
    
    flag = propertype (program, img1, VFF_TYP_BIT, FALSE) ||
      propertype (program, img1, VFF_TYP_1_BYTE, FALSE);
    if (!flag) exit(1);
    
    img2 = createimage ((unsigned long) nrows,
                        (unsigned long) ncols,
                        (unsigned long) VFF_TYP_1_BYTE,
                        (unsigned long) 1,
                        (unsigned long) 1,
                        "created by voro",
                        (unsigned long) 0,
                        (unsigned long) 0,
                        (unsigned long)
                        VFF_MS_NONE,
                        (unsigned long)
                        VFF_MAPTYP_NONE,
                        (unsigned long)
                        VFF_LOC_IMPLICIT,
                        (unsigned long) 0);

   memset((SY_VOID*)img2->imagedata, 0, nrows*ncols*sizeof(char));
 
  }
    
/* -main_before_lib_call_end */

/* -main_library_call */

  Sy_SetVerbose (is_verbose);

  if (is_verbose)
    fprintf (stderr, "%s: loading SyIpos from %s...\n", argv[0], 
             voro->i2_file);

  if (voro->i2_flag){
    Sy_ManageInput (voro->i2_file);
    while (((iohandle = Sy_NextIOhandle()) != NULL) && no_ipo_found){
      switch (data_type = Sy_GetIOproperty (iohandle, TYPE).type){
      case SY_IPD_STRIP:
        no_ipo_found = 0;
        ipd = (SyIpoDAC*) (Sy_GetIOproperty(iohandle, DATA).data);
        iohandle_point = iohandle;
        break;
      case SY_TRIAN_STRIP:
        no_trian_found = 0;
        tri = (SyTriDAC*) (Sy_GetIOproperty(iohandle, DATA).data);
        iohandle_point = iohandle;
        break;
      }
    }
  }
  if (no_ipo_found && no_trian_found){
    Sy_SimpleError ("No SyIpo/TRIAN file found in %s\n", voro->i2_file);
  }

  if (is_verbose){ 
    switch(data_type){
    case SY_IPD_STRIP:
      fprintf (stderr, "@%s:  SyIpos: %d  SyIcns: %d\n", 
               argv[0], ipd->ipodim, ipd->icndim);
      break;
    case SY_TRIAN_STRIP:
      fprintf (stderr, "@%s:  points: %d  edges: %d\n", 
               argv[0], tri->pdim, tri->edim);
      break;
    }
  }

  switch(data_type){
  case SY_IPD_STRIP:
    vd.pdim  = ipd->ipodim;  
    vd.point = (SyPoint*) ipd->ipo;
    break;
  case SY_TRIAN_STRIP:
    vd.pdim  = tri->pdim;
    vd.point = tri->point;
    break;
  }

  vd.edge   = Sy_AllocEdge  (3*vd.pdim);
  vd.vertex = Sy_AllocVertex(2*vd.pdim);

  if (is_timer) Sy_InitTimer();

  Sy_Voronoi (&vd, is_verbose);

  if (is_timer){ 
    delta = Sy_GetTimer(1);
    fprintf (stderr, "Processing speed: %6.0f generators/sec\n", 
             vd.pdim/delta.utime);
  }
  if (is_verbose) 
    fprintf (stderr, "\n%s: points: %d  edges: %d  vertices: %d \n",
             argv[0], vd.pdim, vd.edim, vd.vdim);
  
/* -main_library_call_end */

/* -main_after_lib_call */
  if (voro->i1_flag && voro->o1_flag){
    Sy_Voro2Khoros(&vd, img2); 
    (void) writeimage(voro->o1_file, img2);
    freeimage(img2);
  }

  if (voro->o2_flag){
    iohandle = Sy_CreateIOhandle ();
    Sy_SetIOproperty (iohandle, FORMAT, (is_ascii) ? SY_ASCII:SY_BINARY);
    Sy_SetIOproperty (iohandle, TYPE, SY_VORO_STRIP);
    Sy_SetIOproperty (iohandle, DATA, (char*) &vd);
    Sy_SetIOproperty (iohandle, IO_MODE, SY_FREE_AFTER_WRITE);
    
    Sy_SetIOproperty (iohandle_point, FORMAT, 
                      (is_ascii) ? SY_ASCII:SY_BINARY);
    
    Sy_PutIOhandle (iohandle_point);
    Sy_PutIOhandle (iohandle);
    Sy_ManageOutput (voro->o2_file);
    
    sprintf (charbuf, "@ writing Voronoi Diagram to %s \n", voro->o2_file);
    Sy_Message ("%s", charbuf);
  }

/* -main_after_lib_call_end */


	khoros_close();
}


/****************************************************************
*
* Routine Name:  gw_usage 
*
*
* Purpose:  prints out the usage for voro 
*
* Input:  none 
*
* Output:  none 
*
* Written By:  automatically generated by ghostwriter
*
****************************************************************/


gw_usage()
{
	fprintf(stderr, "voro :\n");

/* -usage_additions */
/* -usage_additions_end */

	fprintf(stderr,"\t-i2  input file of raster crack data  (infile)\n");
	fprintf(stderr, "\n");
	fprintf(stderr, "\n");
	fprintf(stderr,"\t[-i1]  input image or file (infile) [crack.vif]\n");
	fprintf(stderr,"\t[-o1]  resulting output image or file (outfile) [voro.vif]\n");
	fprintf(stderr,"\t[-o2]  resulting output Voronoi file (outfile) [test.vor]\n");
	fprintf(stderr,"\t[-timer]  selects one or zero (boolean) [true]\n");
	fprintf(stderr,"\t[-verbose]  selects one or zero (boolean) [true]\n");
	fprintf(stderr,"\t[-ascii]  selects one or zero (boolean) [true]\n");
	fprintf(stderr, " \n");
	fprintf(stderr, "\n");
	fprintf(stderr, "\t[-V] Gives the version for voro\n");
	fprintf(stderr, "\t[-U] Gives the usage for voro\n");
	fprintf(stderr, "\t[-P] Prompts for command line options\n");
	fprintf(stderr, "\t[-A [file1]] Creates the answer file called voro.ans or file1 \n");
	fprintf(stderr, "\t[-a [file1]] Uses voro.ans or file1 as the answer file \n\n\n");

}


/****************************************************************
*
* Routine Name:  gw_getargs 
*
*
* Purpose:  gets arguments off command line for voro
*
* Input:  none 
*
* Output:  none 
*
* Written By:  automatically generated by ghostwriter
*
****************************************************************/


gw_get_args()
{

char	*i2_pstr = 
	"\nEnter: (i2) input file of raster crack data \n        {infile}: "; 
char	*i1_pstr = 
	"\nEnter: (i1) input image or file \n        {infile [crack.vif] }: "; 
char	*o1_pstr = 
	"\nEnter: (o1) resulting output image or file \n        {outfile [voro.vif] }: "; 
char	*o2_pstr = 
	"\nEnter: (o2) resulting output Voronoi file \n        {outfile [test.vor] }: "; 
char	*timer_pstr = 
	"\nEnter: (timer) selects one or zero \n        {boolean, (y/n)  [yes] }: "; 
char	*verbose_pstr = 
	"\nEnter: (verbose) selects one or zero \n        {boolean, (y/n)  [yes] }: "; 
char	*ascii_pstr = 
	"\nEnter: (ascii) selects one or zero \n        {boolean, (y/n)  [yes] }: "; 
char  answer_file[512], temp[512];



int	fid = -1,
	error, query, ok, count;
	int prompt_flag = FALSE;

	/*
	 * print version if necessary
	 */
	vgparml(fid, "-V", "noprompt", &query, 0, 0, &error);
	if (error == 0)
	{
	    printf ("\n%s: %s: Version %d.%d\n\n",
	    "voro", RELEASE_NAME, RELEASE_MAJOR, RELEASE_MINOR);
	    exit(1);
	}

	/*
	 * print usage if necessary
	 */
	vgparml(fid, "-U", "noprompt", &query, 0, 0, &error);
	if (error == 0)
	{
	    gw_usage();
	    exit(1);
	}

	/*
	 * see if interactive prompting is desired
	 */
	vgparml(fid, "-P", "noprompt", &query, 0, 0, &error);
	if (error == 0)
	    prompt_flag = TRUE;

	/*
	 * see if answer file is to be used
	 */
	vgparms(fid, "-a", "noprompt", answer_file, 0, 0, &error);
	if (error == 0)
	{
	     if (VStrlen(answer_file) == 0)
		sprintf(answer_file, "voro.ans");
	     while (fid < 0)
	     {
		if ((fid = open(answer_file, O_RDONLY, 0666))== -1)
		{
		    fprintf(stderr, "voro: can't open answer file \"%s\"",
			    answer_file); 
		    fprintf(stderr, "please re-enter: ");
		    (void) gets(answer_file);
		    if (VStrlen(answer_file) == 0)
			sprintf(answer_file, "voro.ans");
		}

	     }

	}

	/*
	 * allocate the voro structure
	 */
	voro = (voro_struct *) 
	       calloc((unsigned)1, (unsigned) sizeof (voro_struct));

	/*
	 * get required arguments for voro
	 */
	if (prompt_flag == TRUE)
	    fprintf(stderr,"\nRequired Arguments:\n\n");

	if (!(voro->i2_flag = vget_infile(fid, prompt_flag,
			"-i2", i2_pstr, &(voro->i2_file))
))
	{
	    if (prompt_flag == TRUE)
	    {
		while(!(voro->i2_flag))
		{
		    fprintf(stderr, "\t\ti2 is required, please re-enter: \n");
		    voro->i2_flag = vget_infile(fid, prompt_flag,
		    "-i2", i2_pstr, &(voro->i2_file))
;

		}
	    }
	    else
	    {
		fprintf(stderr, "Error: '-i2' is a required argument\n");
		gw_usage();
		exit(1);
	    }
	}

	/*
	 * get optional arguments for voro
	 */
	if (prompt_flag == TRUE)
	   fprintf(stderr,"\nOptional Arguments:\n\n");

	voro->i1_flag = vget_infile(fid, prompt_flag,
	"-i1", i1_pstr, &(voro->i1_file))
;

	voro->o1_flag = vget_outfile(fid, prompt_flag,
	"-o1", o1_pstr, &(voro->o1_file))
;

	voro->o2_flag = vget_outfile(fid, prompt_flag,
	"-o2", o2_pstr, &(voro->o2_file))
;

	voro->timer_flag = vget_logic(fid, prompt_flag,
	"-timer", timer_pstr, &(voro->timer_logic), 1);

	voro->verbose_flag = vget_logic(fid, prompt_flag,
	"-verbose", verbose_pstr, &(voro->verbose_logic), 1);

	voro->ascii_flag = vget_logic(fid, prompt_flag,
	"-ascii", ascii_pstr, &(voro->ascii_logic), 1);


}



