/*
 *  $Id$
 * 
 *  Author: Robert L. Ogniewicz, Swiss Federal Institute of Technology ETH
 *          Zurich, Switzerland (rlo@vision.ethz.ch)
 */

/* Copyright Notice
 * ================
 * Copyright (C) 1993 by Robert L. Ogniewicz
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
 */

static char sy_AutomaticId[] = "$Id$";

#include <stdio.h>
#include <math.h>
#include "Sy_Skeleton.h"
#include "Sy_Intrinsic.h"

static int _sy_vsel_dim;

static int Sy_SortBirth3 (char *ra, char *rb)
{
  if ( ((SyRegul*)ra)->birth[0] > ((SyRegul*)rb)->birth[0] ) return (1);
  else
  if ( ((SyRegul*)ra)->birth[0] < ((SyRegul*)rb)->birth[0] ) return (-1);
  else
    return (0);
}
  
void Sy_OutputBirthSetHistogramNew (SySkelDAC *vsk, char *filename, 
				    float threshold, float val3d, int is3d)
{
  static int Sy_SortBirth3(char *ra, char *rb);
  SyRegul* buf;
  FILE*  fpout;
  int    mem, i, cnt;

  buf = (SyRegul*) Sy_Malloc (mem = (sizeof(SyRegul)*(vsk->vdim+1)) );
  memcpy ((char*) buf, (char*) vsk->regul, mem);
  
  qsort ((char*) (buf+1), vsk->vdim, sizeof(SyRegul), Sy_SortBirth3); 

  fpout = fopen (filename, "w");
  for (i=1, cnt = 0; i <= vsk->vdim; i++){
    if ((buf[i].birth[0] < SY_POS_INFTY) &&
	(buf[i].birth[0] > SY_NEG_INFTY) &&
	(buf[i].birth[0] > threshold) ){
      if (is3d)
	fprintf (fpout, "%8.3f %8.3f %8.3f\n", buf[i].birth[0], val3d, 
		 buf[i].birth[0]);
      else
	fprintf (fpout, "%8.3f %8.3f\n", buf[i].birth[0], buf[i].birth[0]);
    }
  }
  fclose(fpout); free(buf);
}

void Sy_AddMScaleInfo (SyMScaleSkelDAC *mvsk, SySkelDAC *vsk, float sigma,
		       int n_iter, int cur_iter)
{
  int i;
  float xden, yden, num;

  if (cur_iter == 0){
    mvsk->pdim    = vsk->pdim;
    mvsk->edim    = vsk->edim;
    mvsk->vdim    = vsk->vdim;
    mvsk->point   = (SyPoint*) Sy_Malloc(sizeof(SyPoint)*(vsk->pdim+1));
    mvsk->edge    = (SyMScaleEdge*) Sy_Malloc(sizeof(SyMScaleEdge)
					     *(vsk->edim+1));
    mvsk->vertex  = (SyMScaleVertex*) Sy_Malloc(sizeof(SyMScaleVertex)*
					       (vsk->vdim+1));
    mvsk->nscale  = n_iter;
    mvsk->sc      = (SyMScaleInfo*) Sy_Malloc (sizeof(SyMScaleInfo)*n_iter);

    mvsk->edge_select   = NULL;
    mvsk->vertex_select = NULL;
    mvsk->edge_select_dim = mvsk->vertex_select_dim = 0;

    mvsk->label = NULL;

    for (i = 1; i <= vsk->pdim; i++) mvsk->point[i] = vsk->point[i];
    for (i = 1; i <= vsk->edim; i++){
      mvsk->edge[i].tail_edge   = vsk->edge[i].tail_edge;
      mvsk->edge[i].head_edge   = vsk->edge[i].head_edge;
      mvsk->edge[i].tail_vertex = vsk->edge[i].tail_vertex;
      mvsk->edge[i].head_vertex = vsk->edge[i].head_vertex;
      mvsk->edge[i].left_point  = vsk->edge[i].left_point;
      mvsk->edge[i].right_point = vsk->edge[i].right_point;
      mvsk->edge[i].rank        = vsk->edge[i].rank;
      mvsk->edge[i].dirx        = vsk->edge[i].dirx;
      mvsk->edge[i].diry        = vsk->edge[i].diry;
      mvsk->edge[i].xoff        = vsk->edge[i].xoff;
      mvsk->edge[i].yoff        = vsk->edge[i].yoff;
      
      mvsk->edge[i].nscale = n_iter;
      mvsk->edge[i].sc = (SyMScaleEdgeInfo*) 
	Sy_Malloc(sizeof(SyMScaleEdgeInfo)* n_iter);
    }
    
    for (i = 1; i <= vsk->vdim; i++){
      mvsk->vertex[i].xden = xden = vsk->vertex[i].xden;
      mvsk->vertex[i].yden = yden = vsk->vertex[i].yden;
      mvsk->vertex[i].num  = num  = vsk->vertex[i].num;
      mvsk->vertex[i].x    = xden/num;
      mvsk->vertex[i].y    = yden/num;
      mvsk->vertex[i].nscale = n_iter;
      mvsk->vertex[i].sc = (SyMScaleVertexInfo*) 
	Sy_Malloc(sizeof(SyMScaleVertexInfo)* n_iter);      
      mvsk->vertex[i].nincident = 0;
      mvsk->vertex[i].incident  = (SyIndex*)NULL;
    }
  }
  
  mvsk->sc[cur_iter].sigma = sigma;
  for (i = 1; i <= vsk->vdim; i++){
    mvsk->vertex[i].sc[cur_iter].birth[0] = vsk->regul[i].birth[0];
    mvsk->vertex[i].sc[cur_iter].birth[1] = vsk->regul[i].birth[1];
    mvsk->vertex[i].sc[cur_iter].birth[2] = vsk->regul[i].birth[2];
    mvsk->vertex[i].sc[cur_iter].radius   = vsk->regul[i].radius;
    vsk->regul[i].is_void_nfurcation      = SY_FALSE; 
  }
}

void Sy_OutputSkelSpaceLog (SyMScaleSkelDAC *mvsk, char *filename, 
			    float threshold)
{
  FILE*  fpout;
  int    i, k;

  fpout = fopen (filename, "w");

  for (i = 1; i < mvsk->vdim; i++){
    if (mvsk->vertex[i].sc[0].birth[0] > threshold){
      fprintf (fpout, "# vertex <%d> at (%f,%f)\n",
	       i, mvsk->vertex[i].x, mvsk->vertex[i].y);
      for (k = 0; k < mvsk->nscale; k++){
	fprintf (fpout, "%f %f %f %f\n", mvsk->sc[k].sigma,
		 mvsk->vertex[i].sc[k].birth[0],
		 mvsk->vertex[i].sc[k].birth[1],
		 mvsk->vertex[i].sc[k].birth[2]);
      }
      fprintf (fpout,"\n\n");

    }

  }
  close(fpout);
}

static int Sy_ScaleSelSort(char *ra, char *rb)
{
  if ( ((SyScaleSel*)ra)->sal > ((SyScaleSel*)rb)->sal ) return (-1);
  else
  if ( ((SyScaleSel*)ra)->sal < ((SyScaleSel*)rb)->sal ) return (1);
  else
    return (0);
}

void Sy_SkelSpaceRegularize (SyMScaleSkelDAC *mvsk, 
			     SySkelDAC *vsk, SyIpoDAC *ipd,
			     SyRegulControl *rcntrl)
{
  SyRegulControl control;

  SyIndex         *v_lookup = NULL, 
                  *e_lookup = NULL;
  SyScaleSel      *sca = NULL, *scb = NULL;
  SyMScaleVertex  *mvertex = NULL;

  int v_sel_dim, e_sel_dim, insign_count0, insign_count1, nremoval;
  int v_ptr, e_ptr, cur_scale, next_scale, i, j, k;
  int no_new_traces, perturb_count, higher_scale, is_notsignif,
      void_nfurcation_count;
  int is_verbose = SY_FALSE, is_smcir_break = SY_FALSE, is_verbose_buf;
  float sigma_lo, sigma_hi, sigma_lo_bound, 
        sigma, sigma_step, sigma_delta = 1.0;
  float threshold, select_threshold, xhit, corr_fac;
  SyIndex tail_vertex, head_vertex;
  int n_iter, cptr, max_cptr;
  int lo_ptr, hi_ptr, med_ptr, tmpint, numhi, num_sal;
  float sigma_lo_tmp, sigma_hi_tmp, sigma_old, hi_min, tmpval, bbox_len,
        max_ellipse, ellipse_b, ellipse_b_smooth;

  int concave_count = 0, concave_count_0 = 0, l, iter;

  SyMScaleVertexInfo *scaleinfo;
  float cluster_ratio, old_cluster_ratio;

  enum SyPruneMethod prune_method;
  
  control                 = *rcntrl;
  threshold               = rcntrl->base_threshold;
  control.no_shrink_estim = SY_FALSE;
  control.edge_select     = mvsk->edge_select;
  control.edge_select_dim = mvsk->edge_select_dim;
  is_verbose_buf          = syIsVerbose;
  Sy_SetVerbose(SY_FALSE);

#ifdef DEBUG
  control.is_concave_point  = (SyBoolean*) Sy_Malloc(sizeof(SyBoolean)*
				  		    (ipd->ipodim+1));
  control.is_concave_vertex = (SyBoolean*) Sy_Malloc(sizeof(SyBoolean)*
						     (vsk->vdim+1));
  control.is_concave_vertex_0 = (SyBoolean*) Sy_Malloc(sizeof(SyBoolean)*
						     (vsk->vdim+1));
#endif

  vsk->is_nfurcation_info = SY_TRUE;

  sigma_lo   = rcntrl->sigma_lo;
  sigma_hi   = rcntrl->sigma_hi;
  sigma_step = rcntrl->sigma_step;
  sigma      = rcntrl->sigma_lo;
  
  n_iter     = 5;
  sigma      = sigma_lo;
  
  v_lookup = (SyIndex*) Sy_Malloc (sizeof(SyIndex)*(vsk->vdim+1));
  e_lookup = (SyIndex*) Sy_Malloc (sizeof(SyIndex)*(vsk->edim+1));
  
  if (vsk->regul != (SyRegul*)NULL) {
    free(vsk->regul); 
    vsk->regul = (SyRegul*)NULL;}

  Sy_RegSmooth (vsk, ipd, Sy_SmoothRegularize, is_verbose, sigma,
		SY_FALSE, NULL);

  Sy_AddMScaleInfo (mvsk, vsk, sigma, n_iter, 0);  

  v_sel_dim = iter = 0;
  select_threshold = threshold;
  while (v_sel_dim <= 0 && iter < 5){
    
    for (i = 1, v_sel_dim = 0; i <= mvsk->vdim; i++){
      if (mvsk->vertex[i].sc[0].birth[0] >= SY_POS_INFTY) continue;
      if (mvsk->vertex[i].sc[0].birth[0] > select_threshold) v_sel_dim++;
    }
    
    if (v_sel_dim <= 0){
      select_threshold *= 0.5;
      iter++;
    }    
  } /* while */
 
  mvsk->vertex_select = (SyIndex*) Sy_Malloc(sizeof(SyIndex)*(v_sel_dim+1));
  mvsk->vertex_select_dim = _sy_vsel_dim = v_sel_dim;

  Sy_SetVerbose(is_verbose_buf);
  if (syIsVerbose){
    sprintf(syStrBuf, "Tracking %d nodes.\nScales:", v_sel_dim);
    Sy_Message(SY_FMT, syStrBuf);
  }
  Sy_SetVerbose(SY_FALSE);

  mvertex = (SyMScaleVertex*) Sy_Malloc(sizeof(SyMScaleVertex)*(v_sel_dim+1));
  for (i = 0; i <= v_sel_dim; i++){
    mvertex[i].sc = NULL;
    mvertex[i].incident = NULL;
  }

  for (i = 1, v_ptr = 0; i <= mvsk->vdim; i++){
    if (mvsk->vertex[i].sc[0].birth[0] >= SY_POS_INFTY) continue;
    if (mvsk->vertex[i].sc[0].birth[0] > select_threshold){
      v_ptr++;
      mvsk->vertex_select[v_ptr] = i;
      mvertex[v_ptr]             = mvsk->vertex[i];
      mvertex[v_ptr].self        = i;
      v_lookup[i]                = v_ptr;
    }
    else {
      v_lookup[i] = -1;
    }
  }

  for (i = 1, e_sel_dim = 0; i <= mvsk->edim; i++){
    tail_vertex = mvsk->edge[i].tail_vertex;
    head_vertex = mvsk->edge[i].head_vertex;
    if (mvsk->vertex[tail_vertex].sc[0].birth[0] >= SY_POS_INFTY) continue;
    if (mvsk->vertex[head_vertex].sc[0].birth[0] >= SY_POS_INFTY) continue;
    if ((mvsk->vertex[tail_vertex].sc[0].birth[0] > select_threshold) ||
	(mvsk->vertex[head_vertex].sc[0].birth[0] > select_threshold) ) 
      e_sel_dim++;
  }
  mvsk->edge_select = (SyIndex*) Sy_Malloc(sizeof(SyIndex)*(e_sel_dim+1));
  mvsk->edge_select_dim = e_sel_dim;

#ifdef DEBUG
  sprintf(syStrBuf, "Selected %d edges", e_sel_dim);
  Sy_Message(SY_FMTNL, syStrBuf);
#endif

  for (i = 1, e_ptr = 0; i <= mvsk->edim; i++){
    tail_vertex = mvsk->edge[i].tail_vertex;
    head_vertex = mvsk->edge[i].head_vertex;
    if (mvsk->vertex[tail_vertex].sc[0].birth[0] >= SY_POS_INFTY) continue;
    if (mvsk->vertex[head_vertex].sc[0].birth[0] >= SY_POS_INFTY) continue;
    if ((mvsk->vertex[tail_vertex].sc[0].birth[0] > select_threshold) ||
	(mvsk->vertex[head_vertex].sc[0].birth[0] > select_threshold) ){
      e_ptr++;
      mvsk->edge_select[e_ptr] = i;
      e_lookup[i] = e_ptr;
    }
    else {
      e_lookup[i] = -1;
    }
  }

  control.edge_select     = mvsk->edge_select;
  control.edge_select_dim = mvsk->edge_select_dim;
  control.is_edge_selection = SY_TRUE;

  for (i = 1; i <= v_sel_dim; i++){
    mvertex[i].sc = (SyMScaleVertexInfo*) 
      Sy_Malloc(sizeof(SyMScaleVertexInfo)* n_iter);
    mvertex[i].sc[0] = mvsk->vertex[mvertex[i].self].sc[0];
  }      

  sca = (SyScaleSel*) Sy_Malloc (sizeof(SyScaleSel)*(v_sel_dim+1));
  scb = (SyScaleSel*) Sy_Malloc (sizeof(SyScaleSel)*(v_sel_dim+1));

  insign_count0 = 0;
  max_ellipse = 0;
  for (cptr = max_cptr = 1; cptr <= ipd->contourdim; cptr++){

#ifdef DEBUG
    sprintf(syStrBuf, "contour <%d> length: %f ", 
	    cptr, ipd->info[cptr].orig_len);
    Sy_Message (SY_FMTNL, syStrBuf);
    sprintf(syStrBuf, "first ellipse length: %f ", 
	    ipd->info[cptr].first_ellipse_len);
    Sy_Message (SY_FMTNL, syStrBuf);
#endif

    if (ipd->info[cptr].first_ellipse_b_smooth > max_ellipse){
      max_ellipse = ipd->info[cptr].first_ellipse_b_smooth;
      max_cptr = cptr;
    }

  }
  bbox_len = 2.0*(ipd->max_x - ipd->min_x + ipd->max_y - ipd->min_y);

#ifdef DEBUG
  sprintf (syStrBuf, "bounding box length: %f ", bbox_len);
  Sy_Message (SY_FMTNL, syStrBuf);
#endif

  ellipse_b        = ipd->info[max_cptr].first_ellipse_b;
  ellipse_b_smooth = ipd->info[max_cptr].first_ellipse_b_smooth;

#ifdef DEBUG
  fprintf (stderr, "ellipse b: %f, smoothed: %f\n",
	   ellipse_b, ellipse_b_smooth);
#endif

  corr_fac = (float)ipd->ipodim/ipd->info[max_cptr].orig_len;
  sigma_hi = ellipse_b_smooth * 2.0 * M_PI * 0.34 * corr_fac;
  sigma_lo_bound = fabs(ellipse_b-ellipse_b_smooth) * 2.0 * 0.34 * corr_fac;
  
  if ((sigma_hi - sigma_lo) < sigma_step){
    n_iter = 1;
    sigma  = sigma_lo;}
  else{
    n_iter = ceil(log((sigma_hi-sigma_lo)/sigma_step)/log(2.0));
    sigma  = sigma_hi;
  }

#ifdef DEBUG
  Sy_Message(SY_FMTNL, "computing filter info ");
#endif

  if (vsk->regul != (SyRegul*)NULL) {
    free(vsk->regul); 
    vsk->regul = (SyRegul*)NULL;
  }

  if (control.is_thorough){
    prune_method = vsk->prune_method;
    vsk->prune_method = SKE_SMCIR;
    Sy_RegSmooth (vsk, ipd, Sy_SmoothRegularize, is_verbose, 
		  3.0 /* constant sigma */, SY_FALSE, NULL /* &control */);  
    vsk->prune_method = prune_method;
    
    for (i = 1; i <= v_sel_dim; i++){
      mvertex[i].smcir.birth[0] = vsk->regul[mvertex[i].self].birth[0];
      mvertex[i].smcir.birth[1] = vsk->regul[mvertex[i].self].birth[1];
      mvertex[i].smcir.birth[2] = vsk->regul[mvertex[i].self].birth[2];
      mvertex[i].smcir.is_void_nfurcation = SY_FALSE;
    }

    vsk->regul_aux = vsk->regul;
    vsk->is_smcir  = SY_TRUE;
    vsk->regul     = (SyRegul*)NULL;
  }
  else{
    for (i = 1; i <= v_sel_dim; i++){
      mvertex[i].smcir.birth[0] = 0.0;
      mvertex[i].smcir.birth[1] = 0.0;
      mvertex[i].smcir.birth[2] = 0.0;
      mvertex[i].smcir.is_void_nfurcation = SY_FALSE;
    }
    vsk->is_smcir  = SY_FALSE;
  }

#ifdef DEBUG
  Sy_Message(SY_FMTNL, "done computing filter info");
#endif

  if (control.is_thorough){
    is_smcir_break = SY_FALSE; void_nfurcation_count = 0;
    for (k = 1; k <= v_sel_dim; k++){
      if (mvertex[k].smcir.birth[1] < control.smcir_search_thres){
	is_smcir_break = SY_TRUE;
	void_nfurcation_count++;
#ifdef DEBUG
	sprintf(syStrBuf, "marked vertex %d ", mvertex[k].self);
	Sy_Message (SY_FMTNL, syStrBuf);
#endif
	mvertex[k].smcir.is_void_nfurcation = SY_TRUE;
      }
    }

#ifdef DEBUG  
    if (is_smcir_break){
      sprintf(syStrBuf, "%d insignificant n-furcations found ",
	      void_nfurcation_count);
      Sy_Message (SY_FMTNL, syStrBuf);
    }
#endif
  } /* if */


  if (vsk->regul != (SyRegul*)NULL) {
    free(vsk->regul); vsk->regul = (SyRegul*)NULL;
  }

  
  Sy_RegSmooth (vsk, ipd, Sy_SmoothRegularize, is_verbose, sigma_lo,
		SY_FALSE, &control);  
	
#ifdef DEBUG		
  for (l = 1, concave_count_0 = 0; l <= vsk->vdim; l++){
    if (control.is_concave_vertex[l] && vsk->regul[l].birth[0] > threshold){
      control.is_concave_vertex_0[l] = SY_TRUE;
      concave_count_0++;
    }
  }
  fprintf (stderr, "initially %d concave vertices\n", concave_count_0);
#endif

  Sy_SetVerbose(is_verbose_buf);
  if (syIsVerbose){
    sprintf(syStrBuf, "[%4.1f] ", sigma);
    Sy_Message (SY_FMTRAW, syStrBuf);
  }
  Sy_SetVerbose(SY_FALSE);

  if (vsk->regul != (SyRegul*)NULL) {
    free(vsk->regul); vsk->regul = (SyRegul*)NULL;
  }

  Sy_RegSmooth (vsk, ipd, Sy_SmoothRegularize, is_verbose, sigma,
		SY_FALSE, &control);  

#ifdef DEBUG
  for (l = 1, concave_count_0 = 0; l <= vsk->vdim; l++){
    if (control.is_concave_vertex[l] && vsk->regul[l].birth[0] > threshold) 
      concave_count_0++;
  }
  fprintf (stderr, "sigma %f, conc. vertices -> %d\n", sigma, concave_count_0);
#endif


  hi_min = SY_POS_INFTY;
  for (v_ptr = 1, k = 1, numhi = 0; v_ptr <= v_sel_dim; v_ptr++){
    mvertex[v_ptr].sc[k].birth[0]=vsk->regul[mvertex[v_ptr].self].birth[0];
    mvertex[v_ptr].sc[k].birth[1]=vsk->regul[mvertex[v_ptr].self].birth[1];
    mvertex[v_ptr].sc[k].birth[2]=vsk->regul[mvertex[v_ptr].self].birth[2];
    if ((tmpval = mvertex[v_ptr].sc[k].birth[0]) > threshold 
	&& mvertex[v_ptr].smcir.is_void_nfurcation == SY_FALSE ){ 
      numhi++;
      if (tmpval < hi_min) hi_min = tmpval;
    }
  }

  Sy_SetVerbose(is_verbose_buf);
  sprintf(syStrBuf, "[%d principal trace(s)] ", numhi);
  Sy_Message (SY_FMT, syStrBuf);
  Sy_SetVerbose(SY_FALSE);

  sigma_lo_tmp = sigma_lo; sigma_hi_tmp = sigma_hi;
  lo_ptr = 0; hi_ptr = 1; med_ptr = 2;

  /* now do the real thing */

  control.no_shrink_estim = SY_TRUE; 

  for (k = 1; k <= n_iter; k++){

    if (vsk->regul != (SyRegul*)NULL){
      free(vsk->regul); 
      vsk->regul = (SyRegul*)NULL;
    }
    sigma_old = sigma;
    sigma = (sigma_lo_tmp + sigma_hi_tmp)/2.0;
    if (sigma < sigma_lo_bound) {
      sigma = sigma_lo_bound;
      sigma_lo_tmp = sigma;
    }

    Sy_SetVerbose(is_verbose_buf);
    if (syIsVerbose){
      sprintf(syStrBuf, "[%4.1f] ", sigma);
      Sy_Message (SY_FMTRAW, syStrBuf);
    }
    Sy_SetVerbose(SY_FALSE);
    
    Sy_RegSmooth (vsk, ipd, Sy_SmoothRegularize, is_verbose, sigma,
		  SY_FALSE, &control);  
    
#ifdef DEBUG
    for (l = 1, concave_count = 0; l <= vsk->vdim; l++){
      if (control.is_concave_vertex[l] && vsk->regul[l].birth[0] > threshold) 
	concave_count++;
    }
    fprintf (stderr, "sigma %f, conc. vertices -> %d\n", sigma, concave_count);
#endif
    
    for (v_ptr = 1; v_ptr <= v_sel_dim; v_ptr++){
      mvertex[v_ptr].sc[med_ptr].birth[0]=vsk->regul[mvertex[v_ptr].self].birth[0];
      mvertex[v_ptr].sc[med_ptr].birth[1]=vsk->regul[mvertex[v_ptr].self].birth[1];
      mvertex[v_ptr].sc[med_ptr].birth[2]=vsk->regul[mvertex[v_ptr].self].birth[2];
    }

    higher_scale = SY_TRUE;
    
    for (i = 1; i <= v_sel_dim; i++){
      sca[i].slope = mvertex[i].sc[med_ptr].slope[0];
      sca[i].self  = mvertex[i].self;
      sca[i].vptr  = i;
      sca[i].sal   = mvertex[i].sc[med_ptr].birth[0];
      scb[i]       = sca[i];
      scb[i].sal   = mvertex[i].sc[hi_ptr].birth[0];
    }
    
    qsort ((char*)(sca+1), v_sel_dim, sizeof(SyScaleSel), 
	   Sy_ScaleSelSort);     
    
    qsort ((char*)(scb+1), v_sel_dim, sizeof(SyScaleSel), 
	   Sy_ScaleSelSort);     
    
    /* check for perturbations */
    for (perturb_count = 0, i = 1; i <= v_sel_dim; i++){
      if (scb[i].sal < threshold) break;
      if (i < v_sel_dim && scb[i+1].sal < threshold) break;

      if (i==1) is_notsignif = 
	(fabs(sca[i].sal-sca[i+1].sal) < threshold) ||
	  (fabs(scb[i].sal-scb[i+1].sal) < threshold);
      else
	if (i==v_sel_dim) is_notsignif = 
	  (fabs(sca[i].sal-sca[i-1].sal) < threshold) ||
	    (fabs(scb[i].sal-scb[i-1].sal) < threshold);
      else
	is_notsignif = ( (fabs(sca[i].sal-sca[i-1].sal) < threshold) &&
	  (fabs(sca[i].sal-sca[i+1].sal) < threshold) ) ||
	    ( (fabs(scb[i].sal-scb[i-1].sal) < threshold) &&
	     (fabs(scb[i].sal-scb[i+1].sal) < threshold) );
      sca[i].signif = !is_notsignif;
      if (is_notsignif) continue;
      if (sca[i].self != scb[i].self) perturb_count++;
    }

    /* check for smoothness of birth3 - traces */
    if (perturb_count == SY_FALSE){
      no_new_traces = SY_TRUE; j = 1;
      while (no_new_traces && (j <= v_sel_dim)){
	if (sca[j].sal < threshold) break;
	j++;
      }
      j--;

#ifdef DEBUG      
      fprintf (stderr, "numhi: %d, j: %d\n", numhi, j);
#endif

      if (j > numhi) no_new_traces = SY_FALSE;
/*      else if (j < numhi){ 
	numhi = max(j,1);
	no_new_traces = SY_FALSE;
      }
*/

    } /* if (perturb_count... */

#ifdef DEBUG
    concave_count = 0; j = 1;
    while (j <= v_sel_dim){
      if (sca[j].sal < threshold) break;
      if (control.is_concave_vertex[sca[j].self] &&
	  control.is_concave_vertex_0[sca[j].self]){
	concave_count++;
	control.is_concave_vertex[sca[j].self] = SY_FALSE;
      }
      j++;
    }
#endif

#ifdef DEBUG
    fprintf (stderr, "sigma %f, conc. vertices -> %d\n", sigma, concave_count);
#endif

    higher_scale = (perturb_count > 0) || 
                   (no_new_traces == SY_FALSE /* 
		    || concave_count > concave_count_0 */ );
                   
    
    if (higher_scale){
      sigma_lo_tmp = sigma;
      tmpint = med_ptr; med_ptr = lo_ptr; lo_ptr = tmpint;
    }
    else{
      sigma_hi_tmp = sigma;
      tmpint = med_ptr; med_ptr = hi_ptr; hi_ptr = tmpint;
    }
  } /* for number of iterations */

  Sy_SetVerbose(is_verbose_buf);
  if (syIsVerbose){
    Sy_Message (SY_FMTNL, "halt");
  }
  Sy_SetVerbose(SY_FALSE);
  
  cluster_ratio = old_cluster_ratio = 0.0;
  for (i = 1, num_sal = 0; i <= v_sel_dim; i++){
    if (scb[i].sal < threshold) break;
    num_sal++;
    old_cluster_ratio = cluster_ratio;
    scaleinfo = &(mvertex[scb[i].vptr].sc[hi_ptr]);
    if (scaleinfo->birth[1] != 0.0){
      cluster_ratio = fabs(1.0 - scaleinfo->birth[0]/scaleinfo->birth[1]);}
    if (scaleinfo->birth[2] != 0.0){
      cluster_ratio = max(fabs(1.0 - scaleinfo->birth[1]/scaleinfo->birth[2]),
			  cluster_ratio);}

    cluster_ratio   = max(cluster_ratio, old_cluster_ratio);
  }
  cluster_ratio = max(control.pyr_cluster_delta, cluster_ratio);

  is_smcir_break = SY_FALSE;
  void_nfurcation_count = 0;

  if (control.is_thorough){
    for (i = 1; i <= v_sel_dim; i++){
      scb[i].smcir_sal = sca[i].smcir_sal = 
	vsk->regul_aux[mvertex[sca[i].vptr].self].birth[1];
      mvertex[sca[i].vptr].smcir.is_void_nfurcation = SY_FALSE;
    }
    
    for (k = 1; k <= v_sel_dim; k++){
      if (scb[k].smcir_sal < control.smcir_search_thres){
	is_smcir_break = SY_TRUE; void_nfurcation_count++;
#ifdef DEBUG
	sprintf(syStrBuf, "marked vertex %d ", mvertex[scb[k].vptr].self);
	Sy_Message (SY_FMTNL, syStrBuf);
#endif
	mvertex[scb[k].vptr].smcir.is_void_nfurcation = SY_TRUE;
      }
    }
  
#ifdef DEBUG
    if (is_smcir_break){
      sprintf(syStrBuf, "%d insignificant n-furcations found ",
	      void_nfurcation_count);
      Sy_Message (SY_FMTNL, syStrBuf);
    }
#endif
  } /* if control.is_thorough... */
  
  control.sigma_hint = higher_scale ? sigma_hi_tmp : sigma; 

  if (vsk->regul != (SyRegul*)NULL) {
    free(vsk->regul); vsk->regul = (SyRegul*)NULL;}

  control.no_shrink_estim = SY_FALSE;

  Sy_SetResidualCompMode (RES_FULL); 

#ifdef DEBUG
  fprintf (stderr, "pruning with sigma %f\n", control.sigma_hint);
#endif

  Sy_RegSmooth (vsk, ipd, Sy_SmoothRegularize, is_verbose, 
		  control.sigma_hint, SY_FALSE, NULL);

  Sy_SetResidualCompMode (RES_NORMAL); 

  if (v_sel_dim > 0){
    if (num_sal >0){
      control.pyr_trigger_hint = /* scb[max(num_sal,1)].sal; */
	vsk->regul[scb[max(num_sal,1)].self].birth[0]; }
    else{
      control.pyr_trigger_hint = threshold;}

    control.is_pyr_trigger_hint = SY_TRUE;
    control.is_pyr_cluster_hint = SY_TRUE; 
    control.cluster_max         = 
      mvertex[scb[max(i-1,1)].vptr].sc[hi_ptr].birth[2];
    control.pyr_cluster_hint    = 100.0*cluster_ratio;

    Sy_SetVerbose(is_verbose_buf);
    if (syIsVerbose){
      sprintf (syStrBuf, 
	       "Trigger level for pyramid: %f\nClustering (at nodes): %f\%", 
	       control.pyr_trigger_hint, control.pyr_cluster_hint);
      Sy_Message (SY_FMTNL, syStrBuf);
    }      
    Sy_SetVerbose(SY_FALSE);
  }

  for (i = 1; i <= v_sel_dim; i++){
    vsk->regul[mvertex[i].self].is_void_nfurcation =
      mvertex[i].smcir.is_void_nfurcation;
  }

  /* adaptation of residual values */

  Sy_Free(control.is_concave_point);  control.is_concave_point = NULL;
  Sy_Free(control.is_concave_vertex); control.is_concave_vertex = NULL;
  Sy_Free(control.is_concave_vertex_0); control.is_concave_vertex_0 = NULL;

  Sy_Free(e_lookup);
  Sy_Free(v_lookup);
  Sy_Free(sca);
  Sy_Free(scb);

  for (i = 0; i <= v_sel_dim; i++){
    Sy_DestroyMScaleVertex(mvertex + i);  
  }

  Sy_Free(mvertex);
  
  Sy_DestroyMScaleSkelDAC (mvsk);

  *rcntrl = control;

  Sy_SetVerbose(is_verbose_buf);
}

static void Sy_InsertSimple (SySkelDAC *vsk, int vindex, float dev);

void Sy_RelaxPruning (SySkelDAC *vsk, SyIpoDAC *ipd, SyRegulControl *rc)
{
  SyEresid *eresid;
  float sigma_lo;
  int i;

  if (rc->sigma_hint <= 3.0) return;
  sigma_lo = max(rc->sigma_hint/4.0,3.0);

  eresid = vsk->eresid;
  vsk->eresid = NULL;
  vsk->is_eresid = SY_FALSE;

  if (vsk->regul) Sy_Free(vsk->regul); 
  vsk->regul = NULL;

  Sy_SetResidualCompMode (RES_FULL); 
  Sy_RegSmooth (vsk, ipd, Sy_SmoothRegularize, SY_FALSE, 
		sigma_lo, SY_FALSE, NULL);  
  Sy_SetResidualCompMode (RES_NORMAL); 

  for (i = 1; i <= vsk->edim; i++){
    if (vsk->hier[i] != 1) eresid[i] = vsk->eresid[i];
  }

  for (i = 1; i <= vsk->vdim; i++){
    vsk->regul[i].birth[0] = SY_NEG_INFTY;
    vsk->regul[i].birth[1] = SY_NEG_INFTY;
    vsk->regul[i].birth[2] = SY_NEG_INFTY;
  }

  for (i = 1; i <= vsk->edim; i++){
    Sy_InsertSimple(vsk, vsk->edge[i].tail_vertex, eresid[i].tail_residual);
    Sy_InsertSimple(vsk, vsk->edge[i].head_vertex, eresid[i].head_residual);
  }
  
  Sy_Free(vsk->eresid); 
  vsk->eresid = NULL;
  vsk->is_eresid = SY_FALSE;
  Sy_Free(eresid);
}

static void Sy_InsertSimple (SySkelDAC *vsk, int vindex, float dev)
{
  float*  birth;

  birth = vsk->regul[vindex].birth;

  if (birth[2] < dev){
    birth[0] = birth[1];
    birth[1] = birth[2];
    birth[2] = dev;
    return;
  }
  if (birth[1] < dev){
    birth[0] = birth[1];
    birth[1] = dev;
    return;
  }
  if (birth[0] < dev){
    birth[0] = dev;
  }
}

