/*
 *  $Id$
 * 
 *  Author: Robert L. Ogniewicz, Swiss Federal Institute of Technology ETH
 *          Zurich, Switzerland (rlo@vision.ethz.ch)
 */

/* Copyright Notice
 * ================
 * Copyright (C) 1993 by Robert L. Ogniewicz
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
 */

static char sy_ChronoId[] = "$Id$"; 

#include "Sy_Skeleton.h"
#include "Sy_Intrinsic.h"

#ifndef HZ
#define SY_HZ 60
#else
#define SY_HZ HZ
#endif

static struct rusage rus0, rus1;
static double wall0, wall1;

double Sy_GetWallClock(void);

double Sy_GetWallClock()
{
  struct timeval tp;
  double retval;
#if defined (hpux)
  gettimeofday(&tp, NULL);
#elif defined(SVR4) || defined(SYSV) 
 gettimeofday(&tp);
#else
 gettimeofday(&tp, NULL);
#endif
  retval = (double)tp.tv_sec + (1.0*tp.tv_usec)/1000000.0;
  return retval;
}

char wt[] = "wallclock:";
char ut[] = "user time:";
char st[] = "system time:";
char pr[] = "page reclaims:";
char pf[] = "page faults:";
char sw[] = "swaps:";
char ic[] = "input count:";
char oc[] = "output count:";
char vs[] = "vol. context swi.:";
char is[] = "invol. cont. swi.:";

void Sy_InitTimer(void)
{
#ifndef hpux
  getrusage(RUSAGE_SELF,&rus0);
#endif
  wall0 = Sy_GetWallClock();
}

void Sy_ShowTimer(void)
{
  float utime0, utime1, stime0, stime1;

#ifndef hpux
  getrusage(RUSAGE_SELF,&rus1);
#endif

  wall1 = Sy_GetWallClock();

  utime0 = SY_MICRO *(float) rus0.ru_utime.tv_sec + (float) rus0.ru_utime.tv_usec;
  utime1 = SY_MICRO *(float) rus1.ru_utime.tv_sec + (float) rus1.ru_utime.tv_usec;
  stime0 = SY_MICRO *(float) rus0.ru_stime.tv_sec + (float) rus0.ru_stime.tv_usec;
  stime1 = SY_MICRO *(float) rus1.ru_stime.tv_sec + (float) rus1.ru_stime.tv_usec;

  fprintf(stderr, "\n*** resource utilization ***");
  fprintf(stderr, "****************************\n");
  fprintf(stderr, "%-18s %8.3f\n", wt, wall1-wall0);
#ifndef hpux
  fprintf(stderr, "%-18s %8.3f", ut, (utime1-utime0)/SY_MICRO);
  fprintf(stderr, " %-18s %8d\n", sw, rus1.ru_nswap   - rus0.ru_nswap);
  fprintf(stderr, "%-18s %8.3f", st, (stime1-stime0)/SY_MICRO);
  fprintf(stderr, " %-18s %8d\n", ic, rus1.ru_inblock - rus0.ru_inblock);
  fprintf(stderr, "%-18s %8d", pr, rus1.ru_minflt  - rus0.ru_minflt);
  fprintf(stderr, " %-18s %8d\n", oc, rus1.ru_oublock - rus0.ru_oublock);
  fprintf(stderr, "%-18s %8d", pf, rus1.ru_majflt  - rus0.ru_majflt);  
  fprintf(stderr, " %-18s %8d\n", vs, rus1.ru_nvcsw   - rus0.ru_nvcsw);
  fprintf(stderr, "%-27s", "");
  fprintf(stderr, " %-18s %8d\n", is, rus1.ru_nivcsw  - rus0.ru_nivcsw);
#endif
  fprintf(stderr, "****************************");
  fprintf(stderr, "****************************\n");
}

SyRusdelta Sy_GetTimer(int display_flag)
{
  SyRusdelta delta;
  float utime0, utime1, stime0, stime1;

  wall1 = Sy_GetWallClock();

#ifndef hpux
  getrusage(RUSAGE_SELF,&rus1);

  utime0 = SY_MICRO *(float) rus0.ru_utime.tv_sec + (float) rus0.ru_utime.tv_usec;
  utime1 = SY_MICRO *(float) rus1.ru_utime.tv_sec + (float) rus1.ru_utime.tv_usec;
  stime0 = SY_MICRO *(float) rus0.ru_stime.tv_sec + (float) rus0.ru_stime.tv_usec;
  stime1 = SY_MICRO *(float) rus1.ru_stime.tv_sec + (float) rus1.ru_stime.tv_usec;
  delta.utime = (utime1-utime0)/SY_MICRO;
  delta.stime = (stime1-stime0)/SY_MICRO;
#endif

  delta.wtime = wall1-wall0;

#if defined (hpux)
  delta.utime = delta.wtime;
#endif

  if (display_flag){
    fprintf(stderr, "\n*** resource utilization ***");
    fprintf(stderr, "****************************\n");
    fprintf(stderr, "%-18s %8.3f\n", wt, wall1-wall0);
#ifndef hpux
    fprintf(stderr, "%-18s %8.3f", ut, (utime1-utime0)/SY_MICRO);
    fprintf(stderr, " %-18s %8d\n", sw, rus1.ru_nswap   - rus0.ru_nswap);
    fprintf(stderr, "%-18s %8.3f", st, (stime1-stime0)/SY_MICRO);
    fprintf(stderr, " %-18s %8d\n", ic, rus1.ru_inblock - rus0.ru_inblock);
    fprintf(stderr, "%-18s %8d", pr, rus1.ru_minflt  - rus0.ru_minflt);
    fprintf(stderr, " %-18s %8d\n", oc, rus1.ru_oublock - rus0.ru_oublock);
    fprintf(stderr, "%-18s %8d", pf, rus1.ru_majflt  - rus0.ru_majflt);  
    fprintf(stderr, " %-18s %8d\n", vs, rus1.ru_nvcsw   - rus0.ru_nvcsw);
    fprintf(stderr, "%-27s", "");
    fprintf(stderr, " %-18s %8d\n", is, rus1.ru_nivcsw  - rus0.ru_nivcsw);
#endif
    fprintf(stderr, "****************************");
    fprintf(stderr, "****************************\n");
  }
  
  return (delta);
}
