/*
 *  $Id$
 * 
 *  Author: Robert L. Ogniewicz, Swiss Federal Institute of Technology ETH
 *          Zurich, Switzerland (rlo@vision.ethz.ch)
 */

/* Copyright Notice
 * ================
 * Copyright (C) 1993 by Robert L. Ogniewicz
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
 */

/*
 * random deletion of SyIpos
 *
 */

static char sy_delipoId[] = "$Id$"; 

#include "Sy_Skeleton.h"
#include "Sy_Intrinsic.h"
#include <sys/types.h>

#if defined(SYSV) || defined(SVR4) || defined(sgi)
#include <math.h>
#include <sys/time.h>
#else
#include <sys/timeb.h> 
#endif

#if ! defined (linux)
char  *initstate(unsigned int, char *, int), *setstate(char *);
#endif

static long randgen_state[32]  =  
      {0x03101962,  0xafa1221f,  0x1ffd8c24,  0x9b563182,  
       0xada1f342,  0x7449e56b,  0x8979620b,  0x27cb47b9,
       0x1cab3b91,  0xab5c5918,  0x42ef11ff,  0xa9845a59,
       0x1eb35789,  0x1a5b2190,  0xa501532d,  0xd272e2a0,
       0x95898887,  0xa36f735d,  0x20ff2542,  0x14158fd6,
       0x1fb66651,  0x5e6b9ef6,  0x123efd9c,  0x2e45e1e0,
       0xc1606f13,  0x110dd021,  0xfe9410ab,  0xf64e3f93,
       0xd1672980,  0x81a6bc12,  0x7ff3cffb,  0xb5eded0e};
       
static int nstate = 128;

typedef struct{ 
           int *del_, 
               *ipo_, 
               *icn_; 
           int del_ptr, 
               ipo_ptr,
               icn_ptr; 
	 } SyIpoPackTable;

int Sy_IpoRandomDelete (SyIpoDAC *ipdIn, SyIpoDAC *ipdOut, float percent, int is_verbose)
{ 
  void Sy_InitIpoRandomDelete(void); 
  long random(void); 
  void Sy_IpoPack(SyIpoDAC *ipd, SyIpoPackTable *table, int is_verbose);

  SyIpoDAC* ipd; 

  SyIpoPackTable table;

  int minDeleted, actDeleted = 0; 

  Sy_InitIpoRandomDelete();

  ipd = ipdOut; 
  ipd->ipodim = ipdIn->ipodim; 
  ipd->icndim = ipdIn->icndim; 

  table.del_ = Sy_AllocInt (ipdIn->ipodim);
  table.ipo_ = Sy_AllocInt (ipdIn->ipodim); 
  table.icn_ = Sy_AllocInt (ipdIn->icndim);

  ipd->ipo = Sy_AllocIpo(ipdIn->ipodim);
  ipd->ipl = Sy_AllocIpl(ipdIn->ipodim);
  ipd->icn = Sy_AllocIcn(ipdIn->icndim);

  (void)memcpy ((char*)ipd->ipo, (char*)ipdIn->ipo, (ipdIn->ipodim+1)*sizeof(SyIpo)); 
  (void)memcpy ((char*)ipd->ipl, (char*)ipdIn->ipl, (ipdIn->ipodim+1)*sizeof(SyIpl)); 
  (void)memcpy ((char*)ipd->icn, (char*)ipdIn->icn, (ipdIn->icndim+1)*sizeof(SyIcn));

  minDeleted = (percent/100.0) * (float) ipd->ipodim; 
  if (minDeleted > ipd->ipodim) minDeleted = ipd->ipodim; 
  if (minDeleted < 0) minDeleted = 0;

  while (actDeleted < minDeleted){

#if defined(SYSV) || defined (SVR4)
    table.del_ptr = floor(drand48()*(ipd->ipodim-1))+1;
#else
    table.del_ptr = random() % (ipd->ipodim-1)+1;
#endif
    if (table.del_[table.del_ptr] != SY_DELETEMARK){
      table.del_[table.del_ptr] = SY_DELETEMARK; 
      actDeleted++; 
    } 
  }

  Sy_IpoPack (ipd, &table, is_verbose);

  free (table.del_); 
  free (table.ipo_); 
  free (table.icn_);

  return (actDeleted); }

void Sy_InitIpoRandomDelete (void)
{ 
#if defined (hpux)
  struct timeval t;

  gettimeofday(&t, NULL);
  srand48(t.tv_usec);
#elif defined(SYSV) || defined(SVR4) || defined(SGIArchitecture) || defined(sgi)
  struct timeval t;

  gettimeofday(&t);
  srand48(t.tv_usec);
#else
  char* chptr;
  struct timeb t; 

  ftime (&t);
  
  chptr = initstate ((unsigned) t.millitm, (char*) randgen_state, nstate); 
  chptr = setstate ((char*) randgen_state);
#endif
}


void Sy_IpoPack (SyIpoDAC *ipd, SyIpoPackTable *table, int is_verbose)
{
  int i;

  Sy_Message (SY_FMTNL, "compressing initiating points"); 

  table->ipo_ptr = 0;
  table->icn_ptr = 0;
  
  /* build compression tables */

  for (i = 1; i <= ipd->ipodim; i++){

    if (table->del_[i] == 0){

      table->ipo_[i] = ++table->ipo_ptr;         /* ipo/ipl-table entry      */

      ipd->ipo[table->ipo_ptr] = ipd->ipo[i];     /* compress ipo/ipl array  */
      ipd->ipl[table->ipo_ptr] = ipd->ipl[i];
    }
  }

  for (i = 1; i <= ipd->icndim; i++){

    ipd->icn[i].tail_ipo    = table->ipo_[ipd->icn[i].tail_ipo];
    ipd->icn[i].head_ipo    = table->ipo_[ipd->icn[i].head_ipo];
  }


  for (i = 1; i <= ipd->icndim; i++){
    
    if (ipd->icn[i].tail_ipo && ipd->icn[i].head_ipo){

      table->icn_[i] = ++table->icn_ptr;        /* icn-table entry      */
      ipd->icn[table->icn_ptr] = ipd->icn[i];     /* pack icn array     */    
    }
  }

  for (i = 1; i <= table->ipo_ptr; i++){

    ipd->ipl[i].tail_icn      = table->icn_[ipd->ipl[i].tail_icn];
    ipd->ipl[i].head_icn      = table->icn_[ipd->ipl[i].head_icn];
  }

  ipd->ipodim = table->ipo_ptr;
  ipd->icndim = table->icn_ptr;

  Sy_Message (SY_FMTNL, "  done ");

  return;
}





