/*
 *  $Id$
 * 
 *  Author: Robert L. Ogniewicz, Swiss Federal Institute of Technology ETH
 *          Zurich, Switzerland (rlo@vision.ethz.ch)
 */

/* Copyright Notice
 * ================
 * Copyright (C) 1993 by Robert L. Ogniewicz
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
 */


static char sy_DrawArcId[] = "$Id$"; 

#include "Sy_Skeleton.h"
#include "Sy_Intrinsic.h"
#include "Sy_Graphics.h"
#include "inipoints.xbm"

#ifndef NOX
#include <X11/Xatom.h>
static Atom proto_atom = None, delete_atom = None;

typedef struct _SyLtEventAction {
   Window  winId;
   char    *tag;
   void (*eventHandler)(XEvent *event, Window winId, 
			unsigned int winNr, char *tag);   
} SyLtEventAction;

static XEvent  LtXevent;

static GC      gc;
static Pixmap  icon;

static XArc     *arcPtr;
static XSegment *segmentPtr;
static int      pointDim, segmentDim;

static Display *display;
static char    *displayName = NULL;
static int     screenNr;

static void Sy_CatchEvent(XEvent *event, Window winId, 
			  unsigned int winNr, char *tag);
static void Sy_ExposeEvent(XExposeEvent *event, Window winId, 
			   unsigned int winNr);

static void Sy_Ipo2XArc (SyIpoDAC *ipd, XArc *arcPtr, int *pointDim, 
			 XSegment *segmentPtr, int *segmentDim, 
			 int *winWidth, int *winHeight, 
			 int xZen, int yZen, float uscale);

#endif

/**********************************************************************
 * function Sy_XIpoDisp (ipd, windowName, iconName, xPos, yPos, 
                        xZen, yZen, uscale, userDisplay)
 * 
 * purpose: displays a SyIpoDAC structure
 *
 * param:   [input] ipd   -  SyIpoDAC data
 *********************************************************************/
int Sy_XArcDisp (SyIpoDAC *ipd, char *windowName, char *iconName, int xPos, int yPos, int xZen, int yZen, float uscale, char *userDisplay)
{
#ifndef NOX
  SyLtEventAction  eventBuff;
  int            winWidth, winHeight;
  int            borderWidth = SY_DEF_BORDERWIDTH;
  int            point_mem, segment_mem, page;
  
  page         = getpagesize();
  point_mem    = (((ipd->ipodim + 1)*sizeof (XArc))/page+1)*page;
  segment_mem  = (((ipd->icndim + 1)*sizeof (XSegment))/page+1)*page;

  arcPtr   = (XArc*)   Sy_Malloc ((unsigned)point_mem);
  segmentPtr = (XSegment*) Sy_Malloc ((unsigned)segment_mem);

  Sy_Ipo2XArc (ipd, arcPtr, &pointDim, segmentPtr, &segmentDim, 
            &winWidth, &winHeight, xZen, yZen, uscale); 

  Sy_DestroyIpo (ipd);

  displayName  = userDisplay != NULL ? userDisplay : NULL;
  if ((display = XOpenDisplay (displayName)) == NULL) return (-1);  
  screenNr     = XDefaultScreen (display);

  eventBuff.winId = XCreateSimpleWindow (display, 
					 RootWindow (display, screenNr), 
					 xPos, yPos, winWidth, 
					 winHeight, borderWidth,
					 BlackPixel (display, screenNr),
					 WhitePixel (display, screenNr));
  
  gc   = XCreateGC (display, eventBuff.winId, 0, 0);
  XSetFillStyle (display, gc, FillSolid);   

  icon = XCreateBitmapFromData (display, eventBuff.winId, ipo_icon_bits, 
                                ipo_icon_width, ipo_icon_height);
  XSetForeground (display, gc, BlackPixel (display, screenNr));
  XSetBackground (display, gc, WhitePixel (display, screenNr));
  XSetStandardProperties (display, eventBuff.winId, windowName, 
			  iconName , icon, NULL, 0, NULL);

  XSelectInput (display, eventBuff.winId, 
		ExposureMask | StructureNotifyMask);

  proto_atom = XInternAtom(display, "WM_PROTOCOLS", False);
  delete_atom = XInternAtom(display, "WM_DELETE_WINDOW", False);
  if ((proto_atom != None) && (delete_atom != None))
    XChangeProperty(display, eventBuff.winId, proto_atom, XA_ATOM, 32,
		    PropModeReplace, (unsigned char*) &delete_atom, 1);

  eventBuff.eventHandler = Sy_CatchEvent;
  XMapWindow (display, eventBuff.winId);
  while (LtXevent.type != Expose && LtXevent.xany.window != eventBuff.winId) 
    XNextEvent(display, &LtXevent);

  while (1){
    if (LtXevent.xany.window == eventBuff.winId)
      eventBuff.eventHandler (&LtXevent, LtXevent.xany.window, 1, NULL);
    XNextEvent (display, &LtXevent);
  }
#else
  Sy_Message("%s", "no X11 graphics support available\n");
#endif
}

#ifndef NOX
static void Sy_CatchEvent (XEvent *event, Window winId, 
			   unsigned int winNr, char *tag)
{
  XClientMessageEvent *me;
  switch (event->type){
  case Expose :
    Sy_ExposeEvent ((XExposeEvent*)event, winId, winNr); 
    break;
  case DestroyNotify:
    exit(2); break;
  case ClientMessage:
    me = (XClientMessageEvent*)event;
    if ((me->window == winId) && (me->data.l[0] == delete_atom)){
      exit(2);
    }
    break;
  default : break;
  }
  return;
}

static void Sy_ExposeEvent (XExposeEvent *event, Window winId, unsigned int winNr)
{
   XArc   *cur_point_block;
   XSegment *cur_segment_block;
   int      count;
   for (cur_point_block = arcPtr, count = 0; count < pointDim/SY_XBLOCK;
        cur_point_block += SY_XBLOCK, count++){
     XFillArcs  (display, winId, gc, cur_point_block, SY_XBLOCK);
   };
   if (pointDim%SY_XBLOCK != 0)
     XFillArcs  (display, winId, gc, cur_point_block, pointDim%SY_XBLOCK);

   for (cur_segment_block = segmentPtr, count = 0; count < segmentDim/SY_XBLOCK;
        cur_segment_block += SY_XBLOCK, count++){
     XDrawSegments (display, winId, gc, cur_segment_block, SY_XBLOCK);
   };
   if (segmentDim%SY_XBLOCK != 0)
     XDrawSegments (display, winId, gc, cur_segment_block, segmentDim%SY_XBLOCK);

   return;
}

static void Sy_Ipo2XArc (SyIpoDAC *ipd, XArc *arcPtr, int *pointDim, 
			 XSegment *segmentPtr, int *segmentDim, 
			 int *winWidth, int *winHeight, 
			 int xZen, int yZen, float uscale)
{
  float    scaleX, scaleY, scaleXY;
  float    xOff,   yOff, 
           xShift, yShift;

  int      maxX, minX, maxY, minY;
  int      dimX, dimY; 
  int      effDimX, effDimY;

  int      tail_ipo, head_ipo;
  int      sDim, i;

  maxX = minX = ipd->ipo[3].x;
  minY = ipd->ipo[3].y;
  maxY = ipd->ipo[ipd->ipodim].y;

  for (i = 1; i <= ipd->ipodim; i++){
     maxX = max(maxX, ipd->ipo[i].x);
     minX = min(minX, ipd->ipo[i].x);
  }

  dimX = maxX - minX;
  dimY = maxY - minY;

  if (uscale > 0){
    effDimX = max(min(dimX * uscale, SY_MAX_WINDIM), SY_MIN_WINDIM);
    effDimY = max(min(dimY * uscale, SY_MAX_WINDIM), SY_MIN_WINDIM);
    xOff    = (xZen > 0) ? xZen : dimX/2 + minX;
    yOff    = (yZen > 0) ? yZen : dimY/2 + minY;
    scaleXY = uscale;
  }
  else{
    effDimX = max(min(dimX, SY_DEF_WINDIM), SY_MIN_WINDIM);
    effDimY = max(min(dimY, SY_DEF_WINDIM), SY_MIN_WINDIM);
    scaleX  = dimX > 0 ? (float )effDimX/(float )dimX : 1.0; 
    scaleY  = dimY > 0 ? (float )effDimY/(float )dimY : 1.0;
    scaleXY = min(scaleX, scaleY);
    xOff    = dimX/2 + minX;
    yOff    = dimY/2 + minY;
  }

  xShift  = effDimX/2 + SY_MIN_BORDER;
  yShift  = effDimY/2 + SY_MIN_BORDER;

  for (i = 1; i <= ipd->ipodim; i++){
    arcPtr[i-1].x = scaleXY * ((float)ipd->ipo[i].x - xOff) + xShift;
    arcPtr[i-1].y = scaleXY * ((float)ipd->ipo[i].y - yOff) + yShift;
    arcPtr[i-1].width= arcPtr[i-1].height = 4;
    arcPtr[i-1].angle1= arcPtr[i-1].angle2 = 360*64;
  }

  for (i = 1, sDim = 0; i <= ipd->icndim; i++) {
    if ((tail_ipo = ipd->icn[i].tail_ipo) && (head_ipo = ipd->icn[i].head_ipo)){
      segmentPtr[sDim].x1 = scaleXY * ((float)ipd->ipo[tail_ipo].x - xOff) + xShift;
      segmentPtr[sDim].y1 = scaleXY * ((float)ipd->ipo[tail_ipo].y - yOff) + yShift; 
      segmentPtr[sDim].x2 = scaleXY * ((float)ipd->ipo[head_ipo].x - xOff) + xShift;
      segmentPtr[sDim].y2 = scaleXY * ((float)ipd->ipo[head_ipo].y - yOff) + yShift;
      sDim++;
    }
  }
  *winWidth   = effDimX + 2*SY_MIN_BORDER;
  *winHeight  = effDimY + 2*SY_MIN_BORDER;
  *segmentDim = sDim;
  *pointDim   = ipd->ipodim;
}
#endif 






