/*
 *  $Id$
 * 
 *  Author: Robert L. Ogniewicz, Swiss Federal Institute of Technology ETH
 *          Zurich, Switzerland (rlo@vision.ethz.ch)
 */

/* Copyright Notice
 * ================
 * Copyright (C) 1993 by Robert L. Ogniewicz
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
 */

static char sy_DrawSkeletonId[] = "$Id$"; 

#include "Sy_Skeleton.h"
#include "Sy_Intrinsic.h"
#include "Sy_Graphics.h"
#include "skeleton.xbm"

#ifndef NOX
#include <X11/Xatom.h>
static Atom proto_atom = None, delete_atom = None;

typedef struct{
   Window  winId;
   char    *tag;
   void   (*eventHandler)();
} SyLtEventAction;

static XEvent  LtXevent;

static GC      gc;
static Pixmap  icon;

static XArc     *arcPtr;
static XPoint   *pointPtr;
static XSegment *segmentPtr;
static int      arcDim, pointDim, segmentDim;

static Display *display;
static char    *displayName = NULL;
static int     screenNr;

static void  Sy_CatchEvent(XEvent *event, Window winId, unsigned int winNr, 
			   char *tag);

static void  Sy_ExposeEvent(XExposeEvent *event, Window winId, 
			    unsigned int winNr);

static void  Sy_CatchEvent2(XEvent *event, Window winId, unsigned int winNr, 
			   char *tag);

static void  Sy_ExposeEvent2(XExposeEvent *event, Window winId, 
			    unsigned int winNr);

static void  Sy_Skel2X(SySkelDAC *vsk, SyIpoDAC *ipd, float thres, 
		       XPoint *pointPtr, int *pointDim, XSegment *segmentPtr, 
		       int *segmentDim, int *winWidth, int *winHeight, int xZen, 
		       int yZen, float uscale, enum SyDisplayMode display_mode);

static void  Sy_Skel2XRaw(SySkelDAC *vsk, SyIpoDAC *ipd, float thres, 
			  XPoint *pointPtr, int *pointDim, XSegment *segmentPtr, 
			  int *segmentDim, enum SyDisplayMode display_mode, 
			  enum SyDisplaySwitch boundary_toggle);

static void Sy_SkeletonSet2X (SyVSkeletonSet *vss, XPoint *pointPtr, int *pointDim, 
			      XSegment *segmentPtr, int *segmentDim, 
			      XArc *arcPtr, int *arcDim, 
			      int *winWidth, int *winHeight, int xZen, int yZen, 
			      float uscale, SyDisplayControl *control);

#endif

/**********************************************************************
 * function Sy_XSkelDisp (vsk, ipd, th, windowName, iconName, xPos, yPos, 
 *                        xZen, yZen, uscale, userDisplay)
 * 
 * purpose: displays a Voronoi-diagram based skeleton
 *
 * param:   [input] vsk   -  V-Skeleton data
 *********************************************************************/
int Sy_XSkelDisp (SySkelDAC *vsk, SyIpoDAC *ipd, float th, char *windowName, 
		  char *iconName, int xPos, int yPos, int xZen, int yZen, 
		  float uscale, char *userDisplay, SyDisplayControl control)
{
#ifndef NOX
  SyLtEventAction  eventBuff;
  int            winWidth, winHeight;
  int            borderWidth = SY_DEF_BORDERWIDTH;

  displayName  = userDisplay != NULL ? userDisplay : NULL;
  if ((display = XOpenDisplay (displayName)) == NULL) return (-1);  
  screenNr     = XDefaultScreen (display);

  pointPtr   = (XPoint*)   Sy_Malloc ((unsigned)((vsk->pdim +1)*sizeof(XPoint)));
  segmentPtr = (XSegment*) Sy_Malloc ((unsigned)((vsk->edim + ipd->icndim +2) * 
				   sizeof(XSegment)));

  if (control.raw_mode == SKE_ON){
    Sy_Skel2XRaw (vsk, ipd, th, pointPtr, &pointDim, segmentPtr, &segmentDim, 
           control.display_mode, control.show_boundary); 
    winWidth  = control.window_dimX;
    winHeight = control.window_dimY;
  }
  else{
    Sy_Skel2X (vsk, ipd, th, pointPtr, &pointDim, segmentPtr, &segmentDim, 
	      &winWidth, &winHeight, xZen, yZen, uscale, control.display_mode); 
  }

  Sy_DestroySkeleton (vsk);
  Sy_DestroyIpo (ipd);

  eventBuff.winId = XCreateSimpleWindow (display, 
					 RootWindow (display, screenNr), 
					 xPos, yPos, winWidth, 
					 winHeight, borderWidth,
					 BlackPixel (display, screenNr),
					 WhitePixel (display, screenNr));
  
  gc   = XCreateGC (display, eventBuff.winId, 0, 0);
  icon = XCreateBitmapFromData (display, eventBuff.winId, vskel_icon_bits, 
                                vskel_icon_width, vskel_icon_height);
  XSetForeground (display, gc, BlackPixel (display, screenNr));
  XSetBackground (display, gc, WhitePixel (display, screenNr));
  XSetStandardProperties (display, eventBuff.winId, windowName, 
			  iconName , icon, NULL, 0, NULL);

  XSelectInput (display, eventBuff.winId, 
		ExposureMask | StructureNotifyMask);

  proto_atom = XInternAtom(display, "WM_PROTOCOLS", False);
  delete_atom = XInternAtom(display, "WM_DELETE_WINDOW", False);
  if ((proto_atom != None) && (delete_atom != None))
    XChangeProperty(display, eventBuff.winId, proto_atom, XA_ATOM, 32,
		    PropModeReplace, (unsigned char*) &delete_atom, 1);

  eventBuff.eventHandler = Sy_CatchEvent;
   
  XMapWindow (display, eventBuff.winId);

  if (control.line_width > 0){
    XSetLineAttributes(display, gc, control.line_width, 
		       LineSolid, CapRound, JoinRound);
  }

  while (LtXevent.type != Expose && LtXevent.xany.window != eventBuff.winId) 
    XNextEvent(display, &LtXevent);
  while (1){
    if (LtXevent.xany.window == eventBuff.winId)
      eventBuff.eventHandler (&LtXevent, LtXevent.xany.window, 1, NULL);
    XNextEvent (display, &LtXevent);
  }
#else
  Sy_Message("%s", "no X11 graphics support available\n");
#endif
}

#ifndef NOX
static void Sy_CatchEvent (XEvent *event, Window winId, unsigned int winNr, 
			   char *tag)
{
  XClientMessageEvent *me;
  switch (event->type){
  case Expose :
    Sy_ExposeEvent ((XExposeEvent*)event, winId, winNr); 
    break;
  case DestroyNotify:
    exit(2); break;
  case ClientMessage:
    me = (XClientMessageEvent*)event;
    if ((me->window == winId) && (me->data.l[0] == delete_atom)){
      exit(2);
    }
    break;
  default : break;
  }
  return;
}

static void Sy_ExposeEvent (XExposeEvent *event, Window winId, 
			    unsigned int winNr)
{
   XPoint   *cur_point_block;
   XSegment *cur_segment_block;
   int      count;

   for (cur_point_block = pointPtr, count = 0; count < pointDim/SY_XBLOCK;
        cur_point_block += SY_XBLOCK, count++){
     XDrawPoints  (display, winId, gc, cur_point_block, SY_XBLOCK, 
		   CoordModeOrigin);
   };
   if (pointDim%SY_XBLOCK != 0)
     XDrawPoints  (display, winId, gc, cur_point_block, pointDim%SY_XBLOCK, 
		   CoordModeOrigin);

   for (cur_segment_block = segmentPtr, count = 0; count < segmentDim/SY_XBLOCK;
        cur_segment_block += SY_XBLOCK, count++){
     XDrawSegments (display, winId, gc, cur_segment_block, SY_XBLOCK);
   };
   if (segmentDim%SY_XBLOCK != 0)
     XDrawSegments (display, winId, gc, cur_segment_block, segmentDim%SY_XBLOCK);

   return;
}

static void Sy_Skel2X (SySkelDAC *vsk, SyIpoDAC *ipd, float thres, 
		       XPoint *pointPtr, int *pointDim, XSegment *segmentPtr, 
		       int *segmentDim, int *winWidth, int *winHeight, 
		       int xZen, int yZen, float uscale, 
		       enum SyDisplayMode display_mode)
{
  SyVertex   vhead, vtail;
  int	   i;
  float    x1, y1, x2, y2;
  float    scaleX, scaleY, scaleXY;
  float    xOff, yOff, 
           xShift, yShift;
  float    rubber;
  float    dirX, dirY;
  int      maxX, minX, minY;
  int      dimX, dimY, sDim; 
  int      effDimX, effDimY;
  int      add_segment, conn_flag;
  int      conn[4];
  int      tail_vertex, head_vertex, 
           left_point, right_point,
           tail_ipo, head_ipo;
  int      tail_flag, head_flag;
  int      is_residual_cut, is_skel, is_render, is_foreground;

  maxX = minX = vsk->point[1].x;
  minY = max(vsk->point[1].y, SY_DEF_MINY);

  for (i = 1; i <= vsk->pdim; i++){
     maxX = max(maxX, vsk->point[i].x);
     minX = min(minX, vsk->point[i].x);
  }

  dimX = maxX-minX;
  dimY = vsk->point[vsk->pdim].y - minY;

  if (uscale > 0){
    effDimX = max(min(dimX * uscale, SY_MAX_WINDIM), SY_MIN_WINDIM);
    effDimY = max(min(dimY * uscale, SY_MAX_WINDIM), SY_MIN_WINDIM);
    xOff    = (xZen > 0) ? xZen : dimX/2 + minX;
    yOff    = (yZen > 0) ? yZen : dimY/2 + minY;
    scaleXY = uscale;
  }
  else{
    effDimX = max(min(dimX, SY_DEF_WINDIM), SY_MIN_WINDIM);
    effDimY = max(min(dimY, SY_DEF_WINDIM), SY_MIN_WINDIM);
    scaleX  = dimX > 0 ? (float )effDimX/(float )dimX : 1.0; 
    scaleY  = dimY > 0 ? (float )effDimY/(float )dimY : 1.0;
    scaleXY = min(scaleX, scaleY);
    xOff    = dimX/2 + minX;
    yOff    = dimY/2 + minY;
  }

  xShift  = effDimX/2 + SY_MIN_BORDER;
  yShift  = effDimY/2 + SY_MIN_BORDER;

  for (i = 1; i <= vsk->pdim; i++){
    pointPtr[i-1].x = scaleXY * (vsk->point[i].x - xOff) + xShift;
    pointPtr[i-1].y = scaleXY * (vsk->point[i].y - yOff) + yShift;
  }

  for (i = 1, sDim = 0; i <= vsk->edim; i++) {
    
    tail_vertex = vsk->edge[i].tail_vertex;
    head_vertex = vsk->edge[i].head_vertex;
    left_point  = vsk->edge[i].left_point;
    right_point = vsk->edge[i].right_point;

    tail_flag   = vsk->regul[tail_vertex].birth[2] > thres;
    head_flag   = vsk->regul[head_vertex].birth[2] > thres;


    is_foreground  = Sy_SkeletonType (vsk, ipd, tail_vertex, i);

    if ((is_foreground && (display_mode == SKE_BACKGROUND)) ||
        (!is_foreground && (display_mode == SKE_FOREGROUND))) continue;


    conn[0]  = ((ipd->ipl[left_point].tail_icn == ipd->ipl[right_point].tail_icn) &&
		(ipd->ipl[left_point].tail_icn != 0));

    conn[1]  = ((ipd->ipl[left_point].tail_icn == ipd->ipl[right_point].head_icn) &&
		(ipd->ipl[left_point].tail_icn != 0));

    conn[2]  = ((ipd->ipl[left_point].head_icn == ipd->ipl[right_point].tail_icn) &&
		(ipd->ipl[left_point].head_icn != 0));

    conn[3]  = ((ipd->ipl[left_point].head_icn == ipd->ipl[right_point].head_icn) &&
		(ipd->ipl[left_point].head_icn != 0));

    conn_flag = (conn[0] || conn[1] || conn[2] || conn[3]);

    is_residual_cut = vsk->is_eresid && 
                      ((vsk->eresid[i].tail_residual > thres) || 
                       (vsk->eresid[i].head_residual > thres));

    is_skel         = tail_flag && head_flag;

    is_render = (is_residual_cut || is_skel) && (!conn_flag || vsk->is_border);

    if (is_render){

      if (!(tail_flag && head_flag)){
	Sy_SimpleResidualCut (vsk, i, thres, &vtail, &vhead);
      }
      else{
	vtail = vsk->vertex[vsk->edge[i].tail_vertex];
	vhead = vsk->vertex[vsk->edge[i].head_vertex];
      }

      x1 = scaleXY * (vtail.xden / vtail.num - xOff) + xShift;
      y1 = scaleXY * (vtail.yden / vtail.num - yOff) + yShift;
      x2 = scaleXY * (vhead.xden / vhead.num - xOff) + xShift;
      y2 = scaleXY * (vhead.yden / vhead.num - yOff) + yShift;

      switch (vsk->edge[i].rank) {
      case 0:
	dirX = x2 - x1;
	dirY = y2 - y1;
	rubber = SY_VALRAD/max(SY_ALV(dirX), SY_ALV(dirY));
	x2 += rubber * dirX;
	y2 += rubber * dirY;
	x1 -= rubber * dirX;
	y1 -= rubber * dirY;
	add_segment = 1;
	break;
      case 1:
	if (!(add_segment = ((SY_ALV(x1) < SY_PLANESIZE) && 
			     (SY_ALV(y1) < SY_PLANESIZE)))) break;
	dirX = x2 - x1;
	dirY = y2 - y1;
	rubber = SY_VALRAD/max(SY_ALV(dirX), SY_ALV(dirY));
	x2 = x1 + rubber * dirX;
	y2 = y1 + rubber * dirY;
	break;
      default:
	if (add_segment = ((SY_ALV(x1) < SY_PLANESIZE) && (SY_ALV(y1) < SY_PLANESIZE)
			   && (SY_ALV(x2) < SY_PLANESIZE) && 
			   (SY_ALV(y2) < SY_PLANESIZE))) break;
	if (add_segment = ((SY_ALV(x1) < SY_PLANESIZE) && (SY_ALV(y1) < SY_PLANESIZE))) {
	  dirX = x2 - x1;
	  dirY = y2 - y1;
	  rubber = SY_VALRAD/max(SY_ALV(dirX), SY_ALV(dirY)); 
	  x2 = x1 + rubber * dirX;
	  y2 = y1 + rubber * dirY;
	  break;
	}
	if (add_segment = ((SY_ALV(x2) < SY_PLANESIZE) && (SY_ALV(y2) < SY_PLANESIZE))) {
	  dirX = x1 - x2;
	  dirY = y1 - y2;
	  rubber = SY_VALRAD/max(SY_ALV(dirX), SY_ALV(dirY)); 
	  x1 = x2 + rubber * dirX;
	  y1 = y2 + rubber * dirY;
	  break;
	}
      }
      if (add_segment) {
	segmentPtr[sDim].x1 = x1;
	segmentPtr[sDim].y1 = y1;
	segmentPtr[sDim].x2 = x2;
	segmentPtr[sDim].y2 = y2;
        sDim++;
      }
    }
  }
  for (i = 1; i <= ipd->icndim; i++) {
    if ((tail_ipo = ipd->icn[i].tail_ipo) && (head_ipo = ipd->icn[i].head_ipo)){
      segmentPtr[sDim].x1 = scaleXY * ((float)ipd->ipo[tail_ipo].x - xOff) + xShift;
      segmentPtr[sDim].y1 = scaleXY * ((float)ipd->ipo[tail_ipo].y - yOff) + yShift; 
      segmentPtr[sDim].x2 = scaleXY * ((float)ipd->ipo[head_ipo].x - xOff) + xShift;
      segmentPtr[sDim].y2 = scaleXY * ((float)ipd->ipo[head_ipo].y - yOff) + yShift;
      sDim++;
    }
  }

  *winWidth   = effDimX + 2*SY_MIN_BORDER;
  *winHeight  = effDimY + 2*SY_MIN_BORDER;
  *segmentDim = sDim;
  *pointDim   = vsk->pdim;
}

static void Sy_Skel2XRaw (SySkelDAC *vsk, SyIpoDAC *ipd, float thres, 
			  XPoint *pointPtr, int *pointDim, XSegment *segmentPtr, 
			  int *segmentDim, enum SyDisplayMode display_mode, 
			  enum SyDisplaySwitch boundary_toggle)
{
  SyVertex   vhead, vtail;
  int	   i;
  float    x1, y1, x2, y2;
  float    rubber;
  float    dirX, dirY;
  int      sDim; 
  int      add_segment, conn_flag;
  int      conn[4];
  int      tail_vertex, head_vertex, 
           left_point, right_point,
           tail_ipo, head_ipo;
  int      tail_flag, head_flag;
  int      is_residual_cut, is_skel, is_foreground, is_render;

  if (boundary_toggle == SKE_ON){
    for (i = 1; i <= vsk->pdim; i++){
      pointPtr[i-1].x = vsk->point[i].x;
      pointPtr[i-1].y = vsk->point[i].y;
    }
  }

  for (i = 1, sDim = 0; i <= vsk->edim; i++) {
    
    tail_vertex = vsk->edge[i].tail_vertex;
    head_vertex = vsk->edge[i].head_vertex;
    left_point  = vsk->edge[i].left_point;
    right_point = vsk->edge[i].right_point;

    is_foreground  = Sy_SkeletonType (vsk, ipd, tail_vertex, i);

    if ((is_foreground && (display_mode == SKE_BACKGROUND)) ||
        (!is_foreground && (display_mode == SKE_FOREGROUND))) continue;

    tail_flag   = vsk->regul[tail_vertex].birth[2] > thres;
    head_flag   = vsk->regul[head_vertex].birth[2] > thres;

    conn[0]  = ((ipd->ipl[left_point].tail_icn == ipd->ipl[right_point].tail_icn) &&
		(ipd->ipl[left_point].tail_icn != 0));

    conn[1]  = ((ipd->ipl[left_point].tail_icn == ipd->ipl[right_point].head_icn) &&
		(ipd->ipl[left_point].tail_icn != 0));

    conn[2]  = ((ipd->ipl[left_point].head_icn == ipd->ipl[right_point].tail_icn) &&
		(ipd->ipl[left_point].head_icn != 0));

    conn[3]  = ((ipd->ipl[left_point].head_icn == ipd->ipl[right_point].head_icn) &&
		(ipd->ipl[left_point].head_icn != 0));

    conn_flag = (conn[0] || conn[1] || conn[2] || conn[3]);

    
    is_residual_cut = vsk->is_eresid && 
                      ((vsk->eresid[i].tail_residual > thres) || 
                       (vsk->eresid[i].head_residual > thres));

    is_skel         = tail_flag && head_flag;

    is_render = (is_residual_cut || is_skel) && (!conn_flag || vsk->is_border);

    if (is_render){

      if (!(tail_flag && head_flag)){
	Sy_SimpleResidualCut (vsk, i, thres, &vtail, &vhead);
      }
      else{
	vtail = vsk->vertex[vsk->edge[i].tail_vertex];
	vhead = vsk->vertex[vsk->edge[i].head_vertex];
      }

      x1 = vtail.xden / vtail.num;
      y1 = vtail.yden / vtail.num;
      x2 = vhead.xden / vhead.num;
      y2 = vhead.yden / vhead.num;

      switch (vsk->edge[i].rank) {
      case 0:
	dirX = x2 - x1;
	dirY = y2 - y1;
	rubber = SY_VALRAD/max(SY_ALV(dirX), SY_ALV(dirY));
	x2 += rubber * dirX;
	y2 += rubber * dirY;
	x1 -= rubber * dirX;
	y1 -= rubber * dirY;
	add_segment = 1;
	break;
      case 1:
	if (!(add_segment = ((SY_ALV(x1) < SY_PLANESIZE) && 
			     (SY_ALV(y1) < SY_PLANESIZE)))) break;
	dirX = x2 - x1;
	dirY = y2 - y1;
	rubber = SY_VALRAD/max(SY_ALV(dirX), SY_ALV(dirY));
	x2 = x1 + rubber * dirX;
	y2 = y1 + rubber * dirY;
	break;
      default:
	if (add_segment = ((SY_ALV(x1) < SY_PLANESIZE) && (SY_ALV(y1) < SY_PLANESIZE)
			   && (SY_ALV(x2) < SY_PLANESIZE) && 
			   (SY_ALV(y2) < SY_PLANESIZE))) break;
	if (add_segment = ((SY_ALV(x1) < SY_PLANESIZE) && (SY_ALV(y1) < SY_PLANESIZE))) {
	  dirX = x2 - x1;
	  dirY = y2 - y1;
	  rubber = SY_VALRAD/max(SY_ALV(dirX), SY_ALV(dirY)); 
	  x2 = x1 + rubber * dirX;
	  y2 = y1 + rubber * dirY;
	  break;
	}
	if (add_segment = ((SY_ALV(x2) < SY_PLANESIZE) && (SY_ALV(y2) < SY_PLANESIZE))) {
	  dirX = x1 - x2;
	  dirY = y1 - y2;
	  rubber = SY_VALRAD/max(SY_ALV(dirX), SY_ALV(dirY)); 
	  x1 = x2 + rubber * dirX;
	  y1 = y2 + rubber * dirY;
	  break;
	}
      }
      if (add_segment) {
	segmentPtr[sDim].x1 = x1;
	segmentPtr[sDim].y1 = y1;
	segmentPtr[sDim].x2 = x2;
	segmentPtr[sDim].y2 = y2;
        sDim++;
      }
    }
  }

  if (boundary_toggle == SKE_ON){
    for (i = 1; i <= ipd->icndim; i++) {
      if ((tail_ipo = ipd->icn[i].tail_ipo) && (head_ipo = ipd->icn[i].head_ipo)){
	segmentPtr[sDim].x1 = ipd->ipo[tail_ipo].x;
	segmentPtr[sDim].y1 = ipd->ipo[tail_ipo].y; 
	segmentPtr[sDim].x2 = ipd->ipo[head_ipo].x;
	segmentPtr[sDim].y2 = ipd->ipo[head_ipo].y;
	sDim++;
      }
    }
  }

  *segmentDim = sDim;
  *pointDim   = (boundary_toggle == SKE_ON)? vsk->pdim : 0;
}
#endif


int Sy_XSkeletonSetDisp (SyVSkeletonSet *vss, char *windowName, char *iconName, 
			 int xPos, int yPos, int xZen, int yZen, 
			 float uscale, char *userDisplay, SyDisplayControl control)
{
#ifndef NOX
  SyLtEventAction eventBuff;
  int             winWidth, winHeight;
  int             borderWidth = SY_DEF_BORDERWIDTH;
  int             arcmaxdim, pointmaxdim, segmentmaxdim, i, k;

  displayName  = userDisplay != NULL ? userDisplay : NULL;
  if ((display = XOpenDisplay (displayName)) == NULL) return (-1);  
  screenNr     = XDefaultScreen (display);

  /* number of arcs, points, segments */

  for (i = 1, arcmaxdim = pointmaxdim = segmentmaxdim = 0; i <= vss->dim; i++){
    arcmaxdim += vss->skel[i].nodedim;
    for (k = 1; k <= vss->skel[i].branchdim; k++){
      pointmaxdim   += 2 * vss->skel[i].branch[k].elementdim;
      segmentmaxdim += 3 * vss->skel[i].branch[k].elementdim;
    }
  }

  arcPtr     = (XArc*)     Sy_Malloc (sizeof(XArc)*arcmaxdim);
  pointPtr   = (XPoint*)   Sy_Malloc (sizeof(XPoint)*pointmaxdim);
  segmentPtr = (XSegment*) Sy_Malloc (sizeof(XSegment)*segmentmaxdim);

  Sy_SkeletonSet2X (vss, pointPtr, &pointDim, segmentPtr, &segmentDim, 
	     arcPtr, &arcDim, &winWidth, &winHeight, xZen, yZen, uscale, 
	     &control); 
  
  eventBuff.winId = XCreateSimpleWindow (display, 
					 RootWindow (display, screenNr), 
					 xPos, yPos, winWidth, 
					 winHeight, borderWidth,
					 BlackPixel (display, screenNr),
					 WhitePixel (display, screenNr));
  
  gc   = XCreateGC (display, eventBuff.winId, 0, 0);
  icon = XCreateBitmapFromData (display, eventBuff.winId, vskel_icon_bits, 
                                vskel_icon_width, vskel_icon_height);
  XSetForeground (display, gc, BlackPixel (display, screenNr));
  XSetBackground (display, gc, WhitePixel (display, screenNr));
  XSetStandardProperties (display, eventBuff.winId, windowName, 
			  iconName , icon, NULL, 0, NULL);

  XSelectInput (display, eventBuff.winId, 
		ExposureMask | StructureNotifyMask);

  proto_atom = XInternAtom(display, "WM_PROTOCOLS", False);
  delete_atom = XInternAtom(display, "WM_DELETE_WINDOW", False);
  if ((proto_atom != None) && (delete_atom != None))
    XChangeProperty(display, eventBuff.winId, proto_atom, XA_ATOM, 32,
		    PropModeReplace, (unsigned char*) &delete_atom, 1);

  eventBuff.eventHandler = Sy_CatchEvent2;
   
  XMapWindow (display, eventBuff.winId);

  if (control.line_width > 0){
    XSetLineAttributes(display, gc, control.line_width, 
		       LineSolid, CapRound, JoinRound);
  }

  while (LtXevent.type != Expose && LtXevent.xany.window != eventBuff.winId) 
    XNextEvent(display, &LtXevent);
  while (1){
    if (LtXevent.xany.window == eventBuff.winId)
      eventBuff.eventHandler (&LtXevent, LtXevent.xany.window, 1, NULL);
    XNextEvent (display, &LtXevent);
  }
#else
  Sy_Message("%s", "no X11 graphics support available\n");
#endif
}

#ifndef NOX
static void Sy_CatchEvent2 (XEvent *event, Window winId, unsigned int winNr, 
			   char *tag)
{
  XClientMessageEvent *me;
  switch (event->type){
  case Expose :
    Sy_ExposeEvent2 ((XExposeEvent*)event, winId, winNr); 
    break;
  case DestroyNotify:
    exit(2); break;
  case ClientMessage:
    me = (XClientMessageEvent*)event;
    if ((me->window == winId) && (me->data.l[0] == delete_atom)){
      exit(2);
    }
    break;
  default : break;
  }
  return;
}

static void Sy_ExposeEvent2 (XExposeEvent *event, Window winId, 
			     unsigned int winNr)
{
   XArc     *cur_arc_block;
   XPoint   *cur_point_block;
   XSegment *cur_segment_block;
   int      count;

   for (cur_arc_block = arcPtr, count = 0; count < arcDim/SY_XBLOCK;
        cur_arc_block += SY_XBLOCK, count++){
     XFillArcs  (display, winId, gc, cur_arc_block, SY_XBLOCK);
   };
   if (arcDim%SY_XBLOCK != 0)
     XFillArcs  (display, winId, gc, cur_arc_block, arcDim%SY_XBLOCK);

   for (cur_point_block = pointPtr, count = 0; count < pointDim/SY_XBLOCK;
        cur_point_block += SY_XBLOCK, count++){
     XDrawPoints  (display, winId, gc, cur_point_block, SY_XBLOCK, 
		   CoordModeOrigin);
   };
   if (pointDim%SY_XBLOCK != 0)
     XDrawPoints  (display, winId, gc, cur_point_block, pointDim%SY_XBLOCK, 
		   CoordModeOrigin);

   for (cur_segment_block = segmentPtr, count = 0; count < segmentDim/SY_XBLOCK;
        cur_segment_block += SY_XBLOCK, count++){
     XDrawSegments (display, winId, gc, cur_segment_block, SY_XBLOCK);
   };
   if (segmentDim%SY_XBLOCK != 0)
     XDrawSegments (display, winId, gc, cur_segment_block, segmentDim%SY_XBLOCK);

   return;
}

static void Sy_SkeletonSet2X (SyVSkeletonSet *vss, XPoint *pointPtr, int *pointDim, 
			      XSegment *segmentPtr, int *segmentDim, 
			      XArc *arcPtr, int *arcDim, 
			      int *winWidth, int *winHeight, int xZen, int yZen, 
			      float uscale, SyDisplayControl *ctrl)
{
  int	   i, k, v;
  float    x1, y1, x2, y2, x01, y01, x02, y02, mx, my, tx, ty;
  float    scaleX, scaleY, scaleXY;
  float    xOff, yOff, 
           xShift, yShift;
  float    dirX, dirY;
  int      maxX, maxY, minX, minY;
  int      dimX, dimY, effDimX, effDimY;
  int      is_foreground;
  int      adim = 0, pdim = 0, sdim = 0;

  union SyGenericPoint *axis;

  maxX = maxY = SY_NEG_INT_INFTY; 
  minX = minY = SY_POS_INT_INFTY;

  for (i = 1; i <= vss->dim; i++){

    if ((vss->skel[i].is_foreground && ctrl->display_mode == SKE_BACKGROUND) ||
        (!vss->skel[i].is_foreground && ctrl->display_mode == SKE_FOREGROUND)) 
      continue;

    for (k = 1; k <= vss->skel[i].branchdim; k++){

      axis = vss->skel[i].branch[k].axis;
      for (v = 1; v <= vss->skel[i].branch[k].elementdim; v++){       
	if (maxX < axis[v].rp.x){ maxX = axis[v].rp.x; }
	if (maxY < axis[v].rp.y){ maxY = axis[v].rp.y; }
	if (minX > axis[v].rp.x){ minX = axis[v].rp.x; }
	if (minY > axis[v].rp.y){ minY = axis[v].rp.y; }
      }

      for (v = 2; v <= vss->skel[i].branch[k].elementdim; v++){  
	mx = 0.5*(axis[v].rp.x + axis[v-1].rp.x);
	my = 0.5*(axis[v].rp.y + axis[v-1].rp.y);
	x1 = mx + vss->skel[i].branch[k].left_bndry[v-1].xoff;
	y1 = my + vss->skel[i].branch[k].left_bndry[v-1].yoff;
	x2 = mx + vss->skel[i].branch[k].right_bndry[v-1].xoff;
	y2 = my + vss->skel[i].branch[k].right_bndry[v-1].yoff;

	if (maxX < x1) maxX = x1;
	if (maxX < x2) maxX = x2;
	if (maxY < y1) maxY = y1;
	if (maxY < y2) maxY = y2;
 
	if (minX > x1) minX = x1;
	if (minX > x2) minX = x2;
	if (minY > y1) minY = y1;
	if (minY > y2) minY = y2;
      }
    }
  }

  dimX = maxX-minX;
  dimY = maxY-minY;

  if (uscale > 0){
    effDimX = max(min(dimX * uscale, SY_MAX_WINDIM), SY_MIN_WINDIM);
    effDimY = max(min(dimY * uscale, SY_MAX_WINDIM), SY_MIN_WINDIM);
    xOff    = (xZen > 0) ? xZen : dimX/2 + minX;
    yOff    = (yZen > 0) ? yZen : dimY/2 + minY;
    scaleXY = uscale;
  }
  else{
    effDimX = max(min(dimX, SY_DEF_WINDIM), SY_MIN_WINDIM);
    effDimY = max(min(dimY, SY_DEF_WINDIM), SY_MIN_WINDIM);
    scaleX  = dimX > 0 ? (float )effDimX/(float )dimX : 1.0; 
    scaleY  = dimY > 0 ? (float )effDimY/(float )dimY : 1.0;
    scaleXY = min(scaleX, scaleY);
    xOff    = dimX/2 + minX;
    yOff    = dimY/2 + minY;
  }

  xShift  = effDimX/2 + SY_MIN_BORDER;
  yShift  = effDimY/2 + SY_MIN_BORDER;


  for (i = 1; i <= vss->dim; i++){

    if ((vss->skel[i].is_foreground && ctrl->display_mode == SKE_BACKGROUND) ||
        (!vss->skel[i].is_foreground && ctrl->display_mode == SKE_FOREGROUND)) 
      continue;

    /* draw nodes */
    for (k = 1; k <= vss->skel[i].nodedim; k++){

      if (ctrl->rank > 0){
	if (vss->skel[i].node[k].hierarchy_level <= 0 ||
	    vss->skel[i].node[k].hierarchy_level > ctrl->rank) continue;
      }

      x1 = vss->skel[i].node[k].loc.rp.x;
      y1 = vss->skel[i].node[k].loc.rp.y;
      arcPtr[adim].x = scaleXY * (x1- xOff) + xShift - 2;
      arcPtr[adim].y = scaleXY * (y1- yOff) + yShift - 2;
      arcPtr[adim].width  = arcPtr[adim].height = 4;
      arcPtr[adim].angle1 = arcPtr[adim].angle2 = 360*64;
      adim++;
    }
    /* draw axes */
    for (k = 1; k <= vss->skel[i].branchdim; k++){

      if (ctrl->rank > 0){
	if (vss->skel[i].branch[k].hierarchy_level <= 0 ||
	    vss->skel[i].branch[k].hierarchy_level > ctrl->rank) continue;
      }

      axis = vss->skel[i].branch[k].axis;

      x1 = axis[1].rp.x; 
      y1 = axis[1].rp.y;
      for (v = 2; v <= vss->skel[i].branch[k].elementdim; v++){
	x2 = axis[v].rp.x;
	y2 = axis[v].rp.y;
	segmentPtr[sdim].x1 = scaleXY * (x1 - xOff) + xShift;
	segmentPtr[sdim].y1 = scaleXY * (y1 - yOff) + yShift; 
	segmentPtr[sdim].x2 = scaleXY * (x2 - xOff) + xShift;
	segmentPtr[sdim].y2 = scaleXY * (y2 - yOff) + yShift;
	sdim++;
	x1 = x2; y1 = y2;
      }
    }

    /* draw boundary */
    for (k = 1; k <= vss->skel[i].branchdim; k++){

      if (ctrl->rank > 0){
	if (vss->skel[i].branch[k].hierarchy_level <= 0 ||
	    vss->skel[i].branch[k].hierarchy_level > ctrl->rank) continue;
      }

      axis = vss->skel[i].branch[k].axis;

      if (vss->skel[i].branch[k].elementdim == 2){
	tx  = 0.25*(axis[2].rp.x - axis[1].rp.x);
	ty  = 0.25*(axis[2].rp.y - axis[1].rp.y);
	mx  = 0.5*(axis[2].rp.x + axis[1].rp.x);
	my  = 0.5*(axis[2].rp.y + axis[1].rp.y);
	x01 = x1 = mx + vss->skel[i].branch[k].left_bndry[1].xoff;
	y01 = y1 = my + vss->skel[i].branch[k].left_bndry[1].yoff;
	x01 -= tx; y01 -= ty;
	x1  += tx; y1  += ty;
	x02 = x2 = mx + vss->skel[i].branch[k].right_bndry[1].xoff;
	y02 = y2 = my + vss->skel[i].branch[k].right_bndry[1].yoff;
	x02 -= tx; y02 -= ty;
	x2  += tx; y2  += ty;
	segmentPtr[sdim].x1 = scaleXY * (x01 - xOff) + xShift;
	segmentPtr[sdim].y1 = scaleXY * (y01 - yOff) + yShift; 
	segmentPtr[sdim].x2 = scaleXY * (x1  - xOff) + xShift;
	segmentPtr[sdim].y2 = scaleXY * (y1  - yOff) + yShift;
	sdim++;
	segmentPtr[sdim].x1 = scaleXY * (x02 - xOff) + xShift;
	segmentPtr[sdim].y1 = scaleXY * (y02 - yOff) + yShift; 
	segmentPtr[sdim].x2 = scaleXY * (x2  - xOff) + xShift;
	segmentPtr[sdim].y2 = scaleXY * (y2  - yOff) + yShift;
	sdim++;
      }
      else{
	for (v = 2; v <= vss->skel[i].branch[k].elementdim; v++){  
	  mx = 0.5*(axis[v].rp.x + axis[v-1].rp.x);
	  my = 0.5*(axis[v].rp.y + axis[v-1].rp.y);
	  x1 = mx + vss->skel[i].branch[k].left_bndry[v-1].xoff;
	  y1 = my + vss->skel[i].branch[k].left_bndry[v-1].yoff;
	  x2 = mx + vss->skel[i].branch[k].right_bndry[v-1].xoff;
	  y2 = my + vss->skel[i].branch[k].right_bndry[v-1].yoff;
	  if (v > 2){
	    segmentPtr[sdim].x1 = scaleXY * (x01 - xOff) + xShift;
	    segmentPtr[sdim].y1 = scaleXY * (y01 - yOff) + yShift; 
	    segmentPtr[sdim].x2 = scaleXY * (x1  - xOff) + xShift;
	    segmentPtr[sdim].y2 = scaleXY * (y1  - yOff) + yShift;
	    sdim++;
	    segmentPtr[sdim].x1 = scaleXY * (x02 - xOff) + xShift;
	    segmentPtr[sdim].y1 = scaleXY * (y02 - yOff) + yShift; 
	    segmentPtr[sdim].x2 = scaleXY * (x2  - xOff) + xShift;
	    segmentPtr[sdim].y2 = scaleXY * (y2  - yOff) + yShift;
	    sdim++;
	  }
	  x01 = x1; y01 = y1; x02 = x2; y02 = y2;
	} /* for all axis elements */
      } /* else */
    } /* for */
  } /* for all skeletons */

  *winWidth   = effDimX + 2*SY_MIN_BORDER;
  *winHeight  = effDimY + 2*SY_MIN_BORDER;
  *segmentDim = sdim;
  *pointDim   = pdim;
  *arcDim     = adim;
}
#endif
