/*
 *  $Id$
 * 
 *  Author: Robert L. Ogniewicz, Swiss Federal Institute of Technology ETH
 *          Zurich, Switzerland (rlo@vision.ethz.ch)
 */

/* Copyright Notice
 * ================
 * Copyright (C) 1993 by Robert L. Ogniewicz
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
 */

static char sy_DrawTrianId[] = "$Id$"; 

#include "Sy_Skeleton.h"
#include "Sy_Intrinsic.h"
#include "Sy_Graphics.h"
#include "inipoints.xbm"
#include "triangulation.xbm"

#ifndef NOX
#include <X11/Xatom.h>
static Atom proto_atom = None, delete_atom = None;

typedef struct{
   Window  winId;
   char    *tag;
   void   (*eventHandler)();
} SyLtEventAction;

static XEvent  LtXevent;

static GC      gc;
static Pixmap  icon;

static XPoint   *pointPtr;
static XSegment *segmentPtr;
static int      pointDim, segmentDim;

static Display *display;
static char    *displayName = NULL;
static int     screenNr;

static void Sy_CatchEvent(XEvent *event, Window winId, unsigned int winNr, char *tag);
static void Sy_ExposeEvent(XExposeEvent *event, Window winId, unsigned int winNr);
static void Sy_Trian2X(SyTriDAC *tri, XPoint *pointPtr, int *pointDim, XSegment *segmentPtr, int *segmentDim, int *winWidth, int *winHeight, int xZen, int yZen, float uscale);

static int   halt = 0;
static float cont_thres = 0.0,
             prox_thres = 0.0,
             narr_thres = 0.0;
#endif

int Sy_XTrianDisp (SyTriDAC *tri, SyWalktask *wt, char *windowName, char *iconName, int xPos, int yPos, int xZen, int yZen, float uscale, char *userDisplay)
{
#ifndef NOX
  SyLtEventAction  eventBuff;
  int            winWidth, winHeight;
  int            borderWidth = SY_DEF_BORDERWIDTH;
  int            point_mem, segment_mem, page;

  if (wt != NULL){
    cont_thres = wt->group_thres;
    prox_thres = wt->pro_thres;
    narr_thres = wt->narr_thres;
  }

  page         = getpagesize();
  point_mem    = (((tri->pdim + 1)*sizeof (XPoint))/page+1)*page;
  segment_mem  = (((tri->edim+ 1)*sizeof (XSegment))/page+1)*page;

  pointPtr   = (XPoint*)   Sy_Malloc ((unsigned)point_mem);
  segmentPtr = (XSegment*) Sy_Malloc ((unsigned)segment_mem);

  Sy_Trian2X (tri, pointPtr, &pointDim, segmentPtr, &segmentDim, 
	      &winWidth, &winHeight, xZen, yZen, uscale); 

  Sy_DestroyTrian (tri);

  displayName  = userDisplay != NULL ? userDisplay : NULL;
  if ((display = XOpenDisplay (displayName)) == NULL) return (-1);  
  screenNr     = XDefaultScreen (display);

  eventBuff.winId = XCreateSimpleWindow (display, 
					 RootWindow (display, screenNr), 
					 xPos, yPos, winWidth, 
					 winHeight, borderWidth,
					 BlackPixel (display, screenNr),
					 WhitePixel (display, screenNr));
  
  gc   = XCreateGC (display, eventBuff.winId, 0, 0);
  icon = XCreateBitmapFromData (display, eventBuff.winId, ipo_icon_bits, 
                                ipo_icon_width, ipo_icon_height);
  XSetForeground (display, gc, BlackPixel (display, screenNr));
  XSetBackground (display, gc, WhitePixel (display, screenNr));
  XSetStandardProperties (display, eventBuff.winId, windowName, 
			  iconName , icon, NULL, 0, NULL);

  XSelectInput (display, eventBuff.winId, 
		ExposureMask | StructureNotifyMask);

  proto_atom = XInternAtom(display, "WM_PROTOCOLS", False);
  delete_atom = XInternAtom(display, "WM_DELETE_WINDOW", False);
  if ((proto_atom != None) && (delete_atom != None))
    XChangeProperty(display, eventBuff.winId, proto_atom, XA_ATOM, 32,
		    PropModeReplace, (unsigned char*) &delete_atom, 1);

  eventBuff.eventHandler = Sy_CatchEvent;
   
  XMapWindow (display, eventBuff.winId);

  while (LtXevent.type != Expose && LtXevent.xany.window != eventBuff.winId) 
    XNextEvent(display, &LtXevent);
  while (1){
    if (LtXevent.xany.window == eventBuff.winId)
      eventBuff.eventHandler (&LtXevent, LtXevent.xany.window, 1, NULL);
    XNextEvent (display, &LtXevent);
  }
#else
  Sy_Message("%s", "no X11 graphics support available\n");
#endif
}

#ifndef NOX
static void Sy_CatchEvent (XEvent *event, Window winId, 
			   unsigned int winNr, char *tag)
{
  XClientMessageEvent *me;
  switch (event->type){
  case Expose :
    Sy_ExposeEvent ((XExposeEvent*)event, winId, winNr); 
    break;
  case DestroyNotify:
    exit(2); break;
  case ClientMessage:
    me = (XClientMessageEvent*)event;
    if ((me->window == winId) && (me->data.l[0] == delete_atom)){
      exit(2);
    }
    break;
  default : break;
  }
  return;
}

static void Sy_ExposeEvent (XExposeEvent *event, Window winId, 
			    unsigned int winNr)
{
   XPoint   *cur_point_block;
   XSegment *cur_segment_block;
   int      count;
   for (cur_point_block = pointPtr, count = 0; count < pointDim/SY_XBLOCK;
        cur_point_block += SY_XBLOCK, count++){
     XDrawPoints  (display, winId, gc, cur_point_block, SY_XBLOCK, 
                   CoordModeOrigin);
   };
   if (pointDim%SY_XBLOCK != 0)
     XDrawPoints  (display, winId, gc, cur_point_block, pointDim%SY_XBLOCK, 
                   CoordModeOrigin);

   for (cur_segment_block = segmentPtr, count = 0; count < segmentDim/SY_XBLOCK;
        cur_segment_block += SY_XBLOCK, count++){
     XDrawSegments (display, winId, gc, cur_segment_block, SY_XBLOCK);
   };
   if (segmentDim%SY_XBLOCK != 0)
     XDrawSegments (display, winId, gc, cur_segment_block, segmentDim%SY_XBLOCK);

   return;
}

static void Sy_Trian2X (SyTriDAC *tri, XPoint *pointPtr, int *pointDim, XSegment *segmentPtr, int *segmentDim, int *winWidth, int *winHeight, int xZen, int yZen, float uscale)
{
  float    scaleX, scaleY, scaleXY;
  float    xOff,   yOff, 
           xShift, yShift;

  int      maxX, minX, maxY, minY;
  int      dimX, dimY; 
  int      effDimX, effDimY;

  int      tail_point, head_point;
  int      sDim, i, is_render, is_linked = 1;

  maxX = minX = tri->point[1].x;
  minY = tri->point[1].y;
  maxY = tri->point[tri->pdim].y;

  for (i = 1; i <= tri->pdim; i++){
     maxX = max(maxX, tri->point[i].x);
     minX = min(minX, tri->point[i].x);
  }

  dimX = maxX - minX;
  dimY = maxY - minY;

  if (uscale > 0){
    effDimX = max(min(dimX * uscale, SY_MAX_WINDIM), SY_MIN_WINDIM);
    effDimY = max(min(dimY * uscale, SY_MAX_WINDIM), SY_MIN_WINDIM);
    xOff    = (xZen > 0) ? xZen : dimX/2 + minX;
    yOff    = (yZen > 0) ? yZen : dimY/2 + minY;
    scaleXY = uscale;
  }
  else{
    effDimX = max(min(dimX, SY_DEF_WINDIM), SY_MIN_WINDIM);
    effDimY = max(min(dimY, SY_DEF_WINDIM), SY_MIN_WINDIM);
    scaleX  = dimX > 0 ? (float )effDimX/(float )dimX : 1.0; 
    scaleY  = dimY > 0 ? (float )effDimY/(float )dimY : 1.0;
    scaleXY = min(scaleX, scaleY);
    xOff    = dimX/2 + minX;
    yOff    = dimY/2 + minY;
  }

  xShift  = effDimX/2 + SY_MIN_BORDER;
  yShift  = effDimY/2 + SY_MIN_BORDER;

  for (i = 1; i <= tri->pdim; i++){
    pointPtr[i-1].x = scaleXY * ((float)tri->point[i].x - xOff) + xShift;
    pointPtr[i-1].y = scaleXY * ((float)tri->point[i].y - yOff) + yShift;
  }

  for (i = 1, sDim = 0; i <= tri->edim; i++) {
    is_render = (tail_point = tri->edge[i].tail_point) && 
                (head_point = tri->edge[i].head_point);

    if (tri->regdim > 0) 
      is_linked = ((tri->regul[i].cont >= cont_thres) &&
		   (tri->regul[i].prox >= prox_thres) &&
		   (tri->regul[i].narr >= narr_thres));
    if (is_render && is_linked){
      segmentPtr[sDim].x1 = scaleXY * ((float)tri->point[tail_point].x - xOff) + xShift;
      segmentPtr[sDim].y1 = scaleXY * ((float)tri->point[tail_point].y - yOff) + yShift; 
      segmentPtr[sDim].x2 = scaleXY * ((float)tri->point[head_point].x - xOff) + xShift;
      segmentPtr[sDim].y2 = scaleXY * ((float)tri->point[head_point].y - yOff) + yShift;
      sDim++;
    }
  }
  *winWidth   = effDimX + 2*SY_MIN_BORDER;
  *winHeight  = effDimY + 2*SY_MIN_BORDER;
  *segmentDim = sDim;
  *pointDim   = tri->pdim;
}


static void Sy_CatchEventV2(XEvent *event, Window winId, unsigned int winNr, char *tag);
static void Sy_ExposeEventV2(XExposeEvent *event, Window winId, unsigned int winNr);

static void  Sy_SkelTrian2X(SySkelDAC *vsk, SyTriDAC *tri, float thres, XPoint *pointPtr, int *pointDim, XSegment *segmentPtr, int *segmentDim, int *winWidth, int *winHeight, int xZen, int yZen, float uscale);
#endif

int Sy_XSkelDispWithTrian (SySkelDAC *vsk, SyTriDAC *tri, float th, char *windowName, char *iconName, int xPos, int yPos, int xZen, int yZen, float uscale, char *userDisplay, SyDisplayControl control)
{
#ifndef NOX
  SyLtEventAction  eventBuff;
  int            winWidth, winHeight;
  int            borderWidth = SY_DEF_BORDERWIDTH;


  displayName  = userDisplay != NULL ? userDisplay : NULL;
  if ((display = XOpenDisplay (displayName)) == NULL) return (-1);  
  screenNr     = XDefaultScreen (display);

  pointPtr   = (XPoint*)   Sy_Malloc ((unsigned)((vsk->pdim +1) * sizeof(XPoint)));
  segmentPtr = (XSegment*) Sy_Malloc ((unsigned)((vsk->edim + tri->edim +2) * 
				   sizeof(XSegment)));

  if (tri->is_pointref == 0){
    Sy_MakeTriPointref(tri);
  }

  Sy_SkelTrian2X (vsk, tri, th, pointPtr, &pointDim, segmentPtr, &segmentDim, 
           &winWidth, &winHeight, xZen, yZen, uscale); 

  Sy_DestroySkeleton (vsk);
  Sy_DestroyTrian (tri);

  eventBuff.winId = XCreateSimpleWindow (display, 
					 RootWindow (display, screenNr), 
					 xPos, yPos, winWidth, 
					 winHeight, borderWidth,
					 BlackPixel (display, screenNr),
					 WhitePixel (display, screenNr));
  
  gc   = XCreateGC (display, eventBuff.winId, 0, 0);
  icon = XCreateBitmapFromData (display, eventBuff.winId, trian_icon_bits, 
                                trian_icon_width, trian_icon_height);
  XSetForeground (display, gc, BlackPixel (display, screenNr));
  XSetBackground (display, gc, WhitePixel (display, screenNr));
  XSetStandardProperties (display, eventBuff.winId, windowName, 
			  iconName , icon, NULL, 0, NULL);

  XSelectInput (display, eventBuff.winId, 
		ExposureMask | StructureNotifyMask);

  proto_atom = XInternAtom(display, "WM_PROTOCOLS", False);
  delete_atom = XInternAtom(display, "WM_DELETE_WINDOW", False);
  if ((proto_atom != None) && (delete_atom != None))
    XChangeProperty(display, eventBuff.winId, proto_atom, XA_ATOM, 32,
		    PropModeReplace, (unsigned char*) &delete_atom, 1);

  eventBuff.eventHandler = Sy_CatchEventV2;
   
  XMapWindow (display, eventBuff.winId);
  while (LtXevent.type != Expose && LtXevent.xany.window != eventBuff.winId) 
    XNextEvent(display, &LtXevent);

  while (1){
    if (LtXevent.xany.window == eventBuff.winId)
      eventBuff.eventHandler (&LtXevent, LtXevent.xany.window, 1, NULL);
    XNextEvent (display, &LtXevent);
  }
#else
  Sy_Message("%s", "no X11 graphics support available\n");
#endif
}

#ifndef NOX
static void Sy_CatchEventV2 (XEvent *event, Window winId, 
unsigned int winNr, char *tag)
{
  XClientMessageEvent *me;
  switch (event->type){
  case Expose :
    Sy_ExposeEvent ((XExposeEvent*)event, winId, winNr); 
    break;
  case DestroyNotify:
    exit(2); break;
  case ClientMessage:
    me = (XClientMessageEvent*)event;
    if ((me->window == winId) && (me->data.l[0] == delete_atom)){
      exit(2);
    }
    break;
  default : break;
  }
  return;
}

static void Sy_ExposeEventV2 (XExposeEvent *event, Window winId, 
			      unsigned int winNr)
{
   XPoint   *cur_point_block;
   XSegment *cur_segment_block;
   int      count;

   for (cur_point_block = pointPtr, count = 0; count < pointDim/SY_XBLOCK;
        cur_point_block += SY_XBLOCK, count++){
     XDrawPoints  (display, winId, gc, cur_point_block, SY_XBLOCK, 
		   CoordModeOrigin);
   };
   if (pointDim%SY_XBLOCK != 0)
     XDrawPoints  (display, winId, gc, cur_point_block, pointDim%SY_XBLOCK, 
		   CoordModeOrigin);

   for (cur_segment_block = segmentPtr, count = 0; count < segmentDim/SY_XBLOCK;
        cur_segment_block += SY_XBLOCK, count++){
     XDrawSegments (display, winId, gc, cur_segment_block, SY_XBLOCK);
   };
   if (segmentDim%SY_XBLOCK != 0)
     XDrawSegments (display, winId, gc, cur_segment_block, segmentDim%SY_XBLOCK);

   return;
}

static void Sy_SkelTrian2X (SySkelDAC *vsk, SyTriDAC *tri, float thres, XPoint *pointPtr, int *pointDim, XSegment *segmentPtr, int *segmentDim, int *winWidth, int *winHeight, int xZen, int yZen, float uscale)
{
  SyVertex   vhead, vtail;
  int	   i;
  float    x1, y1, x2, y2;
  float    scaleX, scaleY, scaleXY;
  float    xOff, yOff, 
           xShift, yShift;
  float    rubber;
  float    dirX, dirY;
  int      maxX, minX, minY;
  int      dimX, dimY, sDim; 
  int      effDimX, effDimY;
  int      add_segment, conn_flag;
  int      tail_vertex, head_vertex, 
           left_point, right_point,
           tail_point, head_point;
  int      tail_flag, head_flag;
  int      is_residual_cut, is_skel, is_render;

  maxX = minX = vsk->point[3].x;
  minY = max(vsk->point[3].y,SY_DEF_MINY);

  for (i = 1; i <= vsk->pdim; i++){
     maxX = max(maxX, vsk->point[i].x);
     minX = min(minX, vsk->point[i].x);
  }

  dimX = maxX-minX;
  dimY = vsk->point[vsk->pdim].y - minY;

  if (uscale > 0){
    effDimX = max(min(dimX * uscale, SY_MAX_WINDIM), SY_MIN_WINDIM);
    effDimY = max(min(dimY * uscale, SY_MAX_WINDIM), SY_MIN_WINDIM);
    xOff    = (xZen > 0) ? xZen : dimX/2 + minX;
    yOff    = (yZen > 0) ? yZen : dimY/2 + minY;
    scaleXY = uscale;
  }
  else{
    effDimX = max(min(dimX, SY_DEF_WINDIM), SY_MIN_WINDIM);
    effDimY = max(min(dimY, SY_DEF_WINDIM), SY_MIN_WINDIM);
    scaleX  = dimX > 0 ? (float )effDimX/(float )dimX : 1.0; 
    scaleY  = dimY > 0 ? (float )effDimY/(float )dimY : 1.0;
    scaleXY = min(scaleX, scaleY);
    xOff    = dimX/2 + minX;
    yOff    = dimY/2 + minY;
  }

  xShift  = effDimX/2 + SY_MIN_BORDER;
  yShift  = effDimY/2 + SY_MIN_BORDER;

  for (i = 1; i <= vsk->pdim; i++){
    pointPtr[i-1].x = scaleXY * (vsk->point[i].x - xOff) + xShift;
    pointPtr[i-1].y = scaleXY * (vsk->point[i].y - yOff) + yShift;
  }

  for (i = 1, sDim = 0; i <= vsk->edim; i++) {
    
    tail_vertex = vsk->edge[i].tail_vertex;
    head_vertex = vsk->edge[i].head_vertex;
    left_point  = vsk->edge[i].left_point;
    right_point = vsk->edge[i].right_point;

    tail_flag   = vsk->regul[tail_vertex].birth[2] > thres;
    head_flag   = vsk->regul[head_vertex].birth[2] > thres;

    conn_flag = Sy_TrianConnected (tri, left_point, right_point);

    is_residual_cut = vsk->is_eresid && 
                      ((vsk->eresid[i].tail_residual > thres) || 
                       (vsk->eresid[i].head_residual > thres));

    is_skel         = tail_flag && head_flag;

    is_render = (is_residual_cut || is_skel) && (!conn_flag || vsk->is_border);

    if (is_render){

      if (!(tail_flag && head_flag)){
	Sy_SimpleResidualCut (vsk, i, thres, &vtail, &vhead);
      }
      else{
	vtail = vsk->vertex[vsk->edge[i].tail_vertex];
	vhead = vsk->vertex[vsk->edge[i].head_vertex];
      }

      x1 = scaleXY * (vtail.xden / vtail.num - xOff) + xShift;
      y1 = scaleXY * (vtail.yden / vtail.num - yOff) + yShift;
      x2 = scaleXY * (vhead.xden / vhead.num - xOff) + xShift;
      y2 = scaleXY * (vhead.yden / vhead.num - yOff) + yShift;

      switch (vsk->edge[i].rank) {
      case 0:
	dirX = x2 - x1;
	dirY = y2 - y1;
	rubber = SY_VALRAD/max(SY_ALV(dirX), SY_ALV(dirY));
	x2 += rubber * dirX;
	y2 += rubber * dirY;
	x1 -= rubber * dirX;
	y1 -= rubber * dirY;
	add_segment = 1;
	break;
      case 1:
	if (!(add_segment = ((SY_ALV(x1) < SY_PLANESIZE) && 
			     (SY_ALV(y1) < SY_PLANESIZE)))) break;
	dirX = x2 - x1;
	dirY = y2 - y1;
	rubber = SY_VALRAD/max(SY_ALV(dirX), SY_ALV(dirY));
	x2 = x1 + rubber * dirX;
	y2 = y1 + rubber * dirY;
	break;
      default:
	if (add_segment = ((SY_ALV(x1) < SY_PLANESIZE) && (SY_ALV(y1) < SY_PLANESIZE)
			   && (SY_ALV(x2) < SY_PLANESIZE) && 
			   (SY_ALV(y2) < SY_PLANESIZE))) break;
	if (add_segment = ((SY_ALV(x1) < SY_PLANESIZE) && (SY_ALV(y1) < SY_PLANESIZE))) {
	  dirX = x2 - x1;
	  dirY = y2 - y1;
	  rubber = SY_VALRAD/max(SY_ALV(dirX), SY_ALV(dirY)); 
	  x2 = x1 + rubber * dirX;
	  y2 = y1 + rubber * dirY;
	  break;
	}
	if (add_segment = ((SY_ALV(x2) < SY_PLANESIZE) && (SY_ALV(y2) < SY_PLANESIZE))) {
	  dirX = x1 - x2;
	  dirY = y1 - y2;
	  rubber = SY_VALRAD/max(SY_ALV(dirX), SY_ALV(dirY)); 
	  x1 = x2 + rubber * dirX;
	  y1 = y2 + rubber * dirY;
	  break;
	}
      }
      if (add_segment) {
	segmentPtr[sDim].x1 = x1;
	segmentPtr[sDim].y1 = y1;
	segmentPtr[sDim].x2 = x2;
	segmentPtr[sDim].y2 = y2;
        sDim++;
      }
    }
  }
  for (i = 1; i <= tri->edim; i++) {
    tail_point = tri->edge[i].tail_point;
    head_point = tri->edge[i].head_point;
    segmentPtr[sDim].x1 = scaleXY * ((float)tri->point[tail_point].x - xOff) + xShift;
    segmentPtr[sDim].y1 = scaleXY * ((float)tri->point[tail_point].y - yOff) + yShift; 
    segmentPtr[sDim].x2 = scaleXY * ((float)tri->point[head_point].x - xOff) + xShift;
    segmentPtr[sDim].y2 = scaleXY * ((float)tri->point[head_point].y - yOff) + yShift;
    sDim++;
  }

  *winWidth   = effDimX + 2*SY_MIN_BORDER;
  *winHeight  = effDimY + 2*SY_MIN_BORDER;
  *segmentDim = sDim;
  *pointDim   = vsk->pdim;
}
#endif









