/*
 *  $Id$
 * 
 *  Author: Robert L. Ogniewicz, Swiss Federal Institute of Technology ETH
 *          Zurich, Switzerland (rlo@vision.ethz.ch)
 */

/* Copyright Notice
 * ================
 * Copyright (C) 1993 by Robert L. Ogniewicz
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
 */

/* 
 * shape-oriented grouping of contour fragments
 * 
 */

static char sy_GrouperId[] = "$Id$"; 

#include "Sy_Skeleton.h"
#include "Sy_Intrinsic.h"

static double group_thres, narr_thres, pro_thres;
static int    maxicn;
static int    icncount;

void Sy_SGroup (SySkelDAC *vsk, SyTriDAC *tri, SyIpoDAC *ipd, SyWalktask *wt)
{ 
    void Sy_SGroupInit(void), Sy_SGroupExit(void);    

    int i;
    int next_edge, current_head_vertex;
    int left_point, right_point,
        common_point, head_edge, tail_edge;

    sprintf (syStrBuf, "%s ", sy_GrouperId);
    Sy_Message (SY_FMTNL, syStrBuf);
 
    if (tri->erefdim <= 0){
      Sy_SimpleError (SY_FMTNL, "missing voro/trian reference array ");
      exit(2);
    }

    syWstate.ipd = ipd;
    syWstate.vsk = vsk;
    syWstate.tri = tri;
    syWstate.group_thres = wt->group_thres;
    syWstate.narr_thres  = wt->narr_thres;
    syWstate.pro_thres   = wt->pro_thres;

    tri->regul  = Sy_AllocTriRegul (tri->edim);
    tri->regdim = tri->edim;

    Sy_AddHardlink (ipd, tri);
    
    vsk->label.p_  = Sy_AllocLabel (vsk->pdim);
 
    Sy_InitSkelTraverse(vsk, ipd, syWstate.verbose, wt);    

    Sy_SGroupInit();

    /* Grouping Pass 1 */
    for (i = 1; i <= vsk->edim; i++){
      if (vsk->edge[i].rank < 2){
	left_point  = vsk->edge[i].left_point;
        common_point = (vsk->label.p_[left_point])? 0: left_point;              

        if (common_point){
          vsk->label.p_[common_point] = i;
          next_edge           = vsk->edge[i].tail_edge;
          current_head_vertex = vsk->edge[i].tail_vertex;
          syWstate.start.this_edge  = i;
          Sy_SimplePolyScan(vsk, ipd, &next_edge, &current_head_vertex);
	}
      }
    } 

    /* Grouping Pass 2 */
    for (i = 1; i <= vsk->edim; i++){
      if (vsk->edge[i].rank >= 2){

	left_point  = vsk->edge[i].left_point;
	right_point = vsk->edge[i].right_point;
        head_edge   = vsk->edge[i].head_edge;
        tail_edge   = vsk->edge[i].tail_edge;
              
	if (! (vsk->label.p_[left_point] && vsk->label.p_[right_point]) ){

	  if ((vsk->edge[i].right_point == vsk->edge[tail_edge].left_point) ||
	      (vsk->edge[i].right_point == vsk->edge[tail_edge].right_point)){
	    common_point = vsk->edge[i].right_point;
	  }
	  else{
	    common_point = vsk->edge[i].left_point;
	  }
	  
	  if (vsk->label.p_[common_point]==0){
	    vsk->label.p_[common_point] = i;
	    next_edge           = vsk->edge[i].tail_edge;
	    current_head_vertex = vsk->edge[i].tail_vertex;
	    syWstate.start.this_edge  = i;
	    Sy_SimplePolyScan(vsk, ipd, &next_edge, &current_head_vertex);
	  }
	  else{
 
	    if ((vsk->edge[i].right_point == vsk->edge[head_edge].left_point) ||
		(vsk->edge[i].right_point == vsk->edge[head_edge].right_point)){
	      common_point = vsk->edge[i].right_point;
	    }
	    else{
	      common_point = vsk->edge[i].left_point;
	    }

	    if (vsk->label.p_[common_point]==0){
	      vsk->label.p_[common_point] = i;
	      next_edge           = vsk->edge[i].head_edge;
	      current_head_vertex = vsk->edge[i].head_vertex;
	      syWstate.start.this_edge  = i;
	      Sy_SimplePolyScan(vsk, ipd, &next_edge, &current_head_vertex);
	    }
	  }
	}
      }
    }

    Sy_SGroupExit();
  }

/**********************************************************************
 * function SY_VOID Sy_SGroupWalk () 
 * 
 * purpose: 
 *
 *********************************************************************/

void Sy_SGroupWalk(void)
{
  void Sy_SGroupProcess(SyEdge *R_edge, SyEdge *proj_edge, int ref_edge, int ref_point);
  void swap(SyEdge *e);

  SyEdge C_edge, L_edge, Cproj_edge, Lproj_edge;

  int c_index, l_index, ref_point, ref_edge; 

  if (syWstate.current.head_vertex == syWstate.start.head_vertex) return;
  if (syWstate.current.next_edge   == 0                   ) return;

/*
  c_index = syWstate.current.next_edge;
  l_index = syWstate.last.next_edge;
*/

  c_index  = syWstate.current.this_edge;
  l_index  = syWstate.last.this_edge;

  C_edge = syWstate.vsk->edge[c_index];
  L_edge = syWstate.vsk->edge[l_index];

  if (C_edge.tail_vertex == L_edge.head_vertex){
    swap (&L_edge); 
  }
  else if (C_edge.head_vertex == L_edge.head_vertex){
    swap (&C_edge); 
    swap (&L_edge); 
  }
  else if (C_edge.head_vertex == L_edge.tail_vertex){
    swap (&C_edge); 
  }

  if ((C_edge.right_point == L_edge.left_point) ||
      (C_edge.right_point == L_edge.right_point)){
    ref_point = C_edge.right_point;
  }
  else if ((C_edge.left_point == L_edge.left_point) ||
           (C_edge.left_point == L_edge.right_point)){
    ref_point = C_edge.left_point;
  }
  else{ 
    Sy_SimpleError(SY_FMTNL, "missing common point ");
    return;
  }

  Cproj_edge = C_edge; swap (&Cproj_edge);
  Lproj_edge = L_edge; swap (&Lproj_edge);
  Cproj_edge.xoff = Lproj_edge.xoff = 2* syWstate.ipd->ipo[ref_point].x;
  Cproj_edge.yoff = Lproj_edge.yoff = 2* syWstate.ipd->ipo[ref_point].y;

  if (! Sy_SGroupConnected (C_edge.left_point, C_edge.right_point)){
    Sy_SGroupProcess (&C_edge, &Lproj_edge, ref_edge=c_index, ref_point);
  }
  else{
    ref_edge = c_index;
    syWstate.tri->regul[syWstate.tri->eref[ref_edge]].cont = SY_POS_INFTY;
    syWstate.tri->regul[syWstate.tri->eref[ref_edge]].narr = SY_POS_INFTY;
    syWstate.tri->regul[syWstate.tri->eref[ref_edge]].prox = SY_POS_INFTY;
  }

  if (! Sy_SGroupConnected (L_edge.left_point, L_edge.right_point)){
    Sy_SGroupProcess (&L_edge, &Cproj_edge, ref_edge=l_index, ref_point);
  }
  else{
    ref_edge = l_index;
    syWstate.tri->regul[syWstate.tri->eref[ref_edge]].cont = SY_POS_INFTY;
    syWstate.tri->regul[syWstate.tri->eref[ref_edge]].narr = SY_POS_INFTY;
    syWstate.tri->regul[syWstate.tri->eref[ref_edge]].prox = SY_POS_INFTY;
  }

  return;
}

#define SY_INFTY 10000000.0

void Sy_SGroupProcess(SyEdge *R_edge, SyEdge *proj_edge, int ref_edge, int ref_point)
{
  int  Sy_SGroupIntersection(SyEdge *e1, SyEdge *e2, SyVertex *v), Sy_SGroupConnected(int left_point, int right_point);

  SyPoint P_left, P_right;

  SyVertex I_v, R_v, T_v, H_v;

  double ref_dist, I_dist;

  int    pdist;
  double T_sdist, H_sdist, rdist; 
  int    left_point, right_point;

  int  ref_vertex; 

  int is_Sy_Intersection = 0,
      is_nop          = 0;

  int    is_near = 0;
  double near_T, near_H;

  double tmp_cont, tmp_narr, tmp_prox;

  int tmp_vote, rindex;
/*
  is_nop = ((R_edge->left_point < 3) || (R_edge->right_point < 3));
  if (is_nop) return; */

  ref_vertex = R_edge->tail_vertex;
  R_v        = syWstate.vsk->vertex[ref_vertex];
  H_v        = syWstate.vsk->vertex[R_edge->head_vertex];

  is_Sy_Intersection = Sy_SGroupIntersection (R_edge, proj_edge, &I_v);
  I_v.num *=2.0;

  ref_dist = (R_edge->rank >=2) ? SY_SQUARE(H_v.xden/H_v.num - R_v.xden/R_v.num) +
                                  SY_SQUARE(H_v.yden/H_v.num - R_v.yden/R_v.num) : SY_INFTY;

  if (is_Sy_Intersection){

    if (R_edge->rank < 2){
      tmp_cont = SY_INFTY;      
    }
    else{
      I_dist   = SY_SQUARE(I_v.xden/I_v.num - R_v.xden/R_v.num) +
	         SY_SQUARE(I_v.yden/I_v.num - R_v.yden/R_v.num);      
      tmp_cont = (I_dist != 0.0)? ref_dist/I_dist : SY_INFTY; 
    }
  }
  
  left_point   = R_edge->left_point;
  right_point  = R_edge->right_point;
  
  P_left  = syWstate.vsk->point[left_point];
  P_right = syWstate.vsk->point[right_point];
  pdist   = SY_SQUARE (P_left.x - P_right.x) + SY_SQUARE (P_left.y - P_right.y);
  rdist   = SY_SQUARE (R_edge->dirx) + SY_SQUARE(R_edge->diry);
  
  T_v    = syWstate.vsk->vertex[R_edge->tail_vertex];
  H_v    = syWstate.vsk->vertex[R_edge->head_vertex];   
  
  T_sdist =  SY_SQUARE(P_left.x -(T_v.xden/T_v.num)) 
           + SY_SQUARE(P_left.y -(T_v.yden/T_v.num));
  H_sdist =  SY_SQUARE(P_left.x -(H_v.xden/H_v.num)) 
           + SY_SQUARE(P_left.y -(H_v.yden/H_v.num));
    
  if (SY_SGN(R_edge->dirx) != SY_SGN(H_v.xden * T_v.num - T_v.xden * H_v.num)){
    R_edge->dirx = - R_edge->dirx;
    R_edge->diry = - R_edge->diry;
  }
       
  near_T = (R_edge->rank >= 1) ? ((  R_edge->dirx * (P_left.x * T_v.num - T_v.xden)
				   + R_edge->diry * (P_left.y * T_v.num - T_v.yden))) : 1.0;
  
  near_H = (R_edge->rank >= 2) ? ((  R_edge->dirx * (P_left.x * H_v.num - H_v.xden)
				   + R_edge->diry * (P_left.y * H_v.num - H_v.yden))) : -1.0;

  if (R_edge->rank >= 1){
    near_T = (near_T > 0.0)?  SY_SQUARE(near_T/T_v.num)/(rdist * T_sdist):
      - SY_SQUARE(near_T/T_v.num)/(rdist * T_sdist);
  }
  if (R_edge->rank >= 2){
    near_H = (near_H > 0.0)?  SY_SQUARE(near_H/H_v.num)/(rdist * H_sdist):
      - SY_SQUARE(near_H/H_v.num)/(rdist * H_sdist);
  }
  
  tmp_prox = min(near_T, -1.0*near_H); 
  tmp_narr = ref_dist/pdist;

  rindex   = syWstate.tri->eref[ref_edge];
  tmp_vote = 0;

  if (syWstate.tri->regul[rindex].cont < tmp_cont){ 
    syWstate.tri->regul[rindex].cont = tmp_cont;
    syWstate.tri->regul[rindex].prox = tmp_prox;
  }
  if (syWstate.tri->regul[rindex].narr < tmp_narr){
    syWstate.tri->regul[rindex].narr = tmp_narr; 
    syWstate.tri->regul[rindex].prox = tmp_prox;
  }
/*
  if (syWstate.tri->regul[rindex].cont < tmp_cont) tmp_vote++;
  if (syWstate.tri->regul[rindex].narr < tmp_narr) tmp_vote++; */
/*  if (syWstate.tri->regul[rindex].prox < tmp_prox) tmp_vote++; */

/*
  if (tmp_vote >= 2){
    syWstate.tri->regul[rindex].cont = tmp_cont;
    syWstate.tri->regul[rindex].prox = tmp_prox;
    syWstate.tri->regul[rindex].narr = tmp_narr; 
  } */
  return;
}


int Sy_SGroupConnected (int left_point, int right_point)
{
  int conn_flag, conn[4];

    conn[0]  = ((syWstate.ipd->ipl[left_point].tail_icn == syWstate.ipd->ipl[right_point].tail_icn) &&
		(syWstate.ipd->ipl[left_point].tail_icn != 0));
    
    conn[1]  = ((syWstate.ipd->ipl[left_point].tail_icn == syWstate.ipd->ipl[right_point].head_icn) &&
		(syWstate.ipd->ipl[left_point].tail_icn != 0));
    
    conn[2]  = ((syWstate.ipd->ipl[left_point].head_icn == syWstate.ipd->ipl[right_point].tail_icn) &&
		(syWstate.ipd->ipl[left_point].head_icn != 0));
    
    conn[3]  = ((syWstate.ipd->ipl[left_point].head_icn == syWstate.ipd->ipl[right_point].head_icn) &&
		(syWstate.ipd->ipl[left_point].head_icn != 0));
    
    conn_flag = (conn[0] || conn[1] || conn[2] || conn[3]);
    return(conn_flag);
}




/**********************************************************************
 * function int Sy_SGroupIntersection (e1, e2, v)
 * 
 * purpose: computes the Sy_Intersection of two straight lines, returns 0 in
 *          case of parallel lines
 *
 * param:   e1, e2 - straight lines
 *          v      - resulting vertex
 *********************************************************************/
int Sy_SGroupIntersection (SyEdge *e1, SyEdge *e2, SyVertex *v)
{	
	double tden;

	(*v).num = (*e2).dirx * (*e1).diry  - (*e2).diry * (*e1).dirx;
      	if ( (*v).num == 0 ) return (0);
      	
      	if ( (*v).num < 0 ) {
      		(*v).num = - (*v).num;
      		tden =  (*e2).diry * ( (*e2).xoff - (*e1).xoff )
	              + (*e2).dirx * ( (*e1).yoff - (*e2).yoff );
	}
	else
		tden =  (*e2).diry * ( (*e1).xoff - (*e2).xoff )
	              + (*e2).dirx * ( (*e2).yoff - (*e1).yoff );
  
	(*v).xden =  (*e1).dirx * tden + (*e1).xoff * (*v).num;
	(*v).yden =  (*e1).diry * tden + (*e1).yoff * (*v).num;
	
	return (1);
}


/**********************************************************************
 * function void swap (e)
 * 
 * purpose: reverts the direction of an edge 
 *
 * param:   e  - edge  
 *********************************************************************/
void swap (SyEdge *e)
{
#define SY_EXCH(x,y) temp = x; x = y; y = temp

	int	temp; /* for 'SY_EXCH' macro */
	
/*	if ( (*e).rank >= 2 ) { */

	SY_EXCH ((*e).tail_vertex, (*e).head_vertex);
	SY_EXCH ((*e).tail_edge, (*e).head_edge);
	SY_EXCH ((*e).left_point, (*e).right_point);
	(*e).dirx = - (*e).dirx;
	(*e).diry = - (*e).diry;
	
}


/**********************************************************************
 * function SY_VOID Sy_SGroupInit() 
 * 
 * purpose: 
 *
 *********************************************************************/

void Sy_SGroupInit(void)
{
  group_thres = (syWstate.group_thres >= 0.0)?  SY_SQUARE(syWstate.group_thres):
                                       - SY_SQUARE(syWstate.group_thres);
  narr_thres  = (syWstate.narr_thres >= 0.0)?  SY_SQUARE(syWstate.narr_thres):
                                       - SY_SQUARE(syWstate.narr_thres);
  pro_thres   = (syWstate.pro_thres  >= 0.0)?  SY_SQUARE(syWstate.pro_thres): 
                                       - SY_SQUARE(syWstate.pro_thres);
  maxicn      = syWstate.ipd->icndim;
  icncount    = syWstate.ipd->icndim;
}

/**********************************************************************
 * function SY_VOID Sy_SGroupExit() 
 * 
 * purpose: 
 *
 *********************************************************************/

void Sy_SGroupExit(void)
{
  syWstate.ipd->icndim = icncount;
}


/**********************************************************************
 * function SY_VOID Sy_SkGroupWalk () 
 * 
 * purpose: 
 *
 *********************************************************************/

void Sy_SkGroupWalk(void)
{
  void skog_process();
  void swap(SyEdge *e);

  SyEdge C_edge, L_edge, Cproj_edge, Lproj_edge;

  int c_index, l_index, ref_point, ref_edge; 
  int C_alive, L_alive;

  if (syWstate.current.head_vertex == syWstate.start.head_vertex) return;
  if (syWstate.current.next_edge   == 0                   ) return;

  c_index = syWstate.current.this_edge;
  l_index = syWstate.last.this_edge;

  C_edge = syWstate.vsk->edge[c_index];
  L_edge = syWstate.vsk->edge[l_index];

  if (C_edge.tail_vertex == L_edge.head_vertex){
    swap (&L_edge); 
  }
  else if (C_edge.head_vertex == L_edge.head_vertex){
    swap (&C_edge); 
    swap (&L_edge); 
  }
  else if (C_edge.head_vertex == L_edge.tail_vertex){
    swap (&C_edge); 
  }

  if ((C_edge.right_point == L_edge.left_point) ||
      (C_edge.right_point == L_edge.right_point)){
    ref_point = C_edge.right_point;
  }
  else if ((C_edge.left_point == L_edge.left_point) ||
           (C_edge.left_point == L_edge.right_point)){
    ref_point = C_edge.left_point;
  }
  else{ 
    Sy_SimpleError (SY_FMTNL, "missing common point ");
    return;
  }

  C_alive = ((syWstate.vsk->regul[C_edge.tail_vertex].birth[2] >= syWstate.thres) &&
	     (syWstate.vsk->regul[C_edge.head_vertex].birth[2] >= syWstate.thres));
  L_alive = ((syWstate.vsk->regul[L_edge.tail_vertex].birth[2] >= syWstate.thres) &&
	     (syWstate.vsk->regul[L_edge.head_vertex].birth[2] >= syWstate.thres));

  if (!(C_alive ^ L_alive)) return;

  Cproj_edge = C_edge; swap (&Cproj_edge);
  Lproj_edge = L_edge; swap (&Lproj_edge);
  Cproj_edge.xoff = Lproj_edge.xoff = 2* syWstate.ipd->ipo[ref_point].x;
  Cproj_edge.yoff = Lproj_edge.yoff = 2* syWstate.ipd->ipo[ref_point].y;

  if (! Sy_SGroupConnected (C_edge.left_point, C_edge.right_point)){
    Sy_SGroupProcess (&C_edge, &Lproj_edge, ref_edge=c_index, ref_point);
  }
  else{
    ref_edge = c_index;
    syWstate.tri->regul[syWstate.tri->eref[ref_edge]].cont = SY_POS_INFTY;
    syWstate.tri->regul[syWstate.tri->eref[ref_edge]].narr = SY_POS_INFTY;
    syWstate.tri->regul[syWstate.tri->eref[ref_edge]].prox = SY_POS_INFTY;
  }

  if (! Sy_SGroupConnected (L_edge.left_point, L_edge.right_point)){
    Sy_SGroupProcess (&L_edge, &Cproj_edge, ref_edge=l_index, ref_point);
  }
  else{
    ref_edge = l_index;
    syWstate.tri->regul[syWstate.tri->eref[ref_edge]].cont = SY_POS_INFTY;
    syWstate.tri->regul[syWstate.tri->eref[ref_edge]].narr = SY_POS_INFTY;
    syWstate.tri->regul[syWstate.tri->eref[ref_edge]].prox = SY_POS_INFTY;
  }

  return;
}












