/*
 *  $Id$
 * 
 *  Author: Robert L. Ogniewicz, Swiss Federal Institute of Technology ETH
 *          Zurich, Switzerland (rlo@vision.ethz.ch)
 */

/* Copyright Notice
 * ================
 * Copyright (C) 1993 by Robert L. Ogniewicz
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
 */

#include <stdio.h>
#include <string.h>
#include "Sy_Idraw.h"

#define SY_IDRAW_CHARBUFLEN 256

static FILE  *sy_idraw_fileptr;

static char   sy_idraw_fgcolor[SY_IDRAW_CHARBUFLEN]="Black";
static char   sy_idraw_bgcolor[SY_IDRAW_CHARBUFLEN]="White";

static char   sy_idraw_creator[SY_IDRAW_CHARBUFLEN]="Unspecified";

static float  sy_idraw_intensity=0.0;
static char   sy_idraw_intensity_string[256]="none";

static float  sy_idraw_fg_red=0.0, sy_idraw_fg_green=0.0, sy_idraw_fg_blue=0.0,
              sy_idraw_bg_red=1.0, sy_idraw_bg_green=1.0, sy_idraw_bg_blue=1.0;

static float  sy_idraw_xscale = 1.0, sy_idraw_yscale = 1.0, 
              sy_idraw_xshift = 0.0, sy_idraw_yshift = 0.0;

static int    sy_idraw_linewidth = 1;

static SyIdrawBoundingBox sy_idraw_bbox = {0,0,SY_IDRAW_DIMX, SY_IDRAW_DIMY};

void Sy_IdrawPrologue (FILE *fileptr)
{
  sy_idraw_fileptr = fileptr;

  fprintf (sy_idraw_fileptr, sy_idraw_comments, 
	   sy_idraw_creator,
	   sy_idraw_bbox.x0, sy_idraw_bbox.y0,
	   sy_idraw_bbox.x1, sy_idraw_bbox.y1);

  fprintf (sy_idraw_fileptr, "%s", sy_idraw_prologue);
}

void Sy_IdrawSetFile (FILE *fileptr){ sy_idraw_fileptr = fileptr; }

void Sy_IdrawSetIntensity (float val)
{
  sy_idraw_intensity = val; 
  if (val < 0.0){
    strcpy(sy_idraw_intensity_string, "none");}
  else 
  if (val == 0.0){
    strcpy (sy_idraw_intensity_string, "0");}
  else{
    sprintf(sy_idraw_intensity_string, "%3.2f", val);}
}

void Sy_IdrawSetCurrentFGColor (char* color, 
				float red, float green, float blue)
{ 
  strncpy(sy_idraw_fgcolor, color, SY_IDRAW_CHARBUFLEN);
  sy_idraw_fgcolor[SY_IDRAW_CHARBUFLEN-1]='\0';
  sy_idraw_fg_red   = red;
  sy_idraw_fg_green = green;
  sy_idraw_fg_blue  = blue;
}

void Sy_IdrawSetCurrentBGColor (char* color, 
				float red, float green, float blue)
{ 
  strncpy(sy_idraw_bgcolor, color, SY_IDRAW_CHARBUFLEN);
  sy_idraw_bgcolor[SY_IDRAW_CHARBUFLEN-1]='\0';
  sy_idraw_bg_red   = red;
  sy_idraw_bg_green = green;
  sy_idraw_bg_blue  = blue;
}  

void Sy_IdrawSetLineWidth (int width)
{
  sy_idraw_linewidth = width;
}
  
void Sy_IdrawLine (int x0, int y0, int x1, int y1)
{
  fprintf (sy_idraw_fileptr, sy_idraw_line_template, 
           sy_idraw_linewidth,
	   sy_idraw_fgcolor,
	   sy_idraw_fg_red, sy_idraw_fg_green, sy_idraw_fg_blue, 
	   sy_idraw_bgcolor,
	   sy_idraw_bg_red, sy_idraw_bg_green, sy_idraw_bg_blue, 
	   sy_idraw_intensity,
	   sy_idraw_xscale, sy_idraw_yscale,
	   sy_idraw_xshift, sy_idraw_yshift,
	   x0, y0, x1, y1);
}

void Sy_IdrawRectangle (int x0, int y0, int x1, int y1)
{
  fprintf (sy_idraw_fileptr, sy_idraw_rectangle_template, 
	   sy_idraw_fgcolor,
	   sy_idraw_fg_red, sy_idraw_fg_green, sy_idraw_fg_blue, 
	   sy_idraw_bgcolor,
	   sy_idraw_bg_red, sy_idraw_bg_green, sy_idraw_bg_blue, 
	   sy_idraw_intensity,
	   sy_idraw_xscale, sy_idraw_yscale,
	   sy_idraw_xshift, sy_idraw_yshift,
	   x0, y0, x1, y1);
}

void Sy_IdrawEllipse (int xcenter, int ycenter, int xrad, int yrad)
{
  fprintf (sy_idraw_fileptr, sy_idraw_ellipse_template, 
	   sy_idraw_fgcolor,
	   sy_idraw_fg_red, sy_idraw_fg_green, sy_idraw_fg_blue, 
	   sy_idraw_bgcolor,
	   sy_idraw_bg_red, sy_idraw_bg_green, sy_idraw_bg_blue, 
	   sy_idraw_intensity,
	   sy_idraw_xscale, sy_idraw_yscale,
	   sy_idraw_xshift, sy_idraw_yshift,
	   xcenter, ycenter, xrad, yrad);
}
  
void Sy_IdrawMLine (SyIdrawPoint point[], int numpoints)
{
  int i;

  fprintf (sy_idraw_fileptr, sy_idraw_mline_template0, 
           sy_idraw_linewidth,
	   sy_idraw_fgcolor,
	   sy_idraw_fg_red, sy_idraw_fg_green, sy_idraw_fg_blue, 
	   sy_idraw_bgcolor,
	   sy_idraw_bg_red, sy_idraw_bg_green, sy_idraw_bg_blue, 
	   sy_idraw_intensity_string, 
	   sy_idraw_xscale, sy_idraw_yscale,
	   sy_idraw_xshift, sy_idraw_yshift, 
	   numpoints);

  for (i = 0; i < numpoints; i++){
    fprintf(sy_idraw_fileptr, "%d %d\n", point[i].x, point[i].y);
  }

  fprintf (sy_idraw_fileptr, sy_idraw_mline_template1, numpoints);
}
  
void Sy_IdrawBSpline (SyIdrawPoint point[], int numpoints)
{
  int i;

  fprintf (sy_idraw_fileptr, sy_idraw_bspline_template0, 
	   sy_idraw_fgcolor,
	   sy_idraw_fg_red, sy_idraw_fg_green, sy_idraw_fg_blue, 
	   sy_idraw_bgcolor,
	   sy_idraw_bg_red, sy_idraw_bg_green, sy_idraw_bg_blue, 
	   sy_idraw_intensity,
	   sy_idraw_xscale, sy_idraw_yscale,
	   sy_idraw_xshift, sy_idraw_yshift,	   
	   numpoints);

  for (i = 0; i < numpoints; i++){
    fprintf(sy_idraw_fileptr, "%d %d\n", point[i].x, point[i].y);
  }

  fprintf (sy_idraw_fileptr, sy_idraw_bspline_template1, numpoints);
}

void Sy_IdrawEpilogue ()
{
  fprintf (sy_idraw_fileptr, "%s", sy_idraw_epilogue);
}

void Sy_IdrawBeginGroup()
{ fprintf(sy_idraw_fileptr, "%s", sy_idraw_begingroup); }

void Sy_IdrawEndGroup()
{ fprintf(sy_idraw_fileptr, "%s", sy_idraw_endgroup); }

void Sy_IdrawSetDefaults()
{
strcpy(sy_idraw_fgcolor,"Black");
strcpy(sy_idraw_bgcolor,"White");
sy_idraw_intensity=0.0;
sy_idraw_fg_red=0.0; sy_idraw_fg_green=0.0; sy_idraw_fg_blue=0.0;
sy_idraw_bg_red=1.0; sy_idraw_bg_green=1.0; sy_idraw_bg_blue=1.0;
}

void Sy_IdrawSetBoundingBox(int x0, int y0, int x1, int y1)
{
  sy_idraw_bbox.x0 = x0; sy_idraw_bbox.y0 = y0;
  sy_idraw_bbox.x1 = x1; sy_idraw_bbox.y1 = y1;
}

int Sy_IdrawIsInBoundingBox(int x, int y)
{
  return ( (x > sy_idraw_bbox.x0) && (x < sy_idraw_bbox.x1) &&
	   (y > sy_idraw_bbox.y0) && (y < sy_idraw_bbox.y1) );
}

void Sy_IdrawSetCreator(char* creator)
{
  strncpy(sy_idraw_creator, creator, SY_IDRAW_CHARBUFLEN);
  sy_idraw_creator[SY_IDRAW_CHARBUFLEN-1]='\0';
}

void Sy_IdrawSetXYScale(float xscale, float yscale)
{
  sy_idraw_xscale = xscale; sy_idraw_yscale = yscale;
}

void Sy_IdrawSetXYTranslate(float xshift, float yshift)
{
  sy_idraw_xshift = xshift; sy_idraw_yshift = yshift;
}


static SyIdrawPoint point[]={{100,100},{200,150},{150,50},{100,100}};
static SyIdrawPoint point2[]={{300,100},{400,150},{350,50},{300,100}};

#ifdef STANDALONE_DEMO 
main()
{
   Sy_IdrawSetCreator("Robert Ogniewicz");
   Sy_IdrawSetBoundingBox(100,100,500,500);

   Sy_IdrawPrologue(stdout);

   Sy_IdrawSetXYScale(0.5,0.5);
   Sy_IdrawSetXYTranslate(200,200);

   Sy_IdrawBeginGroup();
   Sy_IdrawSetCurrentFGColor("Red", 1,0,0);
   Sy_IdrawLine(0,0,100,100);
   Sy_IdrawLine(20,0,300,700);
   Sy_IdrawEndGroup();
   Sy_IdrawSetDefaults();
   Sy_IdrawBeginGroup();
   Sy_IdrawSetIntensity(0.75);
   Sy_IdrawLine(0,0,150,100);
   Sy_IdrawLine(20,0,350,700);
   Sy_IdrawEndGroup();
   Sy_IdrawSetDefaults();
   Sy_IdrawSetCurrentFGColor("Green", 0,1,0);
   Sy_IdrawEllipse(400,500,40,20);
   Sy_IdrawRectangle(700,700,800,800);
   Sy_IdrawSetCurrentFGColor("Blue", 0,0,1);
   Sy_IdrawMLine(point, 4);
   Sy_IdrawBSpline(point2, 4);
   Sy_IdrawEpilogue();
}
#endif



