%{

/*
 *  $Id$
 * 
 *  Author: Robert L. Ogniewicz, Division of Applied Sciences,
 *          Harvard University, Cambridge, Massachusetts (rlo@hrl.harvard.edu)
 */

/* Copyright Notice
 * ================
 * Copyright (C) 1995 by Robert L. Ogniewicz
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
 */

 /* lex lexical rules for I/O routines */

#include "Sy_Lang.h"

/* #define DEBUG 1 */

#ifdef DEBUG
#define lexprintf(str) printf ("%s: %s\n", str, yytext) 
#else   
#define lexprintf(str) 
#endif
#define lexreturn(token) if (!is_comment) return(token)
int is_comment = 0, is_comment_par = 0;


void lexdata_f() {
  printf ("data: %s\n", yytext); 
}

#ifdef DEBUG
#define lexdata() lexdata_f()
#else
#define lexdata() 
#endif

%}

DIGIT	 [0-9]
INTEGER  [-+]?{DIGIT}+
EXP      [DEde][-+]?{DIGIT}+
REAL     ({INTEGER}"."{DIGIT}*({EXP})?|"."{DIGIT}+({EXP})?|{INTEGER}{EXP})
HEXDIGIT [0-9A-Fa-f]
HEX      "0x"?{HEXDIGIT}+
IDENT    [A-Za-z][A-Za-z0-9_]*
ASSIGN   "="
ATERM    ";"
STOP     "end."
NEWLINE  "\n"
COMMENTSKIP  "//"
COMMENTBEGIN "/*"
COMMENTEND   "*/"
BEGINLOADSEQ "beginloadseq"
ENDLOADSEQ   "endloadseq"[ \n]*";"
STRING   \"[^"]*\"
SYMBOL  [^0-9A-Za-z \n\t]
%%
[ \t]+          { if (is_data) lexdata(); }

{BEGINLOADSEQ}  { if (is_data) lexdata(); else {
                  lexprintf ("begin load"); strcpy(tmp_str, yytext); 
		  yylval.ival = BEGINLOADSEQ; lexreturn (BEGINLOADSEQ); }}

{ENDLOADSEQ}    { lexprintf ("end load"); is_data = 0; }

{INTEGER}	{ if (is_data) lexdata(); else {
                  lexprintf ("int"); sscanf(yytext, "%d", &tmp_int); 
		  yylval.ival = tmp_int; lexreturn(INTEGER); } }

{REAL}    	{ if (is_data) lexdata(); else {
                  lexprintf ("real"); sscanf(yytext, "%f", &tmp_float); 
		  yylval.fval = tmp_float; lexreturn (REAL); } }

object          { if (is_data) lexdata(); else {
                  lexprintf ("key");  strcpy(tmp_str, yytext);
		  yylval.ival = OBJECT; lexreturn (OBJECT); } }

memory          { if (is_data) lexdata(); else {
                  yylval.ival = MEMORY; lexreturn (MEMORY); } }
shmem           { if (is_data) lexdata(); else {
                  yylval.ival = SHMEM; lexreturn (SHMEM); } }
file            { if (is_data) lexdata(); else {
                  yylval.ival = FILEIO; lexreturn (FILEIO); } }

{IDENT}         { if (is_data) lexdata(); else {
                  lexprintf ("ident"); strcpy(tmp_str, yytext);
		  strcpy(yylval.strval, yytext); lexreturn (IDENT); } }

{HEX}           { if (is_data) lexdata(); else { 
                  lexprintf ("hex"); sscanf(yytext, "%i", &tmp_int);
		  yylval.ival = tmp_int; lexreturn (HEX); } }

{ASSIGN}        { if (is_data) lexdata(); else {
                  lexprintf ("assign"); strcpy(tmp_str, yytext); 
		  yylval.ival = ASSIGN; lexreturn (ASSIGN); } }

{ATERM}         { if (is_data) lexdata(); else {
                  lexprintf ("term"); strcpy(tmp_str, yytext); 
		  yylval.ival = ATERM; lexreturn (ATERM); } }

{STOP}          { if (is_data) lexdata(); else {
                  lexprintf ("stop"); strcpy(tmp_str, yytext); 
		  yylval.ival = STOP; lexreturn (STOP); } }

{COMMENTSKIP}   { if (!is_data) is_comment = 1; }
{NEWLINE}       { if (!is_comment_par && !is_data) is_comment = 0; }
{COMMENTBEGIN}  { if (!is_data) {is_comment_par = 1; is_comment = 1; } }
{COMMENTEND}    { if (!is_data) {is_comment_par = 0; is_comment = 0; } }

{STRING}        { if (is_data) lexdata(); else {
                  lexprintf ("string"); strcpy(tmp_str, yytext);
		  strcpy(yylval.strval, yytext); lexreturn (STRING); } }

{SYMBOL}        { if (is_data) lexdata(); else { 
                  lexprintf ("symbol"); strcpy(tmp_str, yytext);
		  yylval.chval = yytext[0]; lexreturn (yytext[0]); } }

