/*
 *  $Id$
 * 
 *  Author: Robert L. Ogniewicz, Swiss Federal Institute of Technology ETH
 *          Zurich, Switzerland (rlo@vision.ethz.ch)
 */

/* Copyright Notice
 * ================
 * Copyright (C) 1993 by Robert L. Ogniewicz
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
 */

/*
 * 
 * All-Purpose Ipo Traverse
 *
 */

static char sy_IpoTraverseId[] = "$Id$"; 

#include "Sy_Skeleton.h"
#include "Sy_Intrinsic.h"

static int verbose_local;

static SyIpoWalkBuff buff_0 = {0,0,0,0};

SyIpoWalkState wi;
static SyWalktask* task;


int Sy_AllocIpoLabel (SyIpoDAC *ipd, int nr_elem)
{
        int    page, plabel_mem;

        page          = getpagesize();
        plabel_mem    = ((++nr_elem * sizeof(SY_I_LABEL))/page+1)*page;
        ipd->label.p_ = (SY_I_LABEL*) Sy_Malloc ((unsigned)plabel_mem);
        (void) memset ((char*) ipd->label.p_, 0, plabel_mem);
        return (plabel_mem);
}

int Sy_AllocIcnLabel (SyIpoDAC *ipd, int nr_elem)
{
        int    page, ilabel_mem;

        page          = getpagesize();
        ilabel_mem    = ((++nr_elem * sizeof(SY_C_LABEL))/page+1)*page;
        ipd->label.c_ = (SY_C_LABEL*) Sy_Malloc ((unsigned)ilabel_mem);
        (void) memset ((char*) ipd->label.c_, 0, ilabel_mem);
        return (ilabel_mem);
}

void Sy_InitIpoTraverse (SyIpoDAC *ipd, SyWalktask *wt)
{
  task          = wt;
  verbose_local = wi.verbose;
  wi.ipd        = ipd;
}

static void Sy_IpoTrace (SyIpoDAC *ipd, int start_ipo, 
			 int start_icn, int nr);

void Sy_IpoTraverse (SyIpoDAC *ipd, SyWalktask *wt)
{
  int  Sy_AllocIcnLabel (SyIpoDAC *ipd, int nr_elem);

  int i, obj_count = 0;

#ifdef DEBUG
  sprintf(syStrBuf, "IPO TRAVERSE)  %s  ", sy_IpoTraverseId);
  Sy_Message (SY_FMTNL, syStrBuf);
#endif

  /* INIT call */
  if (task->user_function[(int)INIT] != NULL) 
    task->user_function[(int)INIT]();

  ipd->label.c_ = NULL;
  ipd->label.p_ = NULL;

  (void) Sy_AllocIcnLabel (ipd, ipd->ipodim);
  
  for (i = 1; i <= ipd->ipodim; i++){
    if (ipd->label.c_[i]) continue;    
    if (ipd->ipl[i].tail_icn == 0){
      wi.start.head_ipo = i;
      wi.start.this_icn = ipd->ipl[i].tail_icn;
      wi.start.next_icn = ipd->ipl[i].head_icn;
      wi.start.pos      = ipd->ipo[i];
      wi.current        = wi.start;
      wi.contour_id     = obj_count+1;

      /* TRAVerse call */
      if (task->user_function[(int)TRAV] != NULL) 
	task->user_function[(int)TRAV]();

      Sy_IpoTrace (ipd, i, ipd->ipl[i].head_icn, ++obj_count);

      /* TERMination call */
      if (task->user_function[(int)TERM] != NULL) 
	task->user_function[(int)TERM]();
    }
    else
    if (ipd->ipl[i].head_icn == 0){
      wi.start.head_ipo = i;
      wi.start.this_icn = ipd->ipl[i].head_icn;
      wi.start.next_icn = ipd->ipl[i].tail_icn;
      wi.start.pos      = ipd->ipo[i];
      wi.current        = wi.start;
      wi.contour_id     = obj_count+1;

      /* TRAVerse call */
      if (task->user_function[(int)TRAV] != NULL) 
	task->user_function[(int)TRAV]();

      Sy_IpoTrace (ipd, i, ipd->ipl[i].tail_icn, ++obj_count);

      /* TERMination call */
      if (task->user_function[(int)TERM] != NULL) 
	task->user_function[(int)TERM]();
    }    
  }

  for (i = 1; i <= ipd->ipodim; i++){
    if (ipd->label.c_[i]) continue;
    wi.start.head_ipo = i;
    wi.start.this_icn = ipd->ipl[i].tail_icn;
    wi.start.next_icn = ipd->ipl[i].head_icn;    
    wi.start.pos      = ipd->ipo[i];
    wi.current        = wi.start;
    wi.contour_id     = obj_count+1;

    /* TRAVerse call */
    if (task->user_function[(int)TRAV] != NULL) 
      task->user_function[(int)TRAV]();

    Sy_IpoTrace (ipd, i, ipd->ipl[i].head_icn, ++obj_count);

    /* TERMination call */
    if (task->user_function[(int)TERM] != NULL) 
      task->user_function[(int)TERM]();
  }

  if (task->user_function[(int)FEXIT] != NULL) 
    task->user_function[(int)FEXIT]();

  if (ipd->label.c_){
    free(ipd->label.c_); ipd->label.c_ = NULL; 
  }
  if (ipd->label.p_){ 
    free(ipd->label.p_); ipd->label.p_ = NULL;
  }
}

static void Sy_IpoTrace (SyIpoDAC *ipd, int start_ipo, int start_icn, int nr)
{
  int    cur_ipo,  cur_icn; 
  int    next_ipo, next_icn;
  SyIpo    cur_pos,  next_pos;

  if (start_icn == 0) return;

  cur_ipo = start_ipo;
  cur_icn = start_icn;
  wi.current.pos = cur_pos = ipd->ipo[cur_ipo];
  wi.contour_id  = nr;

  do{
    ipd->label.c_[cur_ipo]= nr;

    if (ipd->icn[cur_icn].tail_ipo == cur_ipo){ 
      next_ipo = ipd->icn[cur_icn].head_ipo;
    }
    else{
      next_ipo = ipd->icn[cur_icn].tail_ipo;
    }

    if (ipd->ipl[next_ipo].tail_icn == cur_icn){
      next_icn = ipd->ipl[next_ipo].head_icn;
    }
    else{
      next_icn = ipd->ipl[next_ipo].tail_icn;
    }
    next_pos = ipd->ipo[next_ipo];

    /* WALK call */
    if (task->user_function[(int)WALK] != NULL) 
      task->user_function[(int)WALK]();
    wi.last = wi.current;

    wi.current.head_ipo = next_ipo;
    wi.current.this_icn = wi.last.next_icn;
    wi.current.next_icn = next_icn;
    wi.current.pos      = next_pos;    

    cur_ipo = next_ipo;
    cur_icn = next_icn;
    cur_pos = next_pos;

  } while ((ipd->label.c_[cur_ipo] == 0) && (next_ipo != 0));

  wi.closed_contour = (next_ipo != 0);
}












