/*
 *  $Id$
 * 
 *  Author: Robert L. Ogniewicz, Swiss Federal Institute of Technology ETH
 *          Zurich, Switzerland (rlo@vision.ethz.ch)
 */

/* Copyright Notice
 * ================
 * Copyright (C) 1993 by Robert L. Ogniewicz
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
 */

/*
 *
 * Lisp Interface
 *
 */ 

static char sy_LispFactsId[] = "$Id$"; 

#include "Sy_Skeleton.h"
#include "Sy_Intrinsic.h"

extern SyWalktask wt;

void Sy_SimplePrintWalk (void)
{
  if (syIsVerbose) fprintf (stderr,"\n");

  fprintf(stderr, "vertex:%5d  edge:%5d  radius: %4.2f\n",
          syWstate.current.head_vertex, syWstate.current.next_edge,
          syWstate.vsk->regul[syWstate.current.head_vertex].radius);
}

static int right_brackets[2] = {1,1};

void Sy_LispFactsInit(void)
{
  fprintf(wt.user_file[0],"(\n");
  fprintf(wt.user_file[1],"(\n");
}

void Sy_LispFactsTraverse(void)
{
 /* if (syWstate.foreground) fprintf (wt.user_file[0],"(1 %d", syWstate.id);
  else               fprintf (wt.user_file[1],"(0 %d", syWstate.id); */
}

void Sy_LispFactsWalk(void)
{
  SyVertex vtmp, vtmp_last;
  SyRegul  rtmp;
  static count = 0;
  int index;  

  index = syWstate.foreground;
  vtmp = syWstate.vsk->vertex[syWstate.current.head_vertex];
  rtmp = syWstate.vsk->regul [syWstate.current.head_vertex];

  if (syWstate.current.head_vertex == syWstate.start.head_vertex) return;

  if (syWstate.vsk->label.v_[syWstate.current.head_vertex] != 0){
    fprintf (wt.user_file[index],"(%4.0f %4.0f %5.1f)\n)\n",
	     vtmp.xden/vtmp.num,
             vtmp.yden/vtmp.num, 
	     rtmp.radius); 
    count = 0; right_brackets[index] = 1;
    return;
  }    

  if (syWstate.last.head_vertex == syWstate.start.head_vertex){
    if (!right_brackets[index]) 
      fprintf (wt.user_file[index], ")\n");

    vtmp_last = syWstate.vsk->vertex[syWstate.last.head_vertex];

    fprintf (wt.user_file[index],"(%d %d \n(%4.0f %4.0f %5.1f)",
	     index, 
	     syWstate.id,
	     vtmp_last.xden/vtmp_last.num,
             vtmp_last.yden/vtmp_last.num, 
	     syWstate.vsk->regul[syWstate.last.head_vertex].radius); 
    count = 1; right_brackets[index] = 0;
  }
  else if (syWstate.vsk->regul[syWstate.last.head_vertex].birth[0] > syWstate.thres){
    if (!right_brackets[index]) 
      fprintf (wt.user_file[index], ")\n");

    vtmp_last = syWstate.vsk->vertex[syWstate.last.head_vertex];

    fprintf (wt.user_file[index],"(%d %d \n(%4.0f %4.0f %5.1f)",
	     index, 
	     syWstate.id,
	     vtmp_last.xden/vtmp_last.num,
             vtmp_last.yden/vtmp_last.num, 
	     syWstate.vsk->regul[syWstate.last.head_vertex].radius); 
    count = 1; right_brackets[index] = 0;
  }

  if (rtmp.birth[0] > syWstate.thres){
    fprintf (wt.user_file[index],"(%4.0f %4.0f %5.1f))\n",
	     vtmp.xden/vtmp.num,
             vtmp.yden/vtmp.num, 
	     rtmp.radius); 
    count = 0; right_brackets[index] = 1;
  }
  else if ((rtmp.birth[2] > syWstate.thres) && (rtmp.birth[1] < syWstate.thres)){
    fprintf (wt.user_file[index],"(%4.0f %4.0f %5.1f))\n",
	     vtmp.xden/vtmp.num,
             vtmp.yden/vtmp.num, 
	     rtmp.radius); 
    count = 0; right_brackets[index] = 1;
  }
  else{
    fprintf (wt.user_file[index],"(%4.0f %4.0f %5.1f)",
	     vtmp.xden/vtmp.num,
             vtmp.yden/vtmp.num, 
	     rtmp.radius); 
    count++; right_brackets[index] = 0;
    if (count >= 4) {fprintf (wt.user_file[index], "\n"); count = 0;}

  }
}

void Sy_LispFactsExit(void)
{
  if (!right_brackets[0]) fprintf (wt.user_file[0], ")\n");
  if (!right_brackets[1]) fprintf (wt.user_file[1], ")\n");

  fprintf(wt.user_file[0],")");
  fprintf(wt.user_file[1],")");

  right_brackets[0] = right_brackets[1] = 1;
}

