/*  Functions from hack's utils library.
    Copyright (C) 1989, 1990, 1991 Free Software Foundation, Inc.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

/* These routines were written as part of a library (by hack), but since most
   people don't have the library, here they are.  */

#ifndef HAVE_VPRINTF
#   if defined(sparc)
#      define HAVE_VPRINTF 1
#   endif
#endif

#ifdef __STDC__
#define SY_VOID void
#else
#define SY_VOID char
#endif

#include <stdio.h>
#if defined(HAVE_STRING_H) || defined(STDC_HEADERS) || defined(SYSV)
#include <string.h>
#else
#include <strings.h>
#endif
#if defined(STDC_HEADERS)
#include <stdlib.h>
#else
#ifdef RX_MEMDBUG
#include <sys/types.h>

#if defined (bsdi) || defined (__bsdi__)
#include <sys/malloc.h>
#else
#include <malloc.h>
#endif

#else
SY_VOID *malloc();
SY_VOID *realloc();
#endif /* ndef RX_MEMDBUG  */
#endif

SY_VOID *ck_malloc();

char *myname;

#ifdef __STDC__
#include <stdarg.h>

/* Print an error message and exit */
void panic(char *str, ...)
{
	va_list iggy;
	fprintf(stderr,"%s: ",myname);
	va_start(iggy,str);
#ifdef HAVE_VPRINTF
	vfprintf(stderr,str,iggy);
#else
#ifdef HAVE_DOPRNT
	_doprnt(str,&iggy,stderr);
#endif
#endif
	va_end(iggy);
	putc('\n',stderr);
	exit(4);
}

void ck_fprintf(FILE *stream, char *str, ...)
{
	va_list iggy;
	va_start(iggy,str);
#ifdef HAVE_VPRINTF
	if (vfprintf(stream,str,iggy) < 0)
	  panic("Couldn't write value");
#else
#ifdef HAVE_DOPRNT
	_doprnt(str,&iggy,stream);
#endif
#endif
	va_end(iggy);
}

#else
#include <varargs.h>

void panic(str,va_alist)
char *str;
va_dcl
{
	va_list iggy;

	fprintf(stderr,"%s: ",myname);
	va_start(iggy);
#ifdef HAVE_VPRINTF
	if (vfprintf(stream,str,iggy) < 0)
	  panic("Couldn't write value");
#else
#ifdef HAVE_DOPRNT
	_doprnt(str,&iggy,stderr);
#endif
#endif
	va_end(iggy);
	putc('\n',stderr);
	exit(4);
}

void ck_fprintf(stream, str, va_alist)
FILE *stream;
char *str;
va_dcl
{
	va_list iggy;
	va_start(iggy);
#ifdef HAVE_VPRINTF
	vfprintf(stream,str,iggy);
#else
#ifdef HAVE_DOPRNT
	_doprnt(str,&iggy,stream);
#endif
#endif
	va_end(iggy);
}

#endif

/* Store information about files opened with ck_fopen
   so that error messages from ck_fread, etc can print the
   name of the file that had the error */
#define SY_N_FILE 32

struct id {
	FILE *fp;
	char *name;
};

static struct id __id_s[SY_N_FILE];

/* Internal routine to get a filename from __id_s */
char *
__fp_name(fp)
FILE *fp;
{
	int n;

	for(n=0;n<SY_N_FILE;n++) {
		if(__id_s[n].fp==fp)
			return __id_s[n].name;
	}
	return "{Unknown file pointer}";
}

/* Panic on failing fopen */
FILE *
ck_fopen(name,mode)
char *name;
char *mode;
{
	FILE	*ret;
	int	n;

	ret=fopen(name,mode);
	if(ret==(FILE *)0)
		panic("Couldn't open file %s",name);
	for(n=0;n<SY_N_FILE;n++) {
		if(ret==__id_s[n].fp) {
			free((SY_VOID *)__id_s[n].name);
			__id_s[n].name=(char *)ck_malloc(strlen(name)+1);
			strcpy(__id_s[n].name,name);
			break;
		}
	}
	if(n==SY_N_FILE) {
		for(n=0;n<SY_N_FILE;n++)
			if(__id_s[n].fp==(FILE *)0)
				break;
		if(n==SY_N_FILE)
			panic("Internal error: too many files open");
		__id_s[n].fp=ret;
		__id_s[n].name=(char *)ck_malloc(strlen(name)+1);
		strcpy(__id_s[n].name,name);
	}
	return ret;
}

/* Panic on failing fwrite */
void
ck_fwrite(ptr,size,nmemb,stream)
char *ptr;
int size,nmemb;
FILE *stream;
{
	if(fwrite(ptr,size,nmemb,stream)!=nmemb)
		panic("couldn't write %d items to %s",nmemb,__fp_name(stream));
}

/* Panic on failing fclose */
void
ck_fclose(stream)
FILE *stream;
{
	if(fclose(stream)==EOF)
		panic("Couldn't close %s",__fp_name(stream));
}

/* Panic on failing malloc */
SY_VOID *
ck_malloc(size)
int size;
{
	SY_VOID *ret;

	if(!size)
		size++;
	ret=malloc(size);
	if(ret==(SY_VOID *)0)
		panic("Couldn't allocate memory");
	return ret;
}

/* Panic on failing malloc */
SY_VOID *
xmalloc(size)
int size;
{
  return ck_malloc (size);
}

/* Panic on failing realloc */
SY_VOID *
ck_realloc(ptr,size)
SY_VOID *ptr;
int size;
{
	SY_VOID *ret;

	if (!ptr)
	  return ck_malloc (size);
	ret=realloc(ptr,size);
	if(ret==(SY_VOID *)0)
		panic("Couldn't re-allocate memory");
	return ret;
}

/* Return a malloc()'d copy of a string */
char *
ck_strdup(str)
char *str;
{
	char *ret;

	ret=(char *)ck_malloc(strlen(str)+2);
	strcpy(ret,str);
	return ret;
}


/* Implement a variable sized buffer of 'stuff'.  We don't know what it is,
   nor do we care, as long as it doesn't mind being aligned by malloc. */

struct buffer {
	int	allocated;
	int	length;
	char	*b;
};

#define SY_MIN_ALLOCATE 50

SY_VOID *
init_buffer()
{
	struct buffer *b;

	b=(struct buffer *)ck_malloc(sizeof(struct buffer));
	b->allocated=SY_MIN_ALLOCATE;
	b->b=(char *)ck_malloc(SY_MIN_ALLOCATE);
	b->length=0;
	return (SY_VOID *)b;
}

void
flush_buffer(bb)
SY_VOID *bb;
{
	struct buffer *b;

	b=(struct buffer *)bb;
	free(b->b);
	b->b=0;
	b->allocated=0;
	b->length=0;
	free(b);
}

int
size_buffer(b)
SY_VOID *b;
{
	struct buffer *bb;

	bb=(struct buffer *)b;
	return bb->length;
}

void
add_buffer(bb,p,n)
SY_VOID *bb;
char *p;
int n;
{
	struct buffer *b;
	int x;
	char * cp;

	b=(struct buffer *)bb;
	if(b->length+n>b->allocated) {
		b->allocated = (b->length + n) * 2;
		b->b=(char *)ck_realloc(b->b,b->allocated);
	}
	
	x = n;
	cp = b->b + b->length;
	while (x--)
	  *cp++ = *p++;
	b->length+=n;
}

void
add1_buffer(bb,ch)
SY_VOID *bb;
int ch;
{
	struct buffer *b;

	b=(struct buffer *)bb;
	if(b->length+1>b->allocated) {
		b->allocated*=2;
		b->b=(char *)ck_realloc(b->b,b->allocated);
	}
	b->b[b->length]=ch;
	b->length++;
}

char *
get_buffer(bb)
SY_VOID *bb;
{
	struct buffer *b;

	b=(struct buffer *)bb;
	return b->b;
}
