
/*
 *  $Id$
 * 
 *  Author: Robert L. Ogniewicz, Swiss Federal Institute of Technology ETH
 *          Zurich, Switzerland (rlo@vision.ethz.ch)
 */

/* Copyright Notice
 * ================
 * Copyright (C) 1993 by Robert L. Ogniewicz
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
 */

static char sy_LweCorr[] = "$Id$"; 

#include "Sy_Skeleton.h"
#include "Sy_Intrinsic.h"
#include "LweTable.h"

int Sy_BilinComputeRadius(float si, float ddx, float *rad)
{

float t, u, r1, r2, r3, r4;
int loc_sigma, loc_ddx, error_flag = 0;

if (SY_ALV(ddx) >= 1.0/(SY_SQRTE*si) ){
  error_flag = 1; ddx = 1.0/(SY_SQRTE*si)-(syLweDeriv2Res/1000.0);}

if (si < syLweSigmaLo || si >= syLweSigmaHi) return(0);

/* if (SY_ALV(ddx) < 0.00001) return(0); */
if (SY_ALV(ddx) < syLweDeriv2Lo){*rad = 1.0/(SY_ALV(ddx)); return(1);}


loc_sigma = floor((si - syLweSigmaLo)/syLweSigmaRes);
loc_ddx  = floor((SY_ALV(ddx) - syLweDeriv2Lo)/syLweDeriv2Res);
#ifdef DEBUG
printf ("loc_sigma = %d, loc_ddx = %d\n", loc_sigma, loc_ddx);
#endif

r1 = syLweTable[loc_sigma][loc_ddx].rad;
r2 = syLweTable[loc_sigma][loc_ddx+1].rad;
r3 = syLweTable[loc_sigma+1][loc_ddx+1].rad;
r4 = syLweTable[loc_sigma+1][loc_ddx].rad;

t = (SY_ALV(ddx) -(syLweDeriv2Lo + loc_ddx*syLweDeriv2Res))/syLweDeriv2Res;
u = (si -(syLweSigmaLo + loc_sigma*syLweSigmaRes))/syLweSigmaRes;

*rad = (1-u)* ( (1-t)*r1 + t*r2 ) + u* ( (1-t) * r4 + t*r3);

return (1);
}


int Sy_BilinComputeShrinkageCorr(float si, float ddx, float *corr)
{
  float t, u, c1, c2, c3, c4, tmp_ddx;
  int loc_sigma, loc_ddx, error_flag = 0;

  if (SY_ALV(ddx) >= 1.0/(SY_SQRTE*si) ){
    error_flag = 1; return(0);
  }
 
  tmp_ddx = 1.0/(SY_SQRTE*si)-(syLweDeriv2Res/1000.0);
  if (ddx < 0.0) ddx = - tmp_ddx; else ddx = tmp_ddx;
  
  if (si < syLweSigmaLo || si >= syLweSigmaHi) return(0);
  
  /* if (SY_ALV(ddx) < 0.00001) return(0); */
  if (SY_ALV(ddx) < syLweDeriv2Lo){
    *corr = (1.0-exp(-0.5*si*si*ddx*ddx))/SY_ALV(ddx);
    if (ddx > 0.0) *corr = - *corr; 
    return(1);
  }
  loc_sigma = floor((si - syLweSigmaLo)/syLweSigmaRes);
  loc_ddx  = floor((SY_ALV(ddx) - syLweDeriv2Lo)/syLweDeriv2Res);
#ifdef DEBUG
  printf ("loc_sigma = %d, loc_ddx = %d\n", loc_sigma, loc_ddx);
#endif
  
  c1 = syLweTable[loc_sigma][loc_ddx].corr;
  c2 = syLweTable[loc_sigma][loc_ddx+1].corr;
  c3 = syLweTable[loc_sigma+1][loc_ddx+1].corr;
  c4 = syLweTable[loc_sigma+1][loc_ddx].corr;
  
  t = (SY_ALV(ddx) -(syLweDeriv2Lo + loc_ddx*syLweDeriv2Res))/syLweDeriv2Res;
  u = (si -(syLweSigmaLo + loc_sigma*syLweSigmaRes))/syLweSigmaRes;
  
  *corr = (1-u)* ( (1-t)*c1 + t*c2 ) + u* ( (1-t) * c4 + t*c3);
  if (ddx > 0.0) *corr = - *corr;
  
  return (1);
}
