/*
 *  $Id$
 * 
 *  Author: Robert L. Ogniewicz, Swiss Federal Institute of Technology ETH
 *          Zurich, Switzerland (rlo@vision.ethz.ch)
 */

/* Copyright Notice
 * ================
 * Copyright (C) 1993 by Robert L. Ogniewicz
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
 */

static char sy_SmoothPotiId[] = "$Id$"; 

#include "Sy_Skeleton.h"
#include "Sy_Intrinsic.h"
#include <math.h>

/* from numerical recipes */
#ifdef SY_SWAP
#undef SY_SWAP
#endif

#define SY_SWAP(a,b) tempr=(a);(a)=(b);(b)=tempr
     
void four1(float data[], int nn, int isign)
{
  int n,mmax,m,j,istep,i;
  double wtemp,wr,wpr,wpi,wi,theta;
  float tempr,tempi;
  
  n=nn << 1;
  j=1;
  for (i=1;i<n;i+=2)
     {
       if (j > i)
	  {
	    SY_SWAP(data[j],data[i]);
	    SY_SWAP(data[j+1],data[i+1]);
	  }
       m=n >> 1;
       while (m >= 2 && j > m)
	  {
	    j -= m;
	    m >>= 1;
	  }
       j += m;
     }
  mmax=2;
  while (n > mmax)
     {
       istep=2*mmax;
       theta=6.28318530717959/(isign*mmax);
       wtemp=sin(0.5*theta);
       wpr = -2.0*wtemp*wtemp;
       wpi=sin(theta);
       wr=1.0;
       wi=0.0;
       for (m=1;m<mmax;m+=2)
	  {
	    for (i=m;i<=n;i+=istep)
	       {
		 j=i+mmax;
		 tempr=wr*data[j]-wi*data[j+1];
		 tempi=wr*data[j+1]+wi*data[j];
		 data[j]=data[i]-tempr;
		 data[j+1]=data[i+1]-tempi;
		 data[i] += tempr;
		 data[i+1] += tempi;
	       }
	    wr=(wtemp=wr)*wpr-wi*wpi+wr;
	    wi=wi*wpr+wtemp*wpi+wi;
	  }
       mmax=istep;
     }
}

#undef SY_SWAP

void realft(float data[], int n, int isign)
{
  int i,i1,i2,i3,i4,n2p3;
  float c1=0.5,c2,h1r,h1i,h2r,h2i;
  double wr,wi,wpr,wpi,wtemp,theta;
  
  theta=3.141592653589793/(double) (n>>1);
  if (isign == 1)
     {
       c2 = -0.5;
       four1(data,n>>1,1);
     } else
	{
	  c2=0.5;
	  theta = -theta;
	}
  wtemp=sin(0.5*theta);
  wpr = -2.0*wtemp*wtemp;
  wpi=sin(theta);
  wr=1.0+wpr;
  wi=wpi;
  n2p3=n+3;
  for (i=2;i<=(n>>2);i++)
     {
       i4=1+(i3=n2p3-(i2=1+(i1=i+i-1)));
       h1r=c1*(data[i1]+data[i3]);
       h1i=c1*(data[i2]-data[i4]);
       h2r = -c2*(data[i2]+data[i4]);
       h2i=c2*(data[i1]-data[i3]);
       data[i1]=h1r+wr*h2r-wi*h2i;
       data[i2]=h1i+wr*h2i+wi*h2r;
       data[i3]=h1r-wr*h2r+wi*h2i;
       data[i4] = -h1i+wr*h2i+wi*h2r;
       wr=(wtemp=wr)*wpr-wi*wpi+wr;
       wi=wi*wpr+wtemp*wpi+wi;
     }
  if (isign == 1)
     {
       data[1] = (h1r=data[1])+data[2];
       data[2] = h1r-data[2];
     } else
	{
	  data[1]=c1*((h1r=data[1])+data[2]);
	  data[2]=c1*(h1r-data[2]);
	  four1(data,n>>1,-1);
	}
}

