/*
 *  $Id$
 * 
 *  Author: Robert L. Ogniewicz, Swiss Federal Institute of Technology ETH
 *          Zurich, Switzerland (rlo@vision.ethz.ch)
 */

/* Copyright Notice
 * ================
 * Copyright (C) 1993 by Robert L. Ogniewicz
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
 */

static char sy_RandomDotsId[] = "$Id$"; 

#include "Sy_Skeleton.h"
#include "Sy_Intrinsic.h"
#include <sys/types.h>

#if defined(SYSV) || defined(SVR4) || defined(sgi)
#include <math.h>
#include <sys/time.h>
#else
#include <math.h>
#include <sys/timeb.h> 
#endif

#if ! defined (linux)
char  *initstate(unsigned int, char *, int), *setstate(char *);
#endif

static long randgen_state[32]  =  
      {0x03101962,  0xafa1221f,  0x1ffd8c24,  0x9b563182,  
       0xada1f342,  0x7449e56b,  0x8979620b,  0x27cb47b9,
       0x1cab3b91,  0xab5c5918,  0x42ef11ff,  0xa9845a59,
       0x1eb35789, 0x1a5b2190,  0xa501532d,  0xd272e2a0,
       0x95898887,  0xa36f735d,  0x20ff2542,  0x14158fd6,
       0x1fb66651,  0x5e6b9ef6,  0x123efd9c,  0x2e45e1e0,
       0xc1606f13,  0x110dd021,  0xfe9410ab,  0xf64e3f93,
       0xd1672980,  0x81a6bc12,  0x7ff3cffb,  0xb5eded0e};
       
static int nstate = 128;

static int Sy_XCompare(SY_CONST_VOID *dot1, SY_CONST_VOID *dot2)
{
  return (((SyDot*)dot1)->x - ((SyDot*)dot2)->x);
}

static int Sy_YCompare(SY_CONST_VOID *dot1, SY_CONST_VOID *dot2)
{
  return (((SyDot*)dot1)->y - ((SyDot*)dot2)->y);
}

int Sy_RandomDots (SyDot *array, unsigned int dimx, unsigned int dimy, 
		   unsigned int max_dots, unsigned int num_dots)
{
  SyDot* dotarray;
  int i;
  int oldi, oldx, oldy;
  unsigned ndots, rdots;
  
  dotarray = (SyDot*) Sy_Malloc ((max_dots+1) * sizeof(SyDot));


#if defined(SYSV) || defined(SVR4)
  ndots = num_dots ? num_dots : floor(drand48()*(max_dots-1))+1;
#else
  ndots = num_dots ? num_dots : random() % (max_dots-1) + 1;
#endif

  (dotarray+ndots)->y = dimy + 1;

  for (i = 0; i < ndots; i++){
#if defined(SYSV) || defined(SVR4)
    (dotarray+i)->x = floor(drand48()*dimx);
    (dotarray+i)->y = floor(drand48()*dimy);
#else
    (dotarray+i)->x = random() % dimx;
    (dotarray+i)->y = random() % dimy;
#endif
  }

  qsort ((char*) dotarray, (int)ndots, sizeof(SyDot), Sy_YCompare);

  oldy = dotarray->y; oldi = 0;
  for (i = 1; i <= ndots; i++){
    if ((dotarray+i)->y != oldy){
      qsort ((char*) (dotarray+oldi), i-oldi, sizeof(SyDot), Sy_XCompare);
      oldi = i; oldy = (dotarray+i)->y;
    }
  }

  if (ndots - oldi >= 1){
    qsort ((char*)(dotarray+oldi), ndots-oldi+1, sizeof(SyDot), Sy_XCompare);
  }

  oldx = dotarray->x;
  oldy = dotarray->y;
  array[0] = *dotarray;
  for (i = 1, rdots = 0; i < ndots; i++){
    if ( ((dotarray+i)->x != oldx) || ((dotarray+i)->y != oldy) ){
      array[++rdots] = *(dotarray+i);
      oldx = (dotarray+i)->x;
      oldy = (dotarray+i)->y;
    }
  }
  free ((char*) dotarray);
  return (++rdots);
}

void Sy_InitRandomDots (void)
{ 
#if defined(hpux)
  struct timeval t;

  gettimeofday(&t, NULL);
  srand48(t.tv_usec);
#elif defined(SYSV) || defined(SVR4) || defined(SGIArchitecture) || defined(sgi)
  struct timeval t;

  gettimeofday(&t, NULL);
  srand48(t.tv_usec);
#else
  char* chptr;
  struct timeb t; 

  ftime (&t);
  
  chptr = initstate ((unsigned) t.millitm, (char*) randgen_state, nstate); 
  chptr = setstate ((char*) randgen_state);
#endif
}


void Sy_InitRandom (void)
{ 
#if defined(hpux)
  struct timeval t;

  gettimeofday(&t, NULL);
  srand48(t.tv_usec);
#elif defined(SYSV) || defined(SVR4) || defined(SGIArchitecture) || defined(sgi)
  struct timeval t;

  gettimeofday(&t);
  srand48(t.tv_usec);
#else
  char* chptr;
  struct timeb t; 

  ftime (&t);
  
  chptr = initstate ((unsigned) t.millitm, (char*) randgen_state, nstate); 
  chptr = setstate ((char*) randgen_state);
#endif
}

float Sy_Random (float lo, float hi)
{
  float ranval;
#if defined(SYSV) || defined(SVR4)
  ranval = (float)drand48();
#else
  ranval = (float)random() / (float)MAXINT;
#endif
  ranval = ranval * (hi-lo) + lo;
  return ranval;
}

float Sy_RandomGauss (float mean, float var)
{
  static int storeflag = 0;
  static float store, factor;
  
  float c, x1, x2;

  if (storeflag) {storeflag = 0; return (store * var + mean);}
  do {
    x1 = 2.0 * Sy_Random(0.0,1.0) - 1.0; 
    x2 = 2.0 * Sy_Random(0.0,1.0) - 1.0;
    c  = x1*x1+x2*x2;
  } while (c >= 1.0);
  factor = sqrt(-2.0*log(c));
  store = x2 * factor; storeflag = 1;
  return(x1 * factor * var + mean);
}

#define NVEC

typedef enum _SyWalkDirection {SY_GO_STRAIGHT, SY_GO_LEFT, SY_GO_RIGHT}
  SyWalkDirection;

SySkContourSet *Sy_AddNoise2ContourSet (SySkContourSet *cs, SyControl *ctrl)
{
  SySkContourSet *newcs;
  SyVector nv, tv, tv0, rv;
  SyIpoFloat *p, *buf;
  float norm, ranval, arg, arg0, det, d2;
  int cptr, i, k, i0, i1, bptr, is_closed, noise_on, maxlen = 0, len;
  SyWalkDirection dir, bumpdir, antibumpdir;
  float maxdev, maxdev2, param0, param1;
  int detsign, toggle = 0;

  Sy_InitRandom();
  maxdev = ctrl->maxnoise;
  maxdev2 = 0.5*maxdev;
  if (ctrl->is_gaussian_noise){
    param0 = 0; param1 = ctrl->sigma;
    ctrl->noisefunc = Sy_RandomGauss;
  }
  else{
    param0 = -1.0*ctrl->maxnoise; param1 = ctrl->maxnoise;
    ctrl->noisefunc = Sy_Random;
  }			    

  newcs = Sy_Malloc(sizeof(SySkContourSet));

  newcs->dim           = cs->dim;
  newcs->maxdim        = cs->dim;
  newcs->is_fg_info    = cs->is_fg_info;
  newcs->is_all_closed = cs->is_all_closed;
  newcs->contour = (SySkContour*)Sy_Malloc (sizeof(SySkContour)*newcs->maxdim);

  for (cptr = 0; cptr < newcs->dim; cptr++){
    newcs->contour[cptr].is_closed      = cs->contour[cptr].is_closed;
    newcs->contour[cptr].is_interior_fg = cs->contour[cptr].is_interior_fg;
    newcs->contour[cptr].fg_left_side   = cs->contour[cptr].fg_left_side;

    /* temporary */
    newcs->contour[cptr].len            = len = cs->contour[cptr].len;
    newcs->contour[cptr].point          = (SyIpoFloat*) 
      Sy_Malloc(sizeof(SyIpoFloat)*(newcs->contour[cptr].len+2));

    if (len > maxlen) maxlen = len;
  }

  buf = (SyIpoFloat*) Sy_Malloc(sizeof(SyIpoFloat)*(maxlen+2));

  for (cptr = 0; cptr < cs->dim; cptr++){
    is_closed = cs->contour[cptr].is_closed;
    p = cs->contour[cptr].point;
    len = cs->contour[cptr].len;
    /* it is assumed that enough space is reserved to hold p[len+1] */
    p[0] = p[len]; p[len+1] = p[1];
    noise_on = SY_FALSE;
    for (i = 1; i <= len; i++){

      switch(ctrl->noise_model){
      case 0:
	if (!noise_on) {
	  noise_on = (Sy_Random(0.0,1.0) < ctrl->onnoise) && (i < len-2);
	  if (noise_on){ 
	    i0 = i; 
	  }
	  else{
	    newcs->contour[cptr].point[i].x = p[i].x;
	    newcs->contour[cptr].point[i].y = p[i].y;
	  }
	  break;
	}

	noise_on = (i < len) && (Sy_Random(0.0,1.0) > ctrl->offnoise);
	
	if (!noise_on){
	  i1 = i;
	  if (i1-i0 < 2){
	    for (k = i0; k <= i1; k++){
	      newcs->contour[cptr].point[k].x = p[k].x;
	      newcs->contour[cptr].point[k].y = p[k].y;
	    }
	    break;
	  }
	  toggle = 0;
	  for (k = i0; k <= i1; k++){
	    
	    tv.dirx = p[k+1].x - p[k-1].x;
	    tv.diry = p[k+1].y - p[k-1].y;
	    norm    = 1.0/SY_NORM(tv.dirx, tv.diry);
	    tv.dirx *= norm;
	    tv.diry *= norm;
	    nv.dirx = -1.0 * tv.diry;
	    nv.diry =        tv.dirx;
	    
	    if (toggle <= 0 && k < i1){
	      ranval = ctrl->noisefunc(param0, param1);
	      toggle = 2;
	    }
	    nv.dirx *= ranval;
	    nv.diry *= ranval;
	    toggle--;
	    
	    newcs->contour[cptr].point[k].x = p[k].x + nv.dirx;
	    newcs->contour[cptr].point[k].y = p[k].y + nv.diry;
	  }
	}
	break;
      case 1:
	if (!noise_on) {
	  noise_on = (Sy_Random(0.0,1.0) < ctrl->onnoise) && (i < len-2);
	  if (noise_on){ 
	    i0 = i; 
	  }
	  else{
	    newcs->contour[cptr].point[i].x = p[i].x;
	    newcs->contour[cptr].point[i].y = p[i].y;
	  }
	  break;
	}
	
	noise_on = (i < len) && (Sy_Random(0.0,1.0) > ctrl->offnoise);
	
	if (!noise_on){
	  i1 = i;
	  if (i1-i0 < 2){
	    for (k = i0; k <= i1; k++){
	      newcs->contour[cptr].point[k].x = p[k].x;
	      newcs->contour[cptr].point[k].y = p[k].y;
	    }
	    break;
	  }
	  fprintf (stderr, "Adding noisy bump of length %d\n", i1-i0);
	  
	  if (Sy_Random(0.0,1.0) > 0.5){
            bumpdir = SY_GO_LEFT;
	    antibumpdir = SY_GO_RIGHT;
	    detsign = 1;
	  }
	  else{
	    bumpdir = SY_GO_RIGHT;
	    antibumpdir = SY_GO_LEFT;
	    detsign = -1;
	  }

	  dir = bumpdir;

	  tv0.dirx = p[i1].x - p[i0].x;
	  tv0.diry = p[i1].y - p[i0].y;
	  arg0 = atan2(tv0.diry, tv0.dirx);
          norm = 1.0/SY_NORM(tv0.dirx, tv0.diry);
	  tv0.dirx *= norm; tv0.diry *= norm;
	  buf[0].x = p[i0-1].x;
	  buf[0].y = p[i0-1].y; bptr = 1;

	  for (k = i0; k <= i0+(i1-i0)/2; k++){

	    switch(dir){
	    case SY_GO_LEFT:
	      arg  = Sy_Random(0.05, M_PI/2.0*0.95) + arg0; break;
	    case SY_GO_STRAIGHT:
	      arg  = Sy_Random(M_PI/2.0* -0.95, M_PI/2.0*0.95) + arg0; break;
	    case SY_GO_RIGHT:
	      arg  = -1.0*Sy_Random(0.05, M_PI/2.0*0.95) + arg0; break;
	    }

	    buf[bptr].x = buf[bptr-1].x + cos(arg); 
	    buf[bptr].y = buf[bptr-1].y + sin(arg); 
	    rv.dirx = buf[bptr].x - buf[0].x;
	    rv.diry = buf[bptr].y - buf[0].y;
	    d2 = sq(rv.dirx)+sq(rv.diry) 
	      - sq(rv.dirx*tv0.dirx + rv.diry*tv0.diry);

/*          fprintf (stderr, "f-> d: %f\n", sqrt(d2)); */

	    det = SY_DET(tv0.dirx, tv0.diry, rv.dirx, rv.diry);

	    if (d2 > sq(maxdev)){
	      dir = SY_SGN(det) == detsign ? antibumpdir : bumpdir;
	    }
	    else if (d2 > sq(maxdev2)){
	      dir = SY_SGN(det) == detsign ? SY_GO_STRAIGHT : bumpdir;
	    }
	    else dir = bumpdir;	

	    bptr++;
	  }/* for */

	  dir = antibumpdir;
	  for (k = i0+(i1-i0)/2 + 1; k <= i1; k++){

	    switch(dir){
	    case SY_GO_LEFT:
	      arg  = Sy_Random(0.05, M_PI/2.0*0.95) + arg0; break;
	    case SY_GO_STRAIGHT:
	      arg  = Sy_Random(M_PI/2.0* -0.95, M_PI/2.0*0.95) + arg0; break;
	    case SY_GO_RIGHT:
	      arg  = -1.0*Sy_Random(0.05, M_PI/2.0*0.95) + arg0; break;
	    }

	    buf[bptr].x = buf[bptr-1].x + cos(arg); 
	    buf[bptr].y = buf[bptr-1].y + sin(arg); 
	    rv.dirx     = buf[bptr].x - buf[0].x;
	    rv.diry     = buf[bptr].y - buf[0].y;

	    d2 = sq(rv.dirx)+sq(rv.diry) 
	      - sq(rv.dirx*tv0.dirx + rv.diry*tv0.diry);

/*	    fprintf (stderr, "r-> d: %f\n", sqrt(d2)); */

	    det = SY_DET(tv0.dirx, tv0.diry, rv.dirx, rv.diry);

	    dir = SY_SGN(det) == detsign ? antibumpdir : bumpdir;

	    bptr++;
	  }/* for */

	  for (k = i0, bptr = 0; k <= i1; k++,bptr++){
	    newcs->contour[cptr].point[k].x = buf[bptr].x;
	    newcs->contour[cptr].point[k].y = buf[bptr].y;
	  }
/*
	  newcs->contour[cptr].point[i1].x = p[i1].x;
	  newcs->contour[cptr].point[i1].y = p[i1].y;
*/	 
	}
      default:
	newcs->contour[cptr].point[i].x = p[i].x;
	newcs->contour[cptr].point[i].y = p[i].y;
	break;
      } /* noise model */
    } /* single contour */
  }/* all contours */ 

  Sy_Free(buf);
  return newcs;
}

