/*
 *  $Id$
 * 
 *  Author: Robert L. Ogniewicz, Swiss Federal Institute of Technology ETH
 *          Zurich, Switzerland (rlo@vision.ethz.ch)
 */

/* Copyright Notice
 * ================
 * Copyright (C) 1993 by Robert L. Ogniewicz
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
 */


/*
 * purpose: all-purpose triangulation traverse
 *
 */

static char sy_TrianTraverseId[] = "$Id$"; 

#include "Sy_Skeleton.h"
#include "Sy_Intrinsic.h"

static SyTriWalkBuff buff_0 = {0,0,0,0};

SyTriWalkState wr;

static SyWalktask* task;

static int depth     = 0, 
           max_depth = 0, 
           nrec      = 0;

static int verbose_local;

void Sy_InitTrianTraverse (SyTriDAC *tri, SyWalktask *wt)
{
  task          = wt;
  verbose_local = wt->verbose;
  wr.tri        = tri;
  wr.verbose    = verbose_local;
  wr.cont_thres = wt->group_thres;
  wr.narr_thres = wt->narr_thres;
  wr.prox_thres = wt->pro_thres;
  wr.is_regul   = tri->regdim > 0;
}

void Sy_TrianTraverse (SyTriDAC *tri, SyWalktask *wt)
{
  void Sy_WalkTriangulation(SyTriDAC *tri, int *next_edge, int *current_head_point, int trid);

  int i, trid = 0;
  int head_edge, tail_edge,
      head_point, tail_point;

  int next_edge, current_head_point;

  int edge_flag, is_linked = 1;

  sprintf (syStrBuf, "TRIANGULATION TRAVERSE %s  ", sy_TrianTraverseId);
  Sy_Message (SY_FMTNL, syStrBuf);

  tri->label.e_ = Sy_AllocEdgeLabel (tri->edim);
  tri->label.p_ = Sy_AllocPointLabel (tri->pdim);

  /* INIT call */
  if (task->user_function[(int)INIT] != NULL) task->user_function[(int)INIT]();

  for (i = 1; i <= tri->edim; i++){
    if (tri->label.e_[i]) continue;

    if (wr.is_regul){
      is_linked = ((tri->regul[i].cont >= wr.cont_thres) &&
                   (tri->regul[i].prox >= wr.prox_thres) &&
                   (tri->regul[i].narr >= wr.narr_thres));
    }
    if (!is_linked){
      continue;
    }
    if (tri->edge[i].tail_edge == 0){
      wr.start.next_edge = next_edge = i;
      wr.start.this_edge = 0;
      wr.start.head_point= current_head_point = tri->edge[i].tail_point;
      wr.current = wr.start;
      wr.id      = tri->label.e_[i] = ++trid;

      /* TRAVerse call */
      if (task->user_function[(int)TRAV] != NULL) task->user_function[(int)TRAV]();

      Sy_WalkTriangulation (tri, &next_edge, &current_head_point, wr.id);

      /* TERMination call */
      if (task->user_function[(int)TERM] != NULL) task->user_function[(int)TERM]();
    }
    else
    if (tri->edge[i].head_edge == 0){
      wr.start.next_edge = next_edge = i;
      wr.start.this_edge = 0;
      wr.start.head_point= current_head_point = tri->edge[i].head_point;
      wr.current = wr.start;
      wr.id      = tri->label.e_[i] = ++trid;

      /* TRAVerse call */
      if (task->user_function[(int)TRAV] != NULL) task->user_function[(int)TRAV]();

      Sy_WalkTriangulation (tri, &next_edge, &current_head_point, wr.id);

      /* TERMination call */
      if (task->user_function[(int)TERM] != NULL) task->user_function[(int)TERM]();
    }
  }

  for (i = 1; i <= tri->edim; i++){
    if (tri->label.e_[i]) continue;
    
    if (wr.is_regul){
      is_linked = ((tri->regul[i].cont >= wr.cont_thres) &&
                   (tri->regul[i].prox >= wr.prox_thres) &&
                   (tri->regul[i].narr >= wr.narr_thres));
    }
    if (!is_linked){
      continue;
    }
    wr.start.next_edge = next_edge = tri->edge[i].head_edge;
    wr.start.this_edge = i;
    wr.start.head_point= current_head_point = tri->edge[i].head_point;
    wr.current = wr.start;
    wr.id      = ++trid;

    /* TRAVerse call */
    if (task->user_function[(int)TRAV] != NULL) task->user_function[(int)TRAV]();
    
    Sy_WalkTriangulation (tri, &next_edge, &current_head_point, wr.id);
    
    /* TERMination call */
    if (task->user_function[(int)TERM] != NULL) task->user_function[(int)TERM]();
  }
  
  /* exit call */
  if (task->user_function[(int)FEXIT] != NULL) task->user_function[(int)FEXIT]();
}
   
void Sy_WalkTriangulation (SyTriDAC *tri, int *next_edge, int *current_head_point, int trid)
{
  void Sy_WalkTriangulation(SyTriDAC *tri, int *next_edge, int *current_head_point, int trid);

  int current_head_point_buff, 
      next_edge_buff,
      last_head_point_buff,
      last_edge_buff, ref_edge;

  int is_linked = 1, walking_flag = 1;

  int tail_point, head_point;

  last_head_point_buff = wr.current.head_point;
  last_edge_buff       = wr.current.next_edge;
  ref_edge             = *next_edge;

  /* WALK call */
  if (task->user_function[(int)WALK] != NULL){
    wr.last               = wr.current;
    wr.current.head_point = *current_head_point;
    wr.current.next_edge  = *next_edge;
    task->user_function[(int)WALK]();
  }

  if (tri->label.p_[*current_head_point] == 0){
    
    current_head_point_buff            = *current_head_point;
    next_edge_buff                     = *next_edge;

    tri->label.p_[*current_head_point] = trid;
    
    while ((next_edge_buff != 0) && walking_flag){

      wr.current.head_point = current_head_point_buff;
      wr.current.next_edge  = next_edge_buff;

      head_point = tri->edge[next_edge_buff].head_point;
      tail_point = tri->edge[next_edge_buff].tail_point;

      if (wr.is_regul){
	is_linked = ((tri->regul[next_edge_buff].cont >= wr.cont_thres) &&
		     (tri->regul[next_edge_buff].prox >= wr.prox_thres) &&
		     (tri->regul[next_edge_buff].narr >= wr.narr_thres));
      }
	
      if (is_linked){
	
	*current_head_point = current_head_point_buff;
	*next_edge          = next_edge_buff;
	
	head_point = tri->edge[*next_edge].head_point;
	tail_point = tri->edge[*next_edge].tail_point;
	
	tri->label.e_[*next_edge] = trid;
	
	if (*current_head_point == tail_point){
	  *current_head_point = head_point;
	  *next_edge          = tri->edge[*next_edge].head_edge;
	}
	else{
	  *current_head_point = tail_point;
	  *next_edge          = tri->edge[*next_edge].tail_edge;
	}

	if (++depth > max_depth) max_depth = depth; nrec++;
	if (syIsVerbose){
	  sprintf (syStrBuf, "> depth:%6d, max.depth:%6d, nrec:%6d\r",
		   depth, max_depth, nrec);
	  Sy_Message (SY_FMT, syStrBuf);
	}

	Sy_WalkTriangulation (tri, next_edge, current_head_point, trid);
	
	wr.current.head_point = current_head_point_buff;
        wr.current.next_edge  = next_edge_buff;
	wr.last.next_edge     = last_edge_buff;
	wr.last.head_point    = last_head_point_buff;

	/* ROT call */
	if (task->user_function[(int)ROT] != NULL) task->user_function[(int)ROT]();
	
	depth--;
	if (syIsVerbose){
	  sprintf (syStrBuf, "> depth:%6d\r", depth); 
	  Sy_Message (SY_FMT, syStrBuf);
	}
      }
      else{
	/* EOSK call */
	if (task->user_function[(int)EOSK] != NULL) task->user_function[(int)EOSK]();
      }

      if (current_head_point_buff == tri->edge[next_edge_buff].tail_point){
	next_edge_buff = tri->edge[next_edge_buff].tail_edge;
      }
      else{
	next_edge_buff = tri->edge[next_edge_buff].head_edge;
      }
      if (next_edge_buff != 0){
	walking_flag = (next_edge_buff != ref_edge); 
      }
    }
  }
}




