/*
 *  $Id$
 * 
 *  Author: Robert L. Ogniewicz, Swiss Federal Institute of Technology ETH
 *          Zurich, Switzerland (rlo@vision.ethz.ch)
 */

/* Copyright Notice
 * ================
 * Copyright (C) 1993 by Robert L. Ogniewicz
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
 */

/*
 * all-purpose Voronoi diagram traverse
 *
 */ 

static char sy_VoroTraverseId[] = "$Id$"; 
#define SY_WALK_H_INTERNAL SY_TRUE

#include "Sy_Skeleton.h"
#include "Sy_Intrinsic.h"

static int depth     = 0, 
           max_depth = 0, 
           nrec      = 0;

static int verbose_local;

static SyWalkBuff buff_0 = {0,0};

SyWalkState syWstate;
static SyWalktask* task;

void Sy_InitVoronoiTraverse (SyVoroDAC *vd, SyWalktask *wt)
{
  task          = wt;
  syWstate.verbose    = wt->verbose;
  syWstate.vd         = vd;
}  

static void Sy_TraverseMessage (SyVoroDAC *vd, int current_head_vertex, 
				int voroid);

void Sy_VoronoiTraverse (SyVoroDAC *vd, SyWalktask *wt)
{
  void Sy_WalkVoronoi (SyVoroDAC *vd, int *next_edge, 
		       int *current_head_vertex, int voroid);
         
  int i, voroid;
  int head_vertex, tail_vertex;
  int left_point, right_point;
  int tail_edge, head_edge;

  int next_edge, current_head_vertex;

  vd->label.e_   = Sy_AllocLabel (vd->edim);
  vd->label.v_   = Sy_AllocLabel (vd->vdim);
    
  task          = wt;
  syWstate.verbose    = wt->verbose;
  syWstate.vd         = vd;
  
  sprintf(syStrBuf, "TRAVERSE (WALK VORONOI)  %s  ", sy_VoroTraverseId);
  Sy_Message (SY_FMTNL, syStrBuf);

  /* INIT call */
  if (task->user_function[(int)INIT] != NULL) task->user_function[(int)INIT]();

  for (i = 1, voroid = 0; i <= vd->edim; i++){

    if ((vd->label.e_[i] == 0) && (vd->edge[i].rank == 1) ){

      head_vertex = vd->edge[i].head_vertex;
      tail_vertex = vd->edge[i].tail_vertex; 
      left_point  = vd->edge[i].left_point;
      right_point = vd->edge[i].right_point;
      tail_edge   = vd->edge[i].tail_edge;
      head_edge   = vd->edge[i].head_edge;

      vd->label.e_[i]     = SY_TMPMARK;
      next_edge           = max (tail_edge, head_edge);
      current_head_vertex = (next_edge == head_edge) ? 
                          vd->edge[i].head_vertex : vd->edge[i].tail_vertex;
      
      if (vd->label.v_[current_head_vertex] == 0){
      
	max_depth            = 0;
        syWstate.id                = ++voroid;
	syWstate.start.head_vertex = current_head_vertex;
        syWstate.start.next_edge   = next_edge;
 	syWstate.current           = buff_0;
	Sy_TraverseMessage (vd, current_head_vertex, voroid);
	  
        /* TRAVerse call */	  
	if (task->user_function[(int)TRAV] != NULL) task->user_function[(int)TRAV]();

	Sy_WalkVoronoi (vd, &next_edge, &current_head_vertex, voroid);

        /* TERMination call */	  
        if (task->user_function[(int)TERM] != NULL) task->user_function[(int)TERM]();
	  
      }
    }
  }/* for */
  /* exit call */
  if (task->user_function[(int)FEXIT] != NULL) task->user_function[(int)FEXIT]();
}
   
void Sy_WalkVoronoi (SyVoroDAC *vd, int *next_edge, 
		     int *current_head_vertex, int voroid)
{
  void Sy_WalkVoronoi (SyVoroDAC *vd, int *next_edge, 
		       int *current_head_vertex, int voroid);

  int current_head_vertex_buff, 
      next_edge_buff, ref_edge;

  int head_flag, tail_flag, walking_flag = 1;

  int tail_vertex, head_vertex, 
      right_point, left_point;

  ref_edge = *next_edge;

  /* WALK call */
  if (task->user_function[(int)WALK] != NULL){
    syWstate.last = syWstate.current;
    syWstate.current.head_vertex = *current_head_vertex;
    syWstate.current.next_edge   = *next_edge;
    task->user_function[(int)WALK]();
  }

  if (vd->label.v_[*current_head_vertex] == 0){
      
    current_head_vertex_buff            = *current_head_vertex;
    next_edge_buff                      = *next_edge;
    vd->label.v_[*current_head_vertex]  = voroid;
    
    while ((next_edge_buff != 0) && walking_flag){

      syWstate.current.head_vertex = current_head_vertex_buff;
      syWstate.current.next_edge   = next_edge_buff;
      
      head_vertex = vd->edge[next_edge_buff].head_vertex;
      tail_vertex = vd->edge[next_edge_buff].tail_vertex;
      left_point  = vd->edge[next_edge_buff].left_point;
      right_point = vd->edge[next_edge_buff].right_point;  
      
      *current_head_vertex  = current_head_vertex_buff;
      *next_edge            = next_edge_buff;
      
      head_vertex = vd->edge[*next_edge].head_vertex;
      tail_vertex = vd->edge[*next_edge].tail_vertex;
      
      vd->label.e_[*next_edge] = voroid;
      
      if (*current_head_vertex == tail_vertex){
	*current_head_vertex = head_vertex;
	*next_edge           = vd->edge[*next_edge].head_edge;
      }
      else{
	*current_head_vertex = tail_vertex;
	*next_edge           = vd->edge[*next_edge].tail_edge;
      }

      if (++depth > max_depth) max_depth = depth; nrec++;
      if (syIsVerbose){
	sprintf(syStrBuf, "> depth:%6d, max.depth:%6d, nrec:%6d\r", 
		depth, max_depth, nrec);
	Sy_Message (SY_FMTNL, syStrBuf);
      }

      Sy_WalkVoronoi (vd, next_edge, current_head_vertex, voroid);
      
      syWstate.current.head_vertex = current_head_vertex_buff;
      syWstate.current.next_edge   = next_edge_buff;
      
      /* ROT call */
      if (task->user_function[(int)ROT] != NULL) 
	task->user_function[(int)ROT]();
	
      depth--;
      if (syIsVerbose){
	sprintf (syStrBuf, "> depth:%6d\r", depth); 
	Sy_Message (SY_FMTNL, syStrBuf);
      }
    
      if (current_head_vertex_buff == vd->edge[next_edge_buff].tail_vertex){
	next_edge_buff = vd->edge[next_edge_buff].tail_edge;
      }
      else{
	next_edge_buff = vd->edge[next_edge_buff].head_edge;
      }
      if (next_edge_buff != 0){
	walking_flag = (next_edge_buff != ref_edge);
      }
    }
  }
}


void Sy_VoronoiRotate (SyVoroDAC *vd, int *next_edge, int *current_head_vertex)
{

  int current_head_vertex_buff, 
      next_edge_buff, ref_edge;

  int walking_flag = 1;

  int tail_vertex, head_vertex, 
      right_point, left_point;

  syWstate.start.head_vertex = syWstate.current.head_vertex = *current_head_vertex;
  syWstate.start.next_edge   = syWstate.current.next_edge   = *next_edge;
  syWstate.current.this_edge = syWstate.start.this_edge;
  syWstate.last              = syWstate.current;

  ref_edge             = *next_edge;
    
  current_head_vertex_buff  = *current_head_vertex;
  next_edge_buff            = *next_edge;

  if (task->user_function[(int)TRAV] != NULL) task->user_function[(int)TRAV]();

  while ((next_edge_buff != 0) && walking_flag){

    head_vertex = vd->edge[next_edge_buff].head_vertex;
    tail_vertex = vd->edge[next_edge_buff].tail_vertex;
    left_point  = vd->edge[next_edge_buff].left_point;
    right_point = vd->edge[next_edge_buff].right_point;
  
    /* ROT call */
    if (task->user_function[(int)ROT] != NULL) task->user_function[(int)ROT]();
	
    if (current_head_vertex_buff == vd->edge[next_edge_buff].tail_vertex){
      next_edge_buff = vd->edge[next_edge_buff].tail_edge;
    }
    else{
      next_edge_buff = vd->edge[next_edge_buff].head_edge;
    }
    if (next_edge_buff != 0){
      walking_flag = (next_edge_buff != ref_edge);
    }

    syWstate.last               = syWstate.current;
    syWstate.current.this_edge  = syWstate.current.next_edge;
    syWstate.current.next_edge  = next_edge_buff;

  }
  if (task->user_function[(int)TERM] != NULL) task->user_function[(int)TERM]();
}


void Sy_VoronoiPolyScan (SyVoroDAC *vd, int *next_edge, int *current_head_vertex)
{

  int current_head_vertex_buff, 
      next_edge_buff, ref_edge;

  int walking_flag = 1;

  int tail_vertex, head_vertex;

  syWstate.start.head_vertex = *current_head_vertex;
  syWstate.start.next_edge   = *next_edge;
  syWstate.last              = syWstate.start;
  syWstate.current           = syWstate.start;

  ref_edge             = *next_edge;
    
  current_head_vertex_buff  = *current_head_vertex;
  next_edge_buff            = *next_edge;

  while ((next_edge_buff != 0) && walking_flag){

    /* SCAN call */
    if (task->user_function[(int)SCAN] != NULL) task->user_function[(int)SCAN]();
	
    if (current_head_vertex_buff == vd->edge[next_edge_buff].tail_vertex){
      current_head_vertex_buff = vd->edge[next_edge_buff].head_vertex;
      next_edge_buff           = vd->edge[next_edge_buff].head_edge;
    }
    else{
      current_head_vertex_buff = vd->edge[next_edge_buff].tail_vertex;
      next_edge_buff           = vd->edge[next_edge_buff].tail_edge;
    }

    syWstate.last = syWstate.current;
    syWstate.current.head_vertex = current_head_vertex_buff;
    syWstate.current.this_edge   = syWstate.last.next_edge;
    syWstate.current.next_edge   = next_edge_buff;

    if (next_edge_buff != 0){
      walking_flag = (next_edge_buff != ref_edge);
    }
  }
  if (task->user_function[(int)TERM] != NULL) task->user_function[(int)TERM]();
}

static void Sy_TraverseMessage (SyVoroDAC *vd, int current_head_vertex, int voroid)
{
  SyVertex vtmp;
  float hotx, hoty;

  vtmp = vd->vertex[current_head_vertex];
  hotx = vtmp.xden/vtmp.num; 
  hoty = vtmp.yden/vtmp.num;
  
  sprintf (syStrBuf, "Voronoi Diagram %3d at (%7.1f,%7.1f)  ", 
	   voroid, hotx, hoty);
  Sy_Message (SY_FMTNL, syStrBuf);
}










