.\" @(#)loadskset.1 1.0 95/06/25
'\"macro stdmacro
.nr X
.TH loadskset 1 "25 June 1995"
.\" Copyright 1995 Robert L. Ogniewicz
.SH NAME
loadskset \- sample routine to demonstrate loading of skeleton-set data
.SH SYNOPSIS
.B loadskset
.I inputfile  
.SH DESCRIPTION
.IX "loadskset command" "" "\fLloadsket\fP \(em loading skeleton-set data"
.I loadskset
is a demonstration program which loads and displays data in the skeleton-set
format [see skeleton(5)] as created by the 
.I \-compile 
option of 
.I prune. 
The corresponding code can be found in the 
.I bin
subdirectory of the distribution in 
.I loadskset.c.
.SH SAMPLE PROGRAM CODE
The following code can be used to load a skeleton-set from 
file. File format and data structure 
.I SyVSkeletonSet 
are described in manual page 
.I skeleton(5):
.LP
.nf
#include <stdio.h>
#include "Sy_Skeleton.h"
#include "Sy_Graphics.h"

main(int argc, char **argv)
{
  SyVSkeletonSet *vss;
  FILE *fp;
  int i;

  if (argc <= 1) {
    fprintf (stderr, "Usage: %s file\n", argv[0]); exit(2);
  }
  
  fp = fopen (argv[1], "r");
  if (fp == NULL){
    fprintf (stderr, "Could not open file `%s'.\n", argv[1]);
    exit(2);
  }

  vss = Sy_ReadSkeletonSet(fp);

  if (vss == NULL){
    fprintf (stderr, "Could not read skeleton set.\n");
    exit(2);
  }

  printf ("> Skeleton set with %d skeleton(s).\n", vss->dim);

  for (i = 1; i <= vss->dim; i++){
    if (vss->skel[i].branchdim > 0 || vss->skel[i].nodedim > 0){
      printf ("Skeleton (%s) no %d contains %d nodes and %d branches\n",
	      vss->skel[i].is_foreground ? "foreground" : "background",
	      vss->skel[i].id,
	      vss->skel[i].nodedim,
	      vss->skel[i].branchdim);
    }
  }
}
.fi
.LP
Let us assume that header files (such as Sy_Skeleton.h) 
are stored in directory
/usr/local/include, and the distribution's libraries (such as libSy.a)
in /usr/local/lib. The program is stored in loadskset.c.
Compiling the above program is done by issuing the following commands:
.IP
.B "gcc -c -I/usr/local/include loadskset.c"

.B "gcc -o loadskset loadskset.o -L/usr/local/lib -lSy -lImLoad -ltiff -ljpeg -lX11 -lm -ly -ll" 
.LP
.SH SEE ALSO
.BR ctran (1),
.BR voro (1),
.BR prune (1),
.BR contours (5),
.BR skeleton (5)
.SH AUTHOR
.nf
Robert L. Ogniewicz
E-Mail: rlo@hrl.harvard.edu
.fi

This work has been carried out at the
.I "Communication Technology Laboratory," 
Swiss Federal Institute of
Technology ETH, Zurich, Switzerland, and the
.I "Division of Applied Sciences," 
Harvard University, Cambridge, USA.
.SH COPYRIGHT
Copyright (C) 1995 Robert L. Ogniewicz 
.SH BUGS
Undoubtedly many...
.SH LITERATURE
Information on Voronoi skeletons is available in the
following references:
.LP
Ogniewicz, R.L., "Skeleton-space: 
A multiscale shape description combining region and boundary information", 
in Proc. Conf. on Computer Vision and
Pattern Recognition, Seattle, WA, pp. 746-751, June 1994.
.LP
Ogniewicz, R.L. and Kuebler, O., "Hierarchic
Voronoi Skeletons", Pattern Recognition, vol. 28, no. 3, 
pp. 343-359, 1995.
.LP
Ogniewicz, R.L. and Ilg, M., 
"Voronoi skeletons: Theory and applications", in
Proc. Conf. on Computer Vision and Pattern Recognition, Champaign,
Illinois, pp. 63-69, June 1992.
.LP
Ogniewicz, R.L. (1993),
Discrete Voronoi Skeletons, Konstanz, Germany: Hartung-Gorre Verlag.
.LP
.\".Ee
