.\" @(#)prune.1 1.0 95/06/25
'\"macro stdmacro
.nr X
.TH prune 1 "25 June 1995"
.\" Copyright 1995 Robert L. Ogniewicz
.SH NAME
prune \- Medial Axis pruning routine 
.SH SYNOPSIS
.B prune
[
.B options...
] 
.I -|inputfile  
[
.I -|outputfile
]
.SH DESCRIPTION
.IX "prune command" "" "\fLprune\fP \(em medial axis (skeleton) pruning"
The purpose of the
.I pruning
command is to extract a robust Medial Axis from Voronoi diagrams.
.I Inputfile 
is a file containing Voronoi diagram data as produced by
.I voro. 
Alternatively,
a \- (dash) argument denotes that I/O will be done using
stdin/stdout instead of files. If no 
.I outputfile
argument is specified, 
.I prune 
will display the pruned skeleton in a window. 
.SH OPTIONS
.PD 1
Abbreviation of options listed below is allowed, as long as the
option string remains nonambiguous.
.TP
.B \-automatic 
Perform automatic pruning. Specification of threshold values or other
parameters is 
.I not 
necessary. Currently uses smoothed chord residual. 
See literature for details.
.TP
.B \-pyramid
Run skeleton-pyramid algorithm on attributed Voronoi diagram (Voronoi
skeleton). The output is a so-called Hierarchic Voronoi Skeleton.
.TP
.B \-thorough
Try very hard to prune spurious branches. Affects only
automatic pruning.
.TP
.B \-compile
Convert internal skeleton representation into a more flexible
and more user-friendly description (so-called skeleton-set).
See additional documentation: skeleton(5) for details.
.TP
.B \-pot
Prune using potential residual. Recommended threshold value
about 5..15.
.TP
.B \-circ
Prune using circularity residual. Recommended threshold value
about 3...4.
.TP
.B \-bicirc
Prune using bi-circularity residual. Recommended threshold value
about 7...10.
.TP
.B \-chord
Prune using chord residual (default). Good pruning results for threshold
values about 3...4. Fast computation.
.TP
.B \-smpot 
Smoothed potential residual. 
.TP
.B \-smchord
.TP
.BI "\-sigma" S
Width 
.I S 
of Gaussian kernel. Affects -smpot and -smchord options.
.TP
.BI "\-trigger " t
Parameter 
.I t 
denotes a trigger level for the buildup of the hierarchy. As long as the
residual of an edge is larger than t times the maximal residual
value found within the skeleton, this edge becomes member of the 
first order skeleton. Note that the value of 
.I t
is given in percent (default 2.0). Try also values about 10%.
.TP
.BI "\-cluster " C
Clustering parameter for skeleton-pyramid construction (in percent).
A value of 10 (default) for 
.I C 
signifies that clusters of edges characterized by residual values 
within a ten percent interval of relative deviation are assigned the 
same rank order value.
.TP 
.B \-rich
Generate and store residual values for each edge, as opposed to computing
only so-called birth-sets (see literature for details). Mostly for
internal use.
.TP
.B \-split
Separate "physically" foreground and background skeleton, splitting
at Voronoi edges which have been generated by adjacent boundary points.
Used mostly for demonstration purposes.
.TP
.BI "\-threshold " T
Regularization threshold for skeletons (affects skeleton-pyramid
generation and display). Only edges of the Voronoi
diagram are processed where significance measure (residual function) > 
.I T.
Default values depend on type of significance measure applied, typically
2...4. More information can be found in literature, see corresponding section.
.TP
.BI "\-rank " R
Display only section of the skeleton which belong to a hierarchical
level (skeleton-pyramid) higher or equal 
.I R.
.TP
.B "\-fg " 
Display foreground skeleton. 
.TP
.B "\-bg " 
Display skeleton of the background (exoskeleton).   
.TP
.B \-raw
Do not draw object boundaries.
.TP
.B \-ascii
Create output in text (ASCII) form. Useful for increased portability between
different machine architectures. Default format consists of multiple
data strips, each strip composed of an ASCII header followed by 
binary data.
.TP
.B \-verbose
Produce some diagnostic output while running.
.TP    
.B \-timer
Output performance measurements such as elapsed time, swaps, page faults etc.
.TP
.BI "\-xcenter " x 
.PD 0
.TP 
.BI "\-ycenter " y
(x,y) coordinates of `focus of interest' for display.
.PD 1
.TP
.BI "\-xwindim " width 
.PD 0
.TP 
.BI "\-ywindim " height
Explicitly set x and y dimensions of display window.
.PD 1
.TP
.BI "\-scale "  s 
scale graphics output by factor s.
.TP
.BI "\-linewidth " L
Set width of lines drawn (X Windows) to 
.I L.
.TP
.BI "\-display " display_name
X11 display name to send the image to.
.SH EXAMPLES
.LP
Pruning a Voronoi skeleton with circularity residual at threshold 3.0,
displaying only the foreground skeleton:
.IP
.B "prune -fg -threshold 3.0 -circ voronoi.dat"
.LP
Pruning a Voronoi skeleton with chord residual (default) and
postprocessing with the skeleton-pyramid algorithm:
.IP
.B "prune -threshold 2.0 -pyramid -rank 1 voronoi.dat"
.LP
Automatic pruning:
.IP
.B "prune -fg -automatic -verbose voronoi.dat"
.LP
Automatic pruning of very noisy data 
including skeleton pyramid evaluation and compilation
into a skeleton-set data structure:
.IP
.B "prune -auto -verb -pyr -thorough -compile voronoi.dat output.dat"
.LP
Running 
.I prune
within a unix pipe (implicit pruning with chord residual and
display with threshold 3.0):
.IP
.B "ctran image - | voro - - | prune - "
.LP  
.SH SEE ALSO
.BR ctran (1),
.BR voro (1),
.BR loadskset (1),
.BR contours (5),
.BR skeleton (5)
.SH AUTHOR
.nf
Robert L. Ogniewicz
E-Mail: rlo@hrl.harvard.edu
.fi

This work has been carried out at the
.I "Communication Technology Laboratory," 
Swiss Federal Institute of
Technology ETH, Zurich, Switzerland, and the
.I "Division of Applied Sciences," 
Harvard University, Cambridge, USA.
.SH COPYRIGHT
Copyright (C) 1995 Robert L. Ogniewicz 
.SH BUGS
Automatic pruning currently assumes that either (i) only one object is
processed at a time or (ii) that all objects in the scene appear at
approximately the same scale. This limitation is going to be
removed with the next major release. 

And, undoubtedly, many more...
.SH LITERATURE
Information on Voronoi skeletons is available in the
following references:
.LP
Ogniewicz, R.L., "Skeleton-space: 
A multiscale shape description combining region and boundary information", 
in Proc. Conf. on Computer Vision and
Pattern Recognition, Seattle, WA, pp. 746-751, June 1994.
.LP
Ogniewicz, R.L. and Kuebler, O., "Hierarchic
Voronoi Skeletons", Pattern Recognition, vol. 28, no. 3, 
pp. 343-359, 1995.
.LP
Ogniewicz, R.L. and Ilg, M., 
"Voronoi skeletons: Theory and applications", in
Proc. Conf. on Computer Vision and Pattern Recognition, Champaign,
Illinois, pp. 63-69, June 1992.
.LP
Ogniewicz, R.L. (1993),
Discrete Voronoi Skeletons, Konstanz, Germany: Hartung-Gorre Verlag.
.LP
.\".Ee
