.\" @(#)skeleton.5 1.0 95/06/25
'\"macro stdmacro
.nr X
.TH skeleton 5 "25 June 1995"
.\" Copyright 1995 Robert L. Ogniewicz
.SH NAME
skeleton \- file format/data structure for contours
.SH DESCRIPTION
.IX "skeleton" "" "\fLskeleton\fP \(em skeleton file format/data structure"

The skeleton file format has been designed to provide a flexible
user-interface between 
.I prune,
the Medial Axis extraction routine, and user processes. A sample
loader program for this format is provided, see manual page
loadskset(1).

In general, the representation of a skeleton primitive (node, branch)
consists of a header (carrying generic information) plus one or
more data segments.
It is possible to place comments /* text */ everywhere outside
the data section. Assignments to variables appear in the form
.IP
.B "var = statement;" 
.LP
where statement can be an integer (myvar = 1;), a real value (myvar =
3.14), a string enclosed in double quotes (myvar = "hello world !"),
or simply a single identifier (basically, a string without blanks,
starting with a non-numeric character) (myvar = true;). Also
the order of assignments within the header is irrelevant. 
 
.SH EXAMPLE: FILE FORMAT
The sample data set represents the Medial Axis of a tiny
rectangle (note the orientation of the coordinate system).
An output similar to the one below can be obtained by running
.IP
.B "ctran -invert perf_rect_tiny.ras - | voro - - | prune - -threshold 1 -compile output.dat"
.LP
A rough sketch of the encoded skeleton:
.nf
skeleton no 1   
 --------------------------        + Node
|+ Node 5          Node 1 +|       * Branch element
| * Branch 4             * |       - Boundary
|  *           Branch 1 *  |
|   *                  *   |
|    *    Branch 3  *      |       ---> X-coord
|     +**************+	   |       |		
|    * Node 4  Node 2 *    |       |  
|   *                  *   |       v
|  *            Branch 2*  |       Y-coord 
| * Branch 5             * |
|+ Node 6          Node 3 +|
 --------------------------
.fi
.LP
Contents of output.dat:
.LP
.nf
/* Skeleton Project File*/

object skeleton_1 {              /* skeleton no 1 */
type          = skeleton;                 
id            = 1;               /* unique ID number */
user_id       = 0;               /* user-id, currently = 0 */
branchdim     = 5;               /* number of skeleton branches */
nodedim       = 6;               /* number of skeleton nodes */
is_foreground = true; 


object node_1 { /* skeleton no 1 */   
type            = skeletonnode;
id              = 1;        
user_id         = 0;
skel_id         = 1;             /* belongs to skeleton no 1 */
hierarchy_level = 0;             /* hierarchy level as obtained 
                                    from the skeleton-pyramid algorithm.
                                    Always = 0 if the -pyramid option for
                                    prune was not used. 
                                  */
is_foreground   = true;
nodetype        = endpoint;       /* only one incident branch */
strength        = 0.000000;       /* saliency measure - currently
                                     smallest component of the
                                     so-called `birth-set'
                                   */
radius          = 1.581139;       /* radius of maximal inscribed disk
                                     centered at this node */
rank            = 1;              /* number of incident branches */
xytype          = real;           /* type of node coordinates (x,y),
                                     currently always `real'
                                   */
                                          
/* position: */
x = 29.500000; y = 4.500000;

/* 
The two non-comment lines below start a loading routine for a
following data segment. The next two lines must not be modified.  
The character sequence #2718sync314beg# is a `magic cookie' used for
synchronization between parser of header language and generic data
(can be binary) loading routines. Accordingly, #2718sync314end# is
used to accomplish synchronization after data input. All data between
beginloadseq(Sy_ReadIncList) and #2718sync314beg# is discarded.
*/

/* list of incident branches: */
beginloadseq (Sy_ReadIncList)
#2718sync314beg#
1

#2718sync314end#
endloadseq;                       /* terminate load sequence */

} /* end node no 1 */ 
.fi
.LP
This was the representation of a single 
.I skeleton node,
five more nodes will follow below. Thereafter, the individual
.I skeleton branches 
will be described.
.LP
.nf
object node_2 { /* skeleton no 1 */
type            = skeletonnode;
id              = 2;
user_id         = 0;
skel_id         = 1;
hierarchy_level = 0;
is_foreground   = true;
nodetype        = internal;
strength        = 3.514719;
radius          = 6.000000;
rank            = 3;
xytype          = real;
/* position: */
x = 25.000000; y = 9.000000;
/* list of incident branches: */
beginloadseq (Sy_ReadIncList)
#2718sync314beg#
1
2
3

#2718sync314end#
endloadseq;

} /* end node no 2 */ 

object node_3 { /* skeleton no 1 */
type            = skeletonnode;
id              = 3;
user_id         = 0;
skel_id         = 1;
hierarchy_level = 0;
is_foreground   = true;
nodetype        = endpoint;
strength        = 0.000000;
radius          = 1.581139;
rank            = 1;
xytype          = real;
/* position: */
x = 29.500000; y = 13.500000;
/* list of incident branches: */
beginloadseq (Sy_ReadIncList)
#2718sync314beg#
2

#2718sync314end#
endloadseq;

} /* end node no 3 */ 

object node_4 { /* skeleton no 1 */
type            = skeletonnode;
id              = 4;
user_id         = 0;
skel_id         = 1;
hierarchy_level = 0;
is_foreground   = true;
nodetype        = internal;
strength        = 3.514719;
radius          = 6.000000;
rank            = 3;
xytype          = real;
/* position: */
x = 8.000000; y = 9.000000;
/* list of incident branches: */
beginloadseq (Sy_ReadIncList)
#2718sync314beg#
4
3
5

#2718sync314end#
endloadseq;

} /* end node no 4 */ 

object node_5 { /* skeleton no 1 */
type            = skeletonnode;
id              = 5;
user_id         = 0;
skel_id         = 1;
hierarchy_level = 0;
is_foreground   = true;
nodetype        = endpoint;
strength        = 0.000000;
radius          = 1.581139;
rank            = 1;
xytype          = real;
/* position: */
x = 3.500000; y = 4.500000;
/* list of incident branches: */
beginloadseq (Sy_ReadIncList)
#2718sync314beg#
4

#2718sync314end#
endloadseq;

} /* end node no 5 */ 

object node_6 { /* skeleton no 1 */
type            = skeletonnode;
id              = 6;
user_id         = 0;
skel_id         = 1;
hierarchy_level = 0;
is_foreground   = true;
nodetype        = endpoint;
strength        = 0.000000;
radius          = 1.581139;
rank            = 1;
xytype          = real;
/* position: */
x = 3.500000; y = 13.500000;
/* list of incident branches: */
beginloadseq (Sy_ReadIncList)
#2718sync314beg#
5

#2718sync314end#
endloadseq;

} /* end node no 6 */ 
.fi
.LP
The following lines contain the representation of a single
.I skeleton branch.
Only data fields which have not been already mentioned 
.I node
will be explained.
.LP
.nf
object branch_1 { /* skeleton no 1 */  /* skeleton branch no 1*/
type            = skeletonbranch;  
id              = 1;                  
user_id         = 0;
skel_id         = 1;                   /* belongs to skeleton no 1 */
hierarchy_level = 0;         
is_foreground   = true;
has_boundary    = true;                /* the representation contains
                                               a list of boundary points which
                                               have generated this specific
                                               skeleton branch */ 
strength        = 3.514719;            /* saliency measure, currently
                                               the maximal residual value
                                               on a branch */
length          = 6.363961;            /* overall branch length */

max_strength    = 3.514719; min_strength = 1.757359;
mean_strength   = 2.636039; var_strength = 0.571909;
                                       /* simple statistics of saliency
                                          measure distribution over the
                                          branch -> min, max, mean value,
                                          and variance.
                                        */
max_radius      = 6.000000; min_radius   = 1.581139;
mean_radius     = 3.952759; var_radius   = 2.950833;
	                               /* statistics of local diameter
                                          distribution over a branch */
tail_branch     = 1; head_branch  = 2; /* references (IDs) of adjacent
                                          branches encountered during
                                          a counterclockwise rotation
                                          about tail node and head node,
                                          respectively
                                        */
tail_node       = 1; head_node    = 2;  /* references (IDs) to 
                                           both terminating nodes */
                                              
elementdim      = 6;                    /* number of branch vertices, i.e.
                                           (x,y) coordinate pairs, 
	                                   INCLUDING both terminating nodes
                                         */    
axistype        = real;

/* symmetry axis: */
beginloadseq (Sy_ReadSymmAxis)               
#2718sync314beg#                         /* (x,y) coordinates of the
29.500000 4.500000                          polygonal chain approximating  
28.500000 5.500000                          the symmetry axis, traversed 
27.500000 6.500000                          from tail node to head node */
26.500000 7.500000
25.500000 8.500000
25.000000 9.000000

#2718sync314end#
endloadseq;

/* radii: */                             /* radius of maximally
                                            inscribed disk at each
beginloadseq (Sy_ReadRadiusList)            branch point */
#2718sync314beg#
1.581139
2.549510
3.535534
4.527693
5.522680
6.000000

#2718sync314end#
endloadseq;

                                       /* references to original boundary: */
beginloadseq (Sy_ReadReferences)    
#2718sync314beg#                       /*
1 1 28 34                              1 = index to left contour (left with
                                       respect to branch traversed from
                                       tail node to head node). Correct for
                                       both internal contour representation and
                                       contours encoded according to the format
                                       specified in manual page contours(5).

                                       1 = index to right contour.
                 
                                       28 = index to left boundary point. 
                                       (the points are numbered consecutively
                                        from 1 to N within the contour 
                                        representation according to manual 
                                        page contours(5).
                                       34 = index to right boundary point. */
1 1 27 36
1 1 26 38
1 1 25 40
1 1 24 42
1 1 76 42

#2718sync314end#
endloadseq;

                                          /* boundary (offset vectors): */
                             /*             
                             Note that the offset vectors are defined with
                             respect to VORONOI EDGES, as every edge has
                             been generated by exactly two boundary points.
                             Consequently, if the branch has 6 vertices (of
                             the polygonal chain), there are only 5 entries
                             for each boundary (left and right) -- makes
                             a total of 10 entries. Let `blpoint' denote the 
	                     coordinates of the boundary point on left 
                             boundary which generates the edge connecting 
                             axis point axis[1] with axis[2].
                             Then
                                    blpoint = 0.5*(axis[2]+axis[1]) 
                                              + left_offset_vector. 
                             */

beginloadseq (Sy_ReadBndryOffsetVectors)  
#2718sync314beg#

0.000000 -2.000000                       /* left boundary offset vectors */
0.000000 -3.000000
0.000000 -4.000000
0.000000 -5.000000
-0.250000 -5.750000

2.000000 0.000000                        /* right boundary offset vectors */
3.000000 0.000000
4.000000 0.000000
5.000000 0.000000
5.750000 0.250000

#2718sync314end#
endloadseq;

} /* end branch no 1 */ 

object branch_2 { /* skeleton no 1 */
type            = skeletonbranch;
id              = 2;
user_id         = 0;
skel_id         = 1;
hierarchy_level = 0;
is_foreground   = true;
has_boundary    = true;
strength        = 3.514719;
length          = 6.363961;
max_strength    = 3.514719; min_strength = 1.757359;
mean_strength   = 2.636039; var_strength = 0.571909;
max_radius      = 6.000000; min_radius   = 1.581139;
mean_radius     = 3.952759; var_radius   = 2.950833;
tail_branch     = 3; head_branch  = 0;
tail_node       = 2; head_node    = 3;
elementdim      = 6;
axistype        = real;
/* symmetry axis: */
beginloadseq (Sy_ReadSymmAxis)
#2718sync314beg#
25.000000 9.000000
25.500000 9.500000
26.500000 10.500000
27.500000 11.500000
28.500000 12.500000
29.500000 13.500000

#2718sync314end#
endloadseq;
/* radii: */
beginloadseq (Sy_ReadRadiusList)
#2718sync314beg#
6.000000
5.522680
4.527693
3.535534
2.549510
1.581139

#2718sync314end#
endloadseq;
/* references to original boundary: */
beginloadseq (Sy_ReadReferences)
#2718sync314beg#
1 1 76 42
1 1 77 44
1 1 78 46
1 1 79 48
1 1 80 50
1 1 50 52

#2718sync314end#
endloadseq;
/* boundary (offset vectors): */
beginloadseq (Sy_ReadBndryOffsetVectors)
#2718sync314beg#
-0.250000 5.750000
0.000000 5.000000
0.000000 4.000000
0.000000 3.000000
0.000000 2.000000
5.750000 -0.250000
5.000000 0.000000
4.000000 0.000000
3.000000 0.000000
2.000000 0.000000

#2718sync314end#
endloadseq;

} /* end branch no 2 */ 

object branch_3 { /* skeleton no 1 */
type            = skeletonbranch;
id              = 3;
user_id         = 0;
skel_id         = 1;
hierarchy_level = 0;
is_foreground   = true;
has_boundary    = true;
strength        = 28.000000;
length          = 17.000000;
max_strength    = 28.000000; min_strength = 14.000000;
mean_strength   = 21.411764; var_strength = 23.882355;
max_radius      = 6.020797; min_radius   = 6.000000;
mean_radius     = 6.018608; var_radius   = 0.000041;
tail_branch     = 1; head_branch  = 5;
tail_node       = 2; head_node    = 4;
elementdim      = 19;
axistype        = real;
/* symmetry axis: */
beginloadseq (Sy_ReadSymmAxis)
#2718sync314beg#
25.000000 9.000000
24.500000 9.000000
23.500000 9.000000
22.500000 9.000000
21.500000 9.000000
20.500000 9.000000
19.500000 9.000000
18.500000 9.000000
17.500000 9.000000
16.500000 9.000000
15.500000 9.000000
14.500000 9.000000
13.500000 9.000000
12.500000 9.000000
11.500000 9.000000
10.500000 9.000000
9.500000 9.000000
8.500000 9.000000
8.000000 9.000000

#2718sync314end#
endloadseq;
/* radii: */
beginloadseq (Sy_ReadRadiusList)
#2718sync314beg#
6.000000
6.020797
6.020797
6.020797
6.020797
6.020797
6.020797
6.020797
6.020797
6.020797
6.020797
6.020797
6.020797
6.020797
6.020797
6.020797
6.020797
6.020797
6.000000

#2718sync314end#
endloadseq;
/* references to original boundary: */
beginloadseq (Sy_ReadReferences)
#2718sync314beg#
1 1 24 76
1 1 23 75
1 1 22 74
1 1 21 73
1 1 20 72
1 1 19 71
1 1 18 70
1 1 17 69
1 1 16 68
1 1 15 67
1 1 14 66
1 1 13 65
1 1 12 64
1 1 11 63
1 1 10 62
1 1 9 61
1 1 8 60
1 1 7 59
1 1 59 41

#2718sync314end#
endloadseq;
/* boundary (offset vectors): */
beginloadseq (Sy_ReadBndryOffsetVectors)
#2718sync314beg#
0.250000 -6.000000
0.000000 -6.000000
0.000000 -6.000000
0.000000 -6.000000
0.000000 -6.000000
0.000000 -6.000000
0.000000 -6.000000
0.000000 -6.000000
0.000000 -6.000000
0.000000 -6.000000
0.000000 -6.000000
0.000000 -6.000000
0.000000 -6.000000
0.000000 -6.000000
0.000000 -6.000000
0.000000 -6.000000
0.000000 -6.000000
-0.250000 -6.000000
0.250000 6.000000
0.000000 6.000000
0.000000 6.000000
0.000000 6.000000
0.000000 6.000000
0.000000 6.000000
0.000000 6.000000
0.000000 6.000000
0.000000 6.000000
0.000000 6.000000
0.000000 6.000000
0.000000 6.000000
0.000000 6.000000
0.000000 6.000000
0.000000 6.000000
0.000000 6.000000
0.000000 6.000000
-0.250000 6.000000

#2718sync314end#
endloadseq;

} /* end branch no 3 */ 

object branch_4 { /* skeleton no 1 */
type            = skeletonbranch;
id              = 4;
user_id         = 0;
skel_id         = 1;
hierarchy_level = 0;
is_foreground   = true;
has_boundary    = true;
strength        = 3.514719;
length          = 6.363961;
max_strength    = 3.514719; min_strength = 1.757359;
mean_strength   = 2.636039; var_strength = 0.571909;
max_radius      = 6.000000; min_radius   = 1.581139;
mean_radius     = 3.952759; var_radius   = 2.950833;
tail_branch     = 3; head_branch  = 0;
tail_node       = 4; head_node    = 5;
elementdim      = 6;
axistype        = real;
/* symmetry axis: */
beginloadseq (Sy_ReadSymmAxis)
#2718sync314beg#
8.000000 9.000000
7.500000 8.500000
6.500000 7.500000
5.500000 6.500000
4.500000 5.500000
3.500000 4.500000

#2718sync314end#
endloadseq;
/* radii: */
beginloadseq (Sy_ReadRadiusList)
#2718sync314beg#
6.000000
5.522680
4.527693
3.535534
2.549510
1.581139

#2718sync314end#
endloadseq;
/* references to original boundary: */
beginloadseq (Sy_ReadReferences)
#2718sync314beg#
1 1 7 41
1 1 6 39
1 1 5 37
1 1 4 35
1 1 3 33
1 1 33 31

#2718sync314end#
endloadseq;
/* boundary (offset vectors): */
beginloadseq (Sy_ReadBndryOffsetVectors)
#2718sync314beg#
0.250000 -5.750000
0.000000 -5.000000
0.000000 -4.000000
0.000000 -3.000000
0.000000 -2.000000
-5.750000 0.250000
-5.000000 0.000000
-4.000000 0.000000
-3.000000 0.000000
-2.000000 0.000000

#2718sync314end#
endloadseq;

} /* end branch no 4 */ 

object branch_5 { /* skeleton no 1 */
type            = skeletonbranch;
id              = 5;
user_id         = 0;
skel_id         = 1;
hierarchy_level = 0;
is_foreground   = true;
has_boundary    = true;
strength        = 3.514719;
length          = 6.363961;
max_strength    = 3.514719; min_strength = 1.757359;
mean_strength   = 2.636039; var_strength = 0.571909;
max_radius      = 6.000000; min_radius   = 1.581139;
mean_radius     = 3.952759; var_radius   = 2.950833;
tail_branch     = 4; head_branch  = 0;
tail_node       = 4; head_node    = 6;
elementdim      = 6;
axistype        = real;
/* symmetry axis: */
beginloadseq (Sy_ReadSymmAxis)
#2718sync314beg#
8.000000 9.000000
7.500000 9.500000
6.500000 10.500000
5.500000 11.500000
4.500000 12.500000
3.500000 13.500000

#2718sync314end#
endloadseq;
/* radii: */
beginloadseq (Sy_ReadRadiusList)
#2718sync314beg#
6.000000
5.522680
4.527693
3.535534
2.549510
1.581139

#2718sync314end#
endloadseq;
/* references to original boundary: */
beginloadseq (Sy_ReadReferences)
#2718sync314beg#
1 1 41 59
1 1 43 58
1 1 45 57
1 1 47 56
1 1 49 55
1 1 55 54

#2718sync314end#
endloadseq;
/* boundary (offset vectors): */
beginloadseq (Sy_ReadBndryOffsetVectors)
#2718sync314beg#
-5.750000 -0.250000
-5.000000 0.000000
-4.000000 0.000000
-3.000000 0.000000
-2.000000 0.000000
0.250000 5.750000
0.000000 5.000000
0.000000 4.000000
0.000000 3.000000
0.000000 2.000000

#2718sync314end#
endloadseq;

} /* end branch no 5 */ 
} /* end skeleton no 1 */ 
.fi
.LP
.SH C DATA STRUCTURES
.LP
.nf
typedef enum _SyBranchNodeType { SY_INTERNAL, SY_END, SY_NILNODE } 
  SyBranchNodeType;

typedef enum _SyBranchElementType {SY_REAL, 
                                   SY_RATIONAL, 
				   SY_INTEGER, 
				   SY_NILBRANCH
} SyBranchElementType;

typedef struct _SyRealPoint {
  float x, y;
} SyRealPoint;

typedef struct _SyIntPoint {
  int x, y;
} SyIntPoint;

typedef struct _SyRationalPoint {
  double xnum, ynum, 
         xden, yden;
} SyRationalPoint;

union SyGenericPoint {
  SyIntPoint      np;
  SyRationalPoint qp;
  SyRealPoint     rp;
};

typedef struct _SyVSkeletonRef {
  int left_contour, right_contour,
      left_point, right_point;
} SyVSkeletonRef;

typedef struct _SyVSkeletonBndryOffset {
  float xoff, 
        yoff;
} SyVSkeletonBndryOffset;

/* NOTE on array cell allocation for all arrays in 
   the data structures listed below: 
   N array elements are stored in array cells
   array[1]...array[N], array[0] remains void... */

typedef struct _SyVSkeletonBranch {
  SyBranchElementType etype;

  int        id, skel_id, user_id;
  int        hierarchy_level;
  SyBoolean  is_foreground,
             has_boundary;
  float      strength, 
             max_strength, min_strength, mean_strength, var_strength,
             length,
             max_radius, min_radius, mean_radius, var_radius;
             
  int        tail_branch, head_branch,
             tail_node, head_node;

  /* maxelementdim, maxaxisdim... etc. are used for internal 
     allocation purposes -- e.g. maxaxisdim denotes the maximal
     number of element of type union SyGenericPoint that the
     array axis[...] can accommodate. axisdim denotes the 
     actual number of elements that contains valid data. */

  int        elementdim, maxelementdim;

  int        axisdim, maxaxisdim, 
             radiusdim, maxradiusdim,
             refdim, maxrefdim,
             bndrydim, maxbndrydim;

  union SyGenericPoint   *axis;        /* array of axis points */
  float                  *radius;      /* array of local radii */
  SyVSkeletonRef         *ref;         /* array of boundary references */
  SyVSkeletonBndryOffset *left_bndry,  /* array of boundary offset vectors 
                                          for left boundary */
                         *right_bndry; /* array of boundary offset vectors 
                                          for right boundary */
} SyVSkeletonBranch;

typedef struct _SyVSkeletonNode {
  SyBranchElementType etype;
  SyBranchNodeType    ntype;

  int       id, skel_id, user_id;
  int       hierarchy_level;
  SyBoolean is_foreground;
  float     strength,
            radius;
  int       rank, maxrank;      /* number of incident branches */
  int       *inbranch;          /* references to all incident branches */
  union SyGenericPoint loc; 
} SyVSkeletonNode;

typedef struct _SyVSkeleton {
  int               id, user_id;
  int               branchdim, nodedim,
                    maxbranchdim, maxnodedim;
  SyBoolean         is_foreground;
  SyVSkeletonBranch *branch;   /* array of skeleton branches */
  SyVSkeletonNode   *node;     /* array of skeleton nodes    */
} SyVSkeleton;

typedef struct _SyVSkeletonSet {
  int         maxdim, dim;
  SyBoolean   is_foreground;   /* true if data solely consists of 
                                  foreground skeletons */
  SyVSkeleton *skel;           /* array of skeletons */
} SyVSkeletonSet;
.fi
.LP
.SH SEE ALSO
.BR ctran (1),
.BR voro (1),
.BR prune (1),
.BR contours (5)
.SH AUTHOR
.nf
Robert L. Ogniewicz
E-Mail: rlo@hrl.harvard.edu
.fi

This work has been carried out at the
.I "Communication Technology Laboratory," 
Swiss Federal Institute of
Technology ETH, Zurich, Switzerland, and the
.I "Division of Applied Sciences," 
Harvard University, Cambridge, USA.
.SH COPYRIGHT
Copyright (C) 1995 Robert L. Ogniewicz 
.SH BUGS
Undoubtedly many...
.\".Ee
